<?xml version="1.0" encoding="ISO-8859-1"?>

<!--
  Convert an article to HTML.
  2000/01/09 Max Kellermann
-->

<xsl:stylesheet	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
		version="1.0">

<xsl:output method="html"/>

<xsl:template match="article">
  <html>
    <head>
      <title><xsl:value-of select="head/title"/></title>
    </head>
    <body bgcolor="white">
      <h4><xsl:value-of select="head/author"/></h4>
      <h1><xsl:value-of select="head/title"/></h1>
      <xsl:apply-templates select="body"/>
        <xsl:variable name="date">
           <xsl:value-of select="substring-before(head/time, ' ')"/>
        </xsl:variable>
	<xsl:variable name="year">
           <xsl:value-of select="substring-before($date, '/')"/>
        </xsl:variable>
	<xsl:variable name="date2">
           <xsl:value-of select="substring-after($date, '/')"/>
        </xsl:variable>
	<xsl:variable name="month">
           <xsl:value-of select="substring-before($date2, '/')"/>
        </xsl:variable>
	<xsl:variable name="day">
           <xsl:value-of select="substring-after($date2, '/')"/>
        </xsl:variable>
	   
      <p>
	<xsl:value-of select="concat($day, '.', $month, '.', $year)"/>
      </p>
    </body>
  </html>
</xsl:template>

<!-- Paragraph -->
<xsl:template match="paragraph">
  <p><xsl:apply-templates/></p>
</xsl:template>

<xsl:template match="section">
  <h3><xsl:apply-templates/></h3>
</xsl:template>

<xsl:template match="subsection">
  <h4><xsl:apply-templates/></h4>
</xsl:template>

<!-- Bilder, Multimedia -->
<xsl:template match="break">
  <br></br>
</xsl:template>

<xsl:template match="image">
  <xsl:variable name="al">left</xsl:variable>
  <xsl:choose>
    <xsl:when test="@align='right'">
      <xsl:variable name="al">right</xsl:variable>
    </xsl:when>
    <xsl:when test="@align='center'">
      <xsl:variable name="al">center</xsl:variable>
    </xsl:when>
  </xsl:choose>
  <img src="{@src}" alt="{@alt}" align="{$al}"></img>
</xsl:template>

<!-- Formatierungen -->

<xsl:template match="emphasized">
  <b><xsl:apply-templates/></b>
</xsl:template>

<xsl:template match="emphasized2">
  <i><xsl:apply-templates/></i>
</xsl:template>

<xsl:template match="emphasized3">
  <u><xsl:apply-templates/></u>
</xsl:template>

<!-- Link -->

<xsl:template match="hyperlink">
  <a href="{@url}"><xsl:apply-templates/></a>
</xsl:template>

<!-- Listen -->

<xsl:template match="ul">
  <ul>
    <xsl:for-each select="paragraph">
      <li><xsl:apply-templates/></li>
    </xsl:for-each>
  </ul>
</xsl:template>

<xsl:template match="ol">
  <ol>
    <xsl:for-each select="paragraph">
      <li><xsl:apply-templates/></li>
    </xsl:for-each>
  </ol>
</xsl:template>

</xsl:stylesheet>
