import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.ResourceBundle;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.MissingResourceException;
import java.net.URL;
import java.net.MalformedURLException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import org.xml.sax.SAXException;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import com.make.xml.DOMUtil;
import com.make.xml.DocumentType;
import com.make.xml.FormatManager;
import com.make.editor.EditorDocument;
import com.make.editor.EditorDocumentType;
import com.make.editor.EditorApplication;
import com.make.editor.EditorModule;
import com.make.editor.filter.Filter;
import com.make.editor.plugins.PluginManager;
import com.make.swing.StatusWindow;
import com.make.util.Debug;

public class LeanEdit implements EditorApplication, FormatManager, 
				 EntityResolver {
    /***static**********************************/
    protected static final ResourceBundle resources
	= ResourceBundle.getBundle("Editor");
    protected static String[] formatNames;
    static {
	// formats
	try {
	    InputStream is = LeanEdit.class.getResourceAsStream("/formats/formats.xml");
	    Element el = DOMUtil.parse(is).getDocumentElement();
	    Element[] fs = DOMUtil.getChildElements(el);
	    formatNames = new String[fs.length];
	    for (int z=0; z<fs.length; z++)
		formatNames[z] = fs[z].getAttribute("name");
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }
    /***properties*************************************/
    private PluginManager pluginManager = new PluginManager();
    protected Hashtable documentTypes = new Hashtable();
    protected Hashtable modules = new Hashtable();
    protected Vector filters = new Vector();
    /***Constructor************************************/
    public LeanEdit() {
	try {
	    // register plugins
	    Document doc = DOMUtil.parse(getClass().getResourceAsStream("plugins/plugins.xml"));
	    pluginManager.registerPlugins(doc.getDocumentElement());
	} catch (Exception e) {
	    System.err.println("exception while parsing plugins.xml");
	    e.printStackTrace();
	}
	try {
	    String modules = resources.getString("modules");
	    StringTokenizer st = new StringTokenizer(modules);
	    while (st.hasMoreTokens()) {
		String moduleName = st.nextToken();
		try {
		    loadModule(moduleName);
		} catch (Exception e) {
		    System.err.println("ERROR: could not load module "+moduleName);
		}
	    }
	} catch (MissingResourceException e) {
	}
    }
    protected void loadModule(String className) 
	throws ClassNotFoundException, NoSuchMethodException,
	       InstantiationException, IllegalAccessException,
	       InvocationTargetException {
	Class cls = Class.forName(className);
	Class[] paramClasses = { EditorApplication.class };
	Constructor constructor = cls.getConstructor(paramClasses);
	Object[] parameters = { this };
	EditorModule module
	    = (EditorModule)constructor.newInstance(parameters);
	modules.put(module.toString(), module);
    }
    /***EditorApplication***************************************/
    public PluginManager getPluginManager() {
	return pluginManager;
    }
    public FormatManager getFormatManager() {
	return this;
    }
    public EntityResolver getEntityResolver() {
	return this;
    }
    public Enumeration getModules() {
	return modules.elements();
    }
    public Enumeration getFilters() {
	return filters.elements();
    }
    public void addFilter(Filter flt) {
	filters.addElement(flt);
    }
    public void removeFilter(Filter flt) {
	filters.removeElement(flt);
    }
    /***FormatManager**********************************/
    public String[] getFormatNames() {
	return formatNames;
    }
    public String getFormatName(Document doc) {
	if (doc==null)
	    return null;
	Element el = doc.getDocumentElement();
	if (el==null)
	    return null;
	return el.getTagName();
    }
    public DocumentType getFormat(String name) {
	EditorDocumentType dt = (EditorDocumentType)documentTypes.get(name);
	if (dt==null)
	    try {
		dt = new EditorDocumentType(this, name);
		documentTypes.put(name, dt);
	    } catch (FileNotFoundException e) {
	    }
	return dt;
    }
    /***EntityResolver********************************/
    public InputSource resolveEntity(String publicId,
				     String systemId)
	throws SAXException, IOException {
	try {
	    URL url = new URL(systemId);
	    String fileName = url.getFile();
	    InputStream is = getClass().getResourceAsStream(fileName);
	    if (is==null) {
		StringTokenizer st = new StringTokenizer(url.getFile(), "/");
		String file = null;
		while (st.hasMoreTokens())
		    file = st.nextToken();
		if (file==null)
		    return DOMUtil.emptyEntityResolver
			.resolveEntity(publicId, systemId);
		is = getClass().getResourceAsStream("/entities/"+file);
		if (is==null)
		    return DOMUtil.emptyEntityResolver
			.resolveEntity(publicId, systemId);
	    }
	    Reader rd = new InputStreamReader(is, "ISO-8859-1");
	    return new InputSource(rd);
	} catch (MalformedURLException e) {
	} catch (IOException e) {
	}
	return DOMUtil.emptyEntityResolver
	    .resolveEntity(publicId, systemId);
    }
    /***main*******************************************/
    public static void main(final String[] args) {
	StatusWindow wnd = new StartWindow(resources, new Runnable() {
	    public void run() {
		try {
		    LeanEdit leanEdit = new LeanEdit();
		    LeanEditSession red = new LeanEditSession(leanEdit);
		    boolean ok = false;
		    if (args.length>0) {
			for (int z=0; z<args.length; z++)
			    if (red.open(new File(args[z])))
				ok = true;
		    }
		    if (!ok)
			red.createNewFrame();
		} catch (ExceptionInInitializerError e) {
		    System.out.println("uncaught exception: " + e);
		    e.printStackTrace();
		    e.getException().printStackTrace();
		} catch (Exception t) {
		    System.out.println("uncaught exception: " + t);
		    t.printStackTrace();
		}
	    }
	});
    }
}
