import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.datatransfer.Clipboard;
import java.io.IOException;
import java.io.InputStream;
import java.io.File;
import java.util.ResourceBundle;
import java.util.Enumeration;
import java.util.Vector;
import javax.swing.AbstractAction;
import javax.swing.Action;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;
import javax.swing.JOptionPane;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import com.make.xml.DOMUtil;
import com.make.editor.EditorApplication;
import com.make.editor.EditorSession;
import com.make.editor.EditorModule;
import com.make.editor.EditorFrame;
import com.make.editor.TopEditorFrame;
import com.make.editor.plugins.PluginManager;
import com.make.editor.plugins.EditorPlugin;
import com.make.datamodel.embedded.EmbeddedManager;
import com.make.swing.InterfacePanel;
import com.make.swing.FrameInterfacePanel;
import com.make.util.Debug;
import com.make.util.ArrayEnumeration;
import com.make.util.AppendEnumeration;
import com.make.swing.ThreadAction;
import com.make.swing.ProxyClipboard;

class XmlFileFilter extends FileFilter {
    /***FileFilter******************************/
    public boolean accept(File f) {
	if (f!=null) {
	    if (f.isDirectory()) return true;
	    String name = f.getName();
	    return name.endsWith(".xml") || name.endsWith(".xz");
	}
	return false;
    }
    public String getDescription() {
	return "eXtensible Markup Language (*.xml, *.xz)";
    }
}

class ZipFileFilter extends FileFilter {
    /***FileFilter******************************/
    public boolean accept(File f) {
	if (f!=null) {
	    if (f.isDirectory()) return true;
	    String name = f.getName();
	    return name.endsWith(".zip");
	}
	return false;
    }
    public String getDescription() {
	return "XML-ZIP (*.zip)";
    }
}

public class LeanEditSession implements EditorSession {
    /***static**********************************/
    protected static final JFileChooser fileChooser = new JFileChooser();
    static {
	fileChooser.addChoosableFileFilter(new XmlFileFilter());
	fileChooser.addChoosableFileFilter(new ZipFileFilter());
    }
    /***properties******************************/
    protected EditorApplication application;
    protected final Toolkit toolkit = Toolkit.getDefaultToolkit();
    protected final Clipboard clipboard
	= new ProxyClipboard(toolkit.getSystemClipboard());
    private Vector additionalActions;
    /***Constructor*********************************/
    public LeanEditSession(EditorApplication _app) {
	application = _app;
	for (Enumeration enum=application.getModules(); enum.hasMoreElements();)
	    ((EditorModule)enum.nextElement()).install(this);
    }
    public void createNewFrame() {
	InterfacePanel ip = createInterfacePanel();
	new TopEditorFrame(this, ip);
	ip.show();
    }
    /***EditorSession***********************************************/
    public Enumeration getActions() { 
	Enumeration da = new ArrayEnumeration(defaultActions);
	return additionalActions==null
	    ?da
	    :new AppendEnumeration(da, additionalActions.elements());
    }
    public InterfacePanel createInterfacePanel() {
	return new FrameInterfacePanel();
    }
    public EditorApplication getApplication() { return application; }
    public Clipboard getClipboard() {
	return clipboard;
    }
    public EditorFrame open(EmbeddedManager em, InterfacePanel ip)
	throws IOException, SAXException {
	if (ip==null)
	    ip = createInterfacePanel();
	TopEditorFrame tef = new TopEditorFrame(this, ip, em);
	ip.show();
	return tef;
    }
    public void addActions(Enumeration enum) {
	if (additionalActions==null)
	    additionalActions = new Vector();
	while (enum.hasMoreElements())
	    additionalActions.addElement((Action)enum.nextElement());
    }
    public void removeActions(Enumeration enum) {
	while (enum.hasMoreElements())
	    additionalActions.removeElement((Action)enum.nextElement());
    }
    /***Actions********************************************/
    private Action[] defaultActions = {
	new ThreadAction(new NewFrameAction()),
	new ThreadAction(new OpenAction()),
	new AboutAction(),
    };
    class NewFrameAction extends AbstractAction {
	NewFrameAction() { super("newframe"); }
        public void actionPerformed(ActionEvent e) {
	    createNewFrame();
	}
    }
    public boolean open(File f) {
	try {
	    if (!f.exists()) {
		JOptionPane.showMessageDialog
		    (null, "Die Datei "+f.getAbsolutePath()+" existiert nicht.",
		     "Fehler", JOptionPane.ERROR_MESSAGE, null);
		return false;
	    }
	    if (f.isDirectory()) {
		JOptionPane.showMessageDialog
		    (null, "Die Datei "+f.getAbsolutePath()+" ist ein Verzeichnis.",
		     "Fehler", JOptionPane.ERROR_MESSAGE, null);
		return false;
	    }
	    if (!f.canRead()) {
		JOptionPane.showMessageDialog
		    (null, "Zugriff auf die Datei "+f.getAbsolutePath()+" verweigert.",
		     "Fehler", JOptionPane.ERROR_MESSAGE, null);
		return false;
	    }
	} catch (SecurityException e) {
	    JOptionPane.showMessageDialog
		(null, "Zugriff auf die Datei "+f.getAbsolutePath()+" verweigert.",
		 "Fehler", JOptionPane.ERROR_MESSAGE, null);
	    return false;
	}
	try {
	    InterfacePanel ip = createInterfacePanel();
	    TopEditorFrame tef = new TopEditorFrame(this, ip, f);
	    ip.show();
	    return true;
	} catch (Exception ee) {
	    JOptionPane.showMessageDialog
		(null, ee.getLocalizedMessage(), 
		 "Fehler beim Laden", JOptionPane.ERROR_MESSAGE, null);
	    return false;
	}
    }
    class OpenAction extends AbstractAction {
	OpenAction() { super("open-file"); }
        public void actionPerformed(ActionEvent e) {
	    fileChooser.getUI().rescanCurrentDirectory(fileChooser);
	    final int chooseRet = fileChooser.showOpenDialog(null);
	    if (chooseRet!=fileChooser.APPROVE_OPTION)
		return;
	    File f = fileChooser.getSelectedFile();
	    if (f!=null) open(f);
	}
    }
    class AboutAction extends AbstractAction {
	AboutAction() { super("about"); }
        public void actionPerformed(ActionEvent e) {
	    new AboutDialog(null).show();
	}
    }
}
