package com.make.datamodel.embedded;

import java.io.File;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.ByteArrayInputStream;
import java.io.OutputStream;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.util.Hashtable;
import org.w3c.dom.Node;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;
import com.make.util.Streams;
import com.make.xml.DOMUtil;
import com.make.xml.DocumentType;
import com.make.xml.FormatManager;

public class FileEmbeddedManager implements EmbeddedManager {
    /***properties*****************************/
    protected FormatManager formatManager;
    protected File path, file;
    /***Constructor*****************************/
    public FileEmbeddedManager(File _f, FormatManager _fm) throws IOException {
	if (!_f.isAbsolute())
	    _f = new File(_f.getAbsolutePath());
	file = _f;
	formatManager = _fm;
	path = new File(file.getParent());
    }
    public String toString() { return file.toString(); }
    /***EmbeddedManager******************************************/
    public String[] getNames() {
	String[] list = path.list();
	int count = 0;
	for (int z=0; z<list.length; z++) {
	    File f = new File(path, list[z]);
	    if (f.isFile())
		list[count++] = list[z];
	}
	String[] names = new String[count];
	System.arraycopy(list, 0, names, 0, count);
	return names;
    }
    public Hashtable getSizes() {
	Hashtable ht = new Hashtable();
	String[] list = path.list();
	for (int z=0; z<list.length; z++) {
	    File f = new File(path, list[z]);
	    if (f.isFile())
		ht.put(list[z], new Long(f.length()));
	}
	return ht;
    }
    public boolean contains(String name) {
	File f = new File(path, name);
	return f.exists() && f.isFile();
    }
    public void replaceEntry(String name, InputStream data)
	throws IOException {
	File f = new File(path, name);
	File temp = new File(f.getPath()+".tmp");
	if (data!=null) {
	    FileOutputStream fos = new FileOutputStream(temp);
	    Streams.copy(data, fos);
	    fos.close();
	}
	if (f.exists())
	    f.delete();
	temp.renameTo(f);
    }
    public InputStream getEntry(String name) throws IOException {
	return new FileInputStream(new File(path, name));
    }
    public void replaceDocument(Node n) throws IOException {
	final Document doc = n.getOwnerDocument();
	DocumentType type = null;
	if (formatManager!=null) {
	    String tn = formatManager.getFormatName(doc);
	    if (tn!=null)
		type = formatManager.getFormat(tn);
	}
	byte[] data = DOMUtil.toByteArray(doc, type);
	replaceEntry(file.getName(), new ByteArrayInputStream(data));
    }
    public Node getDocument() throws IOException, SAXException {
	return DOMUtil.parse(getEntry(file.getName())).getDocumentElement();
    }
    public boolean equals(Object o) {
	return o instanceof FileEmbeddedManager &&
	    ((FileEmbeddedManager)o).path.equals(path) &&
	    ((FileEmbeddedManager)o).file.equals(file);
    }
}
