package com.make.datamodel.embedded;

import java.io.IOException;
import java.io.InputStream;
import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.Serializable;
import java.util.Hashtable;
import java.util.Enumeration;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;
import com.make.util.Streams;
import com.make.xml.DOMUtil;

public class MemoryEmbeddedManager 
    implements EmbeddedManager, Serializable {
    /***properties*******************************/
    protected Hashtable entries = new Hashtable();
    protected Node document;
    protected String name;
    /***Constructor*****************************/
    public MemoryEmbeddedManager() { name = "[RAM]"; }
    public MemoryEmbeddedManager(String _n) { name = _n; }
    public String toString() { return name; }
    /***EmbeddedManager***************************/
    public String[] getNames() throws IOException {
	String[] names = new String[entries.size()];
	int z=0;
	for (Enumeration enum=entries.keys(); enum.hasMoreElements();)
	    names[z++] = (String)enum.nextElement();
	return names;
    }
    public Hashtable getSizes() throws IOException {
	Hashtable sz = new Hashtable();
	for (Enumeration enum=entries.keys(); enum.hasMoreElements();) {
	    String name = (String)enum.nextElement();
	    sz.put(name, new Long(((byte[])entries.get(name)).length));
	}
	return sz;
    }
    public boolean contains(String name) { 
	return entries.containsKey(name); 
    }
    public void replaceEntry(String name, InputStream data)
	throws IOException {
	if (data==null)
	    entries.remove(name);
	else {
	    ByteArrayOutputStream os = new ByteArrayOutputStream();
	    Streams.copy(data, os);
	    entries.put(name, os.toByteArray());
	}
    }
    public InputStream getEntry(String name) throws IOException {
	byte[] data = (byte[])entries.get(name);
	return data==null?null:new ByteArrayInputStream(data);
    }
    public void replaceDocument(Node n) throws IOException {
	document = n;
    }
    public Node getDocument() throws IOException, SAXException {
	return document;
    }
    public boolean equals(Object o) { return o==this; }
    /***Serialization****************************/
    private void writeObject(java.io.ObjectOutputStream out)
	throws IOException {
	out.writeByte(1);
	out.writeUTF(name);
	out.writeObject(entries);
	byte[] data = document==null
	    ?null
	    :DOMUtil.toByteArray(document.getOwnerDocument());
	out.writeObject(data);
    }
    private void readObject(java.io.ObjectInputStream in)
	throws IOException, ClassNotFoundException {
	in.readByte();
	name = in.readUTF();
	entries = (Hashtable)in.readObject();
	byte[] data = (byte[])in.readObject();
	try {
	    document = data==null
		?null
		:DOMUtil.parse(data).getDocumentElement();
	} catch (SAXException e) {
	    throw new IOException(e.getLocalizedMessage());
	}
    }
}
