package com.make.datamodel.embedded;

import java.util.Enumeration;
import java.util.Vector;
import java.util.Hashtable;
import java.io.File;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.ByteArrayInputStream;
import java.io.OutputStream;
import java.io.FileOutputStream;
import java.util.zip.ZipFile;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;
import org.w3c.dom.Node;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;
import com.make.util.Streams;
import com.make.xml.DOMUtil;
import com.make.xml.DocumentType;
import com.make.xml.FormatManager;

public class ZipEmbeddedManager implements EmbeddedManager {
    /***CONST***********************************/
    public static final String DOCUMENT_NAME = "document.xml";
    public static final String DUMMY_NAME = "_dummy";
    /***properties*****************************/
    protected FormatManager formatManager;
    protected File file;
    protected ZipFile images;
    /***Constructor*****************************/
    public ZipEmbeddedManager(File _f, FormatManager _fm) throws IOException {
	file = _f; formatManager = _fm;
	open();
    }
    public String toString() { return file.getName(); }
    /***internal**********************************/
    protected void open() throws IOException {
	try {
	    images = new ZipFile(file);
	} catch (IOException e) {
	    FileOutputStream fos = new FileOutputStream(file);
	    ZipOutputStream zos = new ZipOutputStream(fos);
	    //zos.setMethod(zos.STORED);
	    zos.putNextEntry(new ZipEntry(DUMMY_NAME));
	    zos.closeEntry();
	    zos.close();
	    images = new ZipFile(file);
	}
    }
    protected static void copy(ZipOutputStream zos, ZipFile zf, ZipEntry ze) 
	throws IOException {
	zos.putNextEntry(new ZipEntry(ze.getName()));
	Streams.copy(zf.getInputStream(ze), zos);
	zos.closeEntry();
    }
    protected static int copy(ZipOutputStream zos, ZipFile zf, String nc) 
	throws IOException {
	int count = 0;
	for (Enumeration enum=zf.entries(); enum.hasMoreElements();) {
	    ZipEntry ze = (ZipEntry)enum.nextElement();
	    if (!ze.getName().equals(nc)) {
		copy(zos, zf, ze);
		count++;
	    }
	}
	return count;
    }
    protected ZipEntry findEntry(String name) {
	if (images==null) return null;
	for (Enumeration enum=images.entries(); enum.hasMoreElements();) {
	    ZipEntry ze = (ZipEntry)enum.nextElement();
	    if (name.equals(ze.getName()))
		return ze;
	}
	return null;
    }
    /***EmbeddedManager******************************************/
    public String[] getNames() {
	Vector entries = new Vector();
	for (Enumeration enum=images.entries(); enum.hasMoreElements();) {
	    ZipEntry ze = (ZipEntry)enum.nextElement();
	    entries.addElement(ze.getName());
	}
	String[] names = new String[entries.size()];
	int z=0;
	for (Enumeration enum=entries.elements(); enum.hasMoreElements();)
	    names[z++] = (String)enum.nextElement();
	return names;
    }
    public Hashtable getSizes() {
	Hashtable ht = new Hashtable();
	for (Enumeration enum=images.entries(); enum.hasMoreElements();) {
	    ZipEntry ze = (ZipEntry)enum.nextElement();
	    ht.put(ze.getName(), new Long(ze.getSize()));
	}
	return ht;
    }
    public boolean contains(String name) {
	return findEntry(name)!=null;
    }
    public void replaceEntry(String name, InputStream data)
	throws IOException {
	File temp = new File(file.getPath()+".tmp");
	FileOutputStream fos = new FileOutputStream(temp);
	ZipOutputStream zos = new ZipOutputStream(fos);
	//zos.setMethod(zos.STORED);
	final int count = copy(zos, images, name);
	if (name!=null && data!=null) {
	    zos.putNextEntry(new ZipEntry(name));
	    Streams.copy(data, zos);
	    zos.closeEntry();
	} else if (count==0) {
	    zos.putNextEntry(new ZipEntry(DUMMY_NAME));
	    zos.closeEntry();
	}
	zos.finish(); zos.close();
	fos.close();
	images.close(); file.delete();
	temp.renameTo(file);
	open();
    }
    public InputStream getEntry(String name) throws IOException {
	ZipEntry ze = findEntry(name);
	return ze==null ? null : images.getInputStream(ze);
    }
    public void replaceDocument(Node n) throws IOException {
	final Document doc = n.getOwnerDocument();
	DocumentType type = null;
	if (formatManager!=null) {
	    String tn = formatManager.getFormatName(doc);
	    if (tn!=null)
		type = formatManager.getFormat(tn);
	}
	byte[] data = DOMUtil.toByteArray(doc, type);
	replaceEntry(DOCUMENT_NAME, new ByteArrayInputStream(data));
    }
    public Node getDocument() throws IOException, SAXException {
	return DOMUtil.parse(getEntry(DOCUMENT_NAME)).getDocumentElement();
    }
    public boolean equals(Object o) {
	return o instanceof ZipEmbeddedManager &&
	    ((ZipEmbeddedManager)o).file.equals(file);
    }
}
