package com.make.editor;

import java.util.Vector;
import java.util.Enumeration;
import java.io.InputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import javax.swing.JOptionPane;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.Attr;
import org.w3c.dom.NodeList;
import org.w3c.dom.NamedNodeMap;
import org.xml.sax.SAXException;
import org.apache.xalan.xpath.XObject;
import org.apache.xalan.xpath.XPath;
import com.make.datamodel.embedded.EmbeddedManager;
import com.make.util.Debug;
import com.make.util.ArrayEnumeration;
import com.make.xml.XPathUtil;
import com.make.xml.DOMUtil;

public final class EditorDocument {
    /***properties*******************************/
    protected Node node;
    protected EditorDocumentType type;
    protected EmbeddedManager embeddedManager;
    protected EditorDocument parent;
    /***Constructor******************************/
    public EditorDocument(Node _el, EditorDocumentType _dt,
			  EmbeddedManager _em) {
	if (Debug.DEBUG && _el!=_el.getOwnerDocument().getDocumentElement())
	    throw new InternalError();
	node = _el; type = _dt;
	embeddedManager = _em;
    }
    public EditorDocument(EditorDocumentType _dt,
			  EmbeddedManager _em) 
	throws IOException, SAXException {
	type = _dt;
	embeddedManager = _em;
	node = embeddedManager.getDocument();
    }
    public EditorDocument(Node _el, EditorDocumentType _dt,
			  EditorDocument _parent) {
	node = _el; type = _dt;
	parent = _parent;
    }
    public EditorDocument(Node _el, EditorDocument _parent) {
	this(_el, _parent.getType(), _parent);
    }
    /***IO*************************************/
    public void writeDocument(EmbeddedManager em) throws IOException {
	em.replaceDocument(node);
    }
    public void writeDocument() throws IOException {
	writeDocument(getEmbeddedManager());
    }
    /***Info*************************************/
    public Enumeration getEmbeddedNames() {
	return type==null
	    ?new Vector().elements()
	    :new ArrayEnumeration(type.getEmbeddedNames(node));
    }
    public void copyEmbedded(EmbeddedManager src, EmbeddedManager dest) {
	if (src.equals(dest))
	    return;
	for (Enumeration enum=getEmbeddedNames(); enum.hasMoreElements();) {
	    String name = (String)enum.nextElement();
	    if (name.equals("_dummy")) continue;
	    InputStream is = null;
	    try {
		is = src.getEntry(name);
		if (is!=null)
		    dest.replaceEntry(name, is);
	    } catch (FileNotFoundException e) {
	    } catch (IOException e) {
		e.printStackTrace();
		JOptionPane.showMessageDialog
		    (null, "Fehler beim Kopieren der eingebetteten Objekte: "+
		     e.getLocalizedMessage(),
		     "Fehler beim Kopieren", JOptionPane.ERROR_MESSAGE);
	    } finally {
		try {
		    if (is!=null)
			is.close();
		} catch (IOException e) {
		}
	    }
	}
    }
    public void putNewEmbeddedManager(EmbeddedManager _em) {
	if (embeddedManager!=null)
	    copyEmbedded(embeddedManager, _em);
	embeddedManager = _em;
    }
    public void setEmbeddedManager(EmbeddedManager _em) {
	embeddedManager = _em;
    }
    public XObject execute(String s) throws SAXException {
	if (type==null) {
	    return XPathUtil.execute(s, node);
	} else {
	    XPath pt = type.parseXPath(s);
	    return XPathUtil.execute(pt, node);
	}
    }
    public EditorDocument[] followPath(String path, boolean create) 
	throws SAXException {
	NodeList nl = execute(path).nodeset();
	EditorDocument[] docs;
	if (nl.getLength()==0 && create) {
	    docs = new EditorDocument[1];
	    Element data = DOMUtil.createDeepUniqueTag((Element)getNode(), path);
	    docs = new EditorDocument[] {
		new EditorDocument(data, this),
	    };
	} else {
	    docs = new EditorDocument[nl.getLength()];
	    for (int z=0; z<docs.length; z++)
		docs[z] = new EditorDocument((Element)nl.item(z), this);
	}
	return docs;
    }
    /***EditorDocument*****************************/
    public Node getNode() { return node; }
    public Document getXMLDocument() {
	return node.getOwnerDocument();
    }
    public EditorDocumentType getType() { return type; }
    public EmbeddedManager getEmbeddedManager() {
	/*if (embeddedManager==null)
	  JOptionPane.showMessageDialog
	  (null, "Bitte speichern Sie das Dokument, bevor Sie Bilder und "+
	  "andere eingebettete Objekte einf\u00fcgen.",
	  "Fehler", JOptionPane.WARNING_MESSAGE);*/
	return parent==null?embeddedManager:parent.getEmbeddedManager();
    }
    public EditorDocument getParent() { return parent; }
}
