package com.make.editor;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.FileNotFoundException;
import java.net.URL;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Random;
import java.util.Date;
import java.text.DateFormat;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.Attr;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import org.apache.xalan.xslt.XSLTProcessor;
import org.apache.xalan.xslt.XSLTProcessorFactory;
import org.apache.xalan.xslt.XSLTInputSource;
import org.apache.xalan.xslt.XSLTResultTarget;
import org.apache.xalan.xpath.XPathFactory;
import org.apache.xalan.xpath.XPathSupport;
import org.apache.xalan.xpath.XPath;
import org.apache.xalan.xpath.XObject;
import org.apache.xalan.xpath.XNumber;
import org.apache.xalan.xpath.XString;
import org.apache.xalan.xpath.XNull;
import org.apache.xalan.xpath.Function;
import com.make.xml.DocumentType;
import com.make.xml.DOMUtil;
import com.make.xml.XPathUtil;
import com.make.util.Debug;
import com.make.util.Strings;

public class EditorDocumentType extends DocumentType {
    /***properties******************************/
    protected EditorDocumentView defaultView;
    protected Hashtable views = new Hashtable();
    protected EditorApplication application;
    protected XSLTProcessor xslp;
    /***Constructor********************************/
    public EditorDocumentType(EditorApplication _app, String name) 
	throws FileNotFoundException {
	super(_app.getClass().getResource("/formats/"+name+"/"),
	      _app.getEntityResolver());
	try {
	    xslp = XSLTProcessorFactory.getProcessorUsingLiaisonName
		("org.apache.xalan.xpath.xdom.XercesLiaison");
	} catch (SAXException e) {
	    e.printStackTrace();
	}
	application = _app;
	// views
	Element v = parse("views.xml");
	if (v==null)
	    v = getMisc("views");
	loadViews(v);
	// lists
	Element lists = parse("lists.xml");
	if (lists!=null)
	    loadLists(lists);
    }
    protected void loadViews(Element pl) {
	if (pl==null) return;
	String defaultName = pl.getAttribute("default");
	Element[] vis = DOMUtil.getChildElements(pl, "view");
	for (int z=0; z<vis.length; z++) {
	    EditorDocumentView view = new EditorDocumentView(this, vis[z]);
	    String name = view.getName();
	    views.put(name, view);
	    if (name.equals(defaultName))
		defaultView = view;
	}
    }
    /***Info*******************************/
    public EditorDocumentView getView(String name) {
	return (EditorDocumentView)views.get(name);
    }
    public EditorDocumentView getDefaultView() { 
	return defaultView;
    }
    public Enumeration getViews() { return views.elements(); }
    public EditorApplication getApplication() { return application; }
    /***DocumentType*****************************/
    public URL getResource(String uri) {
	return getClass().getResource("/formats/"+name+'/'+uri);
    }
    /***Functions*******************************/
    static class FuncRandomInt32 extends Function {
	Random random = new Random();
	public XObject execute(XPath path, XPathSupport execContext, 
			       Node context, int opPos, Vector args) {
	    return new XNumber(random.nextInt());
	}
    }
    static class FuncNow extends Function {
	DateFormat format = Strings.storeDateFormats[Strings.DATETIME];
	public XObject execute(XPath path, XPathSupport execContext, 
			       Node context, int opPos, Vector args) {
	    return new XString(format.format(new Date()));
	}
    }
    static {
	XPath.installFunction("random-int32", new FuncRandomInt32());
	XPath.installFunction("now", new FuncNow());
    }
    /***Misc.***********************************/
    public Document createNewDocument(EditorSession session) {
	Document result = null;
	try {
	    URL url = getResource("new.xml");
	    if (xslp!=null && url!=null) {
		InputStream is = url.openStream();
		Reader rd = new InputStreamReader(is, "ISO-8859-1");
		Document xmlDoc = DOMUtil.createDocument();
		DOMUtil.setUserData(xmlDoc, "editorSession", session);
		xmlDoc.appendChild(xmlDoc.createElement("null"));
		result = DOMUtil.createDocument();
		synchronized(xslp) {
		    xslp.process(new XSLTInputSource(xmlDoc), 
				 new XSLTInputSource(rd),
				 new XSLTResultTarget(result));
		}
	    }
	} catch (Exception e) {
	    e.printStackTrace(System.err);
	    result = null;
	}
	if (result==null) {
	    result = DOMUtil.createDocument();
	    result.appendChild(result.createElement(name));
	}
	return result;
    }
}
