package com.make.editor;

import java.awt.Frame;
import java.awt.Component;
import java.awt.BorderLayout;
import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import java.util.Hashtable;
import java.util.Enumeration;
import java.text.ParseException;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeSupport;
import javax.swing.Action;
import javax.swing.AbstractAction;
import javax.swing.JOptionPane;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import com.make.util.Debug;
import com.make.swing.InterfacePanel;
import com.make.swing.PanelContent;
import com.make.swing.bt.FrameDescription;
import com.make.editor.plugins.EditorPlugin;
import com.make.editor.plugins.EditorComponent;

public abstract class EditorPanel extends DefaultPanel 
    implements EditorFrame, PropertyChangeListener {
    /***properties*************************/
    private PropertyChangeSupport pcs = new PropertyChangeSupport(this);
    protected EditorComponent editorComponent;
    protected Component installedComponent;
    protected EditorPlugin plugin;
    private EditorDocument document;
    protected EditorDocumentView view;
    protected Hashtable components = new Hashtable();
    /***Constructor*****************************/
    public EditorPanel(EditorSession _sess, InterfacePanel _ip, LayoutManager lm) {
	super(_sess, _ip, lm);
    }
    public EditorPanel(EditorSession _sess, InterfacePanel _ip) {
	this(_sess, _ip, new BorderLayout());
    }
    /***View*********************************/
    protected void setEditorPlugin(EditorPlugin _plugin, EditorDocumentView _v) {
	if (plugin==_plugin && view==_v) return;
	final EditorComponent oldComponent = editorComponent;
	final EditorDocumentView oldView = view;
	// remove old component
	if (editorComponent!=null) {
	    if (_plugin!=null && document!=null &&
		getDocument()==null)
		return;
	    editorComponent.removePropertyChangeListener(this);
	    panel.remove(installedComponent);
	    installedComponent = null;
	    editorComponent.dispose();
	    editorComponent = null;
	}
	if (Debug.DEBUG && components.size()>0) {
	    System.err.println("some unregistered components:");
	    for (Enumeration enum=components.elements(); 
		 enum.hasMoreElements();)
		System.err.println(enum.nextElement());
	    throw new InternalError();
	}
	// install new plugin
	plugin = _plugin;
	// create new component
	if (plugin!=null && document!=null) {
	    view = _v;
	    // component
	    editorComponent = plugin.createComponent(this);
	    editorComponent.setDocument(document);
	    editorComponent.focusGained();
	    installedComponent = editorComponent.getComponent();
	    if (installedComponent!=null)
		panel.add("Center", installedComponent);
	    editorComponent.addPropertyChangeListener(this);
	} else
	    view = null;
	panel.validate();
	panel.repaint();
	// fire
	if (editorComponent!=oldComponent)
	    firePropertyChange(COMPONENT, oldComponent, editorComponent);
	if (view!=oldView)
	    firePropertyChange(VIEW, oldView, view);
    }
    protected void setView(EditorDocumentView _vn) {
	if (_vn==null)
	    setEditorPlugin(null, null);
	else if (_vn!=view) {
	    EditorDocumentType edt = document.getType();
	    EditorPlugin pl = _vn.loadPlugin(); 
	    setEditorPlugin(pl, _vn);
	}
    }
    public void setDocument(EditorDocument _doc, EditorPlugin _plugin, 
			    EditorDocumentView _v) {
	EditorDocument oldDocument = document;
	setEditorPlugin(null, null);
	document = _doc;
	try {
	    setEditorPlugin(_plugin, _v);
	} catch (Exception e) {
	    System.err.println("Exception caught during component creation");
	    e.printStackTrace();
	    setEditorPlugin(null, null);
	}
	firePropertyChange(DOCUMENT, oldDocument, document);
    }
    public void setDocument(EditorDocument _doc) {
	if (_doc==null) {
	    setDocument(null, null, null);
	    interfacePanel.setTitle(null);
	} else {
	    EditorDocumentType edt = _doc.getType();
	    EditorDocumentView view = edt==null
		?new EditorDocumentView(getApplication().getPluginManager())
		    :edt.getDefaultView();
	    EditorPlugin plugin = view.loadPlugin();
	    setDocument(_doc, plugin, view);
	    interfacePanel.setTitle(_doc.getEmbeddedManager().toString());
	}
    }
    /***Util********************************/
    protected void firePropertyChange(String name, Object o, Object n) {
	pcs.firePropertyChange(name, o, n);
    }
    /***Info**********************************/
    public final EditorSession getSession() { 
	return (EditorSession)super.getSwingSession(); 
    }
    public final EditorApplication getApplication() { 
	return getSession().getApplication(); 
    }
    /***EditorPanel******************************/
    protected boolean documentClosing() {
	return true;
    }
    /***EditorFrame********************************/
    public EditorDocumentView getView() { return view; }
    public void registerComponent(Node key, EditorComponent ec) {
	Object ov = components.put(key, ec);
	if (Debug.DEBUG && ov!=null)
	    throw new InternalError();
    }
    public void unregisterComponent(Node key, EditorComponent ec) {
	if (Debug.DEBUG) {
	    Object ov = components.get(key);
	    if (ov!=ec)
		throw new InternalError();
	}
	components.remove(key);
    }
    public EditorComponent getComponent(Node key) {
	return key==null
	    ?editorComponent
	    :(EditorComponent)components.get(key);
    }
    public EditorDocument getDocument() {
	if (document==null)
	    return null;
	else {
	    try {
		editorComponent.flushDocument();
		return document;
	    } catch (ParseException e) {
		JOptionPane.showMessageDialog(getFrame(), e.getLocalizedMessage(),
					      "Eingabefehler", JOptionPane.ERROR_MESSAGE);
		return null;
	    }
	}
    }
    public void addActions(Enumeration enum) {
	interfacePanel.getButtonManager().installActions(enum);
    }
    public void removeActions(Enumeration enum) {
	interfacePanel.getButtonManager().uninstallActions(enum);
    }
    public void addPropertyChangeListener(PropertyChangeListener l) {
	pcs.addPropertyChangeListener(l);
    }
    public void removePropertyChangeListener(PropertyChangeListener l) {
	pcs.removePropertyChangeListener(l);
    }
    /***PanelContent**************************/
    public boolean panelClosing() {
	return super.panelClosing() && documentClosing();
    }
    public void panelClosed() {
	setDocument(null);
	if (editorComponent!=null) {
	    editorComponent.dispose();
	    editorComponent = null;
	}
	super.panelClosed();
    }
    /***PropertChangeListener**********************/
    public void propertyChange(PropertyChangeEvent evt) {
	if (evt.getSource()==editorComponent &&
	    evt.getPropertyName().equals(editorComponent.COMPONENT)) {
	    if (installedComponent!=null)
		panel.remove(installedComponent);
	    installedComponent = (Component)evt.getNewValue();
	    if (installedComponent!=null)
		panel.add("Center", installedComponent);
	    panel.validate();
	}
    }
    /***Actions********************************/
    public abstract class DocumentAction extends AbstractAction {
	public DocumentAction(String ac) { super(ac); setEnabled(false); }
	public abstract void actionPerformed(ActionEvent evt, EditorDocument document);
	public void actionPerformed(ActionEvent evt) {
	    EditorDocument doc = getDocument();
	    if (doc!=null)
		actionPerformed(evt, doc);
	}
	public final void update() {
	    update(document);
	}
	public void update(EditorDocument doc) {
	    setEnabled(doc!=null);
	}
    }
}
