package com.make.editor;

import java.awt.Frame;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.io.IOException;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.Reader;
import java.io.Writer;
import java.io.PrintWriter;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.ByteArrayInputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.ResourceBundle;
import java.util.Hashtable;
import java.util.Enumeration;
import java.text.ParseException;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeSupport;
import javax.swing.Action;
import javax.swing.AbstractAction;
import javax.swing.JOptionPane;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import org.xml.sax.SAXException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.apache.xalan.xslt.XSLTProcessor;
import org.apache.xalan.xslt.XSLTProcessorFactory;
import org.apache.xalan.xslt.XSLTInputSource;
import org.apache.xalan.xslt.XSLTResultTarget;
import org.apache.fop.apps.Driver;
import org.apache.fop.apps.Version;
import org.apache.fop.apps.FOPException;
import com.make.editor.plugins.EditorPlugin;
import com.make.editor.plugins.EditorComponent;
import com.make.editor.plugins.ViewSelector;
import com.make.editor.plugins.ComponentListener;
import com.make.editor.plugins.ComponentEvent;
import com.make.xml.DOMUtil;
import com.make.xml.FormatManager;
import com.make.util.Debug;
import com.make.swing.ButtonManager;
import com.make.swing.InterfacePanel;
import com.make.swing.ThreadAction;
import com.make.swing.ThreadWindow;
import com.make.swing.bt.FrameDescription;
import com.make.datamodel.embedded.EmbeddedManager;
import com.make.datamodel.embedded.ProxyEmbeddedManager;
import com.make.datamodel.embedded.FileEmbeddedManager;
import com.make.datamodel.embedded.ZipEmbeddedManager;
import com.make.datamodel.embedded.MemoryEmbeddedManager;
import com.make.datamodel.embedded.CachedEmbeddedManager;
import com.make.editor.embedded.TransformEmbeddedManager;
import com.make.xml.RtfToXml;

class XmlFileFilter extends FileFilter {
    /***FileFilter******************************/
    public boolean accept(File f) {
	if (f!=null) {
	    if (f.isDirectory()) return true;
	    String name = f.getName();
	    return name.endsWith(".xml") || 
		name.endsWith(".wml");
	}
	return false;
    }
    public String getDescription() {
	return "eXtensible Markup Language (*.xml)";
    }
}

class ZipFileFilter extends FileFilter {
    /***FileFilter******************************/
    public boolean accept(File f) {
	if (f!=null) {
	    if (f.isDirectory()) return true;
	    String name = f.getName();
	    return name.endsWith(".zip");
	}
	return false;
    }
    public String getDescription() {
	return "XML-ZIP-Dateien (*.zip)";
    }
}

class HtmlFileFilter extends FileFilter {
    /***FileFilter******************************/
    public boolean accept(File f) {
	if (f!=null) {
	    if (f.isDirectory()) return true;
	    String name = f.getName();
	    return name.endsWith(".html") || name.endsWith(".htm");
	}
	return false;
    }
    public String getDescription() {
	return "HTML-Dateien (*.html, *.htm)";
    }
}

class PDFFileFilter extends FileFilter {
    /***FileFilter******************************/
    public boolean accept(File f) {
	if (f!=null) {
	    if (f.isDirectory()) return true;
	    String name = f.getName();
	    return name.endsWith(".pdf");
	}
	return false;
    }
    public String getDescription() {
	return "PDF-Dateien (*.pdf)";
    }
}

class RTFFileFilter extends FileFilter {
    /***FileFilter******************************/
    public boolean accept(File f) {
	if (f!=null) {
	    if (f.isDirectory()) return true;
	    String name = f.getName();
	    return name.endsWith(".rtf");
	}
	return false;
    }
    public String getDescription() {
	return "RTF-Dateien (*.rtf)";
    }
}

public class TopEditorFrame extends EditorPanel 
    implements ComponentListener {
    /***static**********************************/
    protected static JFileChooser fileChooser = new JFileChooser();
    protected static ResourceBundle resources
	= ResourceBundle.getBundle("com.make.editor.EditorFrame");
    protected static FrameDescription staticFrameDescription;
    static {
	// filters
	FileFilter zipFilter = new ZipFileFilter();
	fileChooser.addChoosableFileFilter(zipFilter);
	fileChooser.addChoosableFileFilter(new XmlFileFilter());
	fileChooser.addChoosableFileFilter(new HtmlFileFilter());
	fileChooser.addChoosableFileFilter(new PDFFileFilter());
	fileChooser.addChoosableFileFilter(new RTFFileFilter());
	fileChooser.setFileFilter(zipFilter);
	// frame description
	try {
	    InputStream is = TopEditorFrame.class.getResourceAsStream("/frame.xml");
	    Document td = DOMUtil.parse(is);
	    staticFrameDescription = new FrameDescription(td.getDocumentElement());
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }
    /***properties******************************/
    protected boolean changed = false, isNew = true;
    protected Action[] newActions;
    protected FrameDescription frameDescription;
    /***Constructor*****************************/
    public TopEditorFrame(EditorSession _sess, InterfacePanel _ip) {
	super(_sess, _ip);
	interfacePanel.getButtonManager().installActions(defaultActions);
	String[] formatNames
	    = getApplication().getFormatManager().getFormatNames();
	newActions = new Action[formatNames.length];
	for (int z=0; z<formatNames.length; z++)
	    newActions[z] = new NewAction(formatNames[z]);
	interfacePanel.getButtonManager().installActions(newActions);
	if (staticFrameDescription!=null)
	    interfacePanel.addFrameDescription(staticFrameDescription);
	for (Enumeration enum=getApplication().getModules(); enum.hasMoreElements();)
	    ((EditorModule)enum.nextElement()).install(this);
	if (Debug.LEAK_DEBUG)
	    System.err.println("+tef "+this);
    }
    public TopEditorFrame(EditorSession _sess, InterfacePanel _ip, 
			  EditorPlugin _plugin, EditorDocument _doc, 
			  EditorDocumentView _v,
			  boolean _new) {
	this(_sess, _ip);
	isNew = _new;
	setDocument(_doc, _plugin, _v);
    }
    public TopEditorFrame(EditorSession _sess, InterfacePanel _ip, 
			  EditorDocument _doc, boolean _new) {
	this(_sess, _ip);
	isNew = _new;
	setDocument(_doc);
    }
    public TopEditorFrame(EditorSession _sess, InterfacePanel _ip, 
			  EmbeddedManager _em) throws IOException, SAXException {
	this(_sess, _ip);
	readFrom(_em);
    }
    public TopEditorFrame(EditorSession _sess, InterfacePanel _ip, 
			  File _file) throws IOException, SAXException {
	this(_sess, _ip);
	readFrom(_file);
    }
    protected void finalize() throws Throwable {
	super.finalize();
	if (Debug.LEAK_DEBUG)
	    System.err.println("-tef "+this);
    }
    /***EditorPanel*********************************/
    protected void setEditorPlugin(EditorPlugin _plugin, EditorDocumentView _v) {
	if (editorComponent!=null) {
	    editorComponent.removeComponentListener(this);
	}
	if (frameDescription!=null) {
	    interfacePanel.removeFrameDescription(frameDescription);
	    frameDescription = null;
	}
	super.setEditorPlugin(_plugin, _v);
	if (editorComponent!=null) {
	    editorComponent.addComponentListener(this);
	    // frame description
	    Object fd = view.getDescription("frame");
	    if (fd!=null) {
		if (fd instanceof Element) {
		    frameDescription = new FrameDescription((Element)fd);
		    view.putDescription("frame", frameDescription);
		} else
		    frameDescription = (FrameDescription)fd;
		interfacePanel.addFrameDescription(frameDescription);
	    }
	}
    }
    public void setDocument(EditorDocument _doc, EditorPlugin _plugin, 
			    EditorDocumentView _v) {
	super.setDocument(_doc, _plugin, _v);
	saveAction.setEnabled(_doc!=null);
	saveAsAction.setEnabled(_doc!=null);
	printAction.update();
	viewsAction.setEnabled(_doc!=null);
	generateIdAction.setEnabled(_doc!=null);
	closeFileAction.setEnabled(_doc!=null);
    }
    protected boolean documentClosing() {
	if (changed) {
	    switch (JOptionPane.showConfirmDialog
		    (getFrame(), 
		     resources.getString("ask.changedSave"),
		     resources.getString("operation.close"), 
		     JOptionPane.YES_NO_CANCEL_OPTION,
		     JOptionPane.QUESTION_MESSAGE)) {
	    case JOptionPane.CANCEL_OPTION: 
		return false;
	    case JOptionPane.YES_OPTION: {
		EditorDocument doc = getDocument();
		if (doc==null || save(doc)) 
		    return false;
	    }
	    }
	}
	return true;
    }
    /***I/O*****************************************/
    public boolean saveAll(EditorDocument doc, EmbeddedManager em) 
	throws IOException {
	EmbeddedManager oem = doc.getEmbeddedManager();
	boolean ok = saveDocument(doc, em);
	doc.putNewEmbeddedManager(em);
	return ok;
    }
    public boolean saveDocument(EditorDocument doc) throws IOException {
	return saveDocument(doc, doc.getEmbeddedManager());
    }
    public boolean writeDocument(EditorDocument doc, EmbeddedManager em) 
	throws IOException {
	ThreadWindow tw = new ThreadWindow
	    (resources.getString("message.save"),
	     resources.getString("operation.save"),
	     100, true);
	try {
	    doc.writeDocument(em);
	} finally {
	    tw.close();
	}
	return true;
    }
    public boolean saveDocument(EditorDocument doc, EmbeddedManager em) 
	throws IOException {
	if (writeDocument(doc, em)) {
	    isNew = false; changed = false;
	    generateIdAction.setEnabled(!isNew);
	    return true;
	} else
	    return false;
    }
    public static EmbeddedManager createEmbeddedManager(File f, FormatManager fm) 
	throws IOException {
	if (f.getName().endsWith(".zip"))
	    return new ZipEmbeddedManager(f, fm);
	else
	    return new FileEmbeddedManager(f, fm);
    }
    public boolean readFrom(File f) throws IOException, SAXException {
	//RTF-Dateien muessen noch uebersetzt werden nach xml
	if (f.getName().endsWith(".rtf")){
	    String[] formats = getApplication().getFormatManager().getFormatNames();
	    String[] rtfFormats = new String[formats.length];
	    int count=0;
	    for (int z=0; z<formats.length; z++){
		String testURL = "/formats/"+formats[z]+
		    "/rtfimport.xml";
		if (getClass().getResource(testURL)!=null){
		    rtfFormats[count++]=formats[z];
		}
	    }
	    if (count==0){
		throw new IOException("no rtf-import formats matching your DTDs found");
	    }
	    //rtfFormats.length=count;
	    String dtdformat = (String) JOptionPane.showInputDialog(getFrame(),"Bitte DTD auswaehlen:","DTD waehlen",JOptionPane.INFORMATION_MESSAGE, null,rtfFormats, rtfFormats[0]);
	    String xmlURL = "/formats/"+dtdformat+"/rtfimport.xml";
	    InputStream cfgStream = getClass().getResourceAsStream(xmlURL);
	    Document cfg;
	    try {
		cfg = DOMUtil.parse(cfgStream);
	    } catch (SAXException e) {
		throw new IOException(e.getLocalizedMessage());
	    }
	    RtfToXml rtf = new RtfToXml(cfg.getDocumentElement());
	    FileInputStream fis = new FileInputStream(f);
	    InputStreamReader isr = new InputStreamReader(fis,"ISO-8859-1");
	    Document doc = rtf.translate(isr);
	    MemoryEmbeddedManager mem = new MemoryEmbeddedManager();
	    mem.replaceDocument(doc.getDocumentElement());
	    readFrom(mem);
	    isNew=true;
	    return true;
	} else 
	    return readFrom(createEmbeddedManager(f, getApplication().getFormatManager()));
    }
    public boolean readFrom(EmbeddedManager _em) 
	throws IOException, SAXException {
	// parse
	Node n = _em.getDocument();
	if (n==null)
	    throw new IOException(resources.getString("error.file.empty"));
	// load document type
	FormatManager fm = getApplication().getFormatManager();
	String tn = fm.getFormatName(n.getOwnerDocument());
	EditorDocumentType docType = tn==null
	    ?null
	    :(EditorDocumentType)fm.getFormat(tn);
	// set document
	setDocument(new EditorDocument(n, docType, _em));
	isNew = false; changed = false;
	generateIdAction.setEnabled(getDocument()!=null && !isNew);
	return true;
    }
    /***ComponentListener****************************/
    public void focusGained(ComponentEvent evt) {}
    public void documentChanged(ComponentEvent evt) {
	changed = true;
    }
    /***PanelContent**************************/
    public void panelClosed() {
	for (Enumeration enum=getApplication().getModules(); enum.hasMoreElements();)
	    ((EditorModule)enum.nextElement()).uninstall(this);
	interfacePanel.getButtonManager().uninstallActions(newActions);
	interfacePanel.getButtonManager().uninstallActions(defaultActions);
	if (staticFrameDescription!=null)
	    interfacePanel.removeFrameDescription(staticFrameDescription);
	if (frameDescription!=null) {
	    interfacePanel.removeFrameDescription(frameDescription);
	    frameDescription = null;
	}
	super.panelClosed();
    }
    /***File*****************************************/
    public boolean saveAs(EditorDocument doc) {
	fileChooser.getUI().rescanCurrentDirectory(fileChooser);
	final int chooseRet = fileChooser.showSaveDialog(getFrame());
	if (chooseRet!=fileChooser.APPROVE_OPTION)
	    return false;
	File f = fileChooser.getSelectedFile();
	if (f.getName().indexOf('.')==-1) {
	    if (fileChooser.getFileFilter() instanceof XmlFileFilter)
		f = new File(f.getPath()+".xml");
	    else if (fileChooser.getFileFilter() instanceof HtmlFileFilter)
		f = new File(f.getPath()+".html");
	    else if (fileChooser.getFileFilter() instanceof PDFFileFilter)
		f = new File(f.getPath()+".pdf");
	    else if (fileChooser.getFileFilter() instanceof RTFFileFilter)
		f = new File(f.getPath()+".rtf");
	    else
		f = new File(f.getPath()+".zip");
	}
	if (f.exists() && JOptionPane.showConfirmDialog
	    (getFrame(), 
	     resources.getString("ask.overwrite"),
	     resources.getString("operation.saveAs"),
	     JOptionPane.YES_NO_OPTION,
	     JOptionPane.QUESTION_MESSAGE)!=JOptionPane.YES_OPTION)
	    return false;
	try {
	    if (f.getName().endsWith(".html") || f.getName().endsWith(".htm")) {
		// export to HTML
		EmbeddedManager em = new TransformEmbeddedManager
		    (new FileEmbeddedManager(f, null), "html", null);
		doc.copyEmbedded(doc.getEmbeddedManager(), em);
		writeDocument(doc, em);
	    } else if (f.getName().endsWith(".rtf")) {
		// export to RTF
		Document inDoc = doc.getNode().getOwnerDocument();
		XSLTInputSource in = new XSLTInputSource(inDoc);
		InputStream styleStream = getClass().getResourceAsStream
		    ("/formats/"+DOMUtil.getTypeName(inDoc)+"/export_rtf.xsl");
		if (styleStream==null)
		    throw new IOException("Kein passender Exportfilter gefunden.");
		XSLTInputSource style = new XSLTInputSource
		    (new InputStreamReader(styleStream, "ISO-8859-1"));
		XSLTResultTarget out = new XSLTResultTarget(new FileOutputStream(f));
		XSLTProcessor processor = XSLTProcessorFactory.getProcessorUsingLiaisonName
		    ("org.apache.xalan.xpath.xdom.XercesLiaison");
		processor.process(in, style, out);
	    } else if (f.getName().endsWith(".fo")) {
		// export to FO
		Node n = doc.getNode();
		Document xdoc = n.getOwnerDocument();
		// open XSLT
		String xslURL = "/formats/"+
		    DOMUtil.getTypeName(xdoc)+
		    "/export_fo.xsl";
		InputStream xslIS = getClass().getResourceAsStream(xslURL);
		if (xslIS==null)
		    throw new IOException("Kein passender Exportfilter gefunden.");
		Reader xslReader = new InputStreamReader(xslIS, "ISO-8859-1");
		// transform
		OutputStream os = new FileOutputStream(f);
		Writer wr = new OutputStreamWriter(os, "ISO-8859-1");
		try {
		    XSLTProcessor processor
			= XSLTProcessorFactory.getProcessorUsingLiaisonName
			("org.apache.xalan.xpath.xdom.XercesLiaison");
		    XSLTResultTarget rt = new XSLTResultTarget(wr);
		    rt.setEncoding("ISO-8859-1");
		    processor.process
			(new XSLTInputSource(xdoc), 
			 new XSLTInputSource(xslReader), 
			 rt);
		} catch (SAXException e) {
		    throw new IOException(e.getLocalizedMessage());
		}
		wr.close(); os.close();
	    } else if (f.getName().endsWith(".pdf")) {
		// export to PDF
		Node n = doc.getNode();
		Document xdoc = n.getOwnerDocument();
		// open XSLT
		String xslURL = "/formats/"+
		    DOMUtil.getTypeName(xdoc)+
		    "/export_fo.xsl";
		InputStream xslIS = getClass().getResourceAsStream(xslURL);
		if (xslIS==null)
		    throw new IOException("Kein passender Exportfilter gefunden.");
		Reader xslReader = new InputStreamReader(xslIS, "ISO-8859-1");
		// transform
		Document result = DOMUtil.createDocument();
		try {
		    XSLTProcessor processor
			= XSLTProcessorFactory.getProcessorUsingLiaisonName
			("org.apache.xalan.xpath.xdom.XercesLiaison");
		    processor.process
			(new XSLTInputSource(xdoc), 
			 new XSLTInputSource(xslReader), 
			 new XSLTResultTarget(result));
		} catch (SAXException e) {
		    throw new IOException(e.getLocalizedMessage());
		}
		// FO->PDF
		String version = Version.getVersion();
		Driver driver = new Driver();
		driver.setRenderer("org.apache.fop.render.pdf.PDFRenderer", version);
		driver.addElementMapping("org.apache.fop.fo.StandardElementMapping");
		driver.addElementMapping("org.apache.fop.svg.SVGElementMapping");
		OutputStream os = new FileOutputStream(f);
		Writer wr = new OutputStreamWriter(os, "ISO-8859-1");
		PrintWriter pwr = new PrintWriter(wr);
		driver.setWriter(pwr);
		try {
		    driver.buildFOTree(result);
		    driver.format();
		} catch (FOPException e) {
		    throw new IOException(e.getLocalizedMessage());
		}
		driver.render();
		pwr.close(); wr.close(); os.close();
	    } else {
		// simply save into new file
		EmbeddedManager em = createEmbeddedManager(f, getApplication().getFormatManager());
		saveAll(doc, em);
		interfacePanel.setTitle(em.toString());
	    }
	    return true;
	} catch (Exception ee) {
	    JOptionPane.showMessageDialog
		(getFrame(), ee.getLocalizedMessage(),
		 resources.getString("operation.saveAs"), 
		 JOptionPane.ERROR_MESSAGE, null);
	    return false;
	}
    }
    public boolean save(EditorDocument doc) {
	if (isNew)
	    return saveAs(doc);
	else 
	    try {
		return saveDocument(doc);
	    } catch (IOException ex) {
		JOptionPane.showMessageDialog
		    (getFrame(), ex.getLocalizedMessage(),
		     resources.getString("operation.save"), JOptionPane.ERROR_MESSAGE, null);
		return false;
	    }
    }
    public void openFile() {
	if (!documentClosing()) return;
	fileChooser.getUI().rescanCurrentDirectory(fileChooser);
	final int chooseRet = fileChooser.showOpenDialog(getFrame());
	if (chooseRet!=fileChooser.APPROVE_OPTION)
	    return;
	File f = fileChooser.getSelectedFile();
	if (f.exists()) try {
	    readFrom(f);
	} catch (IOException ee) {
	    JOptionPane.showMessageDialog
		(getFrame(), ee.getLocalizedMessage(), 
		 resources.getString("operation.open"), 
		 JOptionPane.ERROR_MESSAGE, null);
	} catch (SAXException e) {
	    JOptionPane.showMessageDialog
		(getFrame(), 
		 resources.getString("error.file.xml.syntax")+'\n'+
		 e.getLocalizedMessage(),
		 resources.getString("operation.open"), 
		 JOptionPane.ERROR_MESSAGE, null);
	}
	else {
	   JOptionPane.showMessageDialog
		(getFrame(), 
		 resources.getString("error.file.exist")+'\n',
		 resources.getString("operation.open"), 
		 JOptionPane.ERROR_MESSAGE, null); 
	}
    }
    /***Actions***************************************/
    private Action saveAction = new SaveAction();
    private Action saveAsAction = new SaveAsAction();
    private PrintAction printAction = new PrintAction();
    private Action generateIdAction = new GenerateIdAction();
    private Action viewsAction = new ViewsAction();
    private Action closeFileAction = new CloseFileAction();
    private Action[] defaultActions = {
	new ThreadAction(viewsAction),
	new OpenAction(),
	new ThreadAction(saveAction),
	saveAsAction,
	new ThreadAction(generateIdAction),
	closeFileAction,
	new ThreadAction(printAction),
    };
    class ViewsAction extends DocumentAction {
	ViewsAction() { super("views"); }
        public void actionPerformed(ActionEvent e, EditorDocument doc) {
	    EditorDocumentView v = ViewSelector.selectView
		(getFrame(), "Select view", "Select view:", 
		 doc.getType().getViews());
	    if (v!=null)
		setView(v);
	}
    }
    class NewAction extends AbstractAction {
	String format;
	NewAction(String _format) { 
	    super("new-"+_format); 
	    format = _format;
	}
        public void actionPerformed(ActionEvent evt) {
	    if (!documentClosing()) return;
	    try {
		EditorDocumentType edt = (EditorDocumentType)getApplication()
		    .getFormatManager().getFormat(format);
		Document doc;
		if (edt==null) {
		    doc = DOMUtil.createDocument();
		    doc.appendChild(doc.createElement(format));
		} else
		    doc = edt.createNewDocument(getSession());
		EmbeddedManager em = new MemoryEmbeddedManager
		    (resources.getString("newDocumentName"));
		setDocument(new EditorDocument(doc.getDocumentElement(),
					       edt, em));
		changed = false; isNew = true;
	    } catch (Exception ex) {
		ex.printStackTrace();
		JOptionPane.showMessageDialog
		    (getFrame(), ex.getLocalizedMessage(), 
		     resources.getString("operation.new"),
		     JOptionPane.ERROR_MESSAGE, null);
	    }
	}
    }
    class OpenAction extends AbstractAction {
	OpenAction() { super("open"); }
        public void actionPerformed(ActionEvent e) {
	    openFile();
	}
    }
    class SaveAction extends DocumentAction {
	SaveAction() { super("save"); }
	public void actionPerformed(ActionEvent e, EditorDocument doc) { 
	    save(doc); 
	}
    }
    class SaveAsAction extends DocumentAction {
	SaveAsAction() { super("saveas"); }
	public void actionPerformed(ActionEvent e, EditorDocument doc) { 
	    saveAs(doc); 
	}
    }
    class GenerateIdAction extends DocumentAction {
	GenerateIdAction() { super("generate-id"); }
	public void actionPerformed(ActionEvent e, EditorDocument doc) {
	    if (JOptionPane.showConfirmDialog
		(getFrame(),
		 resources.getString("ask.generateId"),
		 resources.getString("operation.generateId"),
		 JOptionPane.YES_NO_OPTION, 
		 JOptionPane.QUESTION_MESSAGE)!=JOptionPane.YES_OPTION)
		return;
	    long newId = new java.util.Random().nextInt();
	    ((Element)doc.getNode()).setAttribute("id", ""+newId);
	    EmbeddedManager em = doc.getEmbeddedManager();
	    while (em instanceof ProxyEmbeddedManager)
		em = ((ProxyEmbeddedManager)em).getOriginal();
	    if (!(em instanceof FileEmbeddedManager ||
		  em instanceof ZipEmbeddedManager ||
		  em instanceof MemoryEmbeddedManager)) {
		em = new MemoryEmbeddedManager(resources.getString("newDocumentName"));
		doc.putNewEmbeddedManager(em);
		try {
		    writeDocument(doc, em);
		} catch (IOException ee) {
		    // darf bei MemoryEmbeddedManager nicht passieren
		    ee.printStackTrace(); 
		}
		isNew = true;
		interfacePanel.setTitle(em.toString());
		TopEditorFrame.this.firePropertyChange(DOCUMENT, doc, null);
		TopEditorFrame.this.firePropertyChange(DOCUMENT, null, doc);
	    }
	    changed = true;
	}
    }
    class CloseFileAction extends DocumentAction {
	CloseFileAction() { super("close-file"); }
        public void actionPerformed(ActionEvent e, EditorDocument doc) {
	    if (!documentClosing()) return;
	    setDocument(null);
	}
    }
    class PrintAction extends DocumentAction {
	public PrintAction() { super("print"); }
	public void actionPerformed(ActionEvent evt, EditorDocument doc) {
	    try {
		// file
		File f = new File("/tmp/leprint.html");
		// export
		EmbeddedManager em = new TransformEmbeddedManager
		    (new FileEmbeddedManager(f, null), "html", null);
		doc.copyEmbedded(doc.getEmbeddedManager(), em);
		if (!writeDocument(doc, em))
		    return;
		// run netscape
		Runtime.getRuntime().exec("netscape -remote openURL("+f.getPath()+",new-window)");
	    } catch (IOException e) {
		JOptionPane.showMessageDialog
		    (getFrame(), 
		     resources.getString("error.print")+'\n'+
		     e.getLocalizedMessage(),
		     resources.getString("operation.print"), 
		     JOptionPane.ERROR_MESSAGE, null);
	    }
	}
	public void update(EditorDocument doc) {
	    if (doc==null)
		setEnabled(false);
	    else {
		String xslURL = "/formats/"+
		    DOMUtil.getTypeName(doc.getXMLDocument())+
		    "/export_html.xsl";
		InputStream is = getClass().getResourceAsStream(xslURL);
		setEnabled(is!=null);
	    }
	}
    }
}
