package com.make.editor.embedded;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import java.util.Hashtable;
import java.util.Enumeration;
import org.w3c.dom.Node;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;
import org.apache.xalan.xslt.XSLTProcessor;
import org.apache.xalan.xslt.XSLTProcessorFactory;
import org.apache.xalan.xslt.XSLTInputSource;
import org.apache.xalan.xslt.XSLTResultTarget;
import com.make.xml.DOMUtil;
import com.make.datamodel.embedded.EmbeddedManager;
import com.make.datamodel.embedded.ProxyEmbeddedManager;

public class TransformEmbeddedManager 
    implements EmbeddedManager {
    /***properties******************************/
    protected XSLTProcessor processor;
    protected EmbeddedManager original;
    protected String storeFormat, displayFormat;
    /***Constructor*****************************/
    public TransformEmbeddedManager(EmbeddedManager _org, String _sfmt, String _dfmt) { 
	try {
	    processor = XSLTProcessorFactory.getProcessorUsingLiaisonName
		("org.apache.xalan.xpath.xdom.XercesLiaison");
	} catch (SAXException e) {
	    e.printStackTrace();
	}
	original = _org; 
	storeFormat = _sfmt; displayFormat = _dfmt;
    }
    public String toString() { return "[XSL:"+original.toString()+"]"; }
    /***ProxyEmbeddedManager*************************/
    public EmbeddedManager getOriginal() {
	return original;
    }
    /***EmbeddedManager***************************/
    public String[] getNames() throws IOException {
	return original.getNames();
    }
    public Hashtable getSizes() throws IOException {
	return original.getSizes();
    }
    public boolean contains(String name) { 
	return original.contains(name);
    }
    public void replaceEntry(String name, InputStream data)
	throws IOException {
	original.replaceEntry(name, data);
    }
    public InputStream getEntry(String name) throws IOException {
	return original.getEntry(name);
    }
    public void replaceDocument(Node n) throws IOException {
	if (storeFormat==null)
	    throw new InternalError();
	if (n!=null) {
	    // source
	    Document src = n.getOwnerDocument();
	    // style
	    String xslURL = "/formats/"+
		DOMUtil.getTypeName(src)+
		"/export_"+storeFormat+".xsl";
	    InputStream xslIS = getClass().getResourceAsStream(xslURL);
	    if (xslIS==null)
		throw new IOException("Kein passender Exportfilter gefunden.");
	    Reader xslReader = new InputStreamReader(xslIS, "ISO-8859-1");
	    // target
	    Document target = DOMUtil.createDocument();
	    // transform
	    ByteArrayOutputStream baos = new ByteArrayOutputStream();
	    OutputStreamWriter osw = new OutputStreamWriter(baos, "ISO-8859-1");
	    try {
		synchronized(processor) {
		    processor.process
			(new XSLTInputSource(src), 
			 new XSLTInputSource(xslReader), 
			 new XSLTResultTarget(target));
		}
	    } catch (SAXException e) {
		throw new IOException(e.getLocalizedMessage());
	    }
	    n = target.getDocumentElement();
	}
	original.replaceDocument(n);
    }
    public Node getDocument() throws IOException, SAXException {
	if (displayFormat==null)
	    throw new InternalError();
	// source
	Node n = original.getDocument();
	if (n==null)
	    return null;
	Document src = n.getOwnerDocument();
	// style
	String xslURL = "/formats/"+displayFormat+
	    "/import_"+DOMUtil.getTypeName(src)+".xsl";
	InputStream xslIS = getClass().getResourceAsStream(xslURL);
	if (xslIS==null)
	    throw new IOException("Kein passender Exportfilter gefunden.");
	Reader xslReader = new InputStreamReader(xslIS, "ISO-8859-1");
	// target
	Document target = DOMUtil.createDocument();
	// transform
	synchronized(processor) {
	    processor.process
		(new XSLTInputSource(src), 
		 new XSLTInputSource(xslReader), 
		 new XSLTResultTarget(target));
	}
	return target.getDocumentElement();
    }
    public boolean equals(Object o) { 
	if (o instanceof TransformEmbeddedManager)
	    return original.equals(((TransformEmbeddedManager)o).original);
	else
	    return original.equals(o);
    }
}
