package com.make.editor.field;

import java.io.FilenameFilter;
import java.lang.reflect.Constructor;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import org.w3c.dom.Element;
import com.make.xml.DOMUtil;
import com.make.util.Strings;
import com.make.datamodel.slot.Slot;
import com.make.datamodel.slot.SlotContainer;
import com.make.swing.field.Field;
import com.make.swing.field.TimeOffsetComboField;
import com.make.swing.il.InputLink;
import com.make.swing.il.InputLinkFactory;
import com.make.swing.il.DefaultInputLinkFactory;
import com.make.editor.EditorDocument;
import com.make.editor.EditorDocumentType;
import com.make.editor.EditorSession;
import com.make.editor.EditorFrame;
import com.make.editor.EditorModule;

public class EditorInputLinkFactory extends DefaultInputLinkFactory {
    /***static*********************************/
    private static Hashtable registeredModules = new Hashtable();
    public static void registerModule(EditorModule mod, Class cls) {
	try {
	    Class[] paramClasses = { 
		EditorModule.class, EditorInputLinkFactory.class
	    };
	    Constructor constr = cls.getConstructor(paramClasses);
	    registeredModules.put(mod, constr);
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }
    /***properties****************************/
    private EditorFrame frame;
    protected EditorDocument document;
    protected EditorDocumentType documentType;
    protected InputLinkFactory[] modules;
    /***Constructor**************************/
    public EditorInputLinkFactory(EditorFrame _frm, EditorDocumentType _type,
				  SlotContainer _sc) {
	super(_sc);
	frame = _frm;
	documentType = _type;
	synchronized(registeredModules) {
	    modules = new InputLinkFactory[registeredModules.size()];
	    Enumeration enum = registeredModules.keys();
	    for (int z=0; z<modules.length; z++) {
		EditorModule module = (EditorModule)enum.nextElement();
		Constructor constr = (Constructor)registeredModules.get(module);
		Object[] params = {
		    module, this
		};
		try {
		    InputLinkFactory ilf = (InputLinkFactory)constr.newInstance(params);
		    modules[z] = ilf;
		} catch (Exception e) {
		    System.err.println("while creating ilf for module "+module);
		    e.printStackTrace();
		    modules[z] = null;
		}
	    }
	}
    }
    public EditorInputLinkFactory(EditorFrame _frm, EditorDocument _doc,
				  SlotContainer _sc) {
	this(_frm, _doc.getType(), _sc);
	document = _doc; 
    }
    /***Info******************************/
    public EditorFrame getFrame() {
	return frame;
    }
    public EditorSession getSession() {
	return frame.getSession();
    }
    public EditorDocument getDocument() {
	return document;
    }
    public EditorDocumentType getDocumentType() {
	return documentType;
    }
    public Slot getSlot(Element vd) {
	return super.getSlot(vd);
    }
    /***Internal******************************/
    protected FilenameFilter loadFilenameFilter(Element vd) {
	Element[] es = DOMUtil.getChildElements(vd, "filetype");
	if (es.length==0)
	    return null;
	String[] ext = new String[es.length];
	for (int z=0; z<es.length; z++)
	    ext[z] = es[z].getAttribute("extension");
	return new ExtensionFilenameFilter(ext);
    }
    /***DefaultInputLinkFactory******************/
    protected Object getList(Element vd) {
	if (documentType==null)
	    return null;
	String name = Strings.trim(vd.getAttribute("list"));
	if (name==null) {
	    Element list = DOMUtil.getUniqueTag(vd, "list");
	    if (list!=null)
		name = Strings.trim(list.getAttribute("name"));
	}
	return name==null
	    ?null
	    :documentType.getList(name);
    }
    protected Field createField(Element vd) {
	String type = getType(vd);
	if (document!=null) {
	    if (type.equals("embeddedselect"))
		return new EmbeddedSelectField(document.getEmbeddedManager(), 
					       loadFilenameFilter(vd));
	    if (type.equals("embedded"))
		return new EmbeddedField(document.getEmbeddedManager(), 
					 loadFilenameFilter(vd));
	}
	if (type.equals("radio"))
	    return new RadioField(vd);
	return super.createField(vd);
    }
    public InputLink createInputLink(Element vd) {
	try {
	    // super
	    InputLink il = super.createInputLink(vd);
	    if (il!=null) return il;
	    // some other type
	    String type = getType(vd);
	    if (type.equals("plugin"))
		return new PluginInputLink(vd, document, frame);
	    // ask module
	    for (int z=0; z<modules.length; z++)
		if (modules[z]!=null) {
		    il = modules[z].createInputLink(vd);
		    if (il!=null) return il;
		}
	} catch (Exception e) {
	    e.printStackTrace(System.err);
	}
	return null;
    }
}
