package com.make.editor.field;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.OutputStream;
import java.io.FileOutputStream;
import java.util.Vector;
import java.awt.Component;
import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.text.ParseException;
import javax.swing.AbstractListModel;
import javax.swing.JPanel;
import javax.swing.JFileChooser;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.ImageIcon;
import javax.swing.filechooser.FileFilter;
import org.w3c.dom.Element;
import com.make.datamodel.embedded.EmbeddedManager;
import com.make.swing.field.DefaultField;
import com.make.util.Streams;

class EmbeddedField extends EmbeddedSelectField {
    /***static**********************************/
    protected static final JFileChooser fileChooser = new JFileChooser();
    static {
	fileChooser.setFileFilter(new FileFilter() {
	    public boolean accept(File f) {
		if (f!=null) {
		    if (f.isDirectory()) 
			return true;
		    else if (f.isFile()) {
			String name = f.getName();
			return name.endsWith(".jpg") || name.endsWith(".jpeg") ||
			    name.endsWith(".gif");
		    }
		}
		return false;
	    }
	    public String getDescription() {
		return "Images (*.jpg, *.gif)";
	    }
	});
    }
    /***properties*************/
    protected JPanel panel = new JPanel(new BorderLayout(), false);
    protected JButton open, save;
    /***Constructor*************/
    public EmbeddedField(EmbeddedManager _em, FilenameFilter _flt) {
	super(_em, _flt);
	panel.add("Center", super.getComponent());
	JPanel p = new JPanel(new BorderLayout(), false);
	open = new JButton
	    (new ImageIcon(getClass().getResource("/com/make/editor/resources/open.gif")));
	open.setActionCommand("open");
	open.addActionListener(this);
	p.add("West", open);
	save = new JButton
	    (new ImageIcon(getClass().getResource("/com/make/editor/resources/save.gif")));
	save.setActionCommand("save");
	p.add("East", save);
	panel.add("East", p);
	save.addActionListener(this);
	setComponent(panel);
    }
    public void errorMsg(String msg, String title) {
	JOptionPane.showMessageDialog(null, msg, title,
				      JOptionPane.ERROR_MESSAGE);
    }
    /***ActionListener*******************************/
    public void actionPerformed(ActionEvent evt) {
	// choose
	if (evt.getSource()==open) {
	    fileChooser.getUI().rescanCurrentDirectory(fileChooser);
	    final int chooseRet = fileChooser.showOpenDialog(null);
	    if (chooseRet!=fileChooser.APPROVE_OPTION)
		return;
	    File f = fileChooser.getSelectedFile();
	    if (!f.exists()) return;
	    if (f.isDirectory()) {
		errorMsg("Die ausgew\u00e4hlte Datei ist ein Verzeichnis.",
			 "Fehler");
		return;
	    }
	    if (!f.canRead()) {
		errorMsg("Zugriff auf die Datei verweigert.", "Fehler");
		return;
	    }
	    // copy
	    if (!embeddedManager.contains(f.getName())) //TODO: anderer Pfad?
		try {
		    InputStream fis = new FileInputStream(f);
		    embeddedManager.replaceEntry(f.getName(), fis);
		} catch (IOException e) {
		    errorMsg("Fehler beim Einlesen der Bilddatei:\n"+
			     e.getLocalizedMessage(), "Fehler");
		    return;
		}
	    // embed
	    reloadList();
	    try {
		setValue(f.getName());
	    } catch (ParseException e) {
	    }
	} else if (evt.getSource()==save) {
	    fileChooser.getUI().rescanCurrentDirectory(fileChooser);
	    String name = null;
	    try {
		name = (String)getValue(); 
	    } catch (ParseException e) {
	    }
	    if (name==null || !embeddedManager.contains(name)) {
		errorMsg("Bitte w\u00e4hlen Sie vorher eine Datei aus der Liste aus.",
			 "Datei speichern");
		return;
	    }
	    final int chooseRet = fileChooser.showSaveDialog(null);
	    if (chooseRet!=fileChooser.APPROVE_OPTION)
		return;
	    File f = fileChooser.getSelectedFile();
	    if (f.exists()) {
		if (JOptionPane.showConfirmDialog
		    (null, "Die Datei existiert bereits. \u00dcberschreiben?",
		     "Datei speichern", JOptionPane.YES_NO_OPTION)!=JOptionPane.YES_OPTION)
		    return;
		if (f.isDirectory()) {
		    errorMsg("Die ausgew\u00e4hlte Datei ist ein Verzeichnis.",
			     "Datei speichern");
		    return;
		}
		if (!f.canWrite()) {
		    errorMsg("Zugriff auf die Datei verweigert.", "Datei speichern");
		    return;
		}
	    }
	    // copy
	    try {
		InputStream is = embeddedManager.getEntry(name);
		if (is==null) return;
		OutputStream os = new FileOutputStream(f);
		Streams.copy(is, os);
		is.close(); os.close();
	    } catch (IOException e) {
		errorMsg("Fehler beim Kopieren der Datei:\n"+
			 e.getLocalizedMessage(), "Datei speichern");
	    }
	} else
	    super.actionPerformed(evt);
    }
}
