package com.make.editor.field;

import java.awt.Component;
import java.text.ParseException;
import javax.swing.JComponent;
import javax.swing.JScrollPane;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import com.make.swing.il.DefaultInputLink;
import com.make.xml.DOMUtil;
import com.make.editor.EditorDocument;
import com.make.editor.EditorFrame;
import com.make.editor.plugins.EditorComponent;
import com.make.editor.plugins.ComponentListener;
import com.make.editor.plugins.EditorPlugin;
import com.make.editor.plugins.ContainerComponent;
import com.make.editor.plugins.PluginManager;

public class PluginInputLink extends DefaultInputLink {
    /***properties*************************/
    protected Component component;
    protected EditorComponent editorComponent;
    protected EditorDocument document;
    /***Constructor************************/
    public PluginInputLink(Element vd, EditorDocument _doc, 
			   EditorFrame _frame) throws SAXException {
	super(vd);
	// path
	String path = DOMUtil.getElementString(vd, "path");
	EditorDocument[] docs = _doc.followPath(path, true);
	document = docs[0];
	// plugin
	String pluginName = DOMUtil.getElementString(vd, "plugin");
	PluginManager pluginManager
	    = _frame.getSession().getApplication().getPluginManager();
	EditorPlugin ep = pluginManager.loadPlugin(pluginName);
	editorComponent = ep.createComponent(_frame);
	editorComponent.setDocument(document);
	component = editorComponent.getComponent();
	// listener
	EditorComponent root = _frame.getComponent(null);
	if (root instanceof ComponentListener)
	    editorComponent.addComponentListener((ComponentListener)root);
	// border
	if (component instanceof JComponent) {
	    Element b = DOMUtil.getUniqueTag(vd, "border");
	    if (b!=null)
		((JComponent)component).setBorder(ContainerComponent.createBorder(b));
	}
	// scroll
	Element sp = DOMUtil.getUniqueTag(vd, "scrollpane");
	if (sp!=null)
	    component = ContainerComponent.createScrollPane(sp, component);
    }
    /***InputLink****************************/
    public Component getComponent() {
	return component;
    }
    public void reload() {
	editorComponent.setDocument(document);
    }
    public void apply() throws ParseException {
	editorComponent.flushDocument();
    }
    public void checkInput() throws ParseException {
	editorComponent.flushDocument();
    }
    public void dispose() {
	component = null;
	if (editorComponent!=null) {
	    editorComponent.dispose();
	    editorComponent = null;
	}
	document = null;
	super.dispose();
    }
}
