package com.make.editor.plugins;

import java.awt.LayoutManager;
import java.awt.Component;
import java.awt.Container;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.util.Vector;
import java.util.Enumeration;
import java.text.ParseException;
import javax.swing.Action;
import javax.swing.AbstractAction;
import javax.swing.Box;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.border.Border;
import javax.swing.BorderFactory;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import org.apache.xalan.xpath.XObject;
import com.make.editor.EditorDocument;
import com.make.editor.EditorFrame;
import com.make.xml.DOMUtil;
import com.make.util.Strings;
import com.make.util.Debug;

public abstract class ContainerComponent extends DefaultEditorComponent
    implements ComponentListener {
    /***Class*****************************************/
    protected class ShowAction extends AbstractAction {
	/***properties*******/
	protected Component component;
	/***Constructor******/
	public ShowAction(String ac, Component c) {
	    super(ac); component = c;
	}
	/***ActionListener****/
	public void actionPerformed(ActionEvent evt) {
	    component.setVisible(!component.isVisible());
	    component.getParent().validate();
	}
    }
    protected class ShowChildAction extends AbstractAction {
	/***properties*******/
	protected Child child;
	/***Constructor******/
	public ShowChildAction(String ac, Child c) {
	    super(ac); child = c;
	}
	/***ActionListener****/
	public void actionPerformed(ActionEvent evt) {
	    child.flipVisibility();
	}
    }
    class CursorLink implements ChangeListener {
	/***properties*****/
	protected EditorCursor cursor;
	protected EditorComponent component;
	protected EditorDocument document;
	/***Constructor******/
	public CursorLink(EditorCursor _cur, EditorComponent _com) {
	    cursor = _cur; component = _com;
	    cursor.addChangeListener(this);
	    stateChanged(null);
	}
	public EditorDocument getDocument() {
	    return document;
	}
	/***ChangeListener****/
	public void stateChanged(ChangeEvent evt) {
	    EditorDocument[] docs = cursor.getSelection();
	    if (docs.length==1) {
		document = docs[0];
		component.setDocument(document);
	    } else {
		document = null;
		component.setDocument(document);
	    }
	}
    }
    protected class Child implements PropertyChangeListener {
	/***properties*********/
	protected EditorComponent editorComponent;
	protected CursorLink cursorLink;
	protected Component component;
	protected Container container;
	protected String parameter;
	protected JScrollPane scrollPane;
	protected Border border;
	protected EditorDocument document;
	/***Constructor********/
	protected Child(Container _c, Element vd) {
	    container = _c;
	    // create component
	    String pluginName = DOMUtil.getElementString(vd, "plugin");
	    EditorPlugin ep = pluginManager.loadPlugin(pluginName);
	    editorComponent = ep.createComponent(editorFrame);
	    editorComponent.addComponentListener(ContainerComponent.this);
	    editorComponent.addPropertyChangeListener(this);
	    // scroll
	    Element sp = DOMUtil.getUniqueTag(vd, "scrollpane");
	    if (sp!=null)
		scrollPane = createScrollPane(sp, null);
	    // border
	    Element b = DOMUtil.getUniqueTag(vd, "border");
	    if (b!=null)
		border = createBorder(b);
	    // show/hide
	    String sa = DOMUtil.getElementString(vd, "show-action");
	    if (sa!=null)
		myActions.addElement(new ShowChildAction(sa, this));
	}
	public Child(Container _c, Element vd, EditorCursor _cur) {
	    this(_c, vd);
	    cursorLink = new CursorLink(_cur, editorComponent);
	    // container
	    container = _c;
	    parameter = Strings.trim(vd.getAttribute("parameter"));
	    component = editorComponent.getComponent();
	    if (component!=null) {
		if (border!=null && component instanceof JComponent)
		    ((JComponent)component).setBorder(border);
		if (scrollPane==null) {
		    if (container!=null)
			container.add(parameter, component);
		} else {
		    if (container!=null)
			container.add(parameter, scrollPane);
		    scrollPane.setViewportView(component);
		}
	    }
	}
	public Child(Container _c, Element vd, EditorDocument _doc) {
	    this(_c, vd);
	    document = _doc;
	    editorComponent.setDocument(document);
	    // container
	    container = _c;
	    parameter = Strings.trim(vd.getAttribute("parameter"));
	    component = editorComponent.getComponent();
	    if (component!=null) {
		if (border!=null && component instanceof JComponent)
		    ((JComponent)component).setBorder(border);
		if (scrollPane==null) {
		    if (container!=null)
			container.add(parameter, component);
		} else {
		    if (container!=null)
			container.add(parameter, scrollPane);
		    scrollPane.setViewportView(component);
		}
	    }
	}
	public void flipVisibility() {
	    if (component!=null) {
		component.setVisible(!component.isVisible());
		component.getParent().validate();
	    }
	}
	public void dispose() {
	    editorComponent.removeComponentListener(ContainerComponent.this);
	    editorComponent.removePropertyChangeListener(this);
	    editorComponent.dispose();
	}
	public Component getComponent() {
	    return scrollPane==null ? component : scrollPane;
	}
	public EditorComponent getEditorComponent() {
	    return editorComponent;
	}
	public EditorDocument getDocument() {
	    return cursorLink==null ? document : cursorLink.getDocument();
	}
	public XObject execute(String s) throws SAXException {
	    return getDocument().execute(s);
	}
	public void flushDocument() throws ParseException {
	    editorComponent.flushDocument();
	}
	public void reloadDocument() {
	    editorComponent.setDocument(getDocument());
	}
	/***PropertyChangeListener********************/
	public void propertyChange(PropertyChangeEvent evt) {
	    if (evt.getSource()==editorComponent &&
		evt.getPropertyName().equals(editorComponent.COMPONENT)) {
		if (component!=null) {
		    if (border!=null && component instanceof JComponent)
			((JComponent)component).setBorder(null);
		    if (scrollPane==null) {
			if (container!=null)
			    container.remove(component);
		    } else
			scrollPane.setViewportView(null);
		}
		component = (Component)evt.getNewValue();
		if (component!=null) {
		    if (border!=null && component instanceof JComponent)
			((JComponent)component).setBorder(border);
		    if (scrollPane==null) {
			if (container!=null)
			    container.add(parameter, component);
		    } else
			scrollPane.setViewportView(component);
		}
		if (scrollPane==null && container!=null)
		    container.validate();
	    }
	}
    }
    /***properties*********************************/
    private Vector children = new Vector();
    private EditorComponent active = null;
    private PluginManager pluginManager;
    private Vector myActions = new Vector();
    private boolean hasFocus = false;
    /***Constructor*******************************/
    public ContainerComponent(EditorFrame _ef) {
	super(_ef);
	pluginManager = editorFrame.getSession()
	    .getApplication().getPluginManager();
    }
    /***Info************************************/
    protected Enumeration getChildren() { return children.elements(); }
    protected Child getChildAt(int idx) { return (Child)children.elementAt(idx); }
    protected EditorComponent getActiveChild() { return active; }
    /***Create***********************************/
    protected EditorCursor[] followCursorPath(Element path) {
	String cursorName = Strings.trim(path.getAttribute("cursor"));
	String pathString = DOMUtil.getElementString(path);
	EditorComponent[] cc = findComponent(pathString);
	EditorCursor[] cs = new EditorCursor[cc.length];
	for (int z=0; z<cc.length; z++)
	    cs[z] = ((EditorCursorComponent)cc[z]).getCursor(cursorName);
	return cs;
    }
    public Child createEditor(Container _c, Element vd, EditorDocument ed) {
	Child ch = new Child(_c, vd, ed);
	children.addElement(ch);
	return ch;
    }
    public Child[] createEditor(Container _c, Element vd) {
	CursorLink cursorLink;
	Element path = DOMUtil.getUniqueTag(vd, "path");
	if ("cursor".equals(path.getAttribute("type"))) {
	    EditorCursor[] cursors = followCursorPath(path);
	    Child[] cs = new Child[cursors.length];
	    for (int z=0; z<cs.length; z++) {
		cs[z] = new Child(_c, vd, cursors[z]);
		children.addElement(cs[z]);
	    }
	    return cs;
	} else {
	    try {
		EditorDocument[] documents = getDocument().followPath
		    (DOMUtil.getElementString(path), true);
		Child[] cs = new Child[documents.length];
		for (int z=0; z<cs.length; z++)
		    cs[z] = createEditor(_c, vd, documents[z]);
		return cs;
	    } catch (SAXException e) {
		e.printStackTrace();
		return new Child[0];
	    }
	}
    }
    public LayoutManager createLayoutManager(Element ls) {
	Element l;
	if ((l=DOMUtil.getUniqueTag(ls, "flowlayout"))!=null)
	    return new FlowLayout();
	else if ((l=DOMUtil.getUniqueTag(ls, "borderlayout"))!=null)
	    return new BorderLayout();
	else 
	    System.err.println("WARNING: no layout");
	return null;
    }
    public static JScrollPane createScrollPane(Element sp, Component c) {
	int vpol; String vp = Strings.trim(sp.getAttribute("vpolicy"));
	if ("never".equals(vp))
	    vpol = JScrollPane.VERTICAL_SCROLLBAR_NEVER;
	else if ("always".equals(vp))
	    vpol = JScrollPane.VERTICAL_SCROLLBAR_ALWAYS;
	else
	    vpol = JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED;
	int hpol; String hp = Strings.trim(sp.getAttribute("hpolicy"));
	if ("never".equals(hp))
	    hpol = JScrollPane.HORIZONTAL_SCROLLBAR_NEVER;
	else if ("always".equals(hp))
	    hpol = JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS;
	else
	    hpol = JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED;
	return new JScrollPane(c, vpol, hpol);
    }
    public static Border createBorder(Element el) {
	return BorderFactory.createEtchedBorder();
    }
    public Component createComponent(Container _c, Element el) {
	String tag = el.getTagName();
	if (tag.equals("editor")) {
	    Child[] ch = createEditor(_c, el);
	    return ch.length==0
		?null
		:ch[0].getComponent();
	} else if (tag.equals("glue")) {
	    String axis = Strings.trim(el.getAttribute("axis"));
	    return axis==null
		?Box.createGlue()
		:("x".equals(axis)
		  ?Box.createHorizontalGlue()
		  :Box.createVerticalGlue());
	} else if (tag.equals("label")) {
	    JLabel label = new JLabel(DOMUtil.getElementString(el));
	    label.setAlignmentX(0.0f);
	    return label;
	} else if (tag.equals("panel") || tag.equals("panelview")) {
	    LayoutManager lm = null;
	    Element ls = DOMUtil.getUniqueTag(el, "layout");
	    if (ls!=null) lm = createLayoutManager(ls);
	    JPanel panel = new JPanel(lm, false);
	    // components
	    Element cps = DOMUtil.getUniqueTag(el, "components");
	    Element[] chs = DOMUtil.getChildElements(cps);
	    for (int z=0; z<chs.length; z++) {
		Element cel = chs[z];
		Component c = createComponent(panel, cel);
		if (c!=null) {
		    String parameter = Strings.trim(cel.getAttribute("parameter"));
		    if (parameter==null)
			panel.add(c);
		    else
			panel.add(parameter, c);
		}
	    }
	    Element sp = DOMUtil.getUniqueTag(el, "scrollpane");
	    Component c = sp!=null
		?(Component)createScrollPane(sp, panel)
		:panel;
	    String sa = DOMUtil.getElementString(el, "show-action");
	    if (sa!=null)
		myActions.addElement(new ShowAction(sa, c));
	    return c;
	} else {
	    System.err.println("WARNING: unknown component tag "+tag);
	    return null;
	}
    }
    public int indexOf(EditorDocument ed) {
	EditorDocument myDoc = getDocument();
	while (ed.getParent()!=myDoc)
	    ed = ed.getParent();
	for (int z=children.size()-1; z>=0; z--)
	    if (((Child)children.elementAt(z)).getDocument()==ed)
		return z;
	return -1;
    }
    /***ComponentListener****************************/
    public void focusGained(ComponentEvent evt) {
	if (hasFocus) {
	    if (active!=null)
		active.focusLost();
	    active = evt.getComponent();
	    active.focusGained();
	} else {
	    active = evt.getComponent();
	    fireFocusGained();
	}
    }
    public void documentChanged(ComponentEvent evt) {
	fireDocumentChanged(evt);
    }
    /***EditorComponent****************************/
    public void flushDocument() throws ParseException {
	for (Enumeration enum=children.elements(); enum.hasMoreElements();)
	    ((Child)enum.nextElement()).flushDocument();
    }
    public abstract Component getComponent();
    public void focusGained() {
	if (Debug.DEBUG && hasFocus)
	    throw new InternalError("focus error for "+this);
	super.focusGained();
	if (active!=null)
	    active.focusGained();
	getEditorFrame().getInterfacePanel().getButtonManager()
	    .installActions(myActions.elements());
	hasFocus = true;
    }
    public void focusLost() {
	if (Debug.DEBUG && !hasFocus)
	    throw new InternalError("focus error for "+this);
	super.focusLost();
	if (active!=null)
	    active.focusLost();
	getEditorFrame().getInterfacePanel().getButtonManager()
	    .uninstallActions(myActions.elements());
	hasFocus = false;
    }
    public void dispose() {
	if (hasFocus)
	    focusLost();
	for (Enumeration enum=children.elements(); enum.hasMoreElements();)
	    ((Child)enum.nextElement()).dispose();
	children = null;
	active = null;
	myActions = null;
	super.dispose();
    }
}
