package com.make.editor.plugins;

import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;
import com.make.xml.DOMUtil;
import com.make.xml.FormatException;

public class PluginManager {
    /***properties**********************************/
    protected Vector pluginDescriptions = new Vector();
    protected Hashtable activePlugins = new Hashtable();
    /***Constructor********************************/
    public void registerPlugin(Element pd) {
	pluginDescriptions.addElement(pd);
    }
    public void registerPlugins(Element pl) {
	Element[] pd = DOMUtil.getChildElements(pl, "plugin");
	for (int z=0; z<pd.length; z++)
	    registerPlugin(pd[z]);
    }
    /***internal************************************/
    protected EditorPlugin loadPlugin(Element plugin) {
	String className = DOMUtil.getElementString(plugin, "class");
	if (className==null)
	    throw new FormatException("class expected");
	try {
	    Class cls = Class.forName(className);
	    return (EditorPlugin)cls.newInstance();
	} catch (ClassNotFoundException e) {
	    throw new FormatException("class "+className+" not found");
	} catch (InstantiationException e) {
	    throw new FormatException("error while loading plugin: "+e);
	} catch (IllegalAccessException e) {
	    throw new FormatException("access to plugin class "+className+" denied");
	} catch (ClassCastException e) {
	    throw new FormatException("wrong class for plugin "+className);
	}
    }
    /***Manager**************************************/
    public EditorPlugin loadPlugin(String name) {
	EditorPlugin plugin = (EditorPlugin)activePlugins.get(name);
	if (plugin==null) {
	    for (Enumeration enum=pluginDescriptions.elements(); 
		 enum.hasMoreElements();) {
		Element pd = (Element)enum.nextElement();
		if (name.equals(pd.getAttribute("name"))) {
		    plugin = loadPlugin(pd);
		    break;
		}
	    }
	    if (plugin!=null)
		activePlugins.put(name, plugin);
	}
	return plugin;
    }
}
