package com.make.editor.plugins.container;

import java.util.Vector;
import java.util.Enumeration;
import javax.swing.Action;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListDataEvent;
import org.apache.xalan.xpath.XPath;
import org.xml.sax.SAXException;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;
import com.make.xml.XPathUtil;
import com.make.xml.DOMUtil;
import com.make.editor.EditorDocument;
import com.make.editor.plugins.EditorComponent;

public class DefaultContainerModel implements DynamicContainerModel {
    /***properties****************************/
    protected EditorComponent component;
    protected EditorDocument document;
    protected XPath rowPath;
    protected Vector items = new Vector();
    protected Vector listeners = new Vector();
    protected Action[] actions;
    /***Constructor********************************/
    public DefaultContainerModel(EditorComponent _ec) {
	component = _ec; 
    }
    public DefaultContainerModel(EditorComponent _ec, EditorDocument _d, 
				 Element vd) 
	throws SAXException {
	this(_ec);
	setDocument(_d, vd);
    }
    public void setDocument(EditorDocument _d, Element vd)
	throws SAXException {
	try {
	    document = _d;
	    rowPath = vd==null
		?null
		:XPathUtil.parse(DOMUtil.getElementString(vd, "path"));
	} finally {
	    reload();
	}
    }
    /***Fire***************************************/
    protected void fireContentsChanged(int idx0, int idx1) {
	ListDataEvent evt = new ListDataEvent
	    (this, ListDataEvent.CONTENTS_CHANGED, idx0, idx1);
	for (Enumeration enum=listeners.elements(); enum.hasMoreElements();)
	    ((ListDataListener)enum.nextElement()).contentsChanged(evt);
    }
    protected void fireIntervalAdded(int idx0, int idx1) {
	ListDataEvent evt = new ListDataEvent
	    (this, ListDataEvent.INTERVAL_ADDED, idx0, idx1);
	for (Enumeration enum=listeners.elements(); enum.hasMoreElements();)
	    ((ListDataListener)enum.nextElement()).intervalAdded(evt);
    }
    protected void fireIntervalRemoved(int idx0, int idx1) {
	ListDataEvent evt = new ListDataEvent
	    (this, ListDataEvent.INTERVAL_ADDED, idx0, idx1);
	for (Enumeration enum=listeners.elements(); enum.hasMoreElements();)
	    ((ListDataListener)enum.nextElement()).intervalRemoved(evt);
    }
    /***ContainerModel******************************/
    public int getItemCount() { return items.size(); }
    public EditorDocument getItem(int idx) {
	return (EditorDocument)items.elementAt(idx);
    }
    public int indexOfItem(EditorDocument doc) {
	return items.indexOf(doc);
    }
    public void addListDataListener(ListDataListener _l) {
	listeners.addElement(_l);
    }
    public void removeListDataListener(ListDataListener _l) {
	listeners.removeElement(_l);
    }
    /***DynamicContainerModel***************************/
    public void reload() {
	if (!items.isEmpty()) {
	    final int oldCount = items.size();
	    items.removeAllElements();
	    fireIntervalRemoved(0, oldCount-1);
	}
	if (document!=null && rowPath!=null) {
	    NodeList nl;
	    try {
		nl = XPathUtil.execute(rowPath, document.getNode()).nodeset();
	    } catch (SAXException e) {
		e.printStackTrace(); return;
	    }
	    final int c = nl.getLength();
	    for (int z=0; z<c; z++)
		items.addElement(new EditorDocument(nl.item(z), document));
	    if (!items.isEmpty())
		fireIntervalAdded(0, items.size()-1);
	}
    }
    public int insertItem(EditorDocument _doc, int idx) {
	if (idx>=getItemCount()) {
	    document.getNode().appendChild(_doc.getNode());
	    items.addElement(_doc);
	} else {
	    Node target = getItem(idx).getNode();
	    target.getParentNode().insertBefore(_doc.getNode(), target);
	    items.insertElementAt(_doc, idx);
	}
	fireIntervalAdded(idx, idx);
	return idx;
    }
    public boolean removeItem(int idx) {
	Node n = getItem(idx).getNode();
	n.getParentNode().removeChild(n);
	items.removeElementAt(idx);
	fireIntervalRemoved(idx, idx);
	return true;
    }
    public void itemChanged(int idx) {
	fireContentsChanged(idx, idx);
    }
    public int moveItem(int idx, int newIdx) {
	EditorDocument doc = getItem(idx);
	removeItem(idx);
	return insertItem(doc, newIdx);
    }
}
