package com.make.editor.plugins.count;

import java.util.Vector;
import java.util.Enumeration;
import java.text.MessageFormat;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.ParseException;
import javax.swing.JLabel;
import javax.swing.Timer;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.CharacterData;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;
import org.apache.xalan.xpath.XPath;
import com.make.datamodel.slot.Slot;
import com.make.datamodel.slot.SlotContainer;
import com.make.xml.DOMUtil;
import com.make.xml.XPathUtil;
import com.make.xml.DocumentType;
import com.make.xml.ElementType;
import com.make.editor.plugins.EditorComponent;
import com.make.editor.plugins.ComponentListener;
import com.make.editor.plugins.ComponentEvent;
import com.make.editor.plugins.DefaultEditorComponent;
import com.make.editor.EditorDocument;
import com.make.editor.EditorFrame;
import com.make.util.Strings;
import com.make.util.Debug;

public class CountComponent extends DefaultEditorComponent 
    implements ActionListener, ComponentListener {
    /***properties*****************************/
    // visual
    protected JLabel component = new JLabel();
    protected EditorComponent rootComponent;
    // data
    protected XPath selection;
    protected MessageFormat label;
    protected Object[] values = new Object[2];
    protected Timer timer = new Timer(1000, this);
    protected DocumentType documentType;
    /***constructor***************************/
    public CountComponent(EditorFrame _ef) {
	super(_ef);
	timer.setRepeats(false);
    }
    /***Internal*************************************/
    protected boolean count(Node n, int[] v, boolean skipRest) {
	// flow?
	boolean flowing = true;
	if (n instanceof Element) {
	    ElementType et = documentType.getElementType(n.getNodeName());
	    if (et==null || !et.isFlowing()) {
		flowing = false;
		skipRest = false;
	    }
	}
	// children
	NodeList nl = n.getChildNodes();
	for (int z=0; z<nl.getLength(); z++)
	    skipRest = count(nl.item(z), v, skipRest);
	// text
	if (n instanceof CharacterData) {
	    String s = ((CharacterData)n).getData();
	    char[] c = s.toCharArray();
	    for (int z=0; z<c.length; z++) {
		if (Character.isLetterOrDigit(c[z]))
		    v[0]++;
		if (Character.isWhitespace(c[z])) {
		    skipRest = false;
		} else {
		    if (!skipRest)
			v[1]++;
		    skipRest = true;
		}
	    }
	}
	// skip?
	return skipRest && flowing;
    }
    protected synchronized void count() {
	EditorDocument doc = getDocument();
	if (doc==null || selection==null) {
	    for (int z=0; z<values.length; z++)
		values[z] = new Integer(0);
	} else
	    try {
		NodeList nl = XPathUtil.execute(selection, doc.getNode()).nodeset();
		int[] v = new int[]{0,0};
		for (int z=nl.getLength()-1; z>=0; z--)
		    count(nl.item(z), v, false);
		values[0] = new Integer(v[0]);
		values[1] = new Integer(v[1]);
	    } catch (SAXException e) {
		e.printStackTrace();
		for (int z=0; z<values.length; z++)
		    values[z] = new Integer(0);
	    }
    }
    protected synchronized void updateDisplay() {
	StringBuffer sb = new StringBuffer();
	label.format(values, sb, null);
	component.setText(sb.toString());
	if (component.getParent()!=null)
	    component.getParent().validate();
    }
    /***EditorComponent********************************/
    public synchronized void setDocument(EditorDocument _doc) {
	super.setDocument(_doc);
	component.setText(null);
	if (rootComponent!=null) {
	    rootComponent.removeComponentListener(this);
	    rootComponent = null;
	}
	if (_doc==null) {
	    documentType = null;
	    selection = null;
	    label = null;
	    timer.stop();
	} else {
	    documentType = _doc.getType();
	    Element vd = (Element)getViewDescription("count");
	    try {
		selection = XPathUtil.parse(DOMUtil.getElementString(vd, "selection"));
	    } catch (SAXException e) {
		e.printStackTrace();
	    }
	    label = new MessageFormat(DOMUtil.getLocalizedElementString(vd, "label"));
	    timer.setRepeats(false);
	    timer.start();
	    rootComponent = editorFrame.getComponent(null);
	    rootComponent.addComponentListener(this);
	}
	// validate
	if (component.getParent()!=null)
	    component.getParent().validate();
    }
    public Component getComponent() { return component; }
    public void dispose() {
	timer.stop();
	timer = null;
	if (rootComponent!=null) {
	    rootComponent.removeComponentListener(this);
	    rootComponent = null;
	}
	super.dispose();
    }
    /***ActionListener********************************/
    public void actionPerformed(ActionEvent evt) {
	count();
	updateDisplay();
    }
    /***ComponentListener*****************************/
    public void focusGained(ComponentEvent evt) {}
    public void documentChanged(ComponentEvent evt) {
	if (timer.isRunning())
	    timer.restart();
	else
	    timer.start();
    }
}
