package com.make.editor.plugins.link;

import java.io.IOException;
import java.io.File;
import java.io.InputStream;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import com.make.datamodel.embedded.EmbeddedManager;
import com.make.datamodel.embedded.FileEmbeddedManager;
import com.make.datamodel.embedded.ZipEmbeddedManager;
import com.make.xml.DOMUtil;
import com.make.editor.EditorFrame;

class XmlFileFilter extends FileFilter {
    /***FileFilter******************************/
    public boolean accept(File f) {
	if (f!=null) {
	    if (f.isDirectory()) return true;
	    String name = f.getName();
	    return name.endsWith(".xml") ||
		name.endsWith(".wml");
	}
	return false;
    }
    public String getDescription() {
	return "eXtensible Markup Language (*.xml)";
    }
}

class ZipFileFilter extends FileFilter {
    /***FileFilter******************************/
    public boolean accept(File f) {
	if (f!=null) {
	    if (f.isDirectory()) return true;
	    String name = f.getName();
	    return name.endsWith(".zip");
	}
	return false;
    }
    public String getDescription() {
	return "XML-ZIP-Dateien (*.zip)";
    }
}

public class FileNodeSelector implements NodeSelector {
    /***properties****************************/
    protected JFileChooser fileChooser = new JFileChooser();
    protected EditorFrame editorFrame;
    /***Constructor****************************/
    public FileNodeSelector(Element vd, EditorFrame _ef) {
	editorFrame = _ef;
	// filters
	FileFilter xmlFilter = new XmlFileFilter();
	fileChooser.addChoosableFileFilter(xmlFilter);
	fileChooser.addChoosableFileFilter(new ZipFileFilter());
	fileChooser.setFileFilter(xmlFilter);
    }
    public static EmbeddedManager createEmbeddedManager(File f) 
	throws IOException {
	if (f.getName().endsWith(".zip"))
	    return new ZipEmbeddedManager(f, null);
	else
	    return new FileEmbeddedManager(f, null);
    }
    /***NodeSelector**************************/
    public Node selectNode() throws IOException, SAXException {
	final int chooseRet = fileChooser.showOpenDialog
	    (editorFrame.getInterfacePanel().getFrame());
	if (chooseRet!=fileChooser.APPROVE_OPTION)
	    return null;
	File f = fileChooser.getSelectedFile();
	if (f.exists()) {
	    EmbeddedManager em = createEmbeddedManager(f);
	    return em.getDocument();
	} else
	    return null;
    }
}
