package com.make.editor.plugins.link;

import java.io.IOException;
import java.awt.Component;
import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Hashtable;
import java.util.Enumeration;
import java.lang.reflect.Constructor;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;
import org.apache.xalan.xpath.XPath;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import com.make.xml.DOMUtil;
import com.make.xml.XPathUtil;
import com.make.datamodel.embedded.EmbeddedManager;
import com.make.editor.EditorFrame;
import com.make.editor.EditorDocument;
import com.make.editor.EditorDocumentType;
import com.make.editor.plugins.ContainerComponent;
import com.make.editor.plugins.EditorComponent;

class LinkComponent extends ContainerComponent 
    implements ActionListener {
    /***static***********************************/
    protected static Hashtable cache = new Hashtable();
    /***properties*********************************/
    protected JPanel panel = new JPanel(new BorderLayout(), false);
    protected Component component;
    protected Object[] assignments;
    protected EditorDocumentType documentType;
    protected NodeSelector selector;
    /***Constructor*******************************/
    public LinkComponent(EditorFrame _ef) {
	super(_ef);
	JButton bt = new JButton("Suchen");
	bt.addActionListener(this);
	panel.add("North", bt);
    }
    /***EditorComponent****************************/
    public void setDocument(EditorDocument doc) {
	super.setDocument(doc);
	// remove old
	if (component!=null)
	    panel.remove(component);
	// type/vd
	documentType = doc.getType();
	Element vd = (Element)getViewDescription("link");
	// selector
	Element sel = DOMUtil.getUniqueTag(vd, "selector");
	String clsName = sel.getAttribute("class");
	try {
	    Class cls = Class.forName(clsName);
	    Class[] paramClasses = { Element.class, EditorFrame.class };
	    Constructor constr = cls.getConstructor(paramClasses);
	    Object[] params = { sel, getEditorFrame() };
	    selector = (NodeSelector)constr.newInstance(params);
	} catch (Exception e) {
	    e.printStackTrace();
	    selector = null;
	}
	// on-select
	Element os = DOMUtil.getUniqueTag(vd, "on-select");
	Element[] oss = DOMUtil.getChildElements(os);
	assignments = new Object[oss.length*2];
	for (int z=0; z<oss.length; z++) {
	    String dest = oss[z].getAttribute("path");
	    String src = DOMUtil.getElementString(oss[z]);
	    XPath pt = (XPath)cache.get(src);
	    if (pt==null) 
		try {
		    pt = XPathUtil.parse(src);
		    cache.put(src, pt);
		} catch (SAXException e) {
		    e.printStackTrace();
		}
	    assignments[z*2] = dest;
	    assignments[z*2+1] = pt;
	}
	// content
	Element content = DOMUtil.getUniqueTag(vd, "content");
	Element[] ch = DOMUtil.getChildElements(content);
	component = createComponent(panel, ch[0]);
	// validate
	panel.validate();
    }
    public Component getComponent() { return panel; }
    /***Internal*****************************/
    protected void onSelect(Node ctx) {
	for (int z=0; z<assignments.length; z+=2) 
	    try {
		String dest = (String)assignments[z];
		XPath pt = (XPath)assignments[z+1];
		String val = XPathUtil.execute(pt, ctx).str();
		DOMUtil.setDeepString(getDocument().getNode(),
				      dest, val);
	    } catch (SAXException e) {
		e.printStackTrace();
	    }
	for (Enumeration enum=getChildren(); enum.hasMoreElements();)
	    ((Child)enum.nextElement()).reloadDocument();
    }
    /***ActionListener***************************/
    public void actionPerformed(ActionEvent evt) {
	new Thread() {
		public void run() {
		    try {
			Node n = selector.selectNode();
			if (n!=null)
			    onSelect(n);
		    } catch (Exception e) {
			JOptionPane.showMessageDialog
			    (getFrame(), e.getLocalizedMessage(),
			     "Error", JOptionPane.ERROR_MESSAGE);
		    }
		}
	    }.start();
    }
}
