package com.make.editor.plugins.simpleform;

import java.util.Vector;
import java.util.Enumeration;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.text.ParseException;
import javax.swing.JComponent;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JOptionPane;
import javax.swing.text.BadLocationException;
import org.w3c.dom.Element;
import com.make.datamodel.slot.Slot;
import com.make.datamodel.slot.SlotContainer;
import com.make.xml.DOMUtil;
import com.make.xml.slot.DOMSlotContainer;
import com.make.editor.plugins.DefaultEditorComponent;
import com.make.editor.EditorDocument;
import com.make.editor.EditorFrame;
import com.make.editor.field.EditorInputLinkFactory;
import com.make.util.Strings;
import com.make.util.Debug;
import com.make.swing.ButtonManager;
import com.make.swing.field.Field;
import com.make.swing.field.FieldListener;
import com.make.swing.field.FieldEvent;
import com.make.swing.il.InputLink;
import com.make.swing.il.InputLinkFactory;
import com.make.swing.il.ContainerInputLink;
import com.make.swing.il.FieldInputLink;

public class SimpleFormComponent extends DefaultEditorComponent {
    class ChangedListener implements FieldListener {
	/***properties************/
	protected Vector installedFields = new Vector();
	/***FieldListener***************/
	public void valueChanged(FieldEvent evt) {
	    fireDocumentChanged();
	}
	public void focusGained(FieldEvent evt) {}
	public void focusLost(FieldEvent evt) {}
	public void actionPerformed(FieldEvent evt) {}
	/***Public**********************/
	public void install(InputLink il) {
	    if (il instanceof FieldInputLink) {
		FieldInputLink fil = (FieldInputLink)il;
		Field f = fil.getField();
		f.addFieldListener(this);
		installedFields.addElement(f);
	    } else if (il instanceof ContainerInputLink) {
		ContainerInputLink cil = (ContainerInputLink)il;
		for (Enumeration enum=cil.getInputLinks(); enum.hasMoreElements();) {
		    InputLink il2 = (InputLink)enum.nextElement();
		    install(il2);
		}
	    }
	}
	public void uninstall() {
	    for (Enumeration enum=installedFields.elements(); enum.hasMoreElements();) {
		Field f = (Field)enum.nextElement();
		f.removeFieldListener(this);
	    }
	    installedFields.removeAllElements();
	}
    }
    /***properties*****************************/
    // visual
    protected Box box;
    protected Vector fields = new Vector();
    // data
    protected ChangedListener changedListener = new ChangedListener();
    /***constructor***************************/
    public SimpleFormComponent(EditorFrame _ef) {
	super(_ef);
	box = new Box(BoxLayout.Y_AXIS) {
	    public Dimension getMaximumSize() {
		return new Dimension(super.getMaximumSize().width,
				     getPreferredSize().height);
	    }
	};
    }
    /***EditorComponent********************************/
    public synchronized void setDocument(EditorDocument _doc) {
	super.setDocument(_doc);
	/*try {
	  flushDocument();
	  } catch (ParseException e) {
	  JOptionPane.showMessageDialog(getFrame(), e.getLocalizedMessage(),
	  "Eingabefehler", JOptionPane.ERROR_MESSAGE);
	  }*/
	final ButtonManager bm = editorFrame.getInterfacePanel().getButtonManager();
	// remove old rows
	for (Enumeration enum=fields.elements(); enum.hasMoreElements();)
	    ((InputLink)enum.nextElement()).uninstall(bm);
	unregisterInputComponent(box);
	Component[] oldRows = box.getComponents();
	for (int z=0; z<oldRows.length; z++)
	    box.remove(oldRows[z]);
	fields.removeAllElements();
	changedListener.uninstall();
	// load new rows
	if (_doc!=null) {
	    Element vd = (Element)getViewDescription("simpleform");
	    Element f = DOMUtil.getUniqueTag(vd, "fields");
	    if (f!=null) {
		SlotContainer sc
		    = new DOMSlotContainer(getDocument().getType(),
					   getDocument().getNode());
		InputLinkFactory factory 
		    = new EditorInputLinkFactory(editorFrame, getDocument(), sc);
		Element[] fs = DOMUtil.getChildElements(f);
		for (int z=0; z<fs.length; z++) {
		    InputLink il = factory.createInputLink(fs[z]);
		    if (il!=null) {
			fields.addElement(il);
			Component c = il.getComponent();
			if (c instanceof JComponent) {
			    JComponent jc = (JComponent)c;
			    Dimension pd = jc.getPreferredSize();
			    Dimension md = jc.getMaximumSize();
			    Dimension d = new Dimension(md.width, pd.height);
			    jc.setMaximumSize(d);
			}
			box.add(c);
			changedListener.install(il);
			il.install(bm);
		    }
		}
	    }
	}
	box.add(box.createGlue());
	registerInputComponent(box);
	// validate
	if (box.getParent()!=null)
	    box.getParent().validate();
    }
    public void flushDocument() throws ParseException {
	for (Enumeration enum=fields.elements(); enum.hasMoreElements();) {
	    InputLink field = (InputLink)enum.nextElement();
	    field.apply();
	}
    }
    public Component getComponent() { return box; }
    public void dispose() {
	final ButtonManager bm = editorFrame.getInterfacePanel().getButtonManager();
	for (Enumeration enum=fields.elements(); enum.hasMoreElements();) {
	    InputLink il = (InputLink)enum.nextElement();
	    il.uninstall(bm);
	    il.dispose();
	}
	super.dispose();
    }
}
