package com.make.editor.plugins.table;

import java.util.Vector;
import java.util.Enumeration;
import java.util.Date;
import java.util.Hashtable;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import javax.swing.Action;
import javax.swing.AbstractAction;
import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.JComponent;
import javax.swing.KeyStroke;
import javax.swing.plaf.basic.BasicTableUI;
import javax.swing.table.TableModel;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableCellEditor;
import javax.swing.event.TableModelListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;
import org.apache.xalan.xpath.XPath;
import com.make.xml.DOMUtil;
import com.make.xml.XPathUtil;
import com.make.datamodel.slot.Slot;
import com.make.datamodel.slot.SimpleSlot;
import com.make.datamodel.slot.SlotContainer;
import com.make.editor.EditorSession;
import com.make.editor.plugins.EditorPlugin;
import com.make.editor.plugins.DefaultEditorComponent;
import com.make.editor.plugins.EditorCursorComponent;
import com.make.editor.plugins.EditorCursor;
import com.make.editor.plugins.AbstractEditorCursor;
import com.make.editor.EditorDocument;
import com.make.editor.EditorFrame;
import com.make.editor.field.EditorInputLinkFactory;
import com.make.util.Strings;
import com.make.util.Debug;
import com.make.swing.field.Field;
import com.make.swing.field.FieldCellEditor;
import com.make.swing.il.FieldInputLink;
import com.make.swing.il.InputLink;
import com.make.swing.il.InputLinkFactory;

public class TableComponent extends DefaultEditorComponent 
    implements EditorCursorComponent, TableModel {
    class TableInputLinkFactory 
	extends EditorInputLinkFactory {
	public TableInputLinkFactory() {
	    super(getEditorFrame(), 
		  TableComponent.this.getDocument(), null);
	}
	public Slot getSlot(Element vd) {
	    return null;
	}
	public String getType(Element vd) {
	    return DOMUtil.getElementString(vd, "field");
	}
    }
    class CellEditor extends FieldCellEditor
	implements TableCellEditor {
	protected TableCellEditor original;
	protected boolean forwarding;
	public CellEditor(TableCellEditor _o) {
	    original = _o;
	}
	public Object getCellEditorValue() {
	    return forwarding
		?original.getCellEditorValue()
		:super.getCellEditorValue();
	}
	//public boolean isCellEditable(EventObject anEvent)
	//public boolean shouldSelectCell(EventObject anEvent)
	public boolean stopCellEditing() {
	    return forwarding
		?original.stopCellEditing()
		:super.stopCellEditing();
	}
	public void cancelCellEditing() {
	    if (forwarding)
		original.cancelCellEditing();
	    else
		super.cancelCellEditing();
	}
	public void addCellEditorListener(CellEditorListener l) {
	    super.addCellEditorListener(l);
	    original.addCellEditorListener(l);
	}
	public void removeCellEditorListener(CellEditorListener l) {
	    super.removeCellEditorListener(l);
	    original.removeCellEditorListener(l);
	}
	public Component getTableCellEditorComponent(JTable table,
						     Object value,
						     boolean isSelected,
						     int row,
						     int column) {
	    Column c = columns[column];
	    Field f = c.getField((Node)rows.elementAt(row));
	    forwarding = f==null;
	    if (forwarding) {
		forwarding = true;
		return original.getTableCellEditorComponent
		    (table, value, isSelected, row, column);
	    } else {
		Node r = (Node)rows.elementAt(row);
		return getComponent(f, c.getValue(r), isSelected);
	    }
	}
    }
    class RowSelectionCursor extends AbstractEditorCursor
	implements ListSelectionListener {
	/***Constructor******/
	public RowSelectionCursor() {
	    table.getSelectionModel().addListSelectionListener(this);
	}
	/***ListSelectionListener*****/
	public void valueChanged(ListSelectionEvent evt) {
	    if (!evt.getValueIsAdjusting()) {
		int[] sel = table.getSelectedRows();
		EditorDocument[] docs = new EditorDocument[sel.length];
		for (int z=0; z<sel.length; z++) {
		    int row = sel[z];
		    docs[z] = new EditorDocument((Node)rows.elementAt(row), 
						 getDocument());
		}
		setSelection(docs);
	    }
	}
    }
    class EditorTableUI extends BasicTableUI 
	implements KeyListener {
	KeyListener original;
	public void keyTyped(KeyEvent evt) {
	    // this is a workaround: registered hotkeys are processed twice;
	    // once by JComponent (keyBindings) and once by BasicTableUI.
	    // this code tries to filter out the second one.
	    KeyStroke keyStroke; int cond;
	    // keyChar
	    keyStroke = KeyStroke.getKeyStroke(evt.getKeyChar());
	    cond = this.table.getConditionForKeyStroke(keyStroke);
	    if (cond!=this.table.UNDEFINED_CONDITION)
		return;
	    // keyCode
	    keyStroke = KeyStroke.getKeyStroke(evt.getKeyChar(), evt.getModifiers());
	    cond = this.table.getConditionForKeyStroke(keyStroke);
	    if (cond!=this.table.UNDEFINED_CONDITION)
		return;
	    original.keyTyped(evt);
	}
	public void keyPressed(KeyEvent evt) {}
	public void keyReleased(KeyEvent evt) {}
	protected KeyListener createKeyListener() {
	    original = super.createKeyListener();
	    return this;
	}
    }
    /***properties*****************************/
    protected XPath rowPath;
    protected Vector rows = new Vector();
    protected String autoNewTag;
    protected boolean editable = false;
    protected int newRow = -1;
    protected Node newNode;
    protected Column[] columns;
    protected Vector listeners = new Vector();
    protected JTable table = new JTable(this);
    protected RowSelectionCursor rowSelectionCursor;
    protected Slot editorSlot = new SimpleSlot();
    /***Constructor***************************/
    public TableComponent(EditorFrame _ef) {
	super(_ef);
	table.setUI(new EditorTableUI());
	TableCellEditor oe = table.getDefaultEditor(String.class);
	table.setDefaultEditor(String.class, new CellEditor(oe));
	registerInputComponent(table);
    }
    protected Node createNewNode() {
	if (Debug.DEBUG && autoNewTag==null)
	    throw new InternalError();
	return getXMLDocument().createElement(autoNewTag);
    }
    protected void loadColumns(Element _c) {
	if (_c==null)
	    columns = new Column[0];
	else {
	    TableInputLinkFactory tilf 
		= new TableInputLinkFactory();
	    Element[] cs = DOMUtil.getChildElements(_c);
	    columns = new Column[cs.length];
	    for (int z=0; z<columns.length; z++)
		columns[z] = AbstractColumn.createColumn(cs[z], tilf);
	}
    }
    protected Action loadAction(Element _a) {
	String tag = _a.getTagName();
	if (tag.equals("insert-element"))
	    return new InsertElementAction(_a.getAttribute("tag"));
	else
	    return super.loadAction(_a);
    }
    protected synchronized void reload() {
	if (table.getCellEditor()!=null)
	    try {
		unregisterInputComponent(table);
		table.getCellEditor().stopCellEditing();
	    } finally {
		registerInputComponent(table);
	    }
	rows.removeAllElements();
	if (rowPath!=null)
	    try {
		Node n = getDocument().getNode();
		NodeList nl = XPathUtil.execute(rowPath, n).nodeset();
		for (int z=0; z<nl.getLength(); z++)
		    rows.addElement(nl.item(z));
		if (autoNewTag==null) {
		    newNode = null;
		    newRow = -1;
		} else {
		    newNode = createNewNode();
		    if (newNode!=null) {
			rows.addElement(newNode);
			newRow = rows.indexOf(newNode);
		    } else
			newRow = -1;
		}
	    } catch (SAXException e) {
		e.printStackTrace();
	    }
	fireTableStructureChanged();
    }
    /***EditorComponent********************************/
    public synchronized void setDocument(EditorDocument _doc) {
	super.setDocument(_doc);
	// config
	Element vd = _doc==null
	    ?null
	    :(Element)getViewDescription("table");
	if (vd==null) {
	    rowPath = null;
	    columns = new Column[0];
	} else {
	    loadActions(DOMUtil.getUniqueTag(vd, "actions"));
	    loadKeyBindings(table, DOMUtil.getUniqueTag(vd, "keybindings"));
	    Element cols = DOMUtil.getUniqueTag(vd, "columns");
	    loadColumns(cols);
	    editable = DOMUtil.childTagExists(vd, "editable");
	    // rows
	    Element rw = DOMUtil.getUniqueTag(vd, "rows");
	    try {
		rowPath = XPathUtil.parse(rw.getAttribute("select"));
	    } catch (SAXException e) {
		rowPath = null;
		e.printStackTrace();
	    }
	    Element an = DOMUtil.getUniqueTag(rw, "auto-new");
	    if (an==null)
		autoNewTag = null;
	    else
		autoNewTag = an.getAttribute("tag");
	}
	reload();
    }
    public Component getComponent() { 
	return table;
    }
    public void focusLost() {
	super.focusLost();
	//Thread.dumpStack();
	if (table.getCellEditor()!=null)
	    try {
		unregisterInputComponent(table);
		table.getCellEditor().stopCellEditing();
	    } finally {
		registerInputComponent(table);
	    }
    }
    /***EditorCursorComponent****************************/
    public EditorCursor getCursor(Object key) {
	if (rowSelectionCursor==null)
	    rowSelectionCursor = new RowSelectionCursor();
	return rowSelectionCursor;
    }
    /***Fire*****************************************/
    public void fireTableDataChanged() {
        fireTableChanged(new TableModelEvent(this));
    }
    public void fireTableStructureChanged() {
        fireTableChanged(new TableModelEvent(this, TableModelEvent.HEADER_ROW));
    }
    public void fireTableRowsInserted(int firstRow, int lastRow) {
        fireTableChanged(new TableModelEvent(this, firstRow, lastRow,
					     TableModelEvent.ALL_COLUMNS, 
					     TableModelEvent.INSERT));
    }
    public void fireTableRowsUpdated(int firstRow, int lastRow) {
        fireTableChanged(new TableModelEvent(this, firstRow, lastRow,
					     TableModelEvent.ALL_COLUMNS, 
					     TableModelEvent.UPDATE));
    }
    public void fireTableRowsDeleted(int firstRow, int lastRow) {
        fireTableChanged(new TableModelEvent(this, firstRow, lastRow,
					     TableModelEvent.ALL_COLUMNS, 
					     TableModelEvent.DELETE));
    }
    public void fireTableCellUpdated(int row, int column) {
        fireTableChanged(new TableModelEvent(this, row, row, column));
    }
    public void fireTableChanged(TableModelEvent e) {
	for (Enumeration enum=listeners.elements(); enum.hasMoreElements();)
	    ((TableModelListener)enum.nextElement()).tableChanged(e);
    }

    /***TableModel************************************/
    public int getRowCount() { 
	return rows.size();
    }
    public int getColumnCount() { 
	return columns==null?1:columns.length;
    }
    public String getColumnName(int columnIndex) {
	return columns==null?"Name":columns[columnIndex].getCaption();
    }
    public Class getColumnClass(int columnIndex) {
	return String.class;
    }
    public boolean isCellEditable(int rowIndex, int columnIndex) {
	return columns[columnIndex].isEditable((Node)rows.elementAt(rowIndex));
    }
    public Object getValueAt(int rowIndex, int columnIndex) {
	Node n = (Node)rows.elementAt(rowIndex);
	return columns[columnIndex].getValue(n);
    }
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
	if (!editable) return;
	// convert to string
	if (aValue!=null)
	    aValue = aValue.toString();
	String value = Strings.trim((String)aValue);
	if (value==null && rowIndex==newRow)
	    return;
	// set value
	Node n = (Node)rows.elementAt(rowIndex);
	columns[columnIndex].setValue(n, value);
	// if this is the auto-new-row, then add this row to the document
	// and create a new one.
	if (autoNewTag!=null && rowIndex==newRow) {
	    document.getNode().appendChild(n);
	    newNode = createNewNode();
	    if (newNode!=null) {
		rows.addElement(newNode);
		newRow = rows.indexOf(newNode);
		fireTableRowsInserted(newRow, newRow);
		// select the next cell
		columnIndex++;
		if (columnIndex>=table.getColumnCount()) {
		    columnIndex = 0;
		    table.getSelectionModel().setSelectionInterval(newRow, newRow);
		}
		table.setColumnSelectionInterval(columnIndex, columnIndex);
	    } else
		newRow = -1;
	}
    }
    public void addTableModelListener(TableModelListener l) {
	listeners.addElement(l);
    }
    public void removeTableModelListener(TableModelListener l) {
	listeners.removeElement(l);
    }
    /***Actions***********************************/
    public Action[] getActions() { return actions; }
    private Action[] actions = {
	new DeleteRowAction(),
	new ClearCellAction(),
	new MoveRowUpAction(),
	new MoveRowDownAction(),
    };
    class DeleteRowAction extends AbstractAction {
	public DeleteRowAction() { super("delete-row"); }
	public void actionPerformed(ActionEvent evt) {
	    int oldSelection = table.getSelectedRow();
	    int[] sel = table.getSelectedRows();
	    for (int z=sel.length-1; z>=0; z--) {
		int row = sel[z];
		if (row!=newRow) {
		    Node n = (Node)rows.elementAt(row);
		    rows.removeElementAt(row);
		    n.getParentNode().removeChild(n);
		    if (row<newRow)
			newRow--;
		    if (row<oldSelection)
			oldSelection--;
		    fireTableRowsDeleted(row, row);
		}
	    }
	    final int count = rows.size();
	    if (count>0) {
		if (oldSelection>=count)
		    oldSelection = count-1;
		if (oldSelection>=0)
		    table.getSelectionModel().setSelectionInterval(oldSelection, oldSelection);
	    }
	}
    }
    class ClearCellAction extends AbstractAction {
	public ClearCellAction() { super("clear-cell"); }
	public void actionPerformed(ActionEvent evt) {
	    int row = table.getSelectedRow();
	    int column = table.getSelectedColumn();
	    if (row!=-1 && column!=-1) {
		setValueAt(null, row, column);
		fireTableCellUpdated(row, column);
	    }
	}
    }
    class MoveRowUpAction extends AbstractAction {
	public MoveRowUpAction() { 
	    super("move-up"); 
	}
	public void actionPerformed(ActionEvent evt) {
	    int row = table.getSelectedRow();
	    if (row==-1 || row==newRow)
		return;
	    int targetRow = row-1;
	    if (targetRow==newRow)
		targetRow--;
	    if (targetRow<0)
		return;
	    Node n = (Node)rows.elementAt(row);
	    Node parent = n.getParentNode();
	    rows.removeElementAt(row);
	    fireTableRowsDeleted(row, row);
	    Node target = (Node)rows.elementAt(targetRow);
	    parent.insertBefore(n, target);
	    rows.insertElementAt(n, targetRow);
	    fireTableRowsInserted(targetRow, targetRow);
	    table.getSelectionModel().setSelectionInterval(targetRow, targetRow);
	}
    }
    class MoveRowDownAction extends AbstractAction {
	public MoveRowDownAction() { 
	    super("move-down"); 
	}
	public void actionPerformed(ActionEvent evt) {
	    int row = table.getSelectedRow();
	    if (row==-1 || row==newRow)
		return;
	    int targetRow = row+1;
	    if (targetRow==newRow)
		targetRow++;
	    targetRow++;
	    if (targetRow==newRow)
		targetRow++;
	    if (targetRow>rows.size())
		return;
	    Node n = (Node)rows.elementAt(row);
	    Node parent = n.getParentNode();
	    rows.removeElementAt(row);
	    fireTableRowsDeleted(row, row);
	    targetRow--;
	    Node target = targetRow==newRow || targetRow>=rows.size()
		?null
		:(Node)rows.elementAt(targetRow);
	    if (targetRow==newRow)
		targetRow--;
	    if (target==null)
		parent.appendChild(n);
	    else
		parent.insertBefore(n, target);
	    rows.insertElementAt(n, targetRow);
	    fireTableRowsInserted(targetRow, targetRow);
	    table.getSelectionModel().setSelectionInterval(targetRow, targetRow);
	}
    }
    abstract class InsertNodeAction extends AbstractAction {
	public InsertNodeAction(String _ac) { super(_ac); }
	protected abstract Node createNode();
	public void actionPerformed(ActionEvent evt) {
	    int row = table.getSelectedRow();
	    if (row==-1)
		row = rows.size()-1;
	    if (row==newRow)
		row++;
	    Node n = createNode();
	    Node target = row==-1 || row>=rows.size()
		?null
		:(Node)rows.elementAt(row);
	    if (row>=rows.size() && row==(newRow+1))
		row--;
	    Node parent = getDocument().getNode();
	    if (target==null)
		parent.appendChild(n);
	    else
		parent.insertBefore(n, target);
	    rows.insertElementAt(n, row);
	    fireTableRowsInserted(row, row);
	    table.getSelectionModel().setSelectionInterval(row, row);
	    if (row<=newRow)
		newRow++;
	}
    }
    class InsertElementAction extends InsertNodeAction {
	String tag;
	public InsertElementAction(String _tag) { 
	    super("insert-"+_tag); 
	    tag = _tag;
	}
	protected Node createNode() {
	    return getXMLDocument().createElement(tag);
	}
    }
}
