package com.make.editor.plugins.text;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.io.InputStream;
import java.io.Reader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.OutputStreamWriter;
import java.io.IOException;
import java.text.DateFormat;
import java.util.Date;
import java.util.Hashtable;
import java.util.Vector;
import javax.swing.Action;
import javax.swing.AbstractAction;
import javax.swing.JOptionPane;
import javax.swing.text.Segment;
import javax.swing.text.Position;
import javax.swing.text.BadLocationException;
import org.apache.xalan.xslt.XSLTProcessor;
import org.apache.xalan.xslt.XSLTProcessorFactory;
import org.apache.xalan.xslt.XSLTInputSource;
import org.apache.xalan.xslt.XSLTResultTarget;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.w3c.dom.Node;
import org.w3c.dom.Text;
import org.w3c.dom.NodeList;
import com.make.xml.DOMWriter;
import com.make.xml.DOMUtil;
import com.make.xml.DocumentType;
import com.make.datamodel.embedded.EmbeddedManager;
import com.make.datamodel.embedded.MemoryEmbeddedManager;
import com.make.util.Streams;
import com.make.util.Strings;
import com.make.editor.EditorFrame;
import com.make.editor.EditorSession;
import com.make.editor.plugins.text.tree.SwingDocument;
import com.make.editor.plugins.text.tree.NoteElement;
import com.make.editor.plugins.text.tree.NumberedList;
import com.make.editor.plugins.text.tree.UnorderedList;
import com.make.editor.plugins.text.tree.BreakElement;
import com.make.editor.plugins.text.tree.StyleElement;
import com.make.editor.plugins.text.tree.ElementSwingElement;
import com.make.editor.plugins.text.dlg.DefaultEditorDialog;
import com.make.editor.plugins.text.view.XmlViewFactory;
import com.make.swing.ButtonManager;
import com.make.swing.text.DefaultEditorKit;
import com.make.swing.text.view.ViewFactory;
import com.make.swing.text.Element;
import com.make.swing.text.Document;
import com.make.swing.text.DocumentEvent;
import com.make.swing.text.Caret;
import com.make.swing.text.TextComponent;

public class TextActions extends DefaultEditorKit {
    /***static*****************************************/
    protected static DataFlavor plainTextFlavor = DataFlavor.plainTextFlavor;
    protected static DataFlavor stringFlavor = DataFlavor.stringFlavor;
    protected static DataFlavor xmlFlavor
	= new DataFlavor("text/xml; charset=ISO-8859-1", "XML");
    protected static DataFlavor xmlJavaFlavor
	= new DataFlavor("application/x-java-serialized-object-leanedit; charset=ISO-8859-1", 
			 "XML mit Bildern");
    protected static final ViewFactory viewFactory
	= new XmlViewFactory();
    /***properties*********************************/
    private TextEditorComponent tec;
    protected EditorFrame editorFrame;
    protected Vector xmlActions = new Vector();
    /***Constructor********************************/
    public TextActions(TextEditorComponent _tec, EditorFrame _f) {
	tec = _tec; editorFrame = _f;
    }
    public void loadActions(org.w3c.dom.Element ac, ButtonManager bm) {
	synchronized(xmlActions) {
	    if (bm!=null)
		bm.uninstallActions(xmlActions.elements());
	    xmlActions.removeAllElements();
	    org.w3c.dom.Element[] acs = DOMUtil.getChildElements(ac);
	    for (int z=0; z<acs.length; z++) {
		Action a;
		String tag = acs[z].getTagName();
		if (tag.equals("formataction"))
		    a = new FormatStyleAction(acs[z]);
		else if (tag.equals("insertaction"))
		    a = new InsertTagAction(acs[z]);
		else if (tag.equals("embedaction"))
		    a = new EmbedTagAction(acs[z]);
		else if (tag.equals("insertdlgaction"))
		    a = new InsertDialogElementAction(acs[z]);
		else {
		    System.err.println("WARNING: unknown action type "+tag);
		    a = null;
		}
		if (a!=null)
		    xmlActions.addElement(a);
	    }
	    if (bm!=null)
		bm.installActions(xmlActions.elements());
	}
    }
    /***DefaultEditorKit*******************************/
    public void installActions(ButtonManager bm) {
	super.installActions(bm);
	bm.installActions(actions);
	synchronized(xmlActions) {
	    bm.installActions(xmlActions.elements());
	}
    }
    public void uninstallActions(ButtonManager bm) {
	super.uninstallActions(bm);
	bm.uninstallActions(actions);
	synchronized(xmlActions) {
	    bm.uninstallActions(xmlActions.elements());
	}
    }
    /***EditorKit**********************************/
    public ViewFactory getViewFactory() { return viewFactory; }
    public Document createDefaultDocument() {
	throw new InternalError();
    }
    /***Actions******************************************/
    private final Action[] actions = {
	//new InsertNoteAction(),
	new EditCopyAction(),
	new EditCutAction(),
	new EditPasteAction(),
        new PaintAllAction(),
    };
    final class EditCopyAction extends DocumentAction {
	public EditCopyAction() { super(copyAction, false); }
	public void actionPerformed(Document d, int start, int end, ActionEvent evt) 
	    throws BadLocationException {
	    if (start==end) return;
	    Clipboard clipboard = tec.getSession().getClipboard();
	    try {
		SwingDocument doc = (SwingDocument)d;
		// plain
		Segment plain = new Segment();
		doc.getText(start, end-start, plain);
		final byte[] plainData = Streams.charArrayToByteArray(plain.array, "ISO-8859-1");
		// xml
		final org.w3c.dom.Node xdoc = doc.copy(start, end);
		MemoryEmbeddedManager mem = new MemoryEmbeddedManager("[Clipboard]");
		final DocumentType docType = doc.getDocumentType();
		if (docType==null) {
		    mem = null;
		} else {
		    mem.replaceDocument(xdoc);
		    EmbeddedManager sem = doc.getEmbeddedManager();
		    String[] names = docType.getEmbeddedNames(xdoc);
		    if (names.length==0)
			mem = null;
		    else {
			int count = 0;
			for (int z=0; z<names.length; z++)
			    try {
				mem.replaceEntry(names[z], sem.getEntry(names[z]));
				count++;
			    } catch (IOException e) {
			    }
			if (count==0) mem = null;
		    }
		} 
		final MemoryEmbeddedManager fem = mem;
		// Transferable
		clipboard.setContents(new Transferable() {
		    protected DataFlavor[] flavors = new DataFlavor[] {
			plainTextFlavor,
			xmlFlavor,
			stringFlavor,
			xmlJavaFlavor,
		    };
		    // internal
		    protected int findFlavor(DataFlavor fl) {
			for (int z=0; z<flavors.length; z++)
			    if (flavors[z].isMimeTypeEqual(fl))
				return z;
			return -1;
		    }
		    // Transferable
		    public DataFlavor[] getTransferDataFlavors() {
			return flavors;
		    }
		    public boolean isDataFlavorSupported(DataFlavor flavor) {
			int idx = findFlavor(flavor);
			return idx!=-1 && (idx!=3 || fem!=null);
		    }
		    public Object getTransferData(DataFlavor flavor) 
			throws UnsupportedFlavorException, IOException {
			switch (findFlavor(flavor)) {
			case 0: return new ByteArrayInputStream(plainData);
			case 1: {
			    org.w3c.dom.Document xd = xdoc.getOwnerDocument();
			    byte[] b = DOMUtil.toByteArray(xd);
			    return new ByteArrayInputStream(b);
			}
			case 2: return new String(plainData, "ISO-8859-1");
			case 3: return fem;
			default: return null;
			}
		    }
		}, tec);
		if (clipboard.getContents(this) instanceof StringSelection) {
		    //workaround for windows jvm 1.1.8
		    clipboard.setContents(new StringSelection(new String(plainData, "ISO-8859-1")),
					  tec);
		}
	    } catch (IOException e) {
		e.printStackTrace();
		editor.getToolkit().beep();
	    }
	}
    }
    final class EditCutAction extends DocumentAction {
	protected EditCopyAction copy = new EditCopyAction();
	public EditCutAction() { super(cutAction); }
        public void actionPerformed(Document doc, int start, int end, ActionEvent evt) 
	    throws BadLocationException {
	    if (start==end) return;
	    copy.actionPerformed(doc, start, end, evt);
	    doc.remove(start, end-start);
	}
    }
    final class EditPasteAction extends DocumentAction {
	public EditPasteAction() { super(pasteAction); }
	protected boolean isDataFlavorSupported(Transferable t, DataFlavor df) {
	    // workaround: NullPointerException mit X11Selection
	    try {
		return t.isDataFlavorSupported(df);
	    } catch (Exception e) {
		return false;
	    }
	}
        public void actionPerformed(Document d, int start, int end, ActionEvent evt) 
	    throws BadLocationException {
	    Clipboard clipboard = tec.getSession().getClipboard();
	    try {
		SwingDocument doc = (SwingDocument)d;
		Transferable t = clipboard.getContents(this);
		if (t==null) {
		    editor.getToolkit().beep(); return;
		}
		if (isDataFlavorSupported(t, xmlJavaFlavor)) {
		    // fetch&parse clipboard
		    org.w3c.dom.Node del;
		    try {
			EmbeddedManager sem = doc.getEmbeddedManager();
			MemoryEmbeddedManager mem = 
			    (MemoryEmbeddedManager)t.getTransferData(xmlJavaFlavor);
			del = mem.getDocument();
			String[] names = mem.getNames();
			for (int z=0; z<names.length; z++)
			    if (!sem.contains(names[z]))
				sem.replaceEntry(names[z], mem.getEntry(names[z]));
		    } catch (IOException e) {
			e.printStackTrace(); throw new InternalError();
		    } catch (SAXException e) {
			JOptionPane.showMessageDialog
			    (null, "Fehler beim Parsen der XML-Zwischenablage:\n"+
			     e.getLocalizedMessage(), "Einf\u00fcgen",
			     JOptionPane.ERROR_MESSAGE);
			return;
		    }
		    if (del==null) {
			editor.getToolkit().beep(); return;
		    }
		    // TODO: pruefung auf selben Root-Tag
		    // find child node
		    Node n = null;
		    NodeList nl = del.getChildNodes();
		    for (int z=0; z<nl.getLength(); z++) {
			n = nl.item(z);
			if (!(n instanceof Text))
			    break;
		    }
		    if (n==null) return;
		    // paste
		    org.w3c.dom.Document xdoc = doc.getXmlDocument();
		    try {
			doc.beginUpdate();
			if (start<end)
			    doc.remove(start, end-start);
			doc.paste(start, DOMUtil.cloneNode(xdoc, n, true));
		    } finally {
			doc.endUpdate();
		    }
		} else if (isDataFlavorSupported(t, xmlFlavor)) {
		    // fetch&parse clipboard
		    InputStream is = (InputStream)t.getTransferData(xmlFlavor);
		    org.w3c.dom.Document cdoc;
		    try {
			cdoc = DOMUtil.parse(is);
		    } catch (SAXException e) {
			JOptionPane.showMessageDialog
			    (null, "Fehler beim Parsen der XML-Zwischenablage:\n"+
			     e.getLocalizedMessage(), "Einf\u00fcgen",
			     JOptionPane.ERROR_MESSAGE);
			return;
		    }
		    org.w3c.dom.Element del = cdoc.getDocumentElement();
		    if (del==null) {
			editor.getToolkit().beep(); return;
		    }
		    // TODO: pruefung auf selben Root-Tag
		    // find child node
		    Node n = null;
		    NodeList nl = del.getChildNodes();
		    for (int z=0; z<nl.getLength(); z++) {
			n = nl.item(z);
			if (!(n instanceof Text))
			    break;
		    }
		    if (n==null) return;
		    // paste
		    org.w3c.dom.Document xdoc = doc.getXmlDocument();
		    try {
			doc.beginUpdate();
			if (start<end)
			    doc.remove(start, end-start);
			doc.paste(start, DOMUtil.cloneNode(xdoc, n, true));
		    } finally {
			doc.endUpdate();
		    }
		} else if (isDataFlavorSupported(t, plainTextFlavor)) {
		    StringBuffer sb = new StringBuffer();
		    Object td = t.getTransferData(plainTextFlavor);
		    if (td instanceof Reader)
			Streams.append(sb, (Reader)td);
		    else
			Streams.append(sb, (InputStream)td, "ISO-8859-1");
		    if (sb.length()>0)
			doc.replace(start, end-start, sb.toString());
		} else if (isDataFlavorSupported(t, DataFlavor.stringFlavor)) {
		    String s = (String)t.getTransferData(DataFlavor.stringFlavor);
		    if (s!=null && s.length()>0)
			doc.replace(start, end-start, s);
		} else
		    editor.getToolkit().beep();
	    } catch (UnsupportedFlavorException e) {
		e.printStackTrace(); editor.getToolkit().beep();
	    } catch (IOException e) {
		e.printStackTrace(); editor.getToolkit().beep();
	    } catch (org.w3c.dom.DOMException e) {
		System.err.println("DOMException code="+e.code);
		e.printStackTrace();
	    }
	}
    }
    /*final class InsertNoteAction extends DocumentAction {
	public InsertNoteAction() { super("insert-note"); }
        public void actionPerformed(Document doc, int start, int end, ActionEvent evt2) 
	    throws BadLocationException {
	    org.w3c.dom.Document xdoc = ((SwingDocument)doc).getXmlDocument();
	    org.w3c.dom.Element el = xdoc.createElement("note");
	    DateFormat df = DateFormat.getDateTimeInstance();
	    EditorSession sess = editorFrame.getSession();
	    String fullName = (String)sess.getUserValue(sess.U_FULL_NAME);
	    if (fullName==null)
		fullName = (String)sess.getUserValue(sess.U_NAME);
	    if (fullName!=null)
		el.setAttribute("author", fullName);
	    el.setAttribute("time", df.format(new Date()));
	    NoteElement nel = new NoteElement(el, (SwingDocument)doc);
	    Caret caret = editor.getCaret();
	    doc.beginUpdate();
	    try {
		DocumentEvent evt = new DocumentEvent(doc);
		if (doc.getRootElement().pasteElementAt(caret.getDot(), nel, evt)) {
		    doc.fireDocumentChanged(evt);
		    caret.setDot(nel.getStartOffset()+1);
		} else
		    JOptionPane.showMessageDialog
			(null, "An dieser Stelle kann keine Bemerkung eingef\u00fcgt werden.",
			 "Einf\u00fcgen", JOptionPane.ERROR_MESSAGE);
	    } finally {
		doc.endUpdate();
	    }
	}
	}*/
    final class FormatNumberedListAction extends EmbedElementAction {
	public FormatNumberedListAction() {
	    super("format-numbered-list");
	}
	public Element createElement(Document doc) {
	    org.w3c.dom.Document xdoc = ((SwingDocument)doc).getXmlDocument();
	    org.w3c.dom.Element el = xdoc.createElement("ol");
	    el.setAttribute("suffix", ".)");
	    return new NumberedList(el, (SwingDocument)doc);
	}
    }
    final class FormatUnorderedListAction extends EmbedElementAction {
	public FormatUnorderedListAction() {
	    super("format-unordered-list");
	}
	public Element createElement(Document doc) {
	    org.w3c.dom.Document xdoc = ((SwingDocument)doc).getXmlDocument();
	    org.w3c.dom.Element el = xdoc.createElement("ul");
	    return new UnorderedList(el, (SwingDocument)doc);
	}
    }
    protected org.w3c.dom.Node createNodeInstance(String style) {
	Document _doc = editor.getDocument();
	SwingDocument doc = (SwingDocument)_doc;
	org.w3c.dom.Document xdoc = doc.getXmlDocument();
	Hashtable ht = doc.getStyle(style);
	if (ht!=null) {
	    org.w3c.dom.Element nc =
		(org.w3c.dom.Element)ht.get(TextEditorStyleSet.NEW_CLONE);
	    if (nc!=null)
		return DOMUtil.cloneNode(xdoc, nc, true);
	    org.w3c.dom.Element xslt
		= (org.w3c.dom.Element)ht.get(TextEditorStyleSet.NEW_XSLT);
	    if (xslt!=null) {
		DOMUtil.setUserData(xdoc, "editorSession", tec.getSession());
		XSLTInputSource docSource = new XSLTInputSource(xdoc);
		XSLTInputSource styleSource = new XSLTInputSource(xslt);
		org.w3c.dom.Document dest = DOMUtil.createDocument();
		XSLTResultTarget target = new XSLTResultTarget(dest);
		try {
		    XSLTProcessor processor
			= XSLTProcessorFactory.getProcessorUsingLiaisonName
			("org.apache.xalan.xpath.xdom.XercesLiaison");
		    processor.process(docSource, styleSource, target);
		} catch (Exception e) {
		    e.printStackTrace();
		}
		org.w3c.dom.Element result1 = dest.getDocumentElement();
		if (result1!=null)
		    return DOMUtil.cloneNode(xdoc, result1, true);
	    }
	}
	return xdoc.createElement(style);
    }
    protected Element createElementInstance(String style) {
	Document _doc = editor.getDocument();
	SwingDocument doc = (SwingDocument)_doc;
	org.w3c.dom.Node n = createNodeInstance(style);
	return n==null
	    ?null
	    :doc.getSwingElementFactory().create(null, n);
    }
    final class FormatStyleAction extends DocumentAction {
	protected String style;
	public FormatStyleAction(org.w3c.dom.Element _el) {
	    super(_el.getAttributeNode("command")==null
		  ?"format-"+_el.getAttribute("style")
		  :_el.getAttribute("command"));
	    style = _el.getAttribute("style");
	}
        public void actionPerformed(Document doc, int start, int end, ActionEvent evt2) 
	    throws BadLocationException {
	    if (start==end) {
		if (style.equals(currentFormat))
		    currentFormat = null;
		else {
		    boolean ok = false;
		    try {
			ElementSwingElement root
			    = (ElementSwingElement)doc.getRootElement();
			if (root.canInsertTag(start, Position.Bias.Forward, style)) {
			    currentFormat = style;
			    ok = true;
			}
		    } catch (BadLocationException e) {
		    }
		    if (!ok) {
			JOptionPane.showMessageDialog
			    (tec.getFrame(),
			     "Die Formatierung kann an dieser Stelle nicht benutzt werden.",
			     "Formatierung", JOptionPane.WARNING_MESSAGE);
			currentFormat = null;
		    }
		}
	    } else {
		Element root = doc.getRootElement();
		try {
		    doc.beginUpdate();
		    DocumentEvent evt = new DocumentEvent(doc);
		    if (root.isDeepStyle(start, Position.Bias.Forward, style) ||
			root.isDeepStyle(end, Position.Bias.Backward, style))
			root.removeStyle(style, start, end, evt);
		    else {
			Element st = createElementInstance(style);
			root.embedSplit(st, start, end, evt);
		    }
		    if (evt.isValid())
			doc.fireDocumentChanged(evt);
		} finally {
		    doc.endUpdate();
		}
	    }
	}
    }
    protected class InsertTagAction extends InsertElementAction {
	protected String style;
	public InsertTagAction(org.w3c.dom.Element _el) {
	    super(_el.getAttributeNode("command")==null
		  ?"insert-"+_el.getAttribute("style")
		  :_el.getAttribute("command"));
	    style = _el.getAttribute("style");
	}
	protected Element createElement(Document adoc) {
	    SwingDocument doc = (SwingDocument)adoc;
	    return createElementInstance(style);
	}
    }
    final class EmbedTagAction extends EmbedElementAction {
	protected String style;
	public EmbedTagAction(org.w3c.dom.Element _el) {
	    super(_el.getAttributeNode("command")==null
		  ?"embed-"+_el.getAttribute("style")
		  :_el.getAttribute("command"));
	    style = _el.getAttribute("style");
	}
	public Element createElement(Document adoc) {
	    SwingDocument doc = (SwingDocument)adoc;
	    return createElementInstance(style);
	}
    }
    final class InsertDialogElementAction extends DocumentAction {
	protected String style;
	public InsertDialogElementAction(org.w3c.dom.Element _el) {
	    super(_el.getAttributeNode("command")==null
		  ?"insert-"+_el.getAttribute("style")
		  :_el.getAttribute("command"));
	    style = _el.getAttribute("style");
	}
        public void actionPerformed(Document doc, int start, int end, ActionEvent evt2) 
	    throws BadLocationException {
	    Hashtable ht = doc.getStyle(style);
	    if (ht==null) {
		editor.getToolkit().beep(); return;
	    }
	    org.w3c.dom.Element dinfo
		= (org.w3c.dom.Element)ht.get(TextEditorStyleSet.EDITOR_DIALOG);
	    if (dinfo==null) {
		editor.getToolkit().beep(); return;
	    }
	    ElementSwingElement root = (ElementSwingElement)doc.getRootElement();
	    if (root.canInsertTag(start, Position.Bias.Forward, style))
		DefaultEditorDialog.createNewElement
		    (tec.getEditorFrame(), 
		     (SwingDocument)doc, style, dinfo, start, end);
	    else
		JOptionPane.showMessageDialog
		    (null, "Das Element kann an dieser Stelle nicht eingef\u00fcgt werden.",
		     "Einf\u00fcgen", JOptionPane.ERROR_MESSAGE);
	}
    }
    public class PaintAllAction extends AbstractAction {
        public PaintAllAction() { super(paintAllAction); }
        public void actionPerformed(ActionEvent e) {
	    XmlTextPane ed = (XmlTextPane)editor;
	    ed.setPaintAll(!ed.getPaintAll());
        }
    }
}
