package com.make.editor.plugins.text;

import java.awt.Font;
import java.awt.Component;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.Transferable;
import java.beans.PropertyChangeListener;
import javax.swing.JPopupMenu;
import javax.swing.Action;
import javax.swing.JScrollPane;
import javax.swing.JOptionPane;
import org.w3c.dom.Element;
import com.make.editor.plugins.text.tree.SwingDocument;
import com.make.editor.plugins.EditorPlugin;
import com.make.editor.plugins.DefaultEditorComponent;
import com.make.editor.EditorDocument;
import com.make.editor.EditorFrame;
import com.make.xml.DOMUtil;
import com.make.util.Debug;
import com.make.swing.ButtonManager;
import com.make.swing.text.DefaultEditorKit;
import com.make.swing.text.DocumentListener;
import com.make.swing.text.DocumentEvent;
import com.make.swing.text.Document;

public class TextEditorComponent extends DefaultEditorComponent 
    implements ClipboardOwner, DocumentListener {
    class ViewDescription {
	public boolean hasScroller;
	public Element actions;
	public TextEditorStyleSet styleSet;
	public ViewDescription(Element vd) {
	    hasScroller = !DOMUtil.childTagExists(vd, "noscroll");
	    actions = DOMUtil.getUniqueTag(vd, "actions");
	    Element styles = DOMUtil.getUniqueTag(vd, "styles");
	    styleSet = new TextEditorStyleSet(styles);
	}
    }
    /***properties*****************************/
    // visual
    protected Component component;
    protected XmlTextPane bodyEditor;
    protected TextEditorStyleSet styleSet;
    protected boolean hasScroller = true;
    protected boolean hasFocus = false;
    // data
    protected SwingDocument bodyDocument;
    protected TextActions textActions;
    /***Constructor***************************/
    public TextEditorComponent(EditorFrame _ef) {
	super(_ef);
	textActions = new TextActions(this, _ef);
	if (Debug.LEAK_DEBUG)
	    System.err.println("+txe "+this);
    }
    protected void finalize() throws Throwable {
	super.finalize();
	if (Debug.LEAK_DEBUG)
	    System.err.println("-txe "+this);
    }
    /***DocumentListener*******************************/
    public void documentChanged(DocumentEvent evt) {
	fireDocumentChanged();
    }
    /***EditorComponent********************************/
    public void setDocument(EditorDocument _doc) {
	super.setDocument(_doc);
	// read view
	Object vd = getViewDescription("text");
	if (vd instanceof Element) {
	    vd = new ViewDescription((Element)vd);
	    putViewDescription("text", vd);
	}
	if (vd==null)
	    styleSet = new TextEditorStyleSet(null);
	else {
	    ViewDescription d = (ViewDescription)vd;
	    hasScroller = d.hasScroller;
	    styleSet = d.styleSet;
	    ButtonManager bm = hasFocus
		?getEditorFrame().getInterfacePanel().getButtonManager()
		:null;
	    textActions.loadActions(d.actions, bm);
	}
	// create swingDocument
	if (bodyDocument!=null)
	    bodyDocument.removeDocumentListener(this);
	bodyDocument = new SwingDocument(document, styleSet);
	bodyDocument.addDocumentListener(this);
	if (bodyEditor==null) {
	    bodyEditor = new XmlTextPane(textActions, bodyDocument, 
					 styleSet, editorFrame);
	    registerInputComponent(bodyEditor);
	    if (hasScroller) {
		JScrollPane scrollPane = new JScrollPane(bodyEditor);
		scrollPane.getViewport().putClientProperty("EnableWindowBlit", Boolean.TRUE);
		component = scrollPane;
	    } else
		component = bodyEditor;
	}
	bodyEditor.setDocument(bodyDocument);
    }
    public Component getComponent() { return component; }
    public void focusGained() {
	super.focusGained();
	bodyEditor.setButtonManager(getEditorFrame().getInterfacePanel().getButtonManager());
	hasFocus = true;
    }
    public void focusLost() {
	super.focusLost();
	bodyEditor.setButtonManager(null);
	hasFocus = false;
    }
    public void dispose() {
	bodyEditor.setButtonManager(null);
	component = null;
	bodyEditor = null;
	styleSet = null;
	bodyDocument = null;
	super.dispose();
    }
    /***ClipboardOwner******************************/
    public void lostOwnership(Clipboard clipboard, Transferable contents) {}
}
