package com.make.editor.plugins.text;

import java.util.Hashtable;
import java.awt.Color;
import java.awt.Font;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import com.make.xml.DOMUtil;
import com.make.editor.plugins.EditorPlugin;
import com.make.editor.plugins.EditorStyleSet;
import com.make.editor.plugins.EditorComponent;

public class TextEditorStyleSet extends EditorStyleSet {
    public static final String EDITOR_DIALOG = "_editordialog";
    public static final String NEW_CLONE = "_new_clone";
    public static final String NEW_XSLT = "_new_xslt";
    public static final String TOOLTIP = "_tooltip";
    // class
    public static final String CLASS = "_class";
    public static final int C_UNKNOWN = 1;
    public static final int C_SECTION = 2;
    public static final int C_PARAGRAPH = 3;
    public static final int C_TEXT = 4;
    public static final int C_IGNORE = 5;
    public static final int C_IMAGE = 6;
    public static final int C_TABLE = 7;
    public static final int C_UL = 8;
    public static final int C_OL = 9;
    public static final int C_NOTE = 10;
    public static final int C_BREAK = 11;
    public static final int C_RESOURCE_IMAGE = 12;
    public static final int C_TABLEROW = 13;
    // shape
    public static final String SHAPE = "_shape";
    public static final byte S_UNKNOWN = 0;
    public static final byte S_RECTANGLE = 1;
    public static final byte S_HSTRUT = 2;
    public static final byte S_VSTRUT = 3;
    /***properties****************************/
    protected Hashtable fonts = new Hashtable();
    /***Constructor****************************/
    public TextEditorStyleSet(Element _styles) {
	super(_styles);
    }
    /***EditorStyleSet*************************/
    public Hashtable loadStyle(Element style) {
	Hashtable ht = style.getTagName().equals("default")
	    ?getStyle("default"):new Hashtable();
	NodeList nl = style.getChildNodes();
	for (int z=0; z<nl.getLength(); z++) {
	    Node n = nl.item(z);
	    if (n instanceof Element) {
		Element el = (Element)n;
		String tag = el.getTagName();
		if (tag.equals("font"))
		    ht.put(tag, getElementString(el));
		else if (tag.equals("fontsize"))
		    ht.put(tag, getElementInteger(el));
		else if (tag.equals("bold") || tag.equals("italic") ||
			 tag.equals("underline"))
		    ht.put(tag, Boolean.TRUE);
		else if (tag.equals("textcolor") || tag.equals("bgcolor"))
		    ht.put(tag, getElementColor(el));
		else if (tag.equals("title"))
		    ht.put(tag, getElementString(el));
		else if (tag.equals("editordialog"))
		    ht.put(EDITOR_DIALOG, el);
		else if (tag.equals("new-clone"))
		    ht.put(NEW_CLONE, DOMUtil.getChildElements(el)[0]);
		else if (tag.equals("new-xslt"))
		    ht.put(NEW_XSLT, DOMUtil.getChildElements(el)[0]);
		else if (tag.equals("tooltip"))
		    ht.put(TOOLTIP, el);
		else if (tag.equals("image"))
		    ht.put(tag, getElementString(el));
		else
		    System.err.println("WARNING: text attribute unknown: "+tag);
	    }
	}
	// class
	String _class = style.getAttribute("class");
	int cls;
	if (_class==null || _class.equals("") || _class.equals("unknown"))
	    cls = C_UNKNOWN;
	else if (_class.equals("ignore"))
	    cls = C_IGNORE;
	else if (_class.equals("section"))
	    cls = C_SECTION;
	else if (_class.equals("paragraph"))
	    cls = C_PARAGRAPH;
	else if (_class.equals("text"))
	    cls = C_TEXT;
	else if (_class.equals("image"))
	    cls = C_IMAGE;
	else if (_class.equals("resimage"))
	    cls = C_RESOURCE_IMAGE;
	else if (_class.equals("table"))
	    cls = C_TABLE;
	else if (_class.equals("tablerow"))
	    cls = C_TABLEROW;
	else if (_class.equals("ol"))
	    cls = C_OL;
	else if (_class.equals("ul"))
	    cls = C_UL;
	else if (_class.equals("note"))
	    cls = C_NOTE;
	else if (_class.equals("break"))
	    cls = C_BREAK;
	else {
	    cls = C_UNKNOWN;
	    System.err.println("WARNING: unknown text style cls: "+_class);
	}
	ht.put(CLASS, new Integer(cls));
	// shape
	String _shape = style.getAttribute("shape");
	byte shape;
	if (_shape==null || _shape.equals("") || _shape.equals("unknown"))
	    shape = S_UNKNOWN;
	else if (_shape.equals("rectangle"))
	    shape = S_RECTANGLE;
	else if (_shape.equals("hstrut"))
	    shape = S_HSTRUT;
	else if (_shape.equals("vstrut"))
	    shape = S_VSTRUT;
	else {
	    shape = S_UNKNOWN;
	    System.err.println("WARNING: unknown text style shape: "+_shape);
	}
	ht.put(SHAPE, new Byte(shape));
	return ht;
    }
    /***Font***********************************/
    public Font getFont(com.make.swing.text.Element attr) {
	String name = getFontName(attr);
	Font fnt = (Font)fonts.get(name);
	return fnt==null?createFont(attr, name):fnt;
    }
    protected String getFontName(com.make.swing.text.Element attr) {
	return (String)attr.getAttribute("font")+"-"+
	    attr.getAttribute("fontsize")+"-"+
	    (attr.containsAttribute("bold", Boolean.TRUE)?"bold-":"-")+
	    (attr.containsAttribute("italic", Boolean.TRUE)?"italic-":"-");
    }
    protected Font createFont(com.make.swing.text.Element attr, String name) {
	String fontName = (String)attr.getAttribute("font");
	if (fontName==null) fontName = "times";
	int style = Font.PLAIN;
	if (attr.containsAttribute("bold", Boolean.TRUE)) style |= Font.BOLD;
	if (attr.containsAttribute("italic", Boolean.TRUE)) style |= Font.ITALIC;
	Integer _size = (Integer)attr.getAttribute("fontsize");
	final int size = _size==null?10:_size.intValue();
	final Font font = new Font(fontName, style, size);
	fonts.put(name, font);
	return font;
    }
}
