package com.make.editor.plugins.text.dlg;

import java.util.Vector;
import java.util.Enumeration;
import java.io.IOException;
import java.io.File;
import java.io.FilenameFilter;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.io.InputStream;
import java.io.FileInputStream;
import java.awt.Frame;
import java.awt.Component;
import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.text.ParseException;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.ButtonGroup;
import javax.swing.ButtonModel;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JRadioButton;
import javax.swing.DefaultComboBoxModel;
import javax.swing.ListCellRenderer;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.BadLocationException;
import com.make.xml.DOMUtil;
import com.make.xml.ElementType;
import com.make.xml.AttributeType;
import com.make.editor.EditorSession;
import com.make.editor.EditorFrame;
import com.make.editor.field.EmbeddedSelectField;
import com.make.datamodel.embedded.EmbeddedManager;
import com.make.datamodel.slot.Slot;
import com.make.datamodel.slot.SlotContainer;
import com.make.editor.EditorDocument;
import com.make.editor.plugins.text.tree.SwingDocument;
import com.make.editor.plugins.text.tree.SwingText;
import com.make.editor.plugins.text.tree.ElementSwingElement;
import com.make.editor.field.EditorInputLinkFactory;
import com.make.util.Strings;
import com.make.util.Streams;
import com.make.util.Debug;
import com.make.swing.field.Field;
import com.make.swing.field.ComboField;
import com.make.swing.field.TextField;
import com.make.swing.il.InputLink;
import com.make.swing.il.DefaultInputLink;
import com.make.swing.il.PanelInputLink;
import com.make.swing.il.Label;
import com.make.swing.text.DocumentEvent;

public class DefaultEditorDialog extends EditorDialog {
    /***Editors***********************************/
    public class AttributeSlot implements Slot {
	/***properties*****************************/
	protected ElementSwingElement element;
	protected String attributeName;
	protected AttributeType type;
	/***Constructor*****************************/
	public AttributeSlot(ElementSwingElement _el, String _an) {
	    element = _el; attributeName = _an;
	    ElementType et = element.getElementType();
	    if (et!=null)
		type = et.getAttribute(_an);
	}
	/***Slot**********************************/
	public void setValue(Object v) throws ParseException {
	    if (Debug.DEBUG && documentEvent==null && !inCheck)
		throw new InternalError("setValue without documentEvent");
	    if (v==null && type!=null && type.isRequired())
		throw new ParseException("Bitte geben Sie hier einen Wert ein.", 0);
	    if (!inCheck)
		element.setAttribute(attributeName, v==null?null:v.toString(),
				     documentEvent);
	}
	public Object getValue() throws ParseException {
	    return element.getAttribute(attributeName);
	}
    }
    class ContentEditor extends DefaultInputLink {
	/***properties**************/
	protected JTextField field = new JTextField(20);
	protected String value;
	/***Constructor************/
	public ContentEditor(Element vd) {
	    super(vd);
	    reload();
	}
	/***InputLink*********************/
	public Component getComponent() { return field; }
	public void reload() {
	    if (newPosition==-1 || (newSelection!=-1 && newSelection!=newPosition)) {
		field.setText("[Markierung]");
		field.setEnabled(false);
	    } else {
		field.setText("");
		field.setEnabled(true);
	    }
	}
	public void apply() throws ParseException {
	    if (field.isEnabled()) {
		ElementSwingElement el = getElement();
		SwingDocument doc = (SwingDocument)el.getDocument();
		SwingText txt = new SwingText(doc.getXmlDocument().createTextNode(value),
					      doc, false);
		el.insertChild(txt, null);
		if (documentEvent!=null)
		    documentEvent.createContainerChange(el).childAdded(el.indexOf(txt));
		field.setEnabled(false);
	    }
	}
	public void checkInput() throws ParseException {
	    if (isNew()) {
		value = Strings.trim(field.getText());
		if (value==null)
		    throw new ParseException
			("Bitte geben Sie bei hier einen Text ein.", 0);
	    }
	}
    }
    class AttributeSlotContainer implements SlotContainer {
	public Slot getSlot(Object key) {
	    if (key instanceof String)
		return new AttributeSlot(getElement(), (String)key);
	    else
		return null;
	}
    }
    class Factory extends EditorInputLinkFactory {
	public Factory() {
	    super(frame, getEditorDocument(), new AttributeSlotContainer());
	}
	/***InputLinkFactory*********************/
	public InputLink createInputLink(Element vd) {
	    String type = vd.getTagName();
	    if (type.equals("content"))
		return new ContentEditor(vd);
	    else
		return super.createInputLink(vd);
	}
    }
    /***properties******************************/
    protected EmbeddedManager embeddedManager;
    protected EditorFrame frame;
    protected EditorDocument document;
    protected DocumentEvent documentEvent;
    protected boolean inCheck = false;
    protected InputLink inputLink;
    /***Constructor******************************/
    public DefaultEditorDialog(EditorFrame _ef, SwingDocument _doc, 
			       String _tag, Element dinfo, 
			       int _np, int _ns) {
	super(_ef.getInterfacePanel().getFrame(), 
	      Strings.trim(dinfo.getAttribute("newtitle"))==null
	      ?dinfo.getAttribute("title"):dinfo.getAttribute("newtitle"),
	      _doc, _tag, _np, _ns);
	frame = _ef;
	document = _doc.getEditorDocument();
	embeddedManager = _doc.getEmbeddedManager();
	createForm(dinfo);
    }
    public DefaultEditorDialog(EditorFrame _ef, 
			       ElementSwingElement _el, Element dinfo) {
	super(_ef.getInterfacePanel().getFrame(), 
	      dinfo.getAttribute("title"), _el);
	frame = _ef;
	document = ((SwingDocument)_el.getDocument()).getEditorDocument();
	embeddedManager = ((SwingDocument)_el.getDocument()).getEmbeddedManager();
	createForm(dinfo);
    }
    protected void finalize() throws Throwable {
	inputLink.uninstall(getButtonManager());
	super.finalize();
    }
    /***Data*****************************************/
    protected boolean applyChanges() {
	if (documentEvent!=null)
	    return false;
	ElementSwingElement el = getElement();
	// 1. check
	try {
	    inCheck = true;
	    inputLink.checkInput();
	} catch (ParseException e) {
	    inputErrorMsg(e.getLocalizedMessage()); 
	    return false;
	} finally {
	    inCheck = false;
	}
	// 2. set values
	SwingDocument doc = (SwingDocument)el.getDocument();
	try {
	    doc.beginUpdate();
	    documentEvent = new DocumentEvent(doc);
	    try {
		inputLink.apply();
	    } catch (ParseException e) {
	    }
	    if (isNew) {
		try {
		    ElementType eet = getElement().getElementType();
		    if (newPosition==newSelection || (eet!=null && eet.isEmpty())) {
			if (!doc.getRootElement().pasteElementAt(newPosition, el, 
								 documentEvent)) {
			    errorMsg("Hier koennen Sie dieses Element nicht einfuegen.", "Fehler");
			    return false; //TODO
			}
		    } else {
			if (!doc.getRootElement().embedSplit(el, newPosition, 
							     newSelection, documentEvent)) {
			    inputErrorMsg("Hier koennen Sie dieses Element nicht einfuegen.");
			    doc.endUpdate();
			    return false; //TODO: Attribute, undo rueckgaengig
			}
		    }
		} catch (BadLocationException e) {
		    e.printStackTrace(); return false;
		}
	    }
	    if (documentEvent.isValid())
		doc.fireDocumentChanged(documentEvent);
	} finally {
	    documentEvent = null;
	    doc.endUpdate();
	}
	return true;
    }
    /***Form************************************/
    protected synchronized void createForm(Element dinfo) {
	ElementSwingElement element = getElement();
	Factory factory = new Factory();
	inputLink = new PanelInputLink
	    (dinfo, DOMUtil.getChildElements(dinfo), factory);
	inputLink.install(getButtonManager());
	// dialog
	getContentPane().add("Center", inputLink.getComponent());
	getContentPane().add("South", createButtonBox());
	pack();
	// TODO:firstField aktivieren
	/*if (firstField!=null)
	  firstField.requestFocus();*/
    }
    /***Static***************************************/
    public static void createNewElement(EditorFrame _f, 
					SwingDocument _doc, String _tag,
					Element dinfo, int _np, int _ns) {
	DefaultEditorDialog gre = new DefaultEditorDialog(_f, _doc, _tag, 
							  dinfo, _np, _ns);
	gre.show();
    }
    public static void editElement(EditorFrame _f, 
				   ElementSwingElement _el, Element dinfo) {
	DefaultEditorDialog gre = new DefaultEditorDialog(_f, _el, dinfo);
	gre.show();
    }
}
