package com.make.editor.plugins.text.dlg;

import java.awt.Frame;
import java.awt.Component;
import java.awt.Container;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JOptionPane;
import com.make.util.Strings;
import com.make.xml.ElementType;
import com.make.editor.EditorDocument;
import com.make.editor.plugins.text.tree.ElementSwingElement;
import com.make.editor.plugins.text.tree.SwingDocument;
import com.make.swing.MakeDialog;
import com.make.swing.text.DocumentEvent;

public abstract class EditorDialog extends MakeDialog {
    /***properties***************************/
    private ElementSwingElement element;
    protected int newPosition = -1, newSelection = -1;
    protected JButton defaultButton;
    /***Constructor*************************/
    protected EditorDialog(Frame _f, String title, ElementSwingElement _el) {
	super(_f, title, false, false);
	element = _el;
    }
    protected EditorDialog(Frame _f, String title, SwingDocument doc, 
			   String tag, int _np, int _ns) {
	super(_f, title, false, true);
	element = (ElementSwingElement)doc.getSwingElementFactory().create
	    (doc.getRootElement(), doc.getXmlDocument().createElement(tag)); 
	newPosition = _np; newSelection = _ns;
    }
    protected EditorDialog(Frame _f, String title, SwingDocument doc, 
			   String tag, int _np) {
	this(_f, title, doc, tag, _np, -1);
    }
    /***Info***********************************/
    public boolean isNew() { return newPosition!=-1; }
    public EditorDocument getEditorDocument() {
	return ((SwingDocument)element.getDocument()).getEditorDocument();
    }
    /***Data************************************/
    protected void fireDefaultDocumentChanged() {
	DocumentEvent evt = new DocumentEvent(element.getDocument());
	evt.createParentChange(element);
	element.getDocument().fireDocumentChanged(evt);
    }
    protected String getAttribute(String name) {
	return element.getAttribute(name);
    }
    protected String getAttribute(String name, String def) {
	String val = getAttribute(name);
	return val==null?def:val;
    }
    protected void setAttribute(String name, String val, DocumentEvent evt) {
	element.setAttribute(name, val, evt);
    }
    protected ElementSwingElement getElement() { return element; }
    protected ElementType getElementType() { return element.getElementType(); }
}
