package com.make.editor.plugins.text.tree;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Attr;
import org.w3c.dom.Text;
import org.w3c.dom.NamedNodeMap;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;
import javax.swing.text.Position;
import javax.swing.text.BadLocationException;
import com.make.xml.ElementType;
import com.make.xml.AttributeType;
import com.make.editor.EditorFrame;
import com.make.editor.plugins.text.TextEditorStyleSet;
import com.make.editor.plugins.text.XmlTextPane;
import com.make.editor.plugins.text.dlg.DefaultEditorDialog;
import com.make.util.Strings;
import com.make.util.Debug;
import com.make.xml.DOMUtil;
import com.make.swing.text.Document;
import com.make.swing.text.Element;
import com.make.swing.text.ElementUndoableEdit;
import com.make.swing.text.DocumentEvent;
import com.make.swing.text.ContainerChange;
import com.make.swing.text.TextComponent;

class AttributeUndoableEdit extends ElementUndoableEdit {
    /***properties**************************/
    protected String name, oldValue, newValue;
    /***Constructor*************************/
    public AttributeUndoableEdit(ElementSwingElement el, String n, 
				 String ov, String nv) {
	super(el); name = n; oldValue = ov; newValue = nv;
    }
    public void undo() {
	super.undo();
	ElementSwingElement el = (ElementSwingElement)element;
	if (Debug.DEBUG && !Strings.equals(el.getAttribute(name), 
					   newValue)) {
	    System.err.println("FATAL ERROR in undo: el = "+el);
	    System.err.println("  value is "+el.getAttribute(name));
	    System.err.println("  should be "+newValue);
	    throw new InternalError();
	}
	el.setAttribute(name, oldValue, el.getDocument().getUndoEvent());
    }
    public void redo() {
	super.redo();
	ElementSwingElement el = (ElementSwingElement)element;
	if (Debug.DEBUG && !Strings.equals(el.getAttribute(name), 
					   oldValue)) {
	    System.err.println("FATAL ERROR in redo: el = "+el);
	    System.err.println("  value is "+el.getAttribute(name));
	    System.err.println("  should be "+oldValue);
	    throw new InternalError();
	}
	el.setAttribute(name, newValue, el.getDocument().getUndoEvent());
    }
}
public abstract class ElementSwingElement extends NodeSwingElement {
    /***properties**************************/
    ElementType elementType;
    /***Constructor*************************/
    public ElementSwingElement(org.w3c.dom.Element _e, SwingDocument _doc,
			       int _length, boolean leaf) {
	super(_e, _doc, _length, leaf);
	if (elementType==null && _doc.getDocumentType()!=null)
	    elementType = _doc.getDocumentType().getElementType(getTagName());
    }
    public ElementSwingElement(org.w3c.dom.Element _e, SwingDocument _doc, 
			       int _length) {
	super(_e, _doc, _length, _e.getTagName());
	if (elementType==null && _doc.getDocumentType()!=null)
	    elementType = _doc.getDocumentType().getElementType(getTagName());
    }
    /***Tooltip***************************************/
    public String getToolTipText(int position, Position.Bias bias) 
	throws BadLocationException {
	String sv = super.getToolTipText(position, bias);
	if (sv!=null) return sv;
	org.w3c.dom.Element tt
	    = (org.w3c.dom.Element)getAttribute((Object)TextEditorStyleSet.TOOLTIP);
	if (tt==null) return null;
	org.w3c.dom.Element[] ttrs = DOMUtil.getChildElements(tt, "tooltiprow");
	if (ttrs.length==0) return null;
	StringBuffer tip = new StringBuffer();
	for (int z=0; z<ttrs.length; z++) {
	    org.w3c.dom.Element r = ttrs[z];
	    NodeList vl = r.getChildNodes();
	    final int vlc = vl.getLength()-1;
	    StringBuffer sb = new StringBuffer();
	    for (int a=0; a<=vlc; a++) {
		Node n = vl.item(a);
		switch (n.getNodeType()) {
		case n.TEXT_NODE:
		    sb.append(n.getNodeValue());
		    break;
		case n.ELEMENT_NODE: {
		    String t2 = n.getNodeName();
		    if (t2.equals("attribute-ref")) {
			String name = ((org.w3c.dom.Element)n).getAttribute("name");
			String val = getAttribute(name);
			if (val!=null) sb.append(val);
		    } else
			System.err.println("WARNING: ignoring tooltip tag "+t2);
		    break;
		}
		default:
		    System.err.println("WARNING: ignoring tooltip node "+n);
		}
	    }
	    String tl = Strings.normalize(sb.toString());
	    if (tl!=null && tl.length()>0) {
		if (z>0) tip.append("<p>");
		String label = Strings.trim(r.getAttribute("label"));
		if (label!=null)
		    tip.append(label+": ");
		tip.append("<b>");
		tip.append(tl);
		tip.append("</b>");
	    }
	}
	if (tip.length()>0)
	    return "<html>"+tip.toString()+"</html>";
	else
	    return null;
    }
    public void fillMenu(final TextComponent editor, int context, 
			 int pos, Position.Bias bias, JPopupMenu menu) 
	throws BadLocationException {
	final org.w3c.dom.Element dinfo
	    = (org.w3c.dom.Element)getAttribute((Object)TextEditorStyleSet.EDITOR_DIALOG);
	if (dinfo!=null) {
	    JMenuItem mi;
	    mi = new JMenuItem(dinfo.getAttribute("title")+"...");
	    mi.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt2) {
		    XmlTextPane ed = (XmlTextPane)editor;
		    EditorFrame ef = ed.getEditorFrame();
		    DefaultEditorDialog.editElement
			(ef, ElementSwingElement.this, dinfo);
		}
	    });
	    menu.add(mi);
	}
	super.fillMenu(editor, context, pos, bias, menu);
    }
    /***ElementSwingElement******************************/
    public org.w3c.dom.Element getXmlElement() { 
	return (org.w3c.dom.Element)getXmlNode(); 
    }
    public String getTagName() { 
	return getXmlElement().getTagName(); 
    }
    public ElementType getElementType() { 
	return elementType; 
    }
    /***NodeSwingElement******************************/
    public Node copyXmlNode(org.w3c.dom.Document doc, int start, int end) 
	throws BadLocationException {
	// copy attributes
	org.w3c.dom.Element el = doc.createElement(getTagName());
	NamedNodeMap nnm = getXmlElement().getAttributes();
	final int ac = nnm.getLength();
	for (int z=0; z<ac; z++) {
	    Attr attr = (Attr)nnm.item(z);
	    AttributeType at = elementType==null
		?null:elementType.getAttribute(attr.getName());
	    if (at!=null)
		switch (at.getType()) {
		case at.T_ID: 
		    el.setAttributeNode(at.createDefault(doc));
		    break;
		default:
		    el.setAttribute(attr.getName(), attr.getValue());
		}
	    else
		el.setAttribute(attr.getName(), attr.getValue());
	}
	if (!isLeaf()) {
	    // index
	    final int first = getIndexAt(start, Position.Bias.Forward);
	    final int last = getIndexAt(end, Position.Bias.Backward);
	    // children
	    for (int z=first; z<=last; z++) {
		Element sel = getElementAtIndex(z);
		if (sel instanceof NodeSwingElement) {
		    NodeSwingElement eel = (NodeSwingElement)sel;
		    final int s2 = Math.max(start, eel.getStartOffset());
		    final int e2 = Math.min(end, eel.getEndOffset());
		    Node xel = eel.copyXmlNode(doc, s2, e2);
		    if (xel!=null)
			el.appendChild(xel);
		}
	    }
	}
	return el;
    }
    private boolean pasteNodeAt(int pos, Element targetChild,
				NodeSwingElement newChild, DocumentEvent evt) 
	throws BadLocationException {
	return (targetChild instanceof NodeSwingElement &&
		((NodeSwingElement)targetChild).pasteNode(pos, newChild.getXmlNode(), 
							  evt)) ||
	    pasteElementAt(pos, newChild, evt);
    }
    private int pasteChildAt(int idx, int pos, Element targetChild,
			     NodeSwingElement nse, DocumentEvent evt) 
	throws BadLocationException {
	if (targetChild instanceof NodeSwingElement &&
	    ((NodeSwingElement)targetChild).pasteNode(targetChild.getStartOffset(), 
						      nse.getXmlNode(), evt))
	    return 1;
	else
	    return pasteChild(idx, nse, evt);
    }
    private boolean pasteNodes(int idx, boolean wasSpace, 
			       Enumeration enum, DocumentEvent evt)
	throws BadLocationException {
	ContainerChange chg = evt.createContainerChange(this);
	NodeSwingElement nse = (NodeSwingElement)enum.nextElement();
	while (enum.hasMoreElements()) {
	    if (wasSpace || nse.leadingSpace)
		idx += insertSpaceChild(idx, evt, chg);
	    int ret = pasteChild(idx, nse, evt);
	    if (ret>0) {
		idx += ret;
		wasSpace = nse.endingSpace;
	    } else
		wasSpace = false;
	    nse = (NodeSwingElement)enum.nextElement();
	}
	if (wasSpace || nse.leadingSpace)
	    idx += insertSpaceChild(idx, evt, chg);
	Element targetChild;
	int targetOffset;
	if (idx>=getElementCount()) {
	    targetChild = getElementAtIndex(idx-1);
	    targetOffset = targetChild.getEndOffset();
	} else {
	    targetChild = getElementAtIndex(idx);
	    targetOffset = targetChild.getStartOffset();
	}
	if (targetChild instanceof NodeSwingElement) {
	    NodeSwingElement targetNSE = ((NodeSwingElement)targetChild);
	    if (targetNSE.pasteNode(targetOffset, nse.getXmlNode(), evt))
		return true;
	}
	return pasteChild(idx, nse, evt)>0;
    }
    public boolean pasteNode(int pos, Node n, DocumentEvent evt) 
	throws BadLocationException {
	if (!canInsertAt(pos))
	    return false;
	SwingElementFactory fac = ((SwingDocument)getDocument()).getSwingElementFactory();
	if (n instanceof org.w3c.dom.Element &&
	    n.hasChildNodes() && !isLeaf()) {
	    final org.w3c.dom.Element el = (org.w3c.dom.Element)n;
	    // find target
	    final int posIndex = getIndexAt(pos);
	    final Element targetChild = getElementAtIndex(posIndex);
	    // create element list
	    NodeList nl = el.getChildNodes();
	    Vector ch = new Vector(nl.getLength());
	    final int cc = nl.getLength();
	    for (int z=0; z<cc; z++) {
		NodeSwingElement se = fac.create(this, nl.item(z));
		if (se!=null)
		    ch.addElement(se);
	    }
	    if (ch.size()==0) 
		return false;
	    if (ch.size()==1) {
		// paste directly when only one element in list
		final NodeSwingElement newChild
		    = (NodeSwingElement)ch.firstElement();
		return pasteNodeAt(pos, targetChild, newChild, evt);
	    }
	    // insert?
	    ContainerChange chg = evt.createContainerChange(this);
	    Enumeration enum = ch.elements();
	    if (pos==targetChild.getStartOffset()) {
		// we don't need to split... good.
		pasteNodes(posIndex, false, enum, evt);
		return true;
	    } else if (pos==targetChild.getEndOffset()) {
		// ok, no split
		pasteNodes(posIndex+1, false, enum, evt);
		return true;
	    } else {
		// we have to split
		int idx = forceSplitChild(posIndex, pos, true, evt);
		if (idx==-1)
		    return pasteNode(targetChild.getStartOffset(), n, evt);
		// paste into first
		boolean wasSpace = false;
		if (targetChild instanceof NodeSwingElement) {
		    NodeSwingElement nse = (NodeSwingElement)enum.nextElement();
		    Integer dci = (Integer)targetChild.getAttribute((Object)TextEditorStyleSet.CLASS);
		    Integer sci = (Integer)nse.getAttribute((Object)TextEditorStyleSet.CLASS);
		    if (dci!=null && sci!=null) {
			final int dc = dci.intValue();
			final int sc = sci.intValue();
			if (dc==sc && 
			    (dc==TextEditorStyleSet.C_PARAGRAPH ||
			     dc==TextEditorStyleSet.C_TEXT)) {
			    int tp = targetChild.getEndOffset();
			    if (!targetChild.canInsertAt(tp))
				tp--;
			    if (((NodeSwingElement)targetChild).pasteNode(tp, nse.getXmlNode(), evt))
				wasSpace = nse.endingSpace;
			    else
				idx += pasteChild(idx, nse, evt);
			    
			    } else
				enum = ch.elements();
		    } else
			enum = ch.elements();
		}
		// paste rest
		pasteNodes(idx, wasSpace, enum, evt);
		return true;
	    }
	}
	NodeSwingElement sel = fac.create(this, n);
	if (sel==null) return false;
	return pasteElementAt(pos, sel, evt);
    }
    /***SwingElement*********************************/
    protected int handleSpace(int index, Element se0, Element se1) {
	ElementType eet = ((SwingDocument)getDocument()).getDocumentType().getElementType(getTagName());
	if (eet!=null && !eet.mayContainText())
	    return 0;
	else
	    return super.handleSpace(index, se0, se1);
    }
    /***Format****************************************/
    public boolean childAllowed(Element el) {
	if (el instanceof ElementSwingElement)
	    return tagAllowed(((ElementSwingElement)el).getTagName());
	if (el instanceof SwingText)
	    return elementType==null ||
		elementType.mayContainText();
	return super.childAllowed(el);
    }
    protected boolean tagAllowed(String tag) {
	return elementType==null || elementType.tagAllowed(tag);
    }
    public boolean canInsertTag(int pos, Position.Bias bias, String tag) 
	throws BadLocationException {
	if (tagAllowed(tag)) return true;
	if (isLeaf()) return false;
	Element el = getElementAt(pos, bias);
	if (el instanceof ElementSwingElement)
	    return ((ElementSwingElement)el).canInsertTag(pos, bias, tag);
	else
	    return false;
    }
    protected abstract Element createEmptyInstance(org.w3c.dom.Element el);
    protected Element createEmptyInstance() {
	org.w3c.dom.Element el = getXmlDocument().createElement(getTagName());
	// copy attributes
	NamedNodeMap nnm = getXmlElement().getAttributes();
	final int count = nnm.getLength();
	for (int z=0; z<count; z++) {
	    Attr attr = (Attr)nnm.item(z);
	    el.setAttribute(attr.getNodeName(), attr.getNodeValue());
	}
	// create instance
	return createEmptyInstance(el);
    }
    public Element createDefaultChild() {
	SwingDocument doc = (SwingDocument)getDocument();
	if (elementType==null)
	    elementType = doc.getDocumentType().getElementType(getTagName());
	if (elementType==null) {
	    return new SwingText(doc);
	} else {
	    Node child = elementType.createDefaultChild(getXmlDocument());
	    return child==null
		?null
		:doc.getSwingElementFactory().create(this, child);
	}
    }
    public boolean canRemoveChild(Element child) {
	if (!super.canRemoveChild(child)) return false;
	if (child instanceof ElementSwingElement) {
	    String tag = ((ElementSwingElement)child).getTagName();
	    if (elementType==null) return true;
	    switch (elementType.getPolicy(tag)) {
	    case elementType.P_MORE:
	    case elementType.P_ONCE:
		return DOMUtil.getChildElements(getXmlElement(), tag).length>1;
	    }
	}
	return true;
    }
    /***Edit********************************************/
    protected void attributeChanged(String name, String oldValue, String newValue) {
    }
    public String getAttribute(String name) {
	return Strings.trim(getXmlElement().getAttribute(name));
    }
    public void setAttribute(String name, String value, DocumentEvent evt) {
	value = Strings.trim(value);
	String oldValue = getXmlElement().getAttribute(name);
	if (Strings.equals(oldValue, value))
	    return;
	if (value==null)
	    getXmlElement().removeAttribute(name);
	else
	    getXmlElement().setAttribute(name, value);
	attributeChanged(name, oldValue, value);
	if (evt!=null)
	    evt.createParentChange(this);
	if (!fresh)
	    getDocument().postEdit(new AttributeUndoableEdit(this, name, oldValue, value));
    }
}
