package com.make.editor.plugins.text.tree;

import java.io.File;
import java.io.InputStream;
import java.io.ByteArrayOutputStream;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.swing.text.BadLocationException;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import com.make.util.Strings;
import com.make.util.Streams;
import com.make.datamodel.embedded.EmbeddedManager;
import com.make.swing.text.Element;
import com.make.swing.text.DocumentEvent;

public class ImageElement extends ElementSwingElement {
    /***properties*********************************/
    protected Icon icon;
    protected boolean isInline = false;
    /***Constructor*****************************/
    public ImageElement(org.w3c.dom.Element el, SwingDocument doc, boolean _inline) {
	super(el, doc, _inline?1:2, true);
	setStyle(doc.getStyle(el.getTagName()));
	isInline = _inline;
	loadImage();
    }
    protected static Icon loadDefaultImage() {
	return new ImageIcon(ImageElement.class.getResource("/com/make/editor/resources/image-error.gif"));
    }
    public void loadImage() {
	icon = null;
	String name = getAttribute("src");
	try {
	    if (name==null) return;
	    EmbeddedManager em = ((SwingDocument)getDocument()).getEmbeddedManager();
	    if (em==null) return;
	    InputStream is = em.getEntry(name);
	    if (is==null) return;
	    ByteArrayOutputStream os = new ByteArrayOutputStream();
	    Streams.copy(is, os);
	    icon = new ImageIcon(os.toByteArray());
	} catch (Exception e) {
	    /*JOptionPane.showMessageDialog
	      (null, "Fehler beim Laden des Bildes '"+name+"'.",
	      "Fehler", JOptionPane.ERROR_MESSAGE);*/
	    icon = loadDefaultImage();
	    if (icon==null)
		icon = loadDefaultImage();
	}
    }
    public Icon getIcon() { return icon; }
    public boolean getInline() { return isInline; }
    /***Format****************************************/
    protected Element createEmptyInstance(org.w3c.dom.Element el) { 
	throw new IllegalStateException(); 
    }
    /***NodeSwingElement****************************/
    public Node copyXmlNode(Document doc, int start, int end) 
	throws BadLocationException {
	if (start<=getStartOffset() || end>getStartOffset())
	    return super.copyXmlNode(doc, start, end);
	else
	    return null;
    }
    /***SwingElement************************************/
    public void getText(StringBuffer sb, int offset, int length) 
	throws BadLocationException {
    }
    public boolean insertString(int offset, String str, DocumentEvent evt) 
	throws BadLocationException {
	if (isInline)
	    return false;
	else if (str.equals("\n")) {
	    Element parent = getParent();
	    Element el = parent.createDefaultChild();
	    if (el==null) return false;
	    final int myIndex = parent.indexOf(this);
	    final int newIndex = offset<=getStartOffset()?myIndex:myIndex+1;
	    parent.insertChild(el, newIndex);
	    evt.createContainerChange(parent).childAdded(newIndex);
	    evt.inserted(parent, el.getStartOffset(), el.getLength());
	    return true;
	} else
	    return false;
    }
    public void remove(int offset, int length, DocumentEvent evt) 
	throws BadLocationException {
	if (length==0) return;
	Element parent = getParent();
	final int myIndex = parent.indexOf(this);
	evt.createContainerChange(parent).childRemoved(myIndex);
	evt.removed(parent, getStartOffset(), getLength());
	parent.removeChild(this);
    }
    public void setAttribute(String name, String value, DocumentEvent evt) {
	super.setAttribute(name, value, evt);
	if (name.equals("src"))
	    loadImage();
    }
}
