package com.make.editor.plugins.text.tree;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import javax.swing.text.BadLocationException;
import java.util.Hashtable;
import java.lang.reflect.Constructor;
import com.make.swing.text.Document;
import com.make.swing.text.DocumentEvent;
import com.make.swing.text.Element;
import com.make.swing.text.ContainerChange;

public abstract class NodeSwingElement extends Element {
    /***properties*********************************/
    private Node node;
    /***Constructor*****************************/
    public NodeSwingElement(Node _n, Document _doc,
			    int _length, boolean leaf) {
	super(_doc, _length, leaf);
	node = _n;
	if (!leaf) {
	    loadChildren();
	    /*if (getElementCount()==0) {
		SwingElement el = createDefaultChild();
		if (el!=null) insertChild(el, null);
		}*/
	}
    }
    public NodeSwingElement(Node _n, Document _doc,
			    int _length, String _style) {
	super(_doc, _length, _style);
	node = _n;
	loadChildren();
	/*if (getElementCount()==0) {
	    SwingElement el = createDefaultChild();
	    if (el!=null) insertChild(el, null);
	    }*/
    }
    protected int handleSpace(int index, Element se0, Element se1) {
	return insertSpaceChild(index+1, null, null);
    }
    protected void loadChildren(NodeList nl) {
	SwingElementFactory fac = ((SwingDocument)getDocument()).getSwingElementFactory();
	Node[] ns = new Node[nl.getLength()];
	for (int z=0; z<ns.length; z++)
	    ns[z] = nl.item(z);
	Element last = null;
	boolean nextSpace=false, lastIgnore=false;
	for (int z=0; z<ns.length; z++) {
	    Node node = ns[z];
	    Element xse = fac.create(this, node);
	    if (xse!=null)
		if (xse.getLength()>0) {
		    appendChildView(xse); 
		    final boolean wasSpace = nextSpace || xse.leadingSpace || 
			(last!=null && last.endingSpace &&
			 (!xse.ignoreOuterSpaces()) && !lastIgnore);
		    nextSpace = false;
		    if (wasSpace) {
			xse.leadingSpace = false;
			if (last==null)
			    leadingSpace = true;
			else {
			    last.endingSpace = false;
			    handleSpace(getElementCount()-2, last, xse);
			}
		    }
		    last = xse;
		    lastIgnore = last.ignoreOuterSpaces();
		} else {
		    nextSpace |= xse.leadingSpace || xse.endingSpace;
		}
	}
	if (nextSpace || (last!=null && last.endingSpace)) {
	    endingSpace = true;
	    if (last!=null)
		last.endingSpace = false;
	}
    }
    protected void loadChildren() {
	loadChildren(getXmlNode().getChildNodes());
    }
    /***Element************************************/
    protected int insertSpaceChild(int index, DocumentEvent evt,
				   ContainerChange chg) {
	SwingText t = new SwingText
	    ((SwingDocument)getDocument(), " ", false);
	if (childAllowed(t)) {
	    insertChild(t, index);
	    if (evt!=null) {
		if (chg==null)
		    chg = evt.createContainerChange(this);
		evt.inserted(this, t.getStartOffset(), t.getLength());
	    }
	    if (chg!=null)
		chg.childAdded(index);
	    return 1;
	} else
	    return 0;
    }
    public boolean insertElementAtIndex(int idx, Element el, DocumentEvent evt) 
	throws BadLocationException {
	if (childAllowed(el)) {
	    insertChild(el, idx);
	    if (evt!=null) {
		evt.createContainerChange(this).childAdded(idx);
		evt.inserted(this, el.getStartOffset(), el.getLength());
	    }
	    return true;
	} else
	    return false;
    }
    /***Modify***********************************/
    public abstract boolean pasteNode(int pos, Node n, DocumentEvent evt) 
	throws BadLocationException;
    /***Data***************************************/
    public org.w3c.dom.Document getXmlDocument() { 
	return getXmlNode().getOwnerDocument(); 
    }
    /***NodeSwingElement************************************/
    public final Node getXmlNode() { return node; }
    public abstract Node copyXmlNode(org.w3c.dom.Document doc, int start, int end) 
	throws BadLocationException;
    /***InsertChild*****************************/
    protected final void insertXmlNode(NodeSwingElement newChild,
				       Element refChild) {
	Node refNode = null;
	if (refChild!=null)
	    for (int z=indexOf(refChild); z<getElementCount(); z++) {
		Element el = getElementAtIndex(z);
		if (el instanceof NodeSwingElement)
		    refNode = ((NodeSwingElement)el).getXmlNode(); break;
	    }
	if (refNode==null)
	    getXmlNode().appendChild(newChild.getXmlNode());
	else
	    getXmlNode().insertBefore(newChild.getXmlNode(), refNode);
    }
    protected final void removeXmlNode(NodeSwingElement child) {
	getXmlNode().removeChild(child.getXmlNode());
    }
    public void insertChild(Element child, int index, Element refChild) {
	super.insertChild(child, index, refChild);
	if (child instanceof NodeSwingElement)
	    insertXmlNode((NodeSwingElement)child, refChild);
    }
    public void removeChild(int index) {
	Element child = getElementAtIndex(index);
	super.removeChild(index);
	if (child instanceof NodeSwingElement)
	    removeXmlNode((NodeSwingElement)child);
    }
}
