package com.make.editor.plugins.text.tree;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.text.Position;
import javax.swing.text.BadLocationException;
import com.make.swing.text.Element;
import com.make.swing.text.DocumentEvent;
import com.make.swing.text.Document;
import com.make.swing.text.SplitNotSupportedException;
import com.make.swing.text.TextComponent;

public class NoteElement extends ElementSwingElement {
    /***properties******************************/
    FixedText prefix, suffix;
    /***Constructor********************************/
    public NoteElement(org.w3c.dom.Element _el, SwingDocument _doc) {
	super(_el, _doc, 0);
	prefix = new FixedText(_doc, "[");
	suffix = new FixedText(_doc, "]");
	insertChild(prefix, 0);
	insertChild(suffix, null);
	prefix.setStyle(_doc.getStyle("emphasized"));
	suffix.setStyle(_doc.getStyle("emphasized"));
	leadingSpace = false; endingSpace = false;
    }
    /***Menu*******************************************/
    public void fillMenu(TextComponent editor, int context, 
			 int pos, Position.Bias bias, JPopupMenu menu) 
	throws BadLocationException {
	JMenuItem mi;
	mi = new JMenuItem("Anmerkung entfernen");
	mi.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent evt2) {
		Document doc = getDocument();
		try {
		    doc.beginUpdate();
		    Element parent = getParent();
		    DocumentEvent evt = new DocumentEvent(doc);
		    final int idx = parent.indexOf(NoteElement.this);
		    evt.removed(parent, getStartOffset(), getLength());
		    parent.removeChild(idx);
		    evt.createContainerChange(parent).childRemoved(idx);
		    getDocument().fireDocumentChanged(evt);
		} finally {
		    doc.endUpdate();
		}
	    }
	});
	menu.add(mi);
	super.fillMenu(editor, context, pos, bias, menu);
    }
    /***Element****************************************/
    public boolean childConservationPossible(Element child) {
	return false;
    }
    public boolean canInsertAt(int pos) throws BadLocationException {
	return super.canInsertAt(pos) &&
	    pos>getStartOffset() && pos<getEndOffset();
    }
    protected Element createEmptyInstance(org.w3c.dom.Element el) {
	return new NoteElement(el, (SwingDocument)getDocument());
    }
    public Element split(int offset, DocumentEvent evt) 
	throws SplitNotSupportedException {
	throw new SplitNotSupportedException(this, offset);
    }
    public void remove(int offset, int length, DocumentEvent evt) 
	throws BadLocationException {
	if (offset==getStartOffset() && length==1)
	    offset++;
	if (offset==(getEndOffset()-1) && length==1)
	    offset--;
	if (length>=(getLength()-1) &&
	    (offset==prefix.getEndOffset() || (offset+length)==suffix.getStartOffset())) {
	    final Element parent = getParent();
	    final int idx = parent.indexOf(this);
	    parent.removeChild(idx);
	    evt.createContainerChange(parent).childRemoved(idx);
	    evt.removed(parent, offset, length);
	} else {
	    if (offset==getStartOffset()) {
		offset = getStartOffset()+1; length--;
	    }
	    if ((offset+length)==getEndOffset())
		length--;
	    super.remove(offset, length, evt);
	}
    }
    public boolean insertString(int offset, String str, DocumentEvent evt) 
	throws BadLocationException {
	return offset>=prefix.getEndOffset() && offset<=suffix.getStartOffset()
	    ?super.insertString(offset, str, evt)
	    :false;
    }
}
