package com.make.editor.plugins.text.tree;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import javax.swing.ButtonGroup;
import javax.swing.JMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JMenu;
import javax.swing.JPopupMenu;
import javax.swing.text.Position;
import javax.swing.text.BadLocationException;
import java.util.Hashtable;
import java.util.Enumeration;
import com.make.xml.DocumentType;
import com.make.xml.ElementType;
import com.make.editor.plugins.text.TextEditorStyleSet;
import com.make.util.Debug;
import com.make.swing.text.Element;
import com.make.swing.text.Document;
import com.make.swing.text.DocumentEvent;
import com.make.swing.text.ContainerChange;
import com.make.swing.text.SplitNotSupportedException;
import com.make.swing.text.TextComponent;

public class Paragraph extends ElementSwingElement {
    /***Constructor**********************************/
    public Paragraph(org.w3c.dom.Element _el, SwingDocument _doc) {
	super(_el, _doc, 1);
    }
    /***Menu*****************************************/
    public void fillMenu(final TextComponent editor, int context, 
			 int pos, Position.Bias bias, JPopupMenu menu) 
	throws BadLocationException {
	if (getParent() instanceof ElementSwingElement) {
	    final ElementSwingElement parent = (ElementSwingElement)getParent();
	    DocumentType edt = ((SwingDocument)getDocument()).getDocumentType();
	    ElementType et = edt.getElementType(parent.getTagName());
	    if (et.getOrder()!=et.O_FIXED) {
		// action
		ActionListener al = new ActionListener() {
		    public void actionPerformed(ActionEvent evt) {
			String tag = evt.getActionCommand();
			org.w3c.dom.Element xel = getXmlDocument().createElement(tag);
			Document doc = getDocument();
			doc.beginUpdate();
			try {
			    DocumentEvent tce = new DocumentEvent(doc);
			    replaceStyle(new Paragraph(xel, (SwingDocument)doc), tce);
			    if (tce.isValid()) doc.fireDocumentChanged(tce);
			} finally {
			    doc.endUpdate();
			}
		    }
		};
		// fill menu
		JMenu format = new JMenu("Absatzformat");
		ButtonGroup bg = new ButtonGroup();
		int count=0;
		for (Enumeration enum=et.getAllowedTags(); enum.hasMoreElements(); count++) {
		    String tag = (String)enum.nextElement();
		    Hashtable style = getDocument().getStyle(tag);
		    Integer cls = (Integer)style.get(TextEditorStyleSet.CLASS);
		    if (cls!=null && cls.intValue()==TextEditorStyleSet.C_PARAGRAPH) {
			String title = (String)style.get("title");
			if (title==null) title = '<'+tag+'>';
			JRadioButtonMenuItem mi = new JRadioButtonMenuItem(title);
			bg.add(mi);
			if (tag.equals(getTagName()))
			    bg.setSelected(mi.getModel(), true);
			else {
			    mi.setActionCommand(tag);
			    mi.addActionListener(al);
			}
			format.add(mi);
		    }
		}
		if (count>1)
		    menu.add(format);
	    }
	}
	// list
	JMenuItem mi = null;
	Element parentElement = getParent();
	if (parentElement instanceof NumberedList) {
	    mi = new JMenuItem("Numerierung entfernen");
	} else if (parentElement instanceof UnorderedList) {
	    mi = new JMenuItem("Aufz\u00e4hlung entfernen");
	}
	if (mi!=null) {
	    mi.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent evt2) {
		    Document doc = getDocument();
		    doc.beginUpdate();
		    DocumentEvent evt = new DocumentEvent(doc);
		    if (removeParentStyle(evt) && evt.isValid())
			doc.fireDocumentChanged(evt);
		    doc.endUpdate();
		}
	    });
	    menu.add(mi);
	}
	super.fillMenu(editor, context, pos, bias, menu);
    }
    /***Info******************************************/
    public Element createDefaultChild() { 
	return new SwingText((SwingDocument)getDocument());
    }
    public boolean ignoreOuterSpaces() { return true; }
    /***Split******************************************/
    protected void moveChildren(Element dest, int destIndex, int first, int last,
				ContainerChange chg) {
	for (; last>=first; last--) {
	    Element c = getElementAtIndex(last);
	    if (chg!=null)
		chg.childRemoved(last);
	    removeChild(last);
	    dest.insertChild(c, destIndex);
	}
    }
    /***Element********************************/
    public void getText(StringBuffer sb, int offset, int length) 
	throws BadLocationException {
	boolean pe = (offset+length)>=getEndOffset();
	if (pe) length = getEndOffset()-offset-1;
	if (length>0)
	    super.getText(sb, offset, length);
	if (pe)
	    sb.append("\n");
    }
    public Element split(int offset, DocumentEvent evt) 
	throws BadLocationException, SplitNotSupportedException {
	//throw new SplitNotSupportedException(this, offset);
	// already splitted?
	if (offset>=getEndOffset())
	    return null;
	Paragraph pnew = (Paragraph)createEmptyInstance();
	if (pnew==null) return null;
	final int lastMove = getElementCount()-1;
	if (offset<=getStartOffset()) {
	    if (lastMove>=0)
		moveChildren(pnew, 0, 0, lastMove, evt.createContainerChange(this));
	} else if (offset<(getEndOffset()-1)) {
	    // find child
	    final int index = getIndexAt(offset);
	    int firstMove = index, destIndex = 0;
	    Element el = getElementAtIndex(index);
	    // split it
	    if (offset>el.getStartOffset())
		try {
		    Element el2 = el.split(offset, evt);
		    if (el2!=null) {
			pnew.insertChild(el2, destIndex++);
			firstMove++;
		    }
		} catch (SplitNotSupportedException e) {
		}
	    // move all children on the right hand into a new instance
	    if (firstMove<=lastMove)
		moveChildren(pnew, destIndex, firstMove, lastMove, 
			     evt.createContainerChange(this));
	}
	if (evt!=null)
	    evt.inserted(this, getEndOffset()-1, 1);
	return pnew;
    }
    /***Data**************************************/
    protected boolean restoreDefaultParagraphFormat(DocumentEvent evt) {
	ElementSwingElement parent = (ElementSwingElement)getParent();
	ElementSwingElement p2 = (ElementSwingElement)parent.getParent();
	ElementType ptype = parent.getElementType();
	if (ptype==null) return false;
	String defTag = ptype.getDefaultChildTag();
	if (defTag==null) return false;
	if (!defTag.equals(getTagName())) {
	    org.w3c.dom.Element xel = getXmlDocument().createElement(defTag);
	    SwingElementFactory fac = ((SwingDocument)getDocument()).getSwingElementFactory();
	    Element newStyle = fac.create(parent, xel);
	    replaceStyle(newStyle, evt);
	    return true;
	} else if (parent.childConservationPossible(this) && p2!=null &&
		   p2.getElementType()!=null) {
	    String defTag2 = p2.getElementType().getDefaultChildTag();
	    if (defTag2!=null) {
		Hashtable style = getDocument().getStyle(defTag2);
		if (style!=null) {
		    Integer cls = (Integer)style.get(TextEditorStyleSet.CLASS);
		    if (cls!=null && cls.intValue()==TextEditorStyleSet.C_PARAGRAPH)
			return removeParentStyle(evt);
		}
	    }
	    return false;
	}
	return false;
    }
    public boolean insertString(int offset, String str, DocumentEvent evt) 
	throws BadLocationException {
	if (offset>=getEndOffset())
	    return false;
	else
	    if (str.equals("\n")) {
		Element parent = getParent();
		final int beforeIndex = parent.indexOf(this)-1;
		if (getLength()>1 && 
		    (offset>getStartOffset() ||
		     beforeIndex<0 ||
		     !(parent.getElementAtIndex(beforeIndex) instanceof Paragraph) ||
		     parent.getElementAtIndex(beforeIndex).getElementCount()>0)) {
		    //TODO: enthaelt weitere Absaetze, erst diese trennen
		    try {
			Paragraph pnew = (Paragraph)split(offset, evt);
			if (pnew!=null) {
			    final int afterIndex = beforeIndex+2;
			    parent.insertChild(pnew, afterIndex);
			    evt.createContainerChange(parent).childAdded(afterIndex);
			    return true;
			} else
			    return false;
		    } catch (SplitNotSupportedException e) {
			return restoreDefaultParagraphFormat(evt);
		    }
		} else if (restoreDefaultParagraphFormat(evt)) {
		    return true;
		} else {
		    Paragraph pnew = (Paragraph)createEmptyInstance();
		    final int idx = getParent().indexOf(this);
		    getParent().insertChild(pnew, idx);
		    if (evt!=null) {
			evt.inserted(getParent(), pnew.getStartOffset(), pnew.getLength());
			evt.createContainerChange(getParent()).childAdded(idx);
		    }
		    return true;
		}
	    } else
		return super.insertString(offset, str, evt);
    }
    public void remove(int offset, int length, DocumentEvent evt) 
	throws BadLocationException {
	final boolean remPE = (offset+length)>=getEndOffset();
	if (remPE) length = getEndOffset()-1-offset;
	super.remove(offset, length, evt);
	if (remPE) {
	    // merge?
	    Element body = getParent();
	    final int nextIndex = body.indexOf(this)+1;
	    // TODO: merge mit paragraps in anderen bodies
	    if (nextIndex<body.getElementCount()) {
		// merge paragraphs
		Element e2 = body.getElementAtIndex(nextIndex);
		if (e2.getClass()==getClass() && body.canRemoveChild(e2)) {
		    evt.removed(this, getEndOffset()-1, 1);
		    Paragraph p2 = (Paragraph)e2;
		    // remove p2
		    body.removeChild(nextIndex);
		    evt.createContainerChange(body).childRemoved(nextIndex);
		    // merge
		    ContainerChange chg = evt.createContainerChange(this);
		    for (int z=0; z<p2.getElementCount();) {
			Element el = p2.getElementAtIndex(z);
			int n = getElementCount();
			if (p2.childConservationPossible(el)) {
			    p2.removeChild(z);
			    insertChild(el, n);
			    chg.childAdded(n);
			    n++;
			} else
			    z++;
		    }
		}
	    }
	}
    }
    public boolean pasteNode(int pos, Node n, DocumentEvent evt) 
	throws BadLocationException {
	if (pos==getEndOffset())
	    pos--;
	return super.pasteNode(pos, n, evt);
    }
    /***Format****************************************/
    protected Element createEmptyInstance(org.w3c.dom.Element el) {
	return new Paragraph(el, (SwingDocument)getDocument());
    }
}
