package com.make.editor.plugins.text.tree;

import java.util.Enumeration;
import java.util.Vector;
import java.util.Hashtable;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.text.Position;
import javax.swing.text.BadLocationException;
import org.w3c.dom.NodeList;
import com.make.swing.text.Element;
import com.make.swing.text.Document;
import com.make.swing.text.DocumentEvent;
import com.make.swing.text.TextComponent;

public class StyleElement extends ElementSwingElement {
    /***Constructor********************************/
    public StyleElement(org.w3c.dom.Element _el, SwingDocument _doc) {
	super(_el, _doc, 0);
    }
    public StyleElement(SwingDocument _doc, String tag) {
	super(((NodeSwingElement)_doc.getRootElement()).getXmlDocument().createElement(tag), 
	      _doc, 0);
    }
    /***Format****************************************/
    public boolean isStyle(String style) {
	return getTagName().equals(style);
    }
    protected Element createEmptyInstance(org.w3c.dom.Element el) {
	return new StyleElement(el, (SwingDocument)getDocument());
    }
    /***Menu*******************************************/
    public void fillMenu(TextComponent editor, int context, 
			 int pos, Position.Bias bias, JPopupMenu menu) 
	throws BadLocationException {
	final String tag = getTagName();
	// remove style
	String title = (String)getAttribute((Object)"title");
	if (title==null) title = '<'+tag+'>';
	JMenuItem mi;
	mi = new JMenuItem("Formatierung '"+title+"' entfernen");
	mi.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent evte) {
		Document doc = getDocument();
		try {
		    doc.beginUpdate();
		    Element parent = getParent();
		    final int idx = parent.indexOf(StyleElement.this);
		    int first=idx, last=idx;
		    final int cnt1 = parent.getElementCount()-1;
		    while (first>0 && (parent.getElementAtIndex(--first).getLength()==0 ||
				       parent.getElementAtIndex(first).isStyle(tag)));
		    while (last<cnt1 && (parent.getElementAtIndex(++last).getLength()==0 ||
					 parent.getElementAtIndex(last).isStyle(tag)));
		    DocumentEvent evt = new DocumentEvent(doc);
		    Element root = doc.getRootElement();
		    root.removeStyle(tag, parent.getElementAtIndex(first).getStartOffset(),
				     parent.getElementAtIndex(last).getEndOffset(), evt);
		    if (evt.isValid())
			doc.fireDocumentChanged(evt);
		} catch (BadLocationException e) {
		    e.printStackTrace();
		} finally {
		    doc.endUpdate();
		}
	    }
	});
	menu.add(mi);
	super.fillMenu(editor, context, pos, bias, menu);
    }
    /***Info****************************************/
    public String toString() {
	return "["+super.toString()+": style='"+getXmlNode().getNodeName()+"']";
    }
}
