package com.make.editor.plugins.text.tree;

import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.Text;
import org.w3c.dom.DOMException;
import javax.swing.text.BadLocationException;
import javax.swing.text.AttributeSet;
import javax.swing.undo.AbstractUndoableEdit;
import com.make.util.Debug;
import com.make.swing.text.Element;
import com.make.swing.text.Document;
import com.make.swing.text.DocumentEvent;
import com.make.swing.text.ElementUndoableEdit;
import com.make.swing.text.ContainerChange;
import com.make.swing.text.SplitNotSupportedException;
import com.make.swing.text.ContainerChange;

class RemoveStringUndoableEdit extends ElementUndoableEdit {
    /***properties**************************/
    protected int position;
    protected String data;
    /***Constructor*************************/
    public RemoveStringUndoableEdit(SwingText txt, int _p, String _d) {
	super(txt); position = _p; data = _d;
	if (DUMP)
	    System.err.println(element+": NEW removeString start="+element.getStartOffset());
    }
    public void undo() {
	super.undo();
	if (DUMP)
	    System.err.println(element+": insertString start="+element.getStartOffset());
	try {
	    element.insertString(position, data, getEvent());
	    //inserted(position, data.length());
	} catch (BadLocationException e) {
	    e.printStackTrace();
	}
    }
    public void redo() {
	super.redo();
	if (DUMP)
	    System.err.println(element+": removeString start="+element.getStartOffset());
	try {
	    element.remove(position, data.length(), getEvent());
	    //removed(position, data.length());
	} catch (BadLocationException e) {
	    e.printStackTrace();
	}
    }
}

class InsertStringUndoableEdit extends ElementUndoableEdit {
    /***properties**************************/
    protected int position;
    protected String data;
    /***Constructor*************************/
    public InsertStringUndoableEdit(SwingText txt, int _p, String _d) {
	super(txt); position = _p; data = _d;
	if (DUMP)
	    System.err.println(element+": NEW insertString start="+element.getStartOffset());
    }
    public void undo() {
	super.undo();
	if (DUMP)
	    System.err.println(element+": removeString start="+element.getStartOffset());
	try {
	    element.remove(position, data.length(), getEvent());
	    //removed(position, data.length());
	} catch (BadLocationException e) {
	    e.printStackTrace();
	}
    }
    public void redo() {
	super.redo();
	if (DUMP)
	    System.err.println(element+": insertString start="+element.getStartOffset());
	try {
	    element.insertString(position, data, getEvent());
	    //inserted(position, data.length());
	} catch (BadLocationException e) {
	    e.printStackTrace();
	}
    }
}

public class SwingText extends NodeSwingElement {
    /***Constructor*********************************/
    public SwingText(Text _t, Document _doc, boolean parse) {
	super(_t, _doc, _t.getLength(), true);
	if (parse) {
	    String data = getText().getData();
	    final int length = data.length();
	    if (length>0) {
		StringBuffer sb = new StringBuffer(length);
		boolean ignoreSpace = true;
		boolean wasSpace = false;
		for (int z=0; z<length; z++) {
		    char ch = data.charAt(z);
		    if (Character.isWhitespace(ch)) {
			wasSpace = true;
		    } else {
			if (wasSpace)
			    if (ignoreSpace)
				leadingSpace = true;
			    else
				sb.append(' ');
			sb.append(ch);
			ignoreSpace = false;
			wasSpace = false;
		    }
		}
		endingSpace = wasSpace;
		String newData = sb.toString();
		getText().setData(newData);
		lengthChanged(newData.length()-length);
	    }
	}
    }
    public SwingText(SwingDocument _doc, String txt, boolean parse) {
	this(_doc.getXmlDocument().createTextNode(txt), _doc, parse);
    }
    public SwingText(SwingDocument _doc, String txt) {
	this(_doc, txt, true);
    }
    public SwingText(SwingDocument _doc) {
	this(_doc, "");
    }
    /***Element******************************/
    protected int insertSpaceChild(int index, DocumentEvent evt,
				   ContainerChange chg) {
	throw new InternalError();
    }
    /***Info********************************************/
    public final Text getText() { return (Text)getXmlNode(); }
    public String toString() {
	return "["+super.toString()+": text='"+getXmlNode().getNodeValue()+"']";
    }
    /***Format**********************************************/
    protected Element createEmptyInstance() { 
	throw new IllegalStateException(); 
    }
    public Element split(int offset, DocumentEvent evt) 
	throws SplitNotSupportedException {
	// already splitted?
	if (offset<=getStartOffset() || offset>=getEndOffset())
	    return null;
	offset -= getStartOffset();
	int oldLength = getLength();
	// split
	Text t = getText();
	Text t2 = t.splitText(offset);
	t.getParentNode().removeChild(t2); // is reinserted later
	// undo
	if (!fresh)
	    getDocument().postEdit(new RemoveStringUndoableEdit
				   (this, offset+getStartOffset(), t2.getData()));
	// length
	lengthChanged(offset-oldLength);
	if (evt!=null)
	    evt.createParentChange(this);
	return new SwingText(t2, getDocument(), false);
    }
    /***NodeSwingElement***********************************/
    public Node copyXmlNode(org.w3c.dom.Document doc, int start, int end) 
	throws BadLocationException {
	if (start>end) throw new BadLocationException("start>end", start);
	if (start<getStartOffset()) throw new BadLocationException("start", start);
	if (end>getEndOffset()) throw new BadLocationException("end", end);
	if (start==end) return null;
	//System.err.println("copyXmlNode start="+start+" end="+end+"; this start="+getStartOffset()+" end="+getEndOffset());
	String data = getText().substringData(start-getStartOffset(), end-start);
	return doc.createTextNode(data);
    }
    public boolean pasteNode(int pos, Node n, DocumentEvent evt) 
	throws BadLocationException {
	return n instanceof Text && insertString(pos, n.getNodeValue(), evt);
    }
    /***Element********************************/
    public void getText(StringBuffer sb, int offset, int length) 
	throws BadLocationException {
	final int end = getEndOffset();
	if (offset<getStartOffset()) 
	    throw new BadLocationException("offset<start", offset);
	if ((offset+length)>end)
	    throw new BadLocationException("offset+length>end", offset);
	if (length>0)
	    sb.append(getText().substringData(offset-getStartOffset(), length));
    }
    public boolean insertString(int offset, String str, DocumentEvent evt) 
	throws BadLocationException {
	if (Debug.DEBUG) {
	    if (offset<getStartOffset())
		throw new BadLocationException("offset<getStartOffset(): "+
					       offset+"<"+getStartOffset(), offset);
	    if (offset>getEndOffset())
		throw new BadLocationException("offset>getEndOffset(): "+
					       offset+">"+getEndOffset(), offset);
	}
	Text txt = getText();
	final int relOffset = offset-getStartOffset();
	if (str.equals(" ") && relOffset>0 &&
	    txt.substringData(relOffset-1, 1).equals(" "))
	    try {
		Element p = getParent();
		Element p2 = p.getParent();
		if (p2==null || !p2.childAllowed(this)) return true;
		final int pIndex = p2.indexOf(p);
		p2.splitChild(pIndex, offset, evt);
		int idx = p2.splitChild(pIndex, offset-1, evt);
		if (idx==-1) idx = pIndex;
		p = p2.getElementAtIndex(idx);
		p2.removeChild(idx);
		SwingText st = (SwingText)p.getElementAtIndex(0);
		p.removeChild(0);
		p2.insertChild(st, idx);
		if (evt!=null) {
		    ContainerChange chg = evt.createContainerChange(p2);
		    chg.childRemoved(idx);
		    chg.childAdded(idx);
		}
		return true;
	    } catch (SplitNotSupportedException e) {
	    }
	try {
	    if (offset>=getEndOffset())
		txt.appendData(str);
	    else
		txt.insertData(relOffset, str);
	    lengthChanged(str.length());
	    if (evt!=null)
		evt.inserted(this, offset, str.length());
	    if (!fresh)
		getDocument().postEdit(new InsertStringUndoableEdit(this, offset, str));
	    return true;
	} catch (DOMException e) {
	    if (Debug.DEBUG) {
		System.err.println("DOMException: "+e.code);
		System.err.println("offset="+offset);
		System.err.println("start="+getStartOffset());
		System.err.println("curlen="+getText().getData().length());
	    }
	    e.printStackTrace();
	    return false;
	}
    }
    public void remove(int offset, int len, DocumentEvent evt) 
	throws BadLocationException {
	if (Debug.DEBUG) {
	    if (offset<getStartOffset())
		throw new InternalError(offset+" < "+getStartOffset());
	    if ((offset+len)>getEndOffset())
		throw new InternalError(offset+" + "+len+" > "+getEndOffset());
	}
	if (len<=0) 
	    return;
	try {
	    if (!fresh)
		getDocument().postEdit
		    (new RemoveStringUndoableEdit(this, offset,
						  getText().substringData(offset-getStartOffset(), 
									  len)));
	    getText().deleteData(offset-getStartOffset(), len);
	    lengthChanged(-len);
	    evt.removed(this, offset, len);
	} catch (DOMException e) {
	    if (Debug.DEBUG) {
		System.err.println("DOMException: "+e.code);
		System.err.println("offset="+offset);
		System.err.println("start="+getStartOffset());
		System.err.println("curlen="+getText().getData().length());
	    }
	    e.printStackTrace();
	}
    }
}
