package com.make.editor.plugins.text.tree;

import java.util.Hashtable;
import org.w3c.dom.Node;
import org.w3c.dom.Text;
import org.w3c.dom.Entity;
import org.w3c.dom.EntityReference;
import com.make.xml.DocumentType;
import com.make.xml.ElementType;
import com.make.xml.AttributeType;
import com.make.editor.plugins.EditorStyleSet;
import com.make.editor.plugins.text.TextEditorStyleSet;
import com.make.swing.text.Element;

public class XmlElementFactory implements SwingElementFactory {
    /***properties*********************/
    protected SwingDocument document;
    protected DocumentType docType;
    protected TextEditorStyleSet styles;
    /***Constructor*********************/
    public XmlElementFactory(SwingDocument _doc, DocumentType _dt, 
			     TextEditorStyleSet _st) {
	document = _doc; docType = _dt; styles = _st;
    }
    protected int getClass(String tag) {
	Hashtable style = styles.getStyle(tag);
	Integer cls = style==null
	    ?null
	    :(Integer)style.get(styles.CLASS);
	return cls==null?styles.C_UNKNOWN:cls.intValue();
    }
    protected byte getShape(String tag) {
	Hashtable style = styles.getStyle(tag);
	Byte shape = style==null
	    ?null
	    :(Byte)style.get(styles.SHAPE);
	return shape==null?styles.S_UNKNOWN:shape.byteValue();
    }
    /***SwingElementFactory*************/
    public NodeSwingElement create(Element parent, Node node) {
	if (node==null)
	    return null;
	if (node instanceof Text)
	    return parent instanceof SwingBody 
		|| parent instanceof TableElement
		?null
		:new SwingText((Text)node, document, true);
	else if (node instanceof org.w3c.dom.Element) {
	    org.w3c.dom.Element el = (org.w3c.dom.Element)node;
	    String tag = el.getTagName();
	    switch (getClass(tag)) {
	    case styles.C_IGNORE: return null;
	    case styles.C_UNKNOWN: 
		System.err.println("WARNING: Unknown tag "+tag);
		return new UnknownSwingElement(el, document);
	    case styles.C_SECTION: 
		if (parent instanceof TableRowElement)
		    return new TableCellElement(el, document);
		else
		    return new Section(el, document);
	    case styles.C_PARAGRAPH: 
		/*if (!(parent instanceof Paragraph) &&
		  !(parent instanceof StyleElement))*/
		return new Paragraph(el, document);
	    case styles.C_TEXT: {
		ElementType eet = docType.getElementType(tag);
		if (eet!=null) {
		    AttributeType[] a = eet.getAttributes();
		    if (a!=null) for (int z=0; z<a.length; z++)
			if (a[z].isReference())
			    return new LinkElement(el, document);
		}
		return new StyleElement(el, document);
	    }
	    case styles.C_IMAGE: {
		byte shape = getShape(tag);
		boolean inline;
		switch (shape) {
		case styles.S_RECTANGLE:
		    inline = true; break;
		case styles.S_HSTRUT: case styles.S_VSTRUT:
		    inline = false;
		default:
		    inline = !(parent instanceof SwingBody || 
			       parent instanceof Section);
		}
		return new ImageElement(el, document, inline);
	    }
	    case styles.C_RESOURCE_IMAGE: {
		byte shape = getShape(tag);
		boolean inline;
		switch (shape) {
		case styles.S_RECTANGLE:
		    inline = true; break;
		case styles.S_HSTRUT: case styles.S_VSTRUT:
		    inline = false; break;
		default:
		    inline = !(parent instanceof SwingBody || 
			       parent instanceof Section);
		}
		return new ResourceImageElement(el, document, inline);
	    }
	    case styles.C_TABLE:
		return new TableElement(el, document);
	    case styles.C_TABLEROW:
		return new TableRowElement(el, document);
	    case styles.C_UL:
		return new UnorderedList(el, document);
	    case styles.C_OL:
		return new NumberedList(el, document);
	    case styles.C_NOTE:
		return new NoteElement(el, document);
	    case styles.C_BREAK:
		return new BreakElement(el, document);
	    default: return null;
	    }
	} else if (node instanceof Entity)
	    return new SwingEntityElement((Entity)node, document);
	else if (node instanceof EntityReference)
	    return new SwingEntityElement((EntityReference)node, document);
	else {
	    System.err.println("Ignoring "+node.getClass().getName());
	    return null;
	}
    }
}
