package com.make.editor.plugins.text.view;

import java.awt.Graphics;
import java.awt.Component;
import java.awt.Shape;
import java.awt.Color;
import java.awt.Rectangle;
import java.awt.Point;
import java.util.Hashtable;
import javax.swing.Icon;
import javax.swing.text.JTextComponent;
import javax.swing.text.BadLocationException;
import com.make.editor.plugins.text.TextEditorStyleSet;
import com.make.editor.plugins.text.tree.ImageElement;
import com.make.swing.text.view.View;
import com.make.swing.text.view.ViewFactory;
import com.make.swing.text.view.Highlighter;
import com.make.swing.text.Element;
import com.make.swing.text.Caret;
import com.make.swing.text.TextComponent;
import com.make.swing.text.DocumentEvent;

public class ImageView extends View {
    /***properties**********************************/
    protected ImageElement element;
    protected int allocX, allocY, allocWidth, allocHeight;
    /***Constructor*************************************/
    public ImageView(ImageElement el) {
	super(el); element = el;
    }
    /***View*****************************************/
    public int getRowStart(int pos) { 
	return element.getInline()?super.getRowStart(pos):getStartOffset(); 
    }
    public int getRowEnd(int pos) {
	return element.getInline()?super.getRowEnd(pos):getEndOffset()-1;
    } 
    public float getPreferredSpan(int axis) {
	Icon icon = element.getIcon();
	if (icon==null) return 16; //TODO
	return axis==X_AXIS
	    ? icon.getIconWidth()
	    : icon.getIconHeight();
    }
    public void paint(Graphics g, Shape allocation) {
	boolean inSelection = false;
	TextComponent tc = getContainer();
	// highlighter
	Highlighter h = tc.getHighlighter();
	h.paintHighlights(g, getStartOffset(), getEndOffset(), 
			  allocation, tc, this);
	final int p0 = getStartOffset();
	final Caret caret = tc.getCaret();
	final int dot = caret.getDot();
	final int mark = caret.getMark();
	final int selStart = Math.min(dot, mark);
	final int selEnd = Math.max(dot, mark);
	inSelection = //(selStart==selEnd && selStart==p0) ||
	    (selStart<=p0 && selEnd>p0);
	if (inSelection) {
	    Color selFG = tc.getSelectedTextColor();
	    g.setXORMode(selFG);
	}
	Rectangle r = allocation.getBounds();
	allocX = r.x; allocY = r.y;
	Icon icon = element.getIcon();
	if (icon==null) {
	    g.drawLine(r.x, r.y, r.x+r.width, r.y+r.height);
	    g.drawLine(r.x+r.width, r.y, r.x, r.y+r.height);
	    g.drawRect(r.x, r.y, r.width, r.height);
	} else
	    icon.paintIcon(getContainer(), g, r.x, r.y);
	if (inSelection)
	    g.setPaintMode();
    }
    public float getAlignment(int axis) {
	if (axis==X_AXIS && !element.getInline()) {
	    String al = element.getAttribute("align");
	    if ("right".equals(al))
		return 1.0f;
	    else if ("center".equals(al))
		return 0.5f;
	    else
		return 0.0f;
	} else
	    return super.getAlignment(axis);
    }
    public Shape modelToView(int pos, Shape a) throws BadLocationException {
	final int p0 = getStartOffset();
	final int p1 = getEndOffset();
	if ((pos>=p0) && (pos<=p1)) {
	    Rectangle r = a.getBounds();
	    if (pos>p0)
		r.x += r.width;
	    r.width = 0;
	    return r;
	}
	throw new BadLocationException(pos + " not in range " + p0 + "," + p1, pos);
    }
    public int viewToModel(float x, float y, Shape a) {
	final int p0 = getStartOffset();
	Rectangle alloc = (Rectangle) a;
	if (x < alloc.x + (alloc.width / 2))
	    return p0;
	return p0+1;
    }
    public void setSize(int width, int height) {
	super.setSize(width, height);
	allocWidth = width; allocHeight = height;
    }
    public int getNextVisualPositionFrom(int pos, int direction) 
	throws BadLocationException {
	final int p0 = getStartOffset();
	switch (direction) {
	case NORTH:
	case SOUTH:
	    if (pos==-1) {
		final int x = getMagicX(pos)-allocX;
		return x<(allocWidth/2) ? p0 : p0+1;
	    } else
		return -1;
	case EAST:
	    if (pos==-1)
		return p0;
	    else if (pos<=p0 && !element.getInline())
		return p0+1;
	    else
		return -1;
	case WEST:
	    if (pos==-1)
		return getEndOffset()-1;
	    else if (pos<=p0)
		return -1;
	    else
		return p0;
	default:
	    throw new IllegalArgumentException("Bad direction: " + direction);
	}
    }
    public void documentChanged(DocumentEvent evt, Shape a, ViewFactory vf) {
	Component host = getContainer();
	if (a!=null) {
	    Rectangle alloc = a instanceof Rectangle?(Rectangle)a:a.getBounds();
	    if (host!=null)
		host.repaint(0, alloc.y, Integer.MAX_VALUE, alloc.height);
	} else {
	    if (host!=null)
		host.repaint();
	}
    }
}
