package com.make.editor.plugins.text.view;

import java.awt.Rectangle;
import java.awt.Graphics;
import java.awt.Shape;
import java.awt.Point;
import java.util.Vector;
import java.util.Enumeration;
import javax.swing.text.JTextComponent;
import javax.swing.text.Caret;
import javax.swing.text.BadLocationException;
import com.make.editor.plugins.text.tree.ElementSwingElement;
import com.make.editor.plugins.text.tree.TableElement;
import com.make.editor.plugins.text.tree.TableRowElement;
import com.make.swing.text.DocumentEvent;
import com.make.swing.text.Element;
import com.make.swing.text.Document;
import com.make.swing.text.view.View;
import com.make.swing.text.view.ViewFactory;
import com.make.swing.text.view.BoxView;
import com.make.swing.text.view.EmptyView;
import com.make.swing.text.view.VerticalView;

class TableRowView extends BoxView implements VerticalView {
    /***properties************************************/
    protected TableView table;
    protected TableRowElement tableRow;
    protected ElementSwingElement[] cells;
    /***Constructor************************************/
    public TableRowView(TableRowElement _el, int axis) {
	super(_el, axis);
	tableRow = _el;
    }
    protected void loadChildren(ViewFactory f) {
	table = (TableView)getParent();
	cells = table.extractCells(tableRow);
	View[] v = new View[cells.length];
	int start = getStartOffset();
	for (int z=0; z<v.length; z++) {
	    ElementSwingElement el = cells[z];
	    if (el==null)
		v[z] = new EmptyView(getElement(), start);
	    else {
		v[z] = f.create(el);
		start = v[z].getEndOffset();
	    }
	}
	replace(0, 0, v);
    }
    /***CompositeView***************************************/
    protected void layoutMajorAxis(int targetSpan, int axis, int[] offsets, int[] spans) {
	final int count = cells.length;
	final int width = Math.max(40, targetSpan / count);
	for (int z=0; z<count; z++) {
	    offsets[z] = z*width;
	    spans[z] = Math.min(width, (int)getView(z).getMaximumSpan(axis));
	}
    }
    protected void layoutMinorAxis(int targetSpan, int axis, int[] offsets, int[] spans) {
	final int count = cells.length;
	for (int z=0; z<count; z++) {
	    offsets[z] = 0;
	    spans[z] = Math.min(targetSpan, (int)getView(z).getMaximumSpan(axis));
	}
    }
    protected int getNextNorthSouthVisualPositionFrom(int pos, int direction)
	throws BadLocationException {
	if (pos==-1) {
	    final int x = getMagicX(pos)-allocX;
	    final int column = table.getColumnIndexAt(x);
	    View v = getView(column);
	    return v.getNextVisualPositionFrom(-1, direction);
	} else {
	    final int vIndex = getViewIndexAtPosition(pos);
	    View v = getView(vIndex);
	    return v.getNextVisualPositionFrom(pos, direction);
	}
    }
}
