package com.make.editor.plugins.text.view;

import com.make.swing.text.Element;
import com.make.swing.text.view.ViewFactory;
import com.make.swing.text.view.View;
import com.make.swing.text.view.StyleView;
import com.make.swing.text.view.UnknownView;
import com.make.swing.text.view.EmptyView;
import com.make.editor.plugins.text.tree.SwingBody;
import com.make.editor.plugins.text.tree.Paragraph;
import com.make.editor.plugins.text.tree.SwingText;
import com.make.editor.plugins.text.tree.FixedText;
import com.make.editor.plugins.text.tree.StyleElement;
import com.make.editor.plugins.text.tree.LinkElement;
import com.make.editor.plugins.text.tree.UnknownSwingElement;
import com.make.editor.plugins.text.tree.ImageElement;
import com.make.editor.plugins.text.tree.ElementSwingElement;
import com.make.editor.plugins.text.tree.SwingEntityElement;
import com.make.editor.plugins.text.tree.TableElement;
import com.make.editor.plugins.text.tree.TableRowElement;
import com.make.editor.plugins.text.tree.TableCellElement;
import com.make.editor.plugins.text.tree.NoteElement;
import com.make.editor.plugins.text.tree.UnorderedList;
import com.make.editor.plugins.text.tree.NumberedList;
import com.make.editor.plugins.text.tree.Section;
import com.make.editor.plugins.text.tree.BreakElement;

public class XmlViewFactory implements ViewFactory {
    public View create(Element elem) {
	if (elem instanceof SwingBody)
	    return new BodyView(elem);
	else if (elem instanceof Section || elem instanceof TableCellElement)
	    return new SectionView(elem);
	else if (elem instanceof Paragraph) {
	    Element parent = elem.getParent();
	    if (parent instanceof UnorderedList)
		return new BulletParagraphView(elem, this);
	    else if (parent instanceof NumberedList)
		return new NumberedParagraphView(elem, this);
	    else
		return new ParagraphView(elem, this);
	} else if (elem instanceof SwingText)
	    return new TextView((SwingText)elem);
	else if (elem instanceof StyleElement ||
		 elem instanceof SwingEntityElement ||
		 elem instanceof LinkElement ||
		 elem instanceof NoteElement)
	    return new StyleView(elem);
	else if (elem instanceof ImageElement)
	    return new ImageView((ImageElement)elem);
	else if (elem instanceof TableElement)
	    return new TableView((TableElement)elem, View.Y_AXIS);
	else if (elem instanceof TableRowElement)
	    return new TableRowView((TableRowElement)elem, View.X_AXIS);
	else if (elem instanceof UnknownSwingElement)
	    return new UnknownView((UnknownSwingElement)elem, this);
	else if (elem instanceof FixedText)
	    return new FixedTextView((FixedText)elem);
	else if (elem instanceof BreakElement)
	    return new BreakView((BreakElement)elem);
	else if (elem instanceof TableCellElement.CellEnd)
	    return new EmptyView(elem, elem.getStartOffset());
	else
	    return new NullView(elem);
    }
}
