package com.make.editor.plugins.tree;

import java.awt.Component;
import java.beans.PropertyChangeListener;
import javax.swing.Action;
import javax.swing.JTree;
import javax.swing.JScrollPane;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import javax.swing.event.TreeModelListener;
import javax.swing.event.TreeModelEvent;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.w3c.dom.Attr;
import com.make.editor.EditorFrame;
import com.make.editor.EditorDocument;
import com.make.editor.plugins.DefaultEditorComponent;

class TreeEditorComponent extends DefaultEditorComponent 
    implements TreeModel {
    class TreeEditor extends JTree {
	public TreeEditor() { super(TreeEditorComponent.this); }
	public String convertValueToText(Object value, boolean selected,
					 boolean expanded, boolean leaf, int row,
					 boolean hasFocus) {
	    Node node = (Node)value;
	    if (node instanceof Text || node instanceof Attr)
		return node.getNodeValue();
	    else if (node instanceof Element) {
		StringBuffer sb = new StringBuffer(node.getNodeName());
		NamedNodeMap nnm = node.getAttributes();
		if (nnm.getLength()>0) {
		    sb.append(" [");
		    for (int z=0; z<nnm.getLength(); z++) {
			Attr attr = (Attr)nnm.item(z);
			sb.append(attr.getName());
			sb.append("=\"");
			sb.append(attr.getValue());
			sb.append("\" ");
		    }
		    sb.append("]");
		}
		return sb.toString();
	    } else
		return node.getNodeName();
	}
    }
    /***properties*********************************/
    protected JScrollPane scrollPane;
    protected JTree tree;
    /***Constructor*******************************/
    public TreeEditorComponent(EditorFrame _ef) {
	super(_ef);
    }
    /***EditorComponent****************************/
    public void setDocument(EditorDocument doc) {
	if (tree!=null)
	    unregisterInputComponent(tree);
	super.setDocument(doc);
	if (tree==null) {
	    tree = new TreeEditor();
	    registerInputComponent(tree);
	    scrollPane = new JScrollPane(tree);
	    scrollPane.getViewport().putClientProperty("EnableWindowBlit", Boolean.TRUE);
	} else
	    tree.setModel(this);
    }
    public Component getComponent() { return scrollPane; }
    /***TreeModel********************************/
    public Object getRoot() { return document.getNode(); }
    public Object getChild(Object parent, int index) {
	return ((Node)parent).getChildNodes().item(index);
    }
    public int getChildCount(Object parent) {
	return ((Node)parent).getChildNodes().getLength();
    }
    public boolean isLeaf(Object node) {
	return !((Node)node).hasChildNodes();
    }
    public void valueForPathChanged(TreePath path, Object newValue) {}
    public int getIndexOfChild(Object parent, Object child) {
	NodeList nl = ((Node)parent).getChildNodes();
	for (int z=0; z<nl.getLength(); z++)
	    if (nl.item(z)==child)
		return z;
	return -1;
    }
    public void addTreeModelListener(TreeModelListener l) {}
    public void removeTreeModelListener(TreeModelListener l) {}
}
