package com.make.swing;

import java.awt.Component;
import java.awt.Container;
import java.awt.Frame;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import javax.swing.JToolBar;
import javax.swing.JMenuBar;
import javax.swing.JPopupMenu;
import javax.swing.JComponent;
import com.make.util.Debug;
import com.make.swing.bt.FrameDescription;
import com.make.swing.bt.ButtonContainer;
import com.make.swing.bt.ButtonDescription;
import com.make.swing.bt.MenuDescription;

public abstract class AbstractInterfacePanel 
    implements InterfacePanel, ButtonContainer {
    /***Class***************************/
    protected class InstalledFrameDescription {
	/***properties********************/
	protected FrameDescription frameDescription;
	protected Vector buttons = new Vector();
	protected Vector components = new Vector();
	/***Constructor********************/
	public InstalledFrameDescription(FrameDescription fd) {
	    frameDescription = fd;
	    JToolBar tb = fd.installToolBar(toolBar, buttons, components);
	    if (tb!=toolBar)
		setToolBar(tb);
	    JMenuBar mb = fd.installMenuBar(menuBar, buttons, components);
	    if (mb!=menuBar) {
		menuBar = mb;
		setMenuBar(menuBar);
	    }
	    Container bb = fd.installButtonBox(buttonBox, buttons, components);
	    if (bb!=buttonBox)
		setButtonBox(bb);
	    buttonManager.installButtons(buttons.elements());
	}
	public void uninstall() {
	    buttonManager.uninstallButtons(buttons.elements());
	    buttons = null;
	    for (Enumeration enum=components.elements(); enum.hasMoreElements();) {
		Component component = (Component)enum.nextElement();
		Container parent = component.getParent();
		if (Debug.DEBUG && parent==null)
		    throw new InternalError();
		parent.remove(component);
		if (parent.getComponentCount()==0 &&
		    component instanceof JComponent) {
		    JComponent jc = (JComponent)component;
		    Component menu = (Component)jc.getClientProperty(MenuDescription.MENU);
		    if (menu!=null) {
			Container mb = menu.getParent();
			mb.remove(menu);
			mb.validate();
		    }
		}
		parent.validate();
	    }
	    components = null;
	}
    }
    /***properties********************************/
    private JToolBar toolBar;
    private JMenuBar menuBar;
    private Container buttonBox;
    private Component component, south;
    private ButtonManager buttonManager = new ButtonManager();
    private PanelContent content;
    private Container container;
    private Hashtable frameDescriptions = new Hashtable();
    /***Constructor**********************************/
    protected void setContainer(Container _c) {
	if (Debug.DEBUG && (toolBar!=null || component!=null))
	    throw new InternalError();
	container = _c;
    }
    /***Internal***************************************/
    protected void setComponent(Component c) {
	if (component!=null)
	    container.remove(component);
	component = c;
	if (component!=null)
	    container.add("Center", component);
	container.validate();
    }
    public void setSouth(Component _s) {
	if (south!=null)
	    container.remove(south);
	south = _s;
	if (south!=null)
	    container.add("South", south);
	container.validate();
    }
    protected PanelContent getContent() { return content; }
    protected void setToolBar(JToolBar tb) {
	if (toolBar!=null)
	    throw new InternalError();
	toolBar = tb;
	container.add("North", toolBar);
	container.validate();
    }
    protected abstract void setMenuBar(JMenuBar mb);
    protected void setButtonBox(Container _bb) {
	buttonBox = _bb;
	setSouth(buttonBox);
    }
    protected JToolBar getToolBar() { return toolBar; }
    protected JMenuBar getMenuBar() { return menuBar; }
    /***InterfacePanel******************************/
    public void addFrameDescription(FrameDescription fd) {
	frameDescriptions.put(fd, new InstalledFrameDescription(fd));
    }
    public void removeFrameDescription(FrameDescription fd) {
	InstalledFrameDescription ifd
	    = (InstalledFrameDescription)frameDescriptions.get(fd);
	ifd.uninstall();
	frameDescriptions.remove(fd);
    }
    public ButtonContainer getButtonContainer() {
	return this;
    }
    public ButtonManager getButtonManager() {
	return buttonManager;
    }
    public void setContent(PanelContent c) {
	content = c;
	setComponent(content==null?null:content.getComponent());
    }
    public abstract void setTitle(String s);
    public Frame getFrame() {
	for (Container c=container; c!=null; c=c.getParent())
	    if (c instanceof Frame)
		return (Frame)c;
	return null;
    }
    public abstract void show();
    /***ButtonContainer********************************/
    public ButtonDescription getButton(String name) {
	for (Enumeration enum=frameDescriptions.elements(); enum.hasMoreElements();) {
	    InstalledFrameDescription ifd = (InstalledFrameDescription)enum.nextElement();
	    FrameDescription fd = ifd.frameDescription;
	    ButtonDescription bd = fd.getButton(name);
	    if (bd!=null)
		return bd;
	}
	return null;
    }
}
