package com.make.swing;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.Action;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import com.make.util.Debug;
import com.make.util.ArrayEnumeration;

final public class ButtonManager {
    /***ActionLink************************************/
    final static class ActionLink implements PropertyChangeListener {
	/***properties*************************/
	protected Action action;
	protected Vector buttons = new Vector();
	/***Constructor**************************/
	public ActionLink(Action _a) { 
	    action = _a;
	    if (action!=null)
		action.addPropertyChangeListener(this);
	}
	public ActionLink(GenericButton _ab) { 
	    buttons.addElement(_ab);
	    _ab.setEnabled(false);
	}
	public void dispose() {
	    if (action!=null) {
		if (Debug.DEBUG)
		    System.err.println("BM.dispose: action="+action.getValue(action.NAME));
		uninstallAction();
	    }
	    if (Debug.DEBUG && !buttons.isEmpty()) {
		System.err.println("BM.dispose: buttons.size="+buttons.size());
		for (Enumeration enum=buttons.elements(); enum.hasMoreElements();)
		    System.err.println("  - bt="+enum.nextElement());
		buttons.removeAllElements();
	    }
	}
	/***bind*********************************/
	protected void bind(GenericButton ab) {
	    ab.addActionListener(action);
	    ab.setEnabled(action.isEnabled());
	}
	protected void unbind(GenericButton ab) {
	    ab.removeActionListener(action);
	    ab.setEnabled(false);
	}
	/***(un)install****************************/
	public void installAction(Action _a) {
	    if (Debug.DEBUG && action!=null)
		throw new InternalError("action is already installed: "+action.getValue(action.NAME));
	    action = _a;
	    for (Enumeration enum=buttons.elements(); enum.hasMoreElements();)
		bind((GenericButton)enum.nextElement());
	    action.addPropertyChangeListener(this);
	}
	public void uninstallAction() {
	    action.removePropertyChangeListener(this);
	    for (Enumeration enum=buttons.elements(); enum.hasMoreElements();)
		unbind((GenericButton)enum.nextElement());
	    action = null;
	}
	public void installButton(GenericButton ab) {
	    buttons.addElement(ab);
	    if (action!=null)
		bind(ab);
	    else
		ab.setEnabled(false);
	}
	public void uninstallButton(GenericButton ab) {
	    buttons.removeElement(ab);
	    if (action!=null)
		unbind(ab);
	    else
		ab.setEnabled(false);
	}
	/***Info*****************************************/
	public Action getAction() { return action; }
	/***PropertyChangeListener*************************/
        public void propertyChange(PropertyChangeEvent e) {
            String propertyName = e.getPropertyName();
            /*if (propertyName.equals(Action.NAME)) {
	      if (item instanceof JMenuItem) {
	      String text = (String)e.getNewValue();
	      item.setText(text);
	      }
	      } else */
	    if (propertyName.equals("enabled")) {
                boolean enabled = ((Boolean)e.getNewValue()).booleanValue();
		for (Enumeration enum=buttons.elements(); enum.hasMoreElements();)
		    ((GenericButton)enum.nextElement()).setEnabled(enabled);
            }
	    
	}
    }
    /***properties****************************/
    protected Hashtable actions = new Hashtable();
    /***Constructor*****************************/
    protected void finalize() {
	if (Debug.DEBUG)
	    for (Enumeration enum=actions.elements(); enum.hasMoreElements();)
		((ActionLink)enum.nextElement()).dispose();
    }
    /***Replace**********************************/
    public void installAction(Action a) {
	String name = (String)a.getValue(a.NAME);
	synchronized(actions) {
	    ActionLink al = (ActionLink)actions.get(name);
	    if (al==null) {
		al = new ActionLink(a);
		actions.put(name, al);
	    } else
		al.installAction(a);
	}
    }
    public void uninstallAction(Action a) {
	String name = (String)a.getValue(a.NAME);
	ActionLink al = (ActionLink)actions.get(a.getValue(a.NAME));
	if (Debug.DEBUG) {
	    if (al==null) 
		throw new InternalError("action "+a.getValue(a.NAME)+
					" not found");
	    if (al.action!=a)
		throw new InternalError("action mismatch");
	}
	al.uninstallAction();
    }
    public void installActions(Enumeration enum) {
	if (enum!=null)
	    while (enum.hasMoreElements())
		installAction((Action)enum.nextElement());
    }
    public void uninstallActions(Enumeration enum) {
	if (enum!=null)
	    while (enum.hasMoreElements())
		uninstallAction((Action)enum.nextElement());
    }
    public void installActions(Action[] a) {
	if (a!=null) 
	    for (int z=0; z<a.length; z++)
		installAction(a[z]);
    }
    public void uninstallActions(Action[] a) {
	if (a!=null)
	    for (int z=0; z<a.length; z++)
		uninstallAction(a[z]);
    }
    public void installButton(GenericButton b) {
	String cmd = b.getActionCommand();
	synchronized(actions) {
	    ActionLink al = (ActionLink)actions.get(cmd);
	    if (al==null) {
		al = new ActionLink(b);
		actions.put(cmd, al);
	    } else
		al.installButton(b);
	}
    }
    public void uninstallButton(GenericButton b) {
	ActionLink al = (ActionLink)actions.get(b.getActionCommand());
	al.uninstallButton(b);
    }
    public void installButtons(GenericButton[] b) {
	for (int z=0; z<b.length; z++)
	    installButton(b[z]);
    }
    public void uninstallButtons(GenericButton[] b) {
	for (int z=0; z<b.length; z++)
	    uninstallButton(b[z]);
    }
    public void installButtons(Enumeration enum) {
	while (enum.hasMoreElements())
	    installButton((GenericButton)enum.nextElement());
    }
    public void uninstallButtons(Enumeration enum) {
	while (enum.hasMoreElements())
	    uninstallButton((GenericButton)enum.nextElement());
    }
    /***Info************************************/
    public Action getAction(String cmd) {
	ActionLink al = (ActionLink)actions.get(cmd);
	return al==null?null:al.getAction();
    }
}
