package com.make.swing;

import java.util.Vector;
import java.awt.Frame;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import javax.swing.JDialog;
import javax.swing.JMenuBar;
import javax.swing.Action;
import javax.swing.AbstractAction;
import com.make.swing.bt.FrameDescription;

public class DialogInterfacePanel extends AbstractInterfacePanel {
    /***properties********************************/
    protected String title, fileName;
    private Frame frame;
    protected JDialog dialog;
    private WindowAdapter closingListener = new WindowAdapter() {
	public void windowClosing(WindowEvent evt) {
	    close();
	}
    };
    /***Constructor**********************************/
    public DialogInterfacePanel(Frame f) {
	frame = f;
	dialog = new JDialog(frame, true);
	getButtonManager().installActions(getActions());
	dialog.setDefaultCloseOperation(dialog.DO_NOTHING_ON_CLOSE);
	dialog.addWindowListener(closingListener);
	setContainer(dialog.getContentPane());
    }
    /***Internal***************************************/
    public boolean close() {
	PanelContent content = getContent();
	if (content==null || content.panelClosing()) {
	    dialog.removeWindowListener(closingListener);
	    dialog.dispose(); 
	    if (content!=null)
		content.panelClosed();
	    getButtonManager().uninstallActions(getActions());
	    return true;
	} else
	    return false;
    }
    protected void updateTitle() {
	if (fileName==null)
	    dialog.setTitle(title);
	else if (title==null)
	    dialog.setTitle(fileName);
	else
	    dialog.setTitle(fileName+" - "+title);
    }
    public void setMenuBar(JMenuBar mb) {
	dialog.setJMenuBar(mb);
    }
    /***InterfacePanel******************************/
    public void addFrameDescription(FrameDescription fd) {
	super.addFrameDescription(fd);
	String t = fd.getTitle();
	if (t!=null) {
	    title = t;
	    updateTitle();
	}
    }
    public void setTitle(String s) { 
	fileName = s; updateTitle();
    }
    public Frame getFrame() { return frame; }
    public void show() {
	dialog.pack();
	dialog.show();
    }
    /***Actions*******************************************/
    protected Action[] getActions() {
	return defaultActions;
    }
    protected Action[] defaultActions = {
	new CloseFrameAction(),
    };
    class CloseFrameAction extends AbstractAction {
	CloseFrameAction() { super("close-frame"); }
        public void actionPerformed(ActionEvent e) {
	    close();
	}
    }
}
