package com.make.swing;

import java.util.Vector;
import java.awt.Frame;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.Action;
import javax.swing.AbstractAction;
import com.make.util.Debug;
import com.make.swing.bt.FrameDescription;

public class FrameInterfacePanel extends AbstractInterfacePanel {
    /***properties********************************/
    protected String title, fileName;
    protected JFrame frame = new JFrame();
    protected int width=500, height=600;
    private WindowAdapter closingListener = new WindowAdapter() {
	public void windowClosing(WindowEvent evt) {
	    close();
	}
    };
    /***Constructor**********************************/
    public FrameInterfacePanel(/*int _w, int _h*/) {
	//width = _w; height = _h;
	getButtonManager().installActions(getActions());
	frame.setDefaultCloseOperation(frame.DO_NOTHING_ON_CLOSE);
	frame.addWindowListener(closingListener);
	MainWindowManager.registerFrame(frame);
	setContainer(frame.getContentPane());
	if (Debug.LEAK_DEBUG)
	    System.err.println("+fip "+this);
    }
    protected void finalize() throws Throwable {
	super.finalize();
	if (Debug.LEAK_DEBUG)
	    System.err.println("-fip "+this);
    }
    /***Internal***************************************/
    public boolean close() {
	PanelContent content = getContent();
	if (content==null || content.panelClosing()) {
	    frame.removeWindowListener(closingListener);
	    frame.dispose();
	    if (content!=null)
		content.panelClosed();
	    getButtonManager().uninstallActions(getActions());
	    if (Debug.DEBUG) {
		setContent(null);
		setSouth(null);
	    }
	    return true;
	} else
	    return false;
    }
    protected void setMenuBar(JMenuBar mb) {
	frame.setJMenuBar(mb);
    }
    protected void updateTitle() {
	if (fileName==null)
	    frame.setTitle(title);
	else if (title==null)
	    frame.setTitle(fileName);
	else
	    frame.setTitle(fileName+" - "+title);
    }
    /***InterfacePanel******************************/
    public void addFrameDescription(FrameDescription fd) {
	super.addFrameDescription(fd);
	String t = fd.getTitle();
	if (t!=null) {
	    title = t;
	    updateTitle();
	}
    }
    public void setTitle(String s) { 
	fileName = s; updateTitle();
    }
    public Frame getFrame() { return frame; }
    public void show() {
	frame.getRootPane().setPreferredSize(new java.awt.Dimension(width, height));
	frame.pack();
	frame.show();
    }
    /***Actions*******************************************/
    protected Action[] getActions() {
	return defaultActions;
    }
    protected Action[] defaultActions = {
	new CloseFrameAction(),
	new ShowToolBarAction(),
    };
    class CloseFrameAction extends AbstractAction {
	CloseFrameAction() { super("close-frame"); }
        public void actionPerformed(ActionEvent e) {
	    close();
	}
    }
    class ShowToolBarAction extends AbstractAction {
	ShowToolBarAction() { super("show-toolbar"); }
        public void actionPerformed(ActionEvent e) {
	    Component c = getToolBar();
	    if (c!=null)
		c.setVisible(!c.isVisible());
	}
    }
}
