package com.make.swing;

import java.awt.Frame;
import java.awt.Component;
import java.awt.Container;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.util.ResourceBundle;
import javax.swing.Action;
import javax.swing.AbstractAction;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JOptionPane;
import javax.swing.JRootPane;
import javax.swing.SwingUtilities;
import javax.swing.event.AncestorListener;
import javax.swing.event.AncestorEvent;
import com.make.swing.field.Field;

public abstract class MakeDialog extends JDialog {
    protected class ConfirmAction extends AbstractAction implements Runnable {
	public ConfirmAction() { super("ok"); }
	public void actionPerformed(ActionEvent evt) {
	    if (inButton) return;
	    inButton = true;
	    new Thread(this, "confirm").start();
	}
	public void run() {
	    try {
		if (applyChanges())
		    dispose();
	    } finally {
		inButton = false;
	    }
	}
    }
    protected class ApplyAction extends AbstractAction implements Runnable {
	public ApplyAction() { super("apply"); }
	public void actionPerformed(ActionEvent evt) {
	    if (inButton) return;
	    inButton = true;
	    new Thread(this, "apply").start();
	}
	public void run() {
	    try {
		if (applyChanges() && type==T_EDIT_NEW)
		    type = T_EDIT;
	    } finally {
		inButton = false;
	    }
	}
    }
    protected class CancelAction extends AbstractAction {
	public CancelAction() { super("cancel"); }
	public void actionPerformed(ActionEvent evt) {
	    if (inButton) return;
	    dispose();
	}
    }
    /***CONST********************************/
    protected static final int T_VIEW = 1;
    protected static final int T_EDIT = 2;
    protected static final int T_EDIT_NEW = 3;
    protected static final int T_ACTION = 4;
    /***static******************************/
    private static final ResourceBundle resources
	= ResourceBundle.getBundle("com.make.swing.Dialog");
    public static final GridBagConstraints labelConstraints = new GridBagConstraints();
    public static final GridBagConstraints fieldConstraints = new GridBagConstraints();
    public static final GridBagConstraints verticalConstraints = new GridBagConstraints();
    public static final GridBagLayout gridBagLayout = new GridBagLayout();
    static {
	labelConstraints.fill = labelConstraints.HORIZONTAL;
	fieldConstraints.fill = fieldConstraints.BOTH;
	fieldConstraints.gridwidth = fieldConstraints.REMAINDER;
	fieldConstraints.weightx = 1.0;
	verticalConstraints.fill = fieldConstraints.BOTH;
	verticalConstraints.gridwidth = fieldConstraints.REMAINDER;
	verticalConstraints.weightx = 1.0;
	verticalConstraints.weighty = 1.0;
    }
    /***properties***************************/
    protected Frame frame;
    protected boolean readOnly, isNew, inButton = false;
    protected int type;
    private final Action[] actions = {
	new ConfirmAction(),
	new ApplyAction(),
	new CancelAction(),
    };
    private final ButtonManager buttonManager = new ButtonManager();
    private GenericButton[] buttons;
    /***Constructor*************************/
    protected MakeDialog(Frame _f, String title, boolean _ro, boolean _n) {
	super(_f, title, true);
	frame = _f; readOnly = _ro; isNew = _n;
	type = readOnly?T_VIEW:(isNew?T_EDIT_NEW:T_EDIT);
	setDefaultCloseOperation(DISPOSE_ON_CLOSE);
	buttonManager.installActions(actions);
    }
    protected MakeDialog(Frame _f, String title, int _t) {
	super(_f, title, true);
	frame = _f; type = _t;
	readOnly = type==T_VIEW;
	isNew = type==T_EDIT_NEW;
	setDefaultCloseOperation(DISPOSE_ON_CLOSE);
	buttonManager.installActions(actions);
    }
    protected void finalize() throws Throwable {
	buttonManager.uninstallButtons(buttons);
	buttonManager.uninstallActions(actions);
	super.finalize();
    }
    /***Info*********************************/
    public boolean isReadOnly() { return readOnly; }
    public boolean isNew() { return isNew; }
    public Frame getFrame() { return frame; }
    public void setNew(boolean _n) { isNew = _n; }
    public ButtonManager getButtonManager() { return buttonManager; }
    /***Data************************************/
    protected boolean applyChanges() { 
	return type==T_VIEW;
    }
    /***Form************************************/
    private void internalSetDefaultButton(JButton bt) {
	bt.addAncestorListener(new AncestorListener() {
	    public void ancestorAdded(AncestorEvent e) { 
		JButton defaultButton = (JButton)e.getComponent();
		JRootPane root = SwingUtilities.getRootPane(defaultButton);
		if (root!=null)
		    root.setDefaultButton(defaultButton);
	    }
	    public void ancestorRemoved(AncestorEvent event) {}
	    public void ancestorMoved(AncestorEvent event) {}
	});
    }
    protected Container createButtonBox() {
	Box box = Box.createHorizontalBox();
	box.add(Box.createHorizontalGlue());
	switch (type) {
	case T_VIEW: {
	    JButton ok = new JButton(resources.getString("close.label"));
	    ok.setActionCommand("ok");
	    internalSetDefaultButton(ok);
	    box.add(ok);
	    buttons = new GenericButton[] { 
		new SwingButton(ok), 
	    };
	    break;
	}
	case T_EDIT: case T_EDIT_NEW: {
	    JButton ok = new JButton(resources.getString("ok.label"));
	    JButton apply = new JButton(resources.getString("apply.label"));
	    JButton cancel = new JButton(resources.getString("cancel.label"));
	    ok.setActionCommand("ok");
	    apply.setActionCommand("apply");
	    cancel.setActionCommand("cancel");
	    internalSetDefaultButton(ok);
	    box.add(ok);
	    box.add(apply);
	    box.add(cancel);
	    buttons = new GenericButton[] { 
		new SwingButton(ok), 
		new SwingButton(apply), 
		new SwingButton(cancel), 
	    };
	    break;
	}
	case T_ACTION: {
	    JButton ok = new JButton(resources.getString("ok.label"));
	    JButton cancel = new JButton(resources.getString("cancel.label"));
	    ok.setActionCommand("ok");
	    cancel.setActionCommand("cancel");
	    internalSetDefaultButton(ok);
	    box.add(ok);
	    box.add(cancel);
	    buttons = new GenericButton[] { 
		new SwingButton(ok), 
		new SwingButton(cancel), 
	    };
	    break;
	}
	default:
	    throw new InternalError();
	}
	buttonManager.installButtons(buttons);
	return box;
    }
    public static void addField(Container parent, String _label, Component c) {
	JLabel label = new JLabel(_label);
	parent.add(label, labelConstraints);
	parent.add(c, fieldConstraints);
	label.setLabelFor(c);
    }
    public static void addField(Container parent, String _label, Field f) {
	addField(parent, _label, f.getComponent());
    }
    public static void addDisabledText(Container parent, String _label, String _v) {
	JTextField tf = new JTextField(_v);
	tf.setEnabled(false);
	addField(parent, _label, tf);
    }
    /***Message*****************************************/
    public void errorMsg(String msg, String title) {
	JOptionPane.showMessageDialog(frame, msg, title,
				      JOptionPane.ERROR_MESSAGE);
    }
    public void inputErrorMsg(String msg) {
	errorMsg(msg, "Eingabefehler");
    }
}
