package com.make.swing;

import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.DateFormat;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JButton;
import com.make.util.Debug;

public class ThreadWindow implements Runnable, ActionListener {
    /***properties***************************/
    protected Thread job, thread;
    protected JFrame frame;
    protected JLabel label;
    protected String text, title;
    protected int wait;
    protected boolean hasCancel, threadOk=false;
    /***Constructor**************************/
    public ThreadWindow(String _text, String _title, int _wait, 
			boolean _cancel) {
	text = _text; title = _title; wait = _wait; 
	hasCancel = _cancel;
	job = Thread.currentThread();
	if (wait==0) {
	    createFrame();
	    positionFrame();
	} else {
	    thread = new Thread(this, "threadWindow "+title);
	    thread.setDaemon(true);
	    thread.start();
	}
    }
    /***Interface**********************/
    public void close() {
	job = null;
	if (thread!=null) {
	    if (thread.isAlive())
		thread.stop();
	    /*if (threadOk)
	      while (thread!=null);
	      else*/
	    thread = null;
	}
	if (frame!=null) {
	    frame.dispose(); frame = null;
	}
    }
    /***Internal*****************************/
    protected void createFrame() {
	frame = new JFrame(title);
	frame.setDefaultCloseOperation(frame.DO_NOTHING_ON_CLOSE);
	JPanel panel = new JPanel(new BorderLayout(), false);
	panel.setBorder(javax.swing.BorderFactory.createEmptyBorder(10,10,10,10));
	label = new JLabel(text); text = null;
	panel.add("Center", label);
	if (hasCancel) {
	    JButton cancel = new JButton("Abbrechen");
	    cancel.addActionListener(this);
	    panel.add("East", cancel);
	}
	frame.getContentPane().add("Center", panel);
	frame.pack();
    }
    protected void positionFrame() {
	Dimension screen = frame.getToolkit().getScreenSize();
	Dimension size = frame.getSize();
	frame.setLocation((screen.width-size.width)/2, 
			  (screen.height-size.height)/2);
	frame.show();
    }
    /***ActionListener***********************/
    public void actionPerformed(ActionEvent evt) {
	if (thread!=null) {
	    if (thread.isAlive())
		thread.stop();
	    while (thread!=null);
	}
	if (job!=null)
	    job.stop();
    }
    /***Runnable****************************/
    public void run() {
	try {
	    threadOk = true;
	    if (job==null  || !job.isAlive()) return;
	    try {
		job.join(wait);
	    } catch (InterruptedException e) {
	    }
	    if (job==null || !job.isAlive()) return;
	    createFrame();
	    positionFrame();
	} finally {
	    threadOk = false;
	    thread = null;
	}
    }
}
