package com.make.swing.bt;

import java.net.URL;
import java.awt.Insets;
import javax.swing.KeyStroke;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.AbstractButton;
import javax.swing.JButton;
import javax.swing.JMenuItem;
import org.w3c.dom.Element;
import com.make.xml.DOMUtil;
import com.make.util.Strings;
import com.make.swing.ButtonManager;

public class ButtonDescription {
    public static final String NAME = "BD_NAME";
    /***properties****************************/
    protected String name, actionCommand, label, tooltip;
    protected char mnemonic = 0;
    protected KeyStroke accelerator;
    protected Icon icon;
    /***Constructor**************************/
    public ButtonDescription(Element vd) {
	name = vd.getAttribute("name");
	actionCommand = Strings.trim(vd.getAttribute("action"));
	if (actionCommand==null)
	    actionCommand = name;
	tooltip = DOMUtil.getLocalizedElementString(vd, "tooltip");
	if (tooltip==null)
	    tooltip = label;
	Element lbl = DOMUtil.getLocalizedUniqueTag(vd, "label");
	label = DOMUtil.getElementString(lbl);
	if (lbl!=null) {
	    String mn = lbl.getAttribute("mnemonic");
	    if (mn!=null && mn.length()>0)
		mnemonic = mn.charAt(0);
	}
	String accel = DOMUtil.getLocalizedElementString(vd, "accelerator");
	if (accel!=null)
	    accelerator = KeyStroke.getKeyStroke(accel);
	String iconString = DOMUtil.getElementString(vd, "icon");
	if (iconString!=null){
	    URL iconURL = getClass().getResource(iconString);
	    if (iconURL!=null)
		icon = new ImageIcon(iconURL);
	}
    }
    /***Info****************************/
    public String getName() { return name; }
    /***Factory*****************************/
    public AbstractButton createToolBarButton() {
        JButton b = new JButton() {
	    public boolean isFocusTraversable() { return false; }
	};
	if (icon==null) {
	    b.setText(label);
	    if (mnemonic!=0)
		b.setMnemonic(mnemonic);
	} else
	    b.setIcon(icon);
        b.setRequestFocusEnabled(false);
        b.setMargin(new Insets(1,1,1,1));
	b.setActionCommand(actionCommand);
	b.setEnabled(false);
	if (tooltip!=null)
	    b.setToolTipText(tooltip);
	b.putClientProperty(NAME, name);
	return b;
    }
    public AbstractButton createButton() {
	JButton b = new JButton(label);
	b.setActionCommand(actionCommand);
	b.setEnabled(false);
	if (tooltip!=null)
	    b.setToolTipText(tooltip);
	b.putClientProperty(NAME, name);
	return b;
    }
    public AbstractButton createMenuItem() {
	JMenuItem mi = new JMenuItem(label, icon);
	if (mnemonic!=0)
	    mi.setMnemonic(mnemonic);
	mi.setActionCommand(actionCommand);
	mi.setEnabled(false);
	if (tooltip!=null)
	    mi.setToolTipText(tooltip);
	if (accelerator!=null)
	    mi.setAccelerator(accelerator);
	mi.putClientProperty(NAME, name);
	return mi;
    }
}
