package com.make.swing.bt;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.util.Hashtable;
import java.util.Vector;
import javax.swing.JComponent;
import javax.swing.JToolBar;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.AbstractButton;
import javax.swing.JButton;
import javax.swing.JSeparator;
import javax.swing.Box;
import org.w3c.dom.Element;
import com.make.xml.DOMUtil;
import com.make.util.Strings;
import com.make.swing.SwingButton;

public class FrameDescription implements ButtonContainer {
    /***properties*************************/
    protected String title;
    protected Hashtable buttons = new Hashtable();
    protected ButtonDescription[] toolBar;
    protected ButtonDescription[] buttonBox;
    protected BarDescription[] menuBar;
    /***Constructor************************/
    public FrameDescription(Element d) {
	title = DOMUtil.getLocalizedElementString(d, "title");
	// buttons
	Element buts = DOMUtil.getUniqueTag(d, "buttons");
	if (buts!=null) {
	    Element[] bts = DOMUtil.getChildElements(buts);
	    for (int z=0; z<bts.length; z++) {
		ButtonDescription bd = new ButtonDescription(bts[z]);
		buttons.put(bd.getName(), bd);
	    }
	}
	// toolbar
	Element toolb = DOMUtil.getUniqueTag(d, "toolbar");
	if (toolb!=null) {
	    Element[] tbs = DOMUtil.getChildElements(toolb);
	    toolBar = new ButtonDescription[tbs.length];
	    for (int z=0; z<tbs.length; z++) {
		String tag = tbs[z].getTagName();
		if (tag.equals("buttonref")) {
		    String name = tbs[z].getAttribute("name");
		    toolBar[z] = getButton(name);
		} else if (!tag.equals("separator"))
		    System.err.println("WARNING: unknown toolbar button "+tag);
	    }
	}
	// buttonbox
	Element bb = DOMUtil.getUniqueTag(d, "buttonbox");
	if (bb!=null) {
	    Element[] tbs = DOMUtil.getChildElements(bb);
	    buttonBox = new ButtonDescription[tbs.length];
	    for (int z=0; z<tbs.length; z++) {
		String tag = tbs[z].getTagName();
		if (tag.equals("buttonref")) {
		    String name = tbs[z].getAttribute("name");
		    buttonBox[z] = getButton(name);
		} else
		    System.err.println("WARNING: unknown buttonBox button "+tag);
	    }
	}
	// menubar
	Element mbar = DOMUtil.getUniqueTag(d, "menubar");
	if (mbar!=null) {    
	    Element[] mbs = DOMUtil.getChildElements(mbar);
	    menuBar = new BarDescription[mbs.length];
	    for (int z=0; z<mbs.length; z++) {
		String tag = mbs[z].getTagName();
		if (tag.equals("menu"))
		    menuBar[z] = new MenuDescription(this, mbs[z]);
		else
		    System.err.println("WARNING: unknown menu container "+tag);
	    }
	}
    }
    /***Info*****************************************/
    public String getTitle() { return title; }
    /***ButtonContainer******************************/
    public ButtonDescription getButton(String name) {
	return (ButtonDescription)buttons.get(name);
    }
    /***Install***************************************/
    public JToolBar installToolBar(JToolBar tb, Vector bts, Vector cps) {
	if (toolBar==null)
	    return tb;
	Component last;
	if (tb==null) {
	    tb = new JToolBar();
	    last = Box.createGlue();
	} else {
	    last = tb.getComponent(tb.getComponentCount()-1);
	    tb.remove(last);
	}
	for (int z=0; z<toolBar.length; z++) {
	    if (toolBar[z]==null) {
		Component c = Box.createRigidArea(new Dimension(5,5));
		tb.add(c);
		cps.addElement(c);
	    } else {
		AbstractButton bt = toolBar[z].createToolBarButton();
		bts.addElement(new SwingButton(bt));
		tb.add(bt);
		cps.addElement(bt);
	    }
	}
	tb.add(last);
	tb.validate();
	return tb;
    }
    public Container installButtonBox(Container box, Vector bts, Vector cps) {
	if (buttonBox==null)
	    return box;
	if (box==null) {
	    box = Box.createHorizontalBox();
	    box.add(Box.createGlue());
	}
	for (int z=0; z<buttonBox.length; z++) 
	    if (buttonBox[z]!=null) {
		AbstractButton bt = buttonBox[z].createButton();
		bts.addElement(new SwingButton(bt));
		box.add(bt);
		cps.addElement(bt);
	    }
	box.validate();
	return box;
    }
    public JMenuBar installMenuBar(JMenuBar mb, Vector bts, Vector cps) {
	if (menuBar==null)
	    return mb;
	if (mb==null)
	    mb = new JMenuBar();
	for (int z=0; z<menuBar.length; z++)
	    if (menuBar[z]!=null)
		menuBar[z].install(mb, bts, cps);
	mb.validate();
	return mb;
    }
}
