package com.make.swing.field;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.util.Vector;
import java.text.ParseException;
import javax.swing.JComboBox;
import javax.swing.ListCellRenderer;
import javax.swing.JList;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListDataEvent;
import com.make.util.Strings;
import com.make.util.Debug;

public class ComboField extends DefaultField 
    implements ListCellRenderer {
    /***properties**************************/
    protected JComboBox component;
    protected Object[] list, translate;
    protected ListCellRenderer renderer;
    /***Constructor************************/
    public ComboField(Vector list, boolean _editable) {
	component = new JComboBox(list);
	component.addActionListener(this);
	component.setEditable(_editable);
	setComponent(component);
    }
    public ComboField(Object[] _list, Object[] _translate, boolean _editable) {
	list = _list; translate = _translate;
	if (translate!=null && translate.length==0)
	    translate = null;
	component = new JComboBox(translate==null?list:translate);
	component.addActionListener(this);
	component.setEditable(_editable);
	if (translate!=null) {
	    renderer = component.getRenderer();
	    component.setRenderer(this);
	}
	setComponent(component);
	if (Debug.DEBUG && translate!=null && translate.length!=list.length) {
	    System.err.println("WARNING: check length of translate list");
	    translate = null;
	}
    }
    public ComboField(Object[] _list, boolean _editable) {
	this(_list, null, _editable);
    }
    /***ListCellRenderer*************************/
    public Component getListCellRendererComponent(JList _list, Object value,
						  int index,
						  boolean isSelected,
						  boolean cellHasFocus) {
	if (translate!=null && value!=null)
	    value = Strings.translate(translate, list, value);
	return renderer.getListCellRendererComponent(_list, value, index,
						     isSelected, cellHasFocus);
    }
    /***Field*******************************/
    public void setValue(Object v) throws ParseException {
	if (v!=null && translate!=null) {
	    Object v2 = Strings.translate(translate, list, v);
	    if (v2==null)
		throw new ParseException(null, 0); //TODO
	}
	component.setSelectedItem(v);
    }
    public Object getValue() throws ParseException {
	return component.getSelectedItem();
    }
    /***ActionListener*********************/
    public void actionPerformed(ActionEvent evt) {
	fireValueChanged();
	super.actionPerformed(evt);
    }    
}
