package com.make.swing.field;

import java.awt.Component;
import java.awt.event.FocusListener;
import java.awt.event.FocusEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.text.ParseException;
import java.util.Vector;
import java.util.Enumeration;
import java.util.ResourceBundle;
import com.make.swing.ButtonManager;

public abstract class DefaultField 
    implements Field, FocusListener, ActionListener {
    /***static******************************/
    public static final ResourceBundle messages
	= ResourceBundle.getBundle("com.make.swing.field.ParseMessages");
    /***properties**************************/
    private Vector listeners = new Vector();
    private Component component;
    /***Constructor*************************/
    public DefaultField() {}
    public DefaultField(Component _c) {
	setComponent(_c);
    }
    /***DefaultField***********************/
    protected void setComponent(Component _c) {
	if (component!=null)
	    component.removeFocusListener(this);
	component = _c;
	if (component!=null)
	    component.addFocusListener(this);
    }
    public void fireValueChanged() {
	FieldEvent evt = new FieldEvent(this);
	for (Enumeration enum=listeners.elements(); enum.hasMoreElements();)
	    ((FieldListener)enum.nextElement()).valueChanged(evt);
    }
    public void fireFocusGained() {
	FieldEvent evt = new FieldEvent(this);
	for (Enumeration enum=listeners.elements(); enum.hasMoreElements();)
	    ((FieldListener)enum.nextElement()).focusGained(evt);
    }
    public void fireFocusLost() {
	FieldEvent evt = new FieldEvent(this);
	for (Enumeration enum=listeners.elements(); enum.hasMoreElements();)
	    ((FieldListener)enum.nextElement()).focusLost(evt);
    }
    public void fireActionPerformed() {
	FieldEvent evt = new FieldEvent(this);
	for (Enumeration enum=listeners.elements(); enum.hasMoreElements();)
	    ((FieldListener)enum.nextElement()).actionPerformed(evt);
    }
    /***Field*****************************/
    public Component getComponent() { return component; }
    public void install(ButtonManager bm) {}
    public void uninstall(ButtonManager bm) {}
    public void addFieldListener(FieldListener _l) {
	listeners.addElement(_l);
    }
    public void removeFieldListener(FieldListener _l) {
	listeners.removeElement(_l);
    }
    public void dispose() {
	setComponent(null);
	listeners = null;
    }
    /***FocusListener************************/
    public void focusGained(FocusEvent evt) {
	if (!evt.isTemporary())
	    fireFocusGained();
    }
    public void focusLost(FocusEvent evt) {
	if (!evt.isTemporary())
	    fireFocusLost();
    }
    /***ActionListener************************/
    public void actionPerformed(ActionEvent evt) {
	fireActionPerformed();
    }
}
