package com.make.swing.field;

import java.awt.Component;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.text.ParseException;
import java.util.Vector;
import java.util.Enumeration;
import javax.swing.JPanel;
import javax.swing.JMenu;
import javax.swing.JButton;
import javax.swing.JPopupMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.SwingConstants;
import javax.swing.plaf.basic.BasicArrowButton;
import com.make.util.Strings;
import com.make.xml.LookupTable;
import com.make.swing.ButtonManager;

public class MenuSelectField extends DefaultField 
    implements FieldListener {
    protected class MenuAction implements ActionListener, MouseListener {
	/***ActionListener************/
	public void actionPerformed(ActionEvent evt) {
	    try {
		setValue(evt.getActionCommand());
	    } catch (ParseException e) {
		JOptionPane.showMessageDialog(null, e.getLocalizedMessage(),
					      "Eingabefehler", JOptionPane.ERROR_MESSAGE);
	    }
	}
	/***MouseListener*************/
	public void mouseClicked(MouseEvent e) {}
	public void mousePressed(MouseEvent evt) {
	    menu.show((Component)evt.getSource(), evt.getX(), evt.getY());
	}
	public void mouseReleased(MouseEvent evt) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
    }
    /***properties**************************/
    protected JPanel panel = new JPanel(new BorderLayout(), false);
    protected LookupTable lookupTable;
    protected Field field;
    protected JPopupMenu menu = new JPopupMenu();
    protected MenuAction menuAction = new MenuAction();
    /***Constructor************************/
    public MenuSelectField(Field _f, LookupTable _lt, Object[] _m) {
	lookupTable = _lt; 
	field = _f;
	field.addFieldListener(this);
	panel.add("Center", field.getComponent());
	setComponent(panel);
	for (int z=0; z<_m.length; z++) {
	    JMenuItem mi = createMenuItem(_m[z]);
	    if (mi==null)
		menu.addSeparator();
	    else
		menu.add(mi);
	}
	JButton bt = new BasicArrowButton(SwingConstants.SOUTH);
	bt.addMouseListener(menuAction);
	panel.add("East", bt);
    }
    /***Internal*************************/
    protected JMenuItem createMenuItem(Object o) {
	if (o instanceof String) {
	    String code = (String)o;
	    Object caption = lookupTable.get(code);
	    if (caption==null) caption = code;
	    JMenuItem mi = new JMenuItem(caption.toString());
	    mi.setActionCommand(code);
	    mi.addActionListener(menuAction);
	    return mi;
	} else if (o instanceof Object[]) {
	    Object[] a = (Object[])o;
	    JMenu m = new JMenu((String)a[0]);
	    for (int z=1; z<a.length; z++) {
		JMenuItem mi = createMenuItem(a[z]);
		if (mi==null)
		    m.addSeparator();
		else
		    m.add(mi);
	    }
	    return m;
	} else
	    return null;
    }
    /***Field*******************************/
    public void install(ButtonManager bm) {
	super.install(bm);
	field.install(bm);
    }
    public void uninstall(ButtonManager bm) {
	super.uninstall(bm);
	field.uninstall(bm);
    }
    public void setValue(Object v) throws ParseException {
	field.setValue(v);
    }
    public Object getValue() throws ParseException {
	return field.getValue();
    }
    public void dispose() {
	panel = null;
	lookupTable = null;
	field.removeFieldListener(this);
	field.dispose();
	field = null;
	super.dispose();
    }
    /***FieldListener*************************/
    public void valueChanged(FieldEvent evt) {
	fireValueChanged();
    }
    public void focusGained(FieldEvent evt) {
	fireFocusGained();
    }
    public void focusLost(FieldEvent evt) {
	fireFocusLost();
    }
    public void actionPerformed(FieldEvent evt) {
	fireActionPerformed();
    }
}
