package com.make.swing.field;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.text.ParseException;
import javax.swing.JComboBox;
import javax.swing.ListCellRenderer;
import javax.swing.JList;
import com.make.util.Strings;
import com.make.util.Debug;
import com.make.xml.LookupTable;

public class SelectField extends DefaultField 
    implements ListCellRenderer {
    /***properties**************************/
    protected final Object nullValue = this;
    protected JComboBox component;
    protected LookupTable lookupTable;
    protected ListCellRenderer renderer;
    protected Object nullDisplay;
    /***Constructor************************/
    public SelectField(LookupTable _lt, boolean showNull) {
	lookupTable = _lt;
	Object[] list = lookupTable==null
	    ?new Object[0]
	    :lookupTable.getList();
	if (showNull) {
	    Object[] l2 = new Object[list.length+1];
	    l2[0] = nullValue;
	    System.arraycopy(list, 0, l2, 1, list.length);
	    list = l2;
	}
	component = new JComboBox(list);
	component.addActionListener(this);
	component.setEditable(false);
	renderer = component.getRenderer();
	component.setRenderer(this);
	setComponent(component);
    }
    public SelectField(LookupTable _lt, Object _nd) {
	this(_lt, true);
	nullDisplay = _nd;
    }
    public SelectField(LookupTable _lt) {
	this(_lt, false);
    }
    /***ListCellRenderer*************************/
    public Component getListCellRendererComponent(JList _list, Object value,
						  int index,
						  boolean isSelected,
						  boolean cellHasFocus) {
	if (value==nullValue)
	    value = null;
	Object vDisplay = value==null && nullDisplay!=null
	    ?nullDisplay
	    :lookupTable.get(value);
	if (vDisplay==null)
	    vDisplay = value;
	return renderer.getListCellRendererComponent
	    (_list, vDisplay, index,
	     isSelected, cellHasFocus);
    }
    /***Field*******************************/
    public void setValue(Object v) throws ParseException {
	if (v==null) v = nullValue;
	component.setSelectedItem(v);
    }
    public Object getValue() throws ParseException {
	Object v = component.getSelectedItem();
	return v==nullValue ? null : v;
    }
    /***ActionListener*********************/
    public void actionPerformed(ActionEvent evt) {
	fireValueChanged();
	super.actionPerformed(evt);
    }    
}
