package com.make.swing.field;

import java.awt.Component;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.util.Vector;
import java.util.Enumeration;
import java.util.Date;
import java.text.ParseException;
import java.text.DateFormat;
import javax.swing.JComboBox;
import javax.swing.JOptionPane;
import javax.swing.ComboBoxModel;
import javax.swing.ListCellRenderer;
import javax.swing.JList;
import javax.swing.SwingUtilities;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListDataEvent;
import com.make.util.Strings;
import com.make.util.Debug;

public class TimeOffsetComboField extends DefaultField
    implements ComboBoxModel, ListCellRenderer {
    /***properties**************************/
    protected JComboBox component;
    protected String[] list;
    protected int selected = -1;
    protected DateFormat displayDateFormat = Strings.displayDateFormats[Strings.DATETIME];
    protected DateFormat storeDateFormat = Strings.storeDateFormats[Strings.DATETIME];
    protected Date userValue = new Date();
    protected Vector listeners = new Vector();
    protected Object selectedItem;
    protected boolean inUserDialog = false;
    protected ListCellRenderer renderer;
    /***Constructor************************/
    public TimeOffsetComboField(String[] _list) {
	list = _list;
	component = new JComboBox(this);
	component.addActionListener(this);
	component.setEditable(false);
	renderer = component.getRenderer();
	component.setRenderer(this);
	setComponent(component);
    }
    /***Fire************************************/
    public void fireContentsChanged(int idx0, int idx1) {
	ListDataEvent evt = new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED,
					      idx0, idx1);
	for (Enumeration enum=listeners.elements(); enum.hasMoreElements();)
	    ((ListDataListener)enum.nextElement()).contentsChanged(evt);
    }
    /***ComboBoxModel***************************/
    public void setSelectedItem(Object anItem) {
	if (anItem==userValue) {
	    if (inUserDialog) return;
	    try {
		inUserDialog = true;
		String str = JOptionPane.showInputDialog("Bitte geben Sie ein Datum ein.");
		if (str==null)
		    return;
		try {
		    userValue = Strings.parseDateInput(str);
		} catch (ParseException e) {
		    JOptionPane.showMessageDialog
			(null, "Die Eingabe konnte nicht als Datum erkannt werden.");
		    return;
		}
		selectedItem = userValue;
		fireContentsChanged(-1, -1);
	    } finally {
		SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			inUserDialog = false;
		    }
		});
	    } 
	} else {
	    if (anItem instanceof String) {
		int z;
		for (z=0; z<list.length; z++)
		    if (list[z].equals(anItem))
			break;
		if (z==list.length) {
		    String[] ol = list;
		    list = new String[ol.length+1];
		    System.arraycopy(ol, 0, list, 0, ol.length);
		    list[ol.length] = (String)anItem;
		}
	    }
	    selectedItem = anItem;
	    fireContentsChanged(-1, -1);
	}
    }
    public Object getSelectedItem() {
	return selectedItem;
    }
    public int getSize() {
	return list.length+1;
    }
    public Object getElementAt(int index) {
	if (index==0)
	    return userValue;
	else
	    return list[index-1];
    }
    public void addListDataListener(ListDataListener l) {
	listeners.addElement(l);
    }
    public void removeListDataListener(ListDataListener l) {
	listeners.removeElement(l);
    }
    /***ListCellRenderer*************************/
    public Component getListCellRendererComponent(JList list, Object value,
						  int index,
						  boolean isSelected,
						  boolean cellHasFocus) {
	if (value instanceof Date)
	    value = index==-1
		?displayDateFormat.format((Date)value)
		:"Benutzerdefiniert";
 	else if (value instanceof String) {
	    String str = (String)value;
	    int idx = str.indexOf('d');
	    if (idx!=-1)
		try {
		    int days = Integer.parseInt(str.substring(1, idx));
		    if (days==1)
			value = "Ein Tag";
		    else
			value = days+" Tage";
		} catch (NumberFormatException e) {
		}
	}
	return renderer.getListCellRendererComponent(list, value, index,
						     isSelected, cellHasFocus);
    }
    /***Field*******************************/
    public void setValue(Object v) throws ParseException {
	String str = (String)v;
	Object si;
	if (str==null)
	    si = null;
	else
	    try {
		Strings.parseTimeSpan(str);
		si = str;
	    } catch (ParseException e) {
		si = storeDateFormat.parse(str);
	    }
	try {
	    inUserDialog = true;
	    setSelectedItem(si);
	} finally {
	    inUserDialog = false;
	}
    }
    public Object getValue() throws ParseException {
	if (selectedItem instanceof Date)
	    return storeDateFormat.format((Date)selectedItem);
	else
	    return selectedItem;
    }
    /***ActionListener*********************/
    public void actionPerformed(ActionEvent evt) {
	fireValueChanged();
	super.actionPerformed(evt);
    }    
}
