package com.make.swing.field;

import java.awt.Component;
import java.awt.event.ActionListener;
import java.text.ParseException;
import javax.swing.JTextField;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;
import com.make.util.Strings;
import com.make.swing.GenericButton;
import com.make.swing.ButtonManager;
import com.make.xml.LookupTable;

public class TranslateTextField extends TextField {
    /***properties**************************/
    protected LookupTable lookupTable;
    protected Object value, displayValue;
    protected boolean editing = false;
    protected boolean changed = false;
    /***Constructor************************/
    public TranslateTextField(int len, LookupTable _lt) {
	super(len);
	lookupTable = _lt;
    }
    public TranslateTextField(LookupTable _lt) {
	this(20, _lt);
    }
    /***DefaultField**********************/
    public void fireValueChanged() {
	super.fireValueChanged();
	changed = true;
    }
    public void fireFocusGained() {
	super.fireFocusGained();
	try {
	    editing = true;
	    super.setValue(value);
	    component.selectAll();
	} catch (ParseException e) {
	}
    }
    public void fireFocusLost() {
	super.fireFocusLost();
	try {
	    sync();
	    editing = false;
	    super.setValue(displayValue);
	} catch (ParseException e) {
	}
    }
    /***Internal****************************/
    protected void sync() {
	if (changed) {
	    if (editing)
		try {
		    value = super.getValue();
		    // TODO: upperCase raus
		    if (value instanceof String)
			value = ((String)value).toUpperCase();
		} catch (ParseException e) {
		    value = null;
		}
	    displayValue = lookupTable.get(value);
	    if (displayValue==null)
		displayValue = value;
	    changed = false;
	}
    }
    /***Field*******************************/
    public void setValue(Object v) throws ParseException {
	value = v;
	changed = true;
	if (editing) {
	    super.setValue(value);
	} else {
	    sync();
	    super.setValue(displayValue);
	}
    }
    public Object getValue() throws ParseException {
	sync();
	return value;
    }
}
