package com.make.swing.il;

import javax.swing.BoxLayout;
import org.w3c.dom.Element;
import com.make.datamodel.slot.Slot;
import com.make.datamodel.slot.SlotContainer;
import com.make.util.Strings;
import com.make.xml.DOMUtil;
import com.make.xml.LookupTable;
import com.make.swing.field.Field;
import com.make.swing.field.TextField;
import com.make.swing.field.TranslateTextField;
import com.make.swing.field.PasswordField;
import com.make.swing.field.IntegerField;
import com.make.swing.field.CheckBoxField;
import com.make.swing.field.DateField;
import com.make.swing.field.SelectField;
import com.make.swing.field.MenuSelectField;
import com.make.swing.field.TimeOffsetComboField;

public class DefaultInputLinkFactory implements InputLinkFactory {
    /***properties*****************************/
    protected SlotContainer slotContainer;
    /***Constructor******************************/
    public DefaultInputLinkFactory(SlotContainer _sc) {
	slotContainer = _sc;
    }
    /***Info********************************/
    public SlotContainer getSlotContainer() {
	return slotContainer;
    }
    /***Misc*************************************/
    protected Object getList(Element vd) {
	return null;
    }
    protected Slot getSlot(Element vd) {
	String path = Strings.trim(vd.getAttribute("path"));
	if (path==null)
	    path = DOMUtil.getElementString(vd, "path");
	if (path==null)
	    path = Strings.trim(vd.getAttribute("name"));
	return slotContainer.getSlot(path);
    }
    protected Object[] loadSelectMenu(Element vd) {
	Element[] ch = DOMUtil.getChildElements(vd);
	String caption = Strings.trim(vd.getAttribute("caption"));
	final int cc = caption==null?0:1;
	Object[] ret = new Object[ch.length+cc];
	if (caption!=null)
	    ret[0] = caption;
	for (int z=0; z<ch.length; z++)
	    ret[z+cc] = loadSelectMenuItem(ch[z]);
	return ret;
    }
    protected Object loadSelectMenuItem(Element vd) {
	String type = vd.getTagName();
	if (type.equals("menu"))
	    return loadSelectMenu(vd);
	else if (type.equals("item"))
	    return DOMUtil.getElementString(vd);
	else if (type.equals("separator"))
	    return null;
	else {
	    System.err.println("WARNING: unknown menu item: "+type);
	    return null;
	}
    }
    protected Field createField(Element vd) {
	String type = getType(vd);
	if (type.equals("text"))
	    return new TextField();
	if (type.equals("translatetext")) {
	    LookupTable lt = (LookupTable)getList(vd);
	    return new TranslateTextField(lt);
	}
	if (type.equals("menuselect")) {
	    LookupTable lt = (LookupTable)getList(vd);
	    Object[] menu = loadSelectMenu(DOMUtil.getUniqueTag(vd, "menu"));
	    Element fvd = DOMUtil.getUniqueTag(vd, "field");
	    Field f = createField(DOMUtil.getChildElements(fvd)[0]);
	    return new MenuSelectField(f, lt, menu);
	}
	if (type.equals("password"))
	    return new PasswordField();
	if (type.equals("integer"))
	    return new IntegerField(true);
	if (type.equals("select")) {
	    LookupTable lt = (LookupTable)getList(vd);
	    String nd = Strings.trim(vd.getAttribute("nullDisplay"));
	    if (nd!=null)
		return new SelectField(lt, nd);
	    return new SelectField(lt, "yes".equals(vd.getAttribute("nullable")));
	} 
	if (type.equals("checkbox")) {
	    String caption = Strings.trim(vd.getAttribute("caption"));
	    if (caption==null)
		caption = DOMUtil.getLocalizedElementString(vd, "caption");
	    return new CheckBoxField(caption, true);
	}
	if (type.equals("datetime"))
	    return new DateField(true, 0);
	if (type.equals("date"))
	    return new DateField(true, 1);
	if (type.equals("time"))
	    return new DateField(true, 2);
	if (type.equals("timeoffset")) {
	    String[] list = (String[])getList(vd);
	    return new TimeOffsetComboField(list);
	}
	return null;
    }
    public String getType(Element vd) {
	return vd.getTagName();
    }
    /***InputLinkFactory**************************/
    public InputLink createInputLink(Element vd) {
	String type = getType(vd);
	if (type.equals("label"))
	    return new Label(DOMUtil.getElementString(vd));
	if (type.equals("glue"))
	    return new Glue();
	if (type.equals("panel")) {
	    Element fs = DOMUtil.getUniqueTag(vd, "fields");
	    if (fs==null) fs = vd;
	    return new PanelInputLink(vd, DOMUtil.getChildElements(fs), this);
	} 
	if (type.equals("grid")) {
	    Element fs = DOMUtil.getUniqueTag(vd, "fields");
	    if (fs==null) fs = vd;
	    return new GridInputLink(vd, DOMUtil.getChildElements(fs), this);
	} 
	if (type.equals("vbox")) {
	    Element fs = DOMUtil.getUniqueTag(vd, "fields");
	    if (fs==null) fs = vd;
	    return new BoxInputLink(vd, BoxLayout.Y_AXIS,
				    DOMUtil.getChildElements(fs), this);
	} 
	if (type.equals("hbox")) {
	    Element fs = DOMUtil.getUniqueTag(vd, "fields");
	    if (fs==null) fs = vd;
	    return new BoxInputLink(vd, BoxLayout.X_AXIS,
				    DOMUtil.getChildElements(fs), this);
	} 
	if (type.equals("dialog"))
	    return new DialogInputLink(vd, this);
	Field fld = createField(vd);
	if (fld!=null)
	    return new FieldInputLink(vd, getSlot(vd), fld);
	return null;
    }
}
