package com.make.swing.il;

import java.awt.Component;
import java.awt.Frame;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Enumeration;
import java.text.ParseException;
import org.w3c.dom.Element;
import javax.swing.JComponent;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.Box;
import javax.swing.SwingUtilities;
import javax.swing.BorderFactory;
import com.make.xml.DOMUtil;
import com.make.util.Strings;

public class DialogInputLink extends PanelInputLink
    implements ActionListener {
    /***properties**************************/
    protected Element vd;
    protected InputLinkFactory inputLinkFactory;
    protected JButton button;
    protected JDialog dialog;
    protected Frame frame;
    /***Constructor*************************/
    public DialogInputLink(Element _vd, InputLinkFactory _ilf) { 
	super(_vd, DOMUtil.getChildElements(DOMUtil.getUniqueTag(_vd, "fields")), _ilf);
	vd = _vd; inputLinkFactory = _ilf;
	// button
	String caption = Strings.trim(vd.getAttribute("caption"));
	char mn = 0;
	if (caption==null) {
	    Element cap = DOMUtil.getLocalizedUniqueTag(vd, "caption");
	    caption = DOMUtil.getElementString(cap);
	    String m = Strings.trim(cap.getAttribute("mnemonic"));
	    if (m!=null)
		mn = m.charAt(0);
	}
	button = new JButton(caption);
	if (mn>0)
	    button.setMnemonic(mn);
	button.setActionCommand("dlg");
	button.addActionListener(this);
    }
    protected static JDialog createDialog(Frame frame, ActionListener al,
					  Component component,
					  Element vd, InputLinkFactory ilf) {
	// dialog
	String title = Strings.trim(vd.getAttribute("title"));
	if (title==null)
	    title = DOMUtil.getLocalizedElementString(vd, "title");
	JDialog dialog = new JDialog(frame, title, true);
	// buttons
	JButton ok = new JButton("OK");
	ok.setActionCommand("ok");
	ok.addActionListener(al);
	JButton apply = new JButton("\u00dcbernehmen");
	apply.setActionCommand("apply");
	apply.addActionListener(al);
	JButton cancel = new JButton("Abbrechen");
	cancel.setActionCommand("cancel");
	cancel.addActionListener(al);
	Box box = Box.createHorizontalBox();
	box.add(Box.createHorizontalGlue());
	box.add(ok);
	box.add(apply);
	box.add(cancel);
	dialog.getContentPane().add("South", box);
	dialog.getRootPane().setDefaultButton(ok);
	// fields
	dialog.getContentPane().add("Center", component);
	((JComponent)component).setBorder
	    (BorderFactory.createEmptyBorder(0, 0, 3, 0));
	// borders
	((JComponent)dialog.getContentPane()).setBorder
	    (BorderFactory.createEmptyBorder(5, 5, 5, 5));
	dialog.setResizable(false);
	dialog.pack();
	// return
	return dialog;
    }
    protected boolean applyInput() {
	try {
	    super.checkInput();
	    super.apply();
	    return true;
	} catch (ParseException e) {
	    JOptionPane.showMessageDialog
		(frame, e.getLocalizedMessage(), 
		 "Eingabefehler", JOptionPane.ERROR_MESSAGE);
	    return false;
	}
    }
    /***ActionListener************************/
    public void actionPerformed(ActionEvent evt) {
	if (evt.getActionCommand().equals("dlg")) {
	    if (dialog==null) {
		Component root = SwingUtilities.getRoot(button);
		frame = root instanceof Frame ? (Frame)root : null;
		dialog = createDialog(frame, this, super.getComponent(),
				      vd, inputLinkFactory);
		vd = null; inputLinkFactory = null;
	    }
	    super.reload();
	    dialog.show();
	} else if (evt.getActionCommand().equals("ok")) {
	    if (applyInput())
		dialog.setVisible(false);
	    else
		super.reload();
	} else if (evt.getActionCommand().equals("apply")) {
	    applyInput();
	} else if (evt.getActionCommand().equals("cancel")) {
	    super.reload();
	    dialog.setVisible(false);
	}
    }
    /***InputLink*************************/
    public Component getComponent() { return button; }
    public void reload() {}
    public void apply() throws ParseException {}
    public void checkInput() throws ParseException {}
    public void dispose() {
	if (dialog!=null) {
	    dialog.dispose();
	    dialog = null;
	}
	super.dispose();
    }
    /***CompositeInputLink****************/
    public Enumeration getInputLinks() {
	return new Enumeration() {
		public boolean hasMoreElements() { return false; }
		public Object nextElement() { throw new InternalError(); }
	    };
    }
}
