package com.make.swing.text;

import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.Point;
import java.awt.Font;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.util.Hashtable;
import javax.swing.JComponent;
import javax.swing.Action;
import javax.swing.border.Border;
import javax.swing.UIManager;
import javax.swing.JPopupMenu;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.plaf.UIResource;
import javax.swing.plaf.ComponentUI;
import javax.swing.text.Position;
import javax.swing.text.BadLocationException;
import com.make.swing.ButtonManager;
import com.make.swing.KeyStrokeButton;
import com.make.util.Debug;
import com.make.swing.text.view.View;
import com.make.swing.text.view.ViewFactory;
import com.make.swing.text.view.Highlighter;
import com.make.swing.text.view.DefaultHighlighter;

public class BasicTextUI extends TextUI {
    protected class MouseHandler extends MouseAdapter {
	public void mousePressed(MouseEvent evt) {
	    if (evt.isPopupTrigger() || 
		((evt.isAltDown() || evt.isMetaDown()) && evt.getClickCount()==1) ||
		(evt.getModifiers() & (evt.BUTTON2_MASK|evt.BUTTON3_MASK))!=0) {
		Document document = editor.getDocument();
		if (document==null) return;
		final Point pt = evt.getPoint();
		final int pos = viewToModel(editor, pt);
		editor.getCaret().setDot(pos);
		//ButtonManager bm = editorFrame.getButtonManager();
		JPopupMenu menu = new JPopupMenu();
		/*menu.add(MenuBarFactory.createMenuItem(bm.getAction("cut-to-clipboard")));
		  menu.add(MenuBarFactory.createMenuItem(bm.getAction("copy-to-clipboard")));
		  menu.add(MenuBarFactory.createMenuItem(bm.getAction("paste-from-clipboard")));*/
		menu.addSeparator();
		try {
		    document.getRootElement().fillMenu(editor, 0, pos, 
						       Position.Bias.Backward, menu);
		} catch (BadLocationException e) {
		    e.printStackTrace();
		}
		menu.show(editor, pt.x, pt.y);
		evt.consume();
	    }
	}
    }
    /***CONST*************************************/
    protected static final Object[] keyBindings = {
        KeyStroke.getKeyStroke(KeyEvent.VK_BACK_SPACE, 0),
	EditorKit.deletePrevCharAction,
        KeyStroke.getKeyStroke(KeyEvent.VK_BACK_SPACE, InputEvent.SHIFT_MASK),
	EditorKit.deletePrevCharAction,
        KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0),
	EditorKit.deleteNextCharAction,
        KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, 0),
	EditorKit.forwardAction,
        KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, 0),
	EditorKit.backwardAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_MASK), 
	EditorKit.copyAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_V, InputEvent.CTRL_MASK), 
	EditorKit.pasteAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_X, InputEvent.CTRL_MASK),
	EditorKit.cutAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, InputEvent.SHIFT_MASK),
	EditorKit.selectionBackwardAction,
	KeyStroke.getKeyStroke("shift KP_LEFT"), 
	EditorKit.selectionBackwardAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, InputEvent.SHIFT_MASK),
	EditorKit.selectionForwardAction,
	KeyStroke.getKeyStroke("shift KP_RIGHT"), 
	EditorKit.selectionForwardAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, InputEvent.CTRL_MASK),
	EditorKit.previousWordAction,
	KeyStroke.getKeyStroke("control KP_LEFT"), 
	EditorKit.previousWordAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, InputEvent.CTRL_MASK),
	EditorKit.nextWordAction,
	KeyStroke.getKeyStroke("control KP_RIGHT"), 
	EditorKit.nextWordAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_LEFT, InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK),
	EditorKit.selectionPreviousWordAction,
	KeyStroke.getKeyStroke("control shift KP_LEFT"), 
	EditorKit.selectionPreviousWordAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT, InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK),
	EditorKit.selectionNextWordAction,
	KeyStroke.getKeyStroke("control shift KP_RIGHT"), 
	EditorKit.selectionNextWordAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_A, InputEvent.CTRL_MASK),
	EditorKit.selectAllAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_HOME, 0),
	EditorKit.beginLineAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_END, 0),
	EditorKit.endLineAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_HOME, InputEvent.SHIFT_MASK),
	EditorKit.selectionBeginLineAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_END, InputEvent.SHIFT_MASK),
	EditorKit.selectionEndLineAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_UP, 0),
	EditorKit.upAction,
	KeyStroke.getKeyStroke("KP_UP"),
	EditorKit.upAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, 0),
	EditorKit.downAction,
	KeyStroke.getKeyStroke("KP_DOWN"),
	EditorKit.downAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_PAGE_UP, 0),
	EditorKit.pageUpAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_PAGE_DOWN, 0),
	EditorKit.pageDownAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_UP, InputEvent.SHIFT_MASK),
	EditorKit.selectionUpAction,
	KeyStroke.getKeyStroke("shift KP_UP"),
	EditorKit.selectionUpAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, InputEvent.SHIFT_MASK),
	EditorKit.selectionDownAction,
	KeyStroke.getKeyStroke("shift KP_DOWN"), 
	EditorKit.selectionDownAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0),
	EditorKit.insertBreakAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, InputEvent.SHIFT_MASK),
	EditorKit.insertSoftBreakAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_BACK_SLASH, InputEvent.CTRL_MASK),
	EditorKit.unselectAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_HOME, InputEvent.CTRL_MASK),
	EditorKit.beginAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_END, InputEvent.CTRL_MASK),
	EditorKit.endAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_HOME, InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK),
	EditorKit.selectionBeginAction,
	KeyStroke.getKeyStroke(KeyEvent.VK_END, InputEvent.CTRL_MASK | InputEvent.SHIFT_MASK),
	EditorKit.selectionEndAction,
    };
    /***static***********************************/
    protected static final Font defaultFont = Font.getFont("helv");
    /***properties********************************/
    protected TextComponent editor;
    protected boolean painted = false;
    protected RootView rootView = new RootView();
    protected UpdateHandler updateHandler = new UpdateHandler();
    protected MouseHandler mouseHandler = new MouseHandler();
    protected KeyStrokeButton[] keyStrokes;
    protected ViewFactory viewFactory;
    protected EditorKit editorKit;
    /***Constructor****************************/
    public BasicTextUI(EditorKit _ek) {
	editorKit = _ek;
    }
    /***Static*************************************/
    public static ComponentUI createUI(JComponent c) {
        return new BasicTextUI(((TextComponent)c).getEditorKit());
    }
    /***Public************************************/
    public void rebuildView() {
	if (editor==null) return;
	Document doc = editor.getDocument();
	if (doc==null) return;
	Element root = doc.getRootElement();
	if (root==null) return;
	rootView.setView(editorKit.getViewFactory().create(root));
    }
    /***internal***************************************/
    protected Caret createCaret() { return new DefaultCaret(); }
    protected String getPropertyPrefix() { return "TextPane"; }
    protected Highlighter createHighlighter() {
        return new BasicHighlighter();
    }
    protected void installDefaults() {
        String prefix = getPropertyPrefix();
        Font f = editor.getFont();
        if (f==null || f instanceof UIResource)
            editor.setFont(UIManager.getFont(prefix + ".font"));
        Color bg = editor.getBackground();
        if (bg==null || bg instanceof UIResource)
            editor.setBackground(UIManager.getColor(prefix + ".background"));
        Color fg = editor.getForeground();
        if (fg==null || fg instanceof UIResource)
            editor.setForeground(UIManager.getColor(prefix + ".foreground"));
        Color color = editor.getCaretColor();
        if (color==null || color instanceof UIResource)
            editor.setCaretColor(UIManager.getColor(prefix + ".caretForeground"));
        Color s = editor.getSelectionColor();
        if (s==null || s instanceof UIResource)
            editor.setSelectionColor(UIManager.getColor(prefix + ".selectionBackground"));
        Color sfg = editor.getSelectedTextColor();
        if (sfg==null || sfg instanceof UIResource)
            editor.setSelectedTextColor(UIManager.getColor(prefix + ".selectionForeground"));
        Color dfg = editor.getDisabledTextColor();
        if (dfg==null || dfg instanceof UIResource)
            editor.setDisabledTextColor(UIManager.getColor(prefix + ".inactiveForeground"));
        Border b = editor.getBorder();
        if (b==null || b instanceof UIResource)
            editor.setBorder(UIManager.getBorder(prefix + ".border"));
        Insets margin = editor.getMargin();
        if (margin==null || margin instanceof UIResource)
            editor.setMargin(UIManager.getInsets(prefix + ".margin"));
        Caret caret = editor.getCaret();
        if (caret==null || caret instanceof UIResource) {
            caret = createCaret();
            editor.setCaret(caret);
            Object o = UIManager.get(prefix + ".caretBlinkRate");
            if (o instanceof Integer) {
                Integer rate = (Integer)o;
                caret.setBlinkRate(rate.intValue());
            }
        }
        Highlighter highlighter = editor.getHighlighter();
        if (highlighter==null || highlighter instanceof UIResource)
            editor.setHighlighter(createHighlighter());
    }
    protected void uninstallDefaults()  {
        if (editor.getCaretColor() instanceof UIResource)
            editor.setCaretColor(null);
        if (editor.getSelectionColor() instanceof UIResource)
            editor.setSelectionColor(null);
        if (editor.getDisabledTextColor() instanceof UIResource)
            editor.setDisabledTextColor(null);
        if (editor.getSelectedTextColor() instanceof UIResource)
            editor.setSelectedTextColor(null);
        if (editor.getBorder() instanceof UIResource)
            editor.setBorder(null);
        if (editor.getMargin() instanceof UIResource)
            editor.setMargin(null);
        if (editor.getCaret() instanceof UIResource)
            editor.setCaret(null);
        if (editor.getHighlighter() instanceof UIResource)
            editor.setHighlighter(null);
    }
    protected void installKeyboardActions(ButtonManager bm) {
	if (Debug.DEBUG && keyStrokes!=null)
	    throw new InternalError("keyStrokes already installed");
	final int count = keyBindings.length/2;
	keyStrokes = new KeyStrokeButton[count];
	for (int z=0; z<count; z++)
	    keyStrokes[z] = new KeyStrokeButton
		(getComponent(), (String)keyBindings[z*2+1],
		 (KeyStroke)keyBindings[z*2], JComponent.WHEN_FOCUSED);
	bm.installButtons(keyStrokes);
    }  
    protected void uninstallKeyboardActions(ButtonManager bm) {
	if (keyStrokes!=null) {
	    bm.uninstallButtons(keyStrokes);
	    keyStrokes = null;
	}
    }
    protected void paintBackground(Graphics g) {
        g.setColor(editor.getBackground());
        Dimension d = editor.getSize();
        g.fillRect(0, 0, d.width, d.height);
    }
    protected final TextComponent getComponent() { return editor; }
    protected void modelChanged() {
        Document doc = editor.getDocument();
	if (doc==null)
	    setView(null);
	else {
	    ViewFactory f = rootView.getViewFactory();
	    Element elem = doc.getRootElement();
	    setView(f.create(elem));
	}
    }
    protected void paintSafely(Graphics g) {
	painted = true;
	// background
	if (editor.isOpaque())
	    paintBackground(g);
	// views
	Rectangle alloc = getVisibleEditorRect();
	rootView.paint(g, alloc);
	// caret
	Caret caret = editor.getCaret();
	if (caret!=null)
	    caret.paint(g);
    }
    /***ComponentUI****************************************/
    public void installUI(JComponent c) {
	editor = (TextComponent)c;
	// listen
	Document doc = editor.getDocument();
	if (doc!=null)
	    doc.addDocumentListener(updateHandler);
	editor.addPropertyChangeListener(updateHandler);
	editor.addMouseListener(mouseHandler);
	// defaults
	installDefaults();
	editor.setOpaque(true);
	editor.setAutoscrolls(true);
	// create root view
	modelChanged();
	// install keymap
	ButtonManager bm = editor.getButtonManager();
	if (bm!=null)
	    installKeyboardActions(bm);
    }
    public void uninstallUI(JComponent c) {
        // model
	editor.removeMouseListener(mouseHandler);
	editor.removePropertyChangeListener(updateHandler);
	Document doc = editor.getDocument();
	if (doc!=null)
	    doc.removeDocumentListener(updateHandler);
        // view
        painted = false;
        uninstallDefaults();
        rootView.setView(null);
        c.removeAll();
        // controller
	ButtonManager bm = editor.getButtonManager();
	if (bm!=null)
	    uninstallKeyboardActions(bm);
    }
    public void update(Graphics g, JComponent c) {
	paint(g, c);
    }
    public final void paint(Graphics g, JComponent c) {
	if (rootView.getView()!=null)
	    paintSafely(g);
    }
    public Dimension getPreferredSize(JComponent c) {
	Document doc = editor.getDocument();
	Insets i = c.getInsets();
	Dimension d = c.getSize();
	if (d.width>(i.left+i.right) && d.height>(i.top+i.bottom))
	    rootView.setSize(d.width-i.left-i.right, d.height-i.top-i.bottom);
	d.width = (int)Math.min((long)rootView.getPreferredSpan(View.X_AXIS)+
				(long)i.left+(long)i.right, Integer.MAX_VALUE);
	d.height = (int)Math.min((long)rootView.getPreferredSpan(View.Y_AXIS)+
				 (long)i.top+(long)i.bottom, Integer.MAX_VALUE);
	return d;
    }
    public Dimension getMinimumSize(JComponent c) {
	Document doc = editor.getDocument();
        Insets i = c.getInsets();
	Dimension d = new Dimension();
	d.width = (int)rootView.getMinimumSpan(View.X_AXIS) + i.left + i.right;
	d.height = (int)rootView.getMinimumSpan(View.Y_AXIS) + i.top + i.bottom;
        return d;
    }
    public Dimension getMaximumSize(JComponent c) {
	Document doc = editor.getDocument();
        Insets i = c.getInsets();
	Dimension d = new Dimension();
	d.width = (int)Math.min((long)rootView.getMaximumSpan(View.X_AXIS)+
				(long)i.left+(long)i.right, Integer.MAX_VALUE);
	d.height = (int)Math.min((long) rootView.getMaximumSpan(View.Y_AXIS)+
				 (long)i.top+(long)i.bottom, Integer.MAX_VALUE);
        return d;
    }
    /***TextUI****************************************/
    public Rectangle modelToView(TextComponent tc, int pos)
	throws BadLocationException {
	if (painted) {
	    Rectangle alloc = getVisibleEditorRect();
	    rootView.setSize(alloc.width, alloc.height);
	    Shape s = rootView.modelToView(pos, alloc);
	    return s.getBounds();
	} else
	    return null;
    }
    public int viewToModel(TextComponent tc, Point pt) {
	if (painted) {
	    Rectangle alloc = getVisibleEditorRect();
	    return rootView.viewToModel(pt.x, pt.y, alloc);
	} else
	    return -1;
    }
    public int getNextVisualPositionFrom(TextComponent t, 
					 int pos, int direction)
	throws BadLocationException {
	if (painted)
	    return rootView.getNextVisualPositionFrom(pos, direction);
	else
	    return -1;
    }
    public int getRowStart(TextComponent target, int pos) {
	return painted
	    ?rootView.getRowStart(pos)
	    :-1;
    }
    public int getRowEnd(TextComponent target, int pos) {
	return painted
	    ?rootView.getRowEnd(pos)
	    :-1;
    }
    public void rebuild() {
	modelChanged();
    }
    public void damageRange(final TextComponent t, final int p0, final int p1) {
        if (painted)
	    SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
			Rectangle alloc = getVisibleEditorRect();
			rootView.setSize(alloc.width, alloc.height);
			try {
			    Shape toDamage = rootView.modelToView(p0, p1, alloc);
			    Rectangle rect = (toDamage instanceof Rectangle) ?
				(Rectangle)toDamage : toDamage.getBounds();
			    editor.repaint(rect.x, rect.y, rect.width, rect.height);
			} catch (BadLocationException e) {
			    e.printStackTrace();
			}
		    }
		});
    }
    public EditorKit getEditorKit(TextComponent tc) { 
	return editorKit;
    }
    public View getRootView(TextComponent tc) {
	return rootView;
    }
    public Font getFont(TextComponent t, Element el) {
	return defaultFont;
    }
    /***Internal*******************************/
    protected Rectangle getVisibleEditorRect() {
        Rectangle alloc = new Rectangle(editor.getSize());
        Insets insets = editor.getInsets();
        alloc.x += insets.left;
        alloc.y += insets.top;
        alloc.width -= insets.left + insets.right;
        alloc.height -= insets.top + insets.bottom;
        return alloc;
    }
    protected final void setView(View v) {
	editor.removeAll();
        rootView.setView(v);
        painted = false;
        editor.revalidate();
    }
    /***misc*************************************/
    public static class BasicHighlighter extends DefaultHighlighter implements UIResource {}
    /***RootView************************************************/
    public final class RootView extends View {
	/***properties**********************/
        protected View view;
	/***Constructor********************/
        public RootView() { super(null); }
	/***RootView**********************/
        public void setView(View v) {
            if (view!=null)
		view.setParent(null);
            view = v;
            if (view!=null)
                view.setParent(this);
        }
	public View getView() { return view; }
	/***View*************************/
        public float getPreferredSpan(int axis) {
	    return view==null?10:view.getPreferredSpan(axis);
        }
        public float getMinimumSpan(int axis) {
	    return view==null?10:view.getMinimumSpan(axis);
        }
        public float getMaximumSpan(int axis) {
	    return Integer.MAX_VALUE;
        }
        public void preferenceChanged(View child, boolean width, boolean height) {
            editor.revalidate();
        }
        public float getAlignment(int axis) {
	    return view==null?0:view.getAlignment(axis);
        }
        public void paint(Graphics g, Shape allocation) {
            if (view!=null) {
                Rectangle alloc = allocation.getBounds();
                view.setSize(alloc.width, alloc.height);
                view.paint(g, allocation);
            }
        }
        public void setParent(View parent) {
            throw new InternalError("Can't set parent on root view");
        }
        public Shape getChildAllocation(int index, Shape a) {
            return a;
        }
        public Shape modelToView(int pos, Shape a) 
	    throws BadLocationException {
	    return view==null?null:view.modelToView(pos, a);
        }
	public Shape modelToView(int p0, int p1, Shape a) 
	    throws BadLocationException {
	    return view==null?null:view.modelToView(p0, p1, a);
	}
        public int viewToModel(float x, float y, Shape a) {
	    return view==null?-1:view.viewToModel(x, y, a);
        }
	public int getRowStart(int pos) {
	    return view==null
		?super.getRowStart(pos)
		:view.getRowStart(pos);
	}
	public int getRowEnd(int pos) {
	    return view==null
		?super.getRowEnd(pos)
		:view.getRowEnd(pos);
	}
        public int getNextVisualPositionFrom(int pos, int direction) 
            throws BadLocationException {
            if (view!=null) {
                int nextPos = view.getNextVisualPositionFrom
		    (pos, direction);
		if (nextPos!=-1)
		    pos = nextPos;
            } 
            return pos;
        }
        public void documentChanged(DocumentEvent e, Shape a, ViewFactory f) {
            if (view!=null) view.documentChanged(e, a, f);
        }
        public Document getDocument() {
            return editor.getDocument();
        }
        public int getStartOffset() {
	    return view==null?getElement().getStartOffset():view.getStartOffset();
        }
        public int getEndOffset() {
	    return view==null?getElement().getEndOffset():view.getEndOffset();
        }
        public Element getElement() {
	    return view==null
		?editor.getDocument().getRootElement()
		:view.getElement();
        }
        public View breakView(int axis, int offset, float len, Shape a) {
            throw new Error("Can't break root view");
        }
        public int getResizeWeight(int axis) {
	    return view==null?0:view.getResizeWeight(axis);
        }
        public void setSize(int width, int height) {
            if (view!=null) view.setSize(width, height);
        }
        public TextComponent getContainer() {
            return editor;
        }
        public ViewFactory getViewFactory() {
            return editorKit.getViewFactory();
        }
    }
    /***UpdateHandler***********************************/
    class UpdateHandler implements PropertyChangeListener, DocumentListener {
	/***PropertyChangeListener************/
        public final void propertyChange(PropertyChangeEvent evt) {
	    if (evt.getSource()==editor)
		if (evt.getPropertyName().equals(editor.DOCUMENT)) {
		    Document oldValue = (Document)evt.getOldValue();
		    if (oldValue!=null)
			oldValue.removeDocumentListener(this);
		    Document newValue = (Document)evt.getNewValue();
		    if (newValue!=null)
			newValue.addDocumentListener(this);
		    modelChanged();
		} else if (evt.getPropertyName().equals(editor.BUTTON_MANAGER)) {
		    ButtonManager oldValue = (ButtonManager)evt.getOldValue();
		    if (oldValue!=null)
			uninstallKeyboardActions(oldValue);
		    ButtonManager newValue = (ButtonManager)evt.getNewValue();
		    if (newValue!=null)
			installKeyboardActions(newValue);
		}
        }
	/***DocumentListener****************/
        public final void documentChanged(DocumentEvent evt) {
            Rectangle alloc = painted?getVisibleEditorRect():null;
            rootView.documentChanged(evt, alloc, rootView.getViewFactory());
        }
    }
}
