package com.make.swing.text;

import com.make.util.Interval;
import com.make.util.InsertInterval;
import com.make.util.RemoveInterval;
import com.make.util.Debug;

public class ContainerChange {
    /***properties*******************************/
    protected Interval changed = new Interval();
    protected Interval added = new InsertInterval();
    protected Interval removed = new RemoveInterval();
    protected Element element;
    /***Constructor********************************/
    public ContainerChange(Element _el) {
	element = _el;
    }
    /***Grow************************************/
    public void childChanged(int i) { 
	//System.err.println("CHANGED "+element+"["+i+"]");
	if (Debug.DEBUG && (i<0 || i>=element.getElementCount())) {
	    System.err.println("FATAL ERROR: wrong changed index: "+i);
	    System.err.println("this = "+this);
	    System.err.println("elementCount = "+element.getElementCount());
	    throw new InternalError();
	}
	changed.include(i);
    }
    public void childAdded(int i) {
	//System.err.println("ADDED "+element+"["+i+"]");
	added.include(i); 
	removed.inserted(i);
	changed.inserted(i);
    }
    public void childRemoved(int i) { 
	//System.err.println("REMOVED "+element+"["+i+"]");
	changed.removed(i);
	if (i>=added.getFirst())
	    if (i>added.getLast()) {
		i -= added.getCount();
		removed.include(i);
	    } else {
		added.removed(i);
	    }
	else {
	    removed.include(i);
	    added.removed(i);
	}
	//if (!added.contains(i))
	//removed.include(i);
	//added.removed(i);
	//changed.removed(i);
    }
    /***Info*************************************/
    public boolean isTouched(int i) { 
	return changed.contains(i) || added.contains(i);
    }
    public Element getElement() { return element; }
    public Interval getChanged() { return changed; }
    public Interval getAdded() { return added; }
    public Interval getRemoved() { return removed; }
    public void dump() {
	if (Debug.DEBUG) {
	    System.err.println("ContainerChange "+this);
	    System.err.println(" - changed = "+getChanged());
	    System.err.println(" - added = "+getAdded());
	    System.err.println(" - removed = "+getRemoved());
	}
    }
    public String toString() {
	return "["+super.toString()+": el="+element+" changed="+getChanged()+" added="+getAdded()+" removed="+getRemoved()+"]";
    }
}
