package com.make.swing.text;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.io.PrintStream;
import javax.swing.Action;
import javax.swing.AbstractAction;
import javax.swing.JPopupMenu;
import javax.swing.text.Position;
import javax.swing.text.BadLocationException;
import com.make.util.Debug;

class ContainerUndoableEdit extends ElementUndoableEdit {
    /***properties************************/
    protected Element child;
    protected int index;
    /***Constructor*************************/
    public ContainerUndoableEdit(Element _el) {
	super(_el.getParent()); 
	child = _el; 
	index = element.indexOf(child);
	//System.err.println(getClass()+" p="+element.getClass()+" idx="+index+" child="+child.getClass());
    }
}

class RemoveChildUndoableEdit extends ContainerUndoableEdit {
    /***Constructor*************************/
    public RemoveChildUndoableEdit(Element _el) {
	super(_el);
	if (DUMP)
	    System.err.println(element+": NEW remove "+index+" - "+child);
    }
    public void undo() {
	super.undo();
	if (DUMP)
	    System.err.println(element+": add "+index+" - "+child);
	element.insertChild(child, index);
	inserted(child.getStartOffset(), child.getLength());
	createContainerChange().childAdded(index);
    }
    public void redo() {
	super.redo();
	if (DUMP)
	    System.err.println(element+": remove "+index+" - "+child);
	if (Debug.DEBUG && element.getElementAtIndex(index)!=child) {
	    System.err.println("FATAL ERROR: elementAt("+index+") is "+
			       element.getElementAtIndex(index));
	    System.err.println(" - should be "+child);
	    throw new InternalError();
	}
	removed(child.getStartOffset(), child.getLength());
	element.removeChild(index);
	createContainerChange().childRemoved(index);
    }
}

class InsertChildUndoableEdit extends ContainerUndoableEdit {
    /***Constructor*************************/
    public InsertChildUndoableEdit(Element _el) {
	super(_el);
	if (DUMP)
	    System.err.println(element+": NEW add "+index+" - "+child);
    }
    public void undo() {
	super.undo();
	if (DUMP)
	    System.err.println(element+": remove "+index+" - "+child);
	if (Debug.DEBUG && element.getElementAtIndex(index)!=child) {
	    System.err.println("FATAL ERROR: elementAt("+index+") is "+element.getElementAtIndex(index));
	    System.err.println(" - should be "+child);
	    throw new InternalError();
	}
	removed(child.getStartOffset(), child.getLength());
	element.removeChild(index);
	createContainerChange().childRemoved(index);
    }
    public void redo() {
	super.redo();
	if (DUMP)
	    System.err.println(element+": add "+index+" - "+child);
	element.insertChild(child, index);
	inserted(child.getStartOffset(), child.getLength());
	createContainerChange().childAdded(index);
    }
}

public abstract class Element {
    /***static*******************************/
    private static final Hashtable nullStyle = new Hashtable();
    /***properties****************************/
    private Document document;
    private Element parent;
    private int start=0, end, length;
    private Vector children;
    private Hashtable style = nullStyle;
    private int index=0;
    public boolean leadingSpace=false, endingSpace=false;
    public boolean fresh = true;
    /***Constructor************************/
    public Element(Document _doc, int _length, boolean leaf) {
	document = _doc; 
	length = _length; 
	end = start+length;
	if (!leaf) children = new Vector();
    }
    public Element(Document _doc, int _length, String _style) {
	document = _doc; 
	length = _length; 
	end = start+length;
	children = new Vector();
	setStyle(_doc.getStyle(_style));
    }
    public String toString() {
	return "["+super.toString()+": start="+getStartOffset()+" end="+getEndOffset()+"]";
    }
    protected final int getIndex() { return index; }
    protected final void offsetIndex(int delta) { index+=delta; }
    public void setParent(Element _p, int _start, int _index) {
	if (_p==null) {
	    if (Debug.DEBUG) {
		if (parent==null)
		    throw new IllegalStateException("parent must not be null");
		if (_start!=0)
		    throw new IllegalStateException("_start must be 0");
		if (_index!=0)
		    throw new IllegalStateException("_index must be 0");
	    }
	    parent = null;
	    moveStart(-getStartOffset());
	    offsetIndex(-getIndex());
	} else {
	    if (Debug.DEBUG) {
		if (parent!=null)
		    throw new IllegalStateException(this+": parent must be null");
		if (start!=0)
		    throw new IllegalStateException(this+": start must be 0");
		if (index!=0)
		    throw new IllegalStateException(this+": index must be 0");
	    }
	    parent = _p;
	    moveStart(_start);
	    offsetIndex(_index);
	    fresh = false;
	}
    }
    public Element getParent() { return parent; }
    public void setStyle(Hashtable _s) { style = _s; }
    protected abstract int insertSpaceChild(int index, DocumentEvent evt,
					    ContainerChange chg);
    /***Split********************************************/
    public Element split(int offset, DocumentEvent evt) 
	throws BadLocationException, SplitNotSupportedException {
	// default for leaf is not splittable
	if (isLeaf())
	    throw new SplitNotSupportedException(this, offset);
	// already splitted?
	if (offset<=getStartOffset() || offset>=getEndOffset())
	    return null;
	// find child
	int index = getIndexAt(offset);
	Element el = getElementAtIndex(index);
	// split it
	Element el2;
	if (offset<=el.getStartOffset()) {
	    removeChild(index);
	    if (evt!=null)
		evt.createContainerChange(this).childRemoved(index);
	    el2 = el;
	} else {
	    el2 = el.split(offset, evt);
	    index++;
	}
	// move all children on the right hand into a new instance
	Element ret = createEmptyInstance();
	if (el2!=null) 
	    ret.insertChild(el2, null);
	ContainerChange chg = null;
	if (evt!=null && index<getElementCount())
	    chg = evt.createContainerChange(this);
	while (index<getElementCount()) {
	    el2 = getElementAtIndex(index);
	    if (chg!=null)
		chg.childRemoved(index);
	    removeChild(index);
	    ret.insertChild(el2, null);
	}
	return ret;
    }
    public final int splitChild(int index, int offset, DocumentEvent evt) 
	throws BadLocationException, SplitNotSupportedException {
	Element child = getElementAtIndex(index);
	if (offset>child.getStartOffset() && offset<child.getEndOffset()) {
	    Element split = child.split(offset, evt);
	    if (split!=null) {
		insertChild(split, index+1);
		if (evt!=null)
		    evt.createContainerChange(this).childAdded(index+1);
		return index+1;
	    }
	}
	return -1;
    }
    protected final int forceSplitChild(int index, int offset, boolean begin, 
					DocumentEvent evt) 
	throws BadLocationException {
	try {
	    return splitChild(index, offset, evt);
	} catch (SplitNotSupportedException e) {
	    Element el = e.getElement();
	    return forceSplitChild
		(index, begin?el.getStartOffset():el.getEndOffset(), begin, evt);
	}
    }
    protected final int insertSplitChild(int index, int offset, boolean begin, 
					 DocumentEvent evt) 
	throws BadLocationException {
	Element el = getElementAtIndex(index);
	if (offset==el.getStartOffset())
	    return index;
	else if (offset==el.getEndOffset())
	    return index+1;
	final int ret = forceSplitChild(index, offset, begin, evt);
	return ret==-1
	    ? (begin?index:index+1)
	    : ret;
    }
    /***Format**************************************************/
    protected abstract Element createEmptyInstance();
    protected int embedChildren(Element parent, int first, int last,
				ContainerChange chg) {
	int ret = 0;
	int curCount = 0;
	for (int z=first; z<=last; z++) {
	    Element el = getElementAtIndex(z);
	    if (!childConservationPossible(el) ||
		(!parent.childAllowed(el) && el.isLeaf())) {
		if (curCount>0) {
		    insertChild(parent, z); 
		    if (chg!=null)
			chg.childAdded(z);
		    ret++; z++; last++;
		    parent = parent.createEmptyInstance();
		    curCount = 0;
		}
		if (chg!=null) {
		    chg.childRemoved(z);
		    chg.childAdded(z);
		}
	    } else {
		removeChild(z);
		if (chg!=null)
		    chg.childRemoved(z);
		curCount += parent.insertChildCleanupStyles(el, curCount, null);
		ret--; z--; last--;
	    }
	}
	if (curCount>0) {
	    ret++;
	    insertChild(parent, last+1); 
	    if (chg!=null)
		chg.childAdded(last+1);
	}
	return ret;
    }
    public boolean embed(Element parent, int start, int end,
			 DocumentEvent evt) 
	throws BadLocationException {
	if (isLeaf() || getElementCount()==0 || start>end) return false;
	final int first = getIndexAt(start);
	final int last = getIndexAt(end);
	if (first==last && getElementAtIndex(first).embed(parent, start, end, evt))
	    return true;
	if (!childAllowed(parent))
	    return false;
	ContainerChange chg = evt==null?null:evt.createContainerChange(this);
	Element firstChild = getElementAtIndex(first);
	final int firstStart = firstChild.getStartOffset();
	int added = 0;
	for (int z=last; z>=first; z--) {
	    Element ch = getElementAtIndex(z);
	    removeChild(z);
	    if (chg!=null)
		chg.childRemoved(z);
	    if (parent.childAllowed(ch)) {
		parent.insertChild(ch, 0);
		added++;
	    } else {
		if (added>0) {
		    insertChild(parent, z);
		    if (chg!=null)
			chg.childAdded(z);
		    parent = parent.createEmptyInstance();
		}
		insertChild(ch, z);
		if (chg!=null)
		    chg.childAdded(z);
	    }
	}
	if (added>0) {
	    insertChild(parent, first);
	    if (chg!=null)
		chg.childAdded(first);
	    if (evt!=null)
		evt.inserted(this, firstStart, firstChild.getStartOffset()-firstStart);
	    //TODO: inserted?
	}
	return true;
    }
    public boolean childAllowed(Element el) {
	return false;
    }
    public boolean childConservationPossible(Element child) {
	return true;
    }
    public boolean canRemoveChild(Element child) {
	return true;
    }
    public boolean canInsertAt(int pos) throws BadLocationException {
	return true;
    }
    public int insertChildCleanupStyles(Element el, int idx, 
					ContainerChange chg) {
	if (childAllowed(el)) {
	    insertChild(el, idx);
	    if (chg!=null)
		chg.childAdded(idx);
	    return 1;
	} else if (el.isLeaf())
	    return 0;
	else {
	    int count = 0;
	    for (int elIndex=0; elIndex<el.getElementCount();) {
		Element child = el.getElementAtIndex(elIndex);
		if (el.childConservationPossible(child)) {
		    el.removeChild(elIndex);
		    int delta = insertChildCleanupStyles(child, idx, chg);
		    count += delta;
		    idx += delta;
		} else
		    elIndex++;
	    }
	    return count;
	}
    }
    protected void pasteChildren(int pos, Element el, DocumentEvent evt) 
	throws BadLocationException {
	pos = forceSplitChild(index, pos, true, evt);
	int index = getIndexAt(pos);
	ContainerChange chg = evt==null?null:evt.createContainerChange(this);
	while (el.getElementCount()>0) {
	    Element child = el.getElementAtIndex(0);
	    el.removeChild(0);
	    insertChildCleanupStyles(child, index++, chg);
	}
    }
    public boolean isStyle(String style) {
	return false;
    }
    public boolean isDeepStyle(int pos, Position.Bias bias, String style) 
	throws BadLocationException {
	if (isStyle(style))
	    return true;
	if (isLeaf() || getElementCount()==0)
	    return false;
	Element ch = getElementAt(pos, bias);
	return ch==null?false:ch.isDeepStyle(pos, bias, style);
    }
    protected int removeStyle(int idx, ContainerChange chg) {
	// returns number of new children
	Element el = getElementAtIndex(idx);
	if (Debug.DEBUG && el.isLeaf())
	    throw new InternalError("the element "+el+" is a leaf");
	removeChild(idx);
	if (chg!=null) chg.childRemoved(idx);
	int count = 0;
	for (int z=el.getElementCount()-1; z>=0; z--) {
	    Element child = el.getElementAtIndex(z);
	    el.removeChild(z);
	    count += insertChildCleanupStyles(child, idx, chg);
	}
	return count;
    }
    public void removeStyle(int idx, DocumentEvent evt) {
	removeStyle(idx, evt==null?null:evt.createContainerChange(this));
    }
    public void removeStyle(String style, int start, int end, DocumentEvent evt) 
	throws BadLocationException {
	if (isLeaf()) return;
	ContainerChange chg = evt==null?null:evt.createContainerChange(this);
	int z = getIndexAt(start);
	Element el = getElementAtIndex(z);
	if (el.isStyle(style)) {
	    final int newZ = forceSplitChild(z, start, true, evt);
	    if (newZ!=-1) {
		z = newZ;
		el = getElementAtIndex(z);
	    }
	    if (el.getEndOffset()>=end) {
		final int newZ2 = forceSplitChild(z, end, false, evt);
		removeStyle(z, chg);
		return;
	    } else
		z += removeStyle(z, chg);
	} else {
	    if (el.getEndOffset()>=end) {
		el.removeStyle(style, start, end, evt);
		return;
	    } else {
		el.removeStyle(style, start, el.getEndOffset(), evt);
		z++;
	    }
	}
	final int count = getElementCount();
	for (; z<count && (el=getElementAtIndex(z)).getEndOffset()<end; z++) {
	    if (el.isStyle(style))
		z += removeStyle(z, chg)-1;
	    else {
		if (el.getLength()==0) {
		    removeChild(z); 
		    if (chg!=null)
			chg.childRemoved(z--);
		} else if (chg==null) {
		    // workaround
		    el.removeStyle(style, el.getStartOffset(),
				   el.getEndOffset(), evt);
		} else {
		    // workaround
		    chg.childRemoved(z); chg.childAdded(z); 
		    el.removeStyle(style, el.getStartOffset(),
				   el.getEndOffset(), null);
		}
	    }
	}
	if (z<count && el.getStartOffset()<end)
	    if (el.isStyle(style)) {
		final int newZ = forceSplitChild(z, end, false, null);
		z += removeStyle(z, chg);
		if (newZ!=-1 && chg!=null)
		    chg.childAdded(z);
	    } else
		el.removeStyle(style, el.getStartOffset(), end, evt);
    }
    public boolean removeParentStyle(DocumentEvent evt) {
	Element parent = getParent();
	Element parent2 = parent.getParent();
	Element newP = parent2.childAllowed(this)
	    ?this
	    :parent2.createDefaultChild();
	if (newP==null) return false;
	ContainerChange chg2 = evt==null?null:evt.createContainerChange(parent2);
	int idx2 = parent2.indexOf(parent);
	// before
	Element splitted;
	if (getStartOffset()>parent.getStartOffset()) {
	    try {
		splitted = parent.split(getStartOffset(), evt);
	    } catch (BadLocationException e) {
		e.printStackTrace(); throw new InternalError();
	    } catch (SplitNotSupportedException e) {
		e.printStackTrace(); throw new InternalError();
	    }
	    idx2++;
	} else {
	    splitted = parent;
	    parent2.removeChild(idx2);
	    if (chg2!=null)
		chg2.childRemoved(idx2);
	}
	// middle
	splitted.removeChild(0);
	final int last = getElementCount()-2;
	if (newP!=this)
	    for (int z=0; z<=last; z++) {
		Element child = getElementAtIndex(0);
		removeChild(0);
		newP.insertChildCleanupStyles(child, z, null);
	    }
	parent2.insertChild(newP, idx2);
	if (chg2!=null)
	    chg2.childAdded(idx2);
	idx2++;
	// after
	if (splitted.getElementCount()>0) {
	    parent2.insertChild(splitted, idx2);
	    if (chg2!=null)
		chg2.childAdded(idx2);
	}
	return true;
    }
    public boolean embedSplit(Element parent, int start, int end,
			      DocumentEvent evt) 
	throws BadLocationException {
	if (isLeaf() || start>=end) 
	    return false;
	if (childAllowed(parent)) {
	    final int oldFirst = getIndexAt(start);
	    final int newFirst = forceSplitChild(oldFirst, start, true, evt);
	    final int oldLast = getIndexAt(end, Position.Bias.Backward);
	    final int newLast = forceSplitChild(oldLast, end, false, null);
	    final int first = newFirst==-1?oldFirst:newFirst;
	    final int last = oldLast;
	    ContainerChange chg = evt.createContainerChange(this);
	    int ec = 0;
	    if (last>=first)
		ec = embedChildren(parent, first, last, chg);
	    if (newLast!=-1)
		chg.childAdded(last+ec+1);
	    return true;
	} else {
	    final int first = getIndexAt(start);
	    int last = getIndexAt(end);
	    if (getElementAtIndex(last).getStartOffset()==end)
		last--;
	    if (first==last)
		return getElementAtIndex(first).embedSplit(parent, start, end, evt);
	    else {
		boolean ok = false;
		Element el;
		el = getElementAtIndex(first);
		if (el.embedSplit(parent.createEmptyInstance(), 
				  start, el.getEndOffset(), evt))
		    ok = true;
		for (int z=first+1; z<last; z++) {
		    el = getElementAtIndex(z);
		    if (el.embedSplit(parent.createEmptyInstance(), el.getStartOffset(),
				      el.getEndOffset(), evt))
			ok = true;
		}
		el = getElementAtIndex(last);
		if (el.embedSplit(parent, el.getStartOffset(), end, evt))
		    ok = true;
		return ok;
	    }
	}
    }
    public void replaceStyle(Element newStyle, DocumentEvent evt) {
	Element parent = getParent();
	final int idx = parent.indexOf(this);
	ContainerChange chg = evt==null?null:evt.createContainerChange(parent);
	parent.removeChild(idx);
	if (chg!=null)
	    chg.childRemoved(idx);
	for (int tz=getElementCount()-1; tz>=0; tz--) {
	    Element ch = getElementAtIndex(tz);
	    if (childConservationPossible(ch)) {
		removeChild(tz);
		newStyle.insertChildCleanupStyles(ch, 0, null);
	    }
	}
	parent.insertChild(newStyle, idx);
	if (chg!=null)
	    chg.childAdded(idx);
    }
    public boolean insertElement(int idx, Element el, DocumentEvent evt) 
	throws BadLocationException {
	if (childAllowed(el)) {
	    insertChild(el, idx);
	    if (evt!=null) {
		evt.createContainerChange(this).childAdded(idx);
		evt.inserted(this, el.getStartOffset(), el.getLength());
	    }
	    return true;
	} else
	    return false;
    }
    public int pasteChild(int idx, Element el, DocumentEvent evt)
	throws BadLocationException {
	if (isLeaf())
	    return 0;
	else if (insertElement(idx, el, evt))
	    return 1;
	else if (el.isLeaf())
	    return 0;
	else {
	    final int count = el.getElementCount();
	    int ret = 0;
	    for (int z=count-1; z>=0; z--) {
		Element child = el.getElementAtIndex(z);
		if (el.childConservationPossible(child)) {
		    el.removeChild(z);
		    ret += pasteChild(idx, child, evt);
		}
	    }
	    return ret;
	}
    }
    public boolean pasteElementAt(int pos, Element el, DocumentEvent evt) 
	throws BadLocationException {
	if (isLeaf()) return false;
	final int idx = getIndexAt(pos);
	if (getElementAtIndex(idx).pasteElementAt(pos, el, evt))
	    return true;
	else if (childAllowed(el)) {
	    int newIdx = forceSplitChild(idx, pos, true, evt);
	    if (newIdx==-1) newIdx = idx;
	    insertChild(el, newIdx);
	    if (evt!=null) {
		evt.createContainerChange(this).childAdded(newIdx);
		evt.inserted(this, el.getStartOffset(), el.getLength());
	    }
	    return true;
	} else
	    return false;
    }
    public boolean insertElementAtIndex(int idx, Element el, DocumentEvent evt) 
	throws BadLocationException {
	if (childAllowed(el)) {
	    insertChild(el, idx);
	    if (evt!=null) {
		evt.createContainerChange(this).childAdded(idx);
		evt.inserted(this, el.getStartOffset(), el.getLength());
	    }
	    return true;
	} else
	    return false;
    }
    public int pasteElements(int pos, Element[] els, DocumentEvent evt)
	throws BadLocationException {
	if (els==null || els.length==0 || isLeaf()) 
	    return 0;
	final int idx = getIndexAt(pos);
	int count = getElementAtIndex(idx).pasteElements(pos, els, evt);
	if (count>0) return count;
	int newIdx = insertSplitChild(idx, pos, true, evt);
	if (newIdx==-1) newIdx = idx;
	ContainerChange chg = evt==null?null:evt.createContainerChange(this);
	boolean wasSpace = false;
	final int oldLen = getLength();
	for (int z=0; z<els.length; z++)
	    if (els[z]!=null) {
		if (wasSpace || els[z].leadingSpace)
		    count += insertSpaceChild(newIdx+count, null, chg);
		count += insertChildCleanupStyles(els[z], newIdx+count, chg);
		wasSpace = els[z].endingSpace;
	    }
	if (wasSpace)
	    count += insertSpaceChild(newIdx+count, null, chg);
	if (count>0 && evt!=null)
	    evt.inserted(this, pos, getLength()-oldLen);
	return count;
    }
    /***Info************************************/
    public final Document getDocument() { return document; }
    public boolean ignoreOuterSpaces() { return false; }
    /***Data**************************************/
    public Element createDefaultChild() { return null; }
    public boolean insertString(int offset, String str, DocumentEvent evt) 
	throws BadLocationException {
	if (isLeaf())
	    return false;
	else {
	    if (getElementCount()==0) {
		// no children - create child
		Element nel = createDefaultChild();
		if (nel==null) return false;
		final int newIndex = 0;
		insertChild(nel, newIndex);
		if (evt!=null) {
		    evt.inserted(this, nel.getStartOffset(), nel.getLength());
		    evt.createContainerChange(this).childAdded(newIndex);
		}
	    }
	    int idx = getIndexAt(offset);
	    Element el = getElementAtIndex(idx);
	    boolean back = false;
	    if (idx>0 && el.getStartOffset()==offset && !str.equals("\n")) {
		idx--; el = getElementAtIndex(idx); back = true;
	    }
	    if (el.insertString(offset, str, evt))
		return true;
	    if (back) {
		idx++;
		el = getElementAtIndex(idx);
		if (getElementAtIndex(idx).insertString(offset, str, evt))
		    return true;
		else {
		    Element nel = createDefaultChild();
		    if (nel!=null) {
			insertChild(nel, idx, el);
			if (offset>=nel.getStartOffset())
			    offset += nel.getLength();
			if (nel.insertString(offset, str, null)) {
			    if (evt!=null) {
				evt.inserted(this, nel.getStartOffset(), nel.getLength());
				evt.createContainerChange(this).childAdded(idx);
			    }
			    return true;
			} else {
			    removeChild(idx);
			    return false;
			}
		    } else
			return false;
		}
	    } else {
		Element nel = createDefaultChild();
		if (nel!=null) {
		    final int newIndex = offset==getEndOffset()?idx+1:idx;
		    insertChild(nel, newIndex);
		    if (offset>=nel.getStartOffset())
			offset += nel.getLength();
		    if (nel.insertString(offset, str, null)) {
			if (evt!=null) {
			    evt.inserted(this, nel.getStartOffset(), nel.getLength());
			    evt.createContainerChange(this).childAdded(newIndex);
			}
			return true;
		    } else {
			removeChild(newIndex);
			return false;
		    }
		} else
		    return false;
	    }
	}
    }
    public void remove(int offset, int length, DocumentEvent evt) 
	throws BadLocationException {
	if (isLeaf()) return;
	if (length==0) return;
	int idx0 = getIndexAt(offset),
	    idx1 = getIndexAt(offset+length);
	if (idx0==idx1) {
	    Element el = getElementAtIndex(idx0);
	    if (offset==el.getStartOffset() && length==el.getLength()) {
		removeChildren(idx0, idx0, evt);
		if (evt!=null)
		    evt.removed(this, offset, length); //TODO?
		if (getElementCount()==0) {
		    insertChild(createDefaultChild(), null);
		    if (evt!=null)
			evt.getContainerChange(this).childAdded(0);
		}
	    } else
		getElementAtIndex(idx0).remove(offset, length, evt);
	} else {
	    Element el0 = getElementAtIndex(idx0),
		el1 = getElementAtIndex(idx1);
	    if (offset==el0.getStartOffset()) {
		idx0--; el0 = null;
	    }
	    if ((offset+length)==el1.getEndOffset())
		idx1++;
	    else
		el1.remove(el1.getStartOffset(), offset+length-el1.getStartOffset(), evt);
	    if ((idx0+1)<idx1) {
		final int o1 = getElementAtIndex(idx0+1).getStartOffset();
		final int l1 = getElementAtIndex(idx1-1).getEndOffset()-o1;
		if (evt!=null)
		    evt.removed(this, o1, l1);
		removeChildren(idx0+1, idx1-1, evt);
		if (el0!=null) el0.remove(offset, el0.getEndOffset()-offset, evt);
	    } else {
		if (el0!=null) el0.remove(offset, el0.getEndOffset()-offset, evt);
	    }
	}
    }
    /***Element***********************/
    public Element getDeepElementAt(int start, int end)
	throws BadLocationException {
	if (start>end)
	    throw new BadLocationException("start>end", start);
	if (isLeaf())
	    return this;
	final int first = getIndexAt(start);
	final int last = getIndexAt(end);
	if (first==last)
	    return getElementAtIndex(first).getDeepElementAt(start, end);
	else
	    return this;
    }
    public Element getLeafElementAt(int pos, Position.Bias bias) throws BadLocationException {
	return isLeaf() 
	    ?this
	    :(children.size()==0?null:getElementAt(pos, bias));
    }
    /***InsertChildView*********************************************/
    protected final void insertChildView(Element newChild, int index, 
					 Element refChild) {
	if (refChild==null) {
	    final int start = getElementCount()==0
		?getStartOffset()
		:getElementAtIndex(getElementCount()-1).getEndOffset();
	    children.addElement(newChild);
	    newChild.setParent(this, start, children.size()-1);
	} else {
	    children.insertElementAt(newChild, index);
	    newChild.setParent(this, refChild.getStartOffset(), index);
	    for (int z=index+1; z<children.size(); z++)
		getElementAtIndex(z).offsetIndex(1);
	}
	childLengthChanged(index, newChild.getLength());
    }
    protected final void appendChildView(Element newChild) {
	insertChildView(newChild, getElementCount(), null);
    }
    /***InsertChild***********************************************/
    public void insertChild(Element child, int index, Element refChild) {
	if (!child.fresh)
	    fresh = false;
	insertChildView(child, index, refChild);
	// undo
	if (!fresh)
	    getDocument().postEdit(new InsertChildUndoableEdit(child));
    }
    public final void insertChild(Element child, Element refChild) {
	insertChild(child, refChild==null?getElementCount():indexOf(refChild), refChild);
    }
    public final void insertChild(Element child, int index) {
	if (Debug.DEBUG) {
	    if (index<0)
		throw new ArrayIndexOutOfBoundsException(index+" < 0");
	    if (index>getElementCount())
		throw new ArrayIndexOutOfBoundsException(index+" > "+getElementCount());
	}
	insertChild(child, index, index==getElementCount()?null:getElementAtIndex(index));
    }
    /***RemoveChild**********************************************/
    protected final void removeChildView(int index) {
	Element child = getElementAtIndex(index);
	// change child length to 0
	childLengthChanged(child, -child.getLength());
	child.offsetIndex(-index);
	// and remove it
	children.removeElementAt(index);
	final int count = children.size();
	for (int z=index; z<count; z++)
	    getElementAtIndex(z).offsetIndex(-1);
	// reset child start
	child.setParent(null, 0, 0);
    }
    public void removeChild(int index) {
	//if (Debug.DEBUG && !canRemoveChild(elementAt(index)))
	//throw new InternalError("can't remove child "+elementAt(index)+" from "+this);
	// undo
	if (!getElementAtIndex(index).fresh)
	    fresh = false;
	if (!fresh)
	    getDocument().postEdit(new RemoveChildUndoableEdit(getElementAtIndex(index)));
	// view
	removeChildView(index);
    }
    public final void removeChild(Element child) {
	removeChild(indexOf(child));
    }
    protected void removeChildren(int first, int last, DocumentEvent evt) {
	ContainerChange chg = evt==null?null:evt.createContainerChange(this);
	for (int z=last; z>=first; z--) {
	    Element ch = getElementAtIndex(z);
	    if (canRemoveChild(ch)) {
		if (chg!=null)
		    chg.childRemoved(z);
		removeChild(z);
	    } else {
		if (chg!=null) { // workaround
		    chg.childRemoved(z);
		    chg.childAdded(z);
		}
		try {
		    ch.remove(ch.getStartOffset(), ch.getLength(), null);
		} catch (BadLocationException e) {
		    e.printStackTrace();
		}
	    }
	}
    }
    protected void removeChild(int index, DocumentEvent evt) {
	removeChildren(index, index, evt);
    }
    /***Element***************************/
    public void lengthChanged(int delta) {
	// correct own length
	length += delta; end += delta;
	// parent has to be notified
	if (parent!=null)
	    parent.childLengthChanged(this, delta);
    }
    private void childLengthChanged(int index, int delta) {
	// now move start of all children after child
	for (int idx = index+1; idx<children.size(); idx++) {
	    Element el = getElementAtIndex(idx);
	    el.moveStart(delta);
	}
	lengthChanged(delta);
    }
    private void childLengthChanged(Element child, int delta) {
	childLengthChanged(children.indexOf(child), delta);
    }
    private void moveStart(int delta) { 
	// move own start
	start += delta; end += delta;
	// all children have to be moved
	if (children!=null)
	    for (Enumeration enum=children.elements(); enum.hasMoreElements();) {
		Element el = (Element)enum.nextElement();
		el.moveStart(delta);
	    }
    }
    public void getText(StringBuffer sb, int offset, int length) 
	throws BadLocationException {
	if (offset<getStartOffset()) 
	    throw new BadLocationException("offset<start", offset);
	if ((offset+length)>getEndOffset()) 
	    throw new BadLocationException("offset+length>end", offset);
	int idx0 = getIndexAt(offset, Position.Bias.Forward), 
	    idx1 = getIndexAt(offset+length, Position.Bias.Backward);
	for (int z=idx0; z<=idx1; z++) {
	    Element el = getElementAtIndex(z);
	    int offs = el.getStartOffset(), len = el.getLength();
	    if (offs<offset) { len -= offset-offs; offs = offset; }
	    if ((offs+len)>(offset+length)) len = offset+length-offs;
	    if (len>0)
		el.getText(sb, offs, len);
	}
    }
    public final int indexOf(Element el) { return el.getIndex(); }
    public final Element getElementAtIndex(int index) {
	return (Element)children.elementAt(index);
    }
    /***Tooltip***************************************/
    public String getToolTipText(int position, Position.Bias bias) 
	throws BadLocationException {
	if (!isLeaf()) {
	    Element el = getElementAt(position, bias);
	    if (el!=null)
		return el.getToolTipText(position, bias);
	}
	return null;
    }
    public void fillMenu(TextComponent container, int context, 
			 int pos, Position.Bias bias, JPopupMenu menu) 
	throws BadLocationException {
	if (!isLeaf()) {
	    Element el = getElementAt(pos, bias);
	    if (el!=null)
		el.fillMenu(container, context, pos, bias, menu);
	}
    }
    /***Element***************************/
    public final int getStartOffset() { return start; }
    public final int getEndOffset() { return end; }
    public final int getLength() { return length; }
    public int getIndexAt(int offset) {
	if (Debug.DEBUG && isLeaf())
	    throw new InternalError("getIndexAt invalid in leaf");
	int lower=0, upper=children.size()-1, mid=0;
	int p0=getStartOffset(), p1;
	if (upper==-1 || offset>=getElementAtIndex(upper).getEndOffset())
	    return upper;
	if (offset<=p0)
	    return lower;
	while (lower<=upper) {
	    mid = (upper+lower)/2;
	    Element elem = getElementAtIndex(mid);
	    p0 = elem.getStartOffset(); 
	    p1 = elem.getEndOffset();
	    if (offset>=p0 && offset<p1)
		return mid;
	    else if (offset<p0)
		upper = mid - 1;
	    else
		lower = mid + 1;
	}
	return offset<p0
	    ?mid
	    :mid+1;
    }
    public int getIndexAt(int offset, Position.Bias bias) {
	int idx = getIndexAt(offset);
	if (bias==Position.Bias.Backward && idx>0) {
	    Element el = getElementAtIndex(idx);
	    if (offset==el.getStartOffset())
		idx--;
	}
	return idx;
    }
    public Element getElementAt(int pos, Position.Bias bias) 
	throws BadLocationException {
	if (pos<getStartOffset() || pos>getEndOffset())
	    throw new BadLocationException("invalid position", pos);
	final int idx = getIndexAt(pos, bias);
	return idx<0?null:getElementAtIndex(idx);
    }
    public int getElementCount() { return children.size(); }
    public boolean isLeaf() { return children==null; }
    /***AttributeSet****************************************/
    public Object getAttribute(Object key) { 
	Object val = style.get(key);
	if (val==null && parent!=null && 
	    (!(key instanceof String) || ((String)key).charAt(0)!='_')) 
	    val = parent.getAttribute(key);
	return val;
    }
    public boolean containsAttribute(Object name, Object value) {
	return value.equals(getAttribute(name));
    }
    /***Debug***************************************/
    protected void dump(PrintStream out, String indent) {
	if (Debug.DEBUG) {
	    out.print(indent+this);
	    if (isLeaf()) {
		out.println();
	    } else {
		out.println(" {");
		for (int z=0; z<getElementCount(); z++)
		    getElementAtIndex(z).dump(out, indent+"  ");
		out.println(indent+"}");
	    }
	}
    }
    public void dump(PrintStream out) {
	if (Debug.DEBUG)
	    dump(out, "");
    }
}
