package com.make.swing.text;

import javax.swing.undo.AbstractUndoableEdit;
import com.make.util.Debug;

public abstract class ElementUndoableEdit extends AbstractUndoableEdit {
    /***CONST********************************/
    public static final boolean DUMP = false;
    /***properties**************************/
    protected Element element;
    private int debugStart = -1;
    private boolean connected;
    /***Constructor*************************/
    public ElementUndoableEdit(Element _el) {
	element = _el;
	if (Debug.DEBUG)
	    debugStart = element.getStartOffset();
	Element el = element;
	while (el.getParent()!=null)
	    el = el.getParent();
	connected = el==element.getDocument().getRootElement();
	if (DUMP)
	    System.err.println(element+": connected="+connected);
    }
    protected DocumentEvent getEvent() { 
	return element.getDocument().getUndoEvent();
    }
    protected ContainerChange createContainerChange() {
	return getEvent().createContainerChange(element);
    }
    /***Public***********************************/
    public final boolean isConnected() { return connected; }
    public void inserted(int pos, int len) {
	if (connected)
	    getEvent().inserted(element, pos, len);
    }
    public void removed(int pos, int len) {
	if (connected)
	    getEvent().removed(element, pos, len);
    }
    /***AbstractUndoableEdit********************/
    public void undo() {
	super.undo();
	if (Debug.DEBUG)
	    if (debugStart!=element.getStartOffset()) {
		System.err.println("FATAL ERROR in undo from "+this);
		System.err.println("  el = "+element);
		System.err.println("  debugStart = "+debugStart+
				   " start="+element.getStartOffset());
		throw new InternalError();
	    }
    }
    public void redo() {
	super.redo();
	if (Debug.DEBUG)
	    if (debugStart!=element.getStartOffset()) {
		System.err.println("FATAL ERROR in redo from "+this);
		System.err.println("  el = "+element);
		System.err.println("  debugStart = "+debugStart+
				   " start="+element.getStartOffset());
		throw new InternalError();
	    }
    }
}
