package com.make.swing.text;

import java.awt.GridBagLayout;
import java.awt.Component;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.*;
import javax.swing.JPanel;
import javax.swing.BorderFactory;
import javax.swing.JTextField;
import javax.swing.JCheckBox;
import javax.swing.JButton;
import javax.swing.Box;
import javax.swing.text.Segment;
import javax.swing.text.BadLocationException;
import com.make.swing.InterfacePanel;
import com.make.swing.PanelContent;
import com.make.swing.MakeDialog;
import com.make.swing.SwingButton;
import com.make.swing.DialogInterfacePanel;

public class ReplaceDialog implements PanelContent, ActionListener {
    /***static************************/
    protected static ResourceBundle resources = ResourceBundle.getBundle("com.make.swing.text.ReplaceDialog");
    
    /***properties************************/
    
    protected JPanel panel = new JPanel(new GridBagLayout(), false);
    protected JTextField searchTextField = new JTextField(20);
    protected JTextField replaceTextField = new JTextField(20);
    protected JCheckBox ignoreCaseField = new JCheckBox(resources.getString("ignoreCase.label"));
    protected TextComponent textcomp;
    protected Document doc;
    protected Caret caret;
    protected SwingButton closeButton;
    protected DialogInterfacePanel interfacePanel;
    protected boolean replace = false;
    protected boolean replacePossible = false;
    protected boolean replaceAll = false;
    protected boolean closed = false;
    protected int lastFoundPos = -1;
    /***Constructor*************************/
    public ReplaceDialog() {
	panel.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
	MakeDialog.addField(panel,resources.getString("searchFor.label") , searchTextField);
	searchTextField.setActionCommand("searchEnter");
	searchTextField.addActionListener(this);
	MakeDialog.addField(panel,resources.getString("replaceWith.label") , replaceTextField);
	replaceTextField.setActionCommand("replaceEnter");
	replaceTextField.addActionListener(this);
	panel.add(ignoreCaseField, MakeDialog.fieldConstraints);
	Box box = Box.createHorizontalBox();
	JButton searchButton = new JButton(resources.getString("search.label"));
	searchButton.setActionCommand("search");
	searchButton.addActionListener(this);
	box.add(searchButton);
	JButton replaceButton = new JButton(resources.getString("replace.label"));
	replaceButton.setActionCommand("replace");
	replaceButton.addActionListener(this);
	box.add(replaceButton);
	JButton replaceAllButton = new JButton(resources.getString("replaceAll.label"));
	replaceAllButton.setActionCommand("replaceAll");
	replaceAllButton.addActionListener(this);
	box.add(replaceAllButton);
	JButton cb = new JButton(resources.getString("close.label"));
	cb.setActionCommand("close-frame");
	closeButton = new SwingButton(cb);
	box.add(cb);
	panel.add(box, MakeDialog.fieldConstraints);
    }

    public void actionPerformed(ActionEvent evt){
	if (doc==null || caret==null) return;

	if (evt.getActionCommand()=="replace"){
	    replace();
	}
	else if (evt.getActionCommand()=="replaceAll"){
	    replaceAll();
	}
	else if(evt.getActionCommand()=="search"){
	    search();
	}
	else if(evt.getActionCommand()=="searchEnter"){
	    replaceTextField.requestFocus();
	}
	else if(evt.getActionCommand()=="replaceEnter"){
	    if (replaceTextField.getText().length()>0) 
		replace();
	    else {
		search();
	    }
	}
	//interfacePanel.close();
    }
    
    public void showDialog(DialogInterfacePanel _ip, TextComponent _textcomp){
	interfacePanel = _ip;
	textcomp = _textcomp;
	doc = _textcomp.getDocument(); 
	caret = _textcomp.getCaret();
	interfacePanel.setContent(this);
	interfacePanel.setTitle(resources.getString("title"));
	interfacePanel.getButtonManager().installButton(closeButton);
	interfacePanel.show();
    };
    
    protected void search(){
        replace=false;
	if(!searchText()) //endReached;
	    panel.getToolkit().beep(); 
    }
    protected void replace(){
	replace=true;
	if(!replaceText()) //endReached;
	    panel.getToolkit().beep();
	if(!replacePossible)
	    replaceText();
    }
    protected void replaceAll(){
	replace=true;
	caret.setDot(0);
	lastFoundPos=-1;
	replacePossible=true;
	replaceAll=true;
	doc.beginUpdate();
	try{
	    while (replaceText()){
	    }
	}
	finally {
	    doc.endUpdate(); 
	}
	replaceAll=false;
    }
  
    protected boolean searchText(){
	int searchLength = searchTextField.getText().length();
	int foundAt=0;
	if(searchLength>0){
	    if(lastFoundPos==caret.getDot())
		caret.setDot(caret.getDot()+1);
	    foundAt=findText(caret.getDot());
	    if(foundAt>=0){
		caret.setDot(foundAt+searchLength);
		caret.moveDot(foundAt);
		textcomp.requestFocus();
		return true;
	    }
	}
	caret.setDot(0);
	replacePossible=false;
	return false;
    }
	
    protected boolean replaceText(){
	int searchLength = searchTextField.getText().length();
	int foundAt=0;
	if(searchLength>0){
	    foundAt=findText(caret.getDot());
	    if(foundAt>=0){
		caret.setDot(foundAt+searchLength);
		caret.moveDot(foundAt);
		if(replacePossible){
		    textcomp.replaceSelection(replaceTextField.getText(),null);
		    caret.setDot(lastFoundPos+replaceTextField.getText().length());
		    if(!replaceAll)
			replacePossible=false;
		}
		else {
		    replacePossible=true;
		}
		textcomp.requestFocus();
		return true;
	    }
	}
	caret.setDot(0);
	replacePossible=false;
	return false;
    }
    
    public int findTextnew(int fromPos){
	Segment seg = new Segment();
	try{
	    long milstart=System.currentTimeMillis();
	    System.err.println("1 "+(System.currentTimeMillis()-milstart));
	    doc.getText(0,doc.getRootElement().getEndOffset(),seg);
	    //System.err.println("2 "+(System.currentTimeMillis()-milstart));
	    int pos = seg.toString().indexOf(searchTextField.getText(),fromPos);
	    System.err.println("3 "+(System.currentTimeMillis()-milstart));
	    lastFoundPos=pos;
	    return pos; 
	}
	catch (BadLocationException e){
	    panel.getToolkit().beep(); 
	    e.printStackTrace();
	}
	//System.err.println("Segment"+seg.toString());
	return -1;
    }
    public int findText(int fromPos){
	Segment seg = new Segment();
	String searchText = searchTextField.getText();
	int searchLength = searchText.length();
	int caretPos = caret.getDot();
	try{
	    long milstart=System.currentTimeMillis();
	    //System.err.println("1 "+(System.currentTimeMillis()-milstart));
	    doc.getText(0,doc.getRootElement().getEndOffset(),seg);
	    if(ignoreCaseField.isSelected()){
		String testString=seg.toString();
		String test;
		while(doc.getRootElement().getEndOffset()>caretPos+searchLength){
		    test=testString.substring(caretPos,caretPos+searchLength);
		    if(test.equalsIgnoreCase(searchText)){
			//System.err.println("found:"+searchText+" at Pos:"+caretPos+"replPoss:"+replacePossible);
			lastFoundPos=caretPos;
			//System.err.println("3 "+(System.currentTimeMillis()-milstart));
			return caretPos;
		    }
		    caretPos++;
		}
	    }
	    else{ // hier kann schnellere Systemroutine benutzt werden.
		int pos = seg.toString().indexOf(searchTextField.getText(),fromPos);
		//System.err.println("3 "+(System.currentTimeMillis()-milstart));
		lastFoundPos=pos;
		return pos; 
	    }
	}
	catch (BadLocationException e){
	    panel.getToolkit().beep(); 
	    e.printStackTrace();
	}
	return -1; //falls nix gefunden;
    }
    public int findTextok(int fromPos){
	Segment seg = new Segment();
	String searchText = searchTextField.getText();
	int searchLength = searchText.length();
	int caretPos = caret.getDot();
	try{
	    long milstart=System.currentTimeMillis();
	    //System.err.println("1 "+(System.currentTimeMillis()-milstart));
	    while(doc.getRootElement().getEndOffset()>caretPos+searchLength){
		doc.getText(caretPos,searchLength,seg);
		//System.err.println("2 "+(System.currentTimeMillis()-milstart));
		if(ignoreCaseField.isSelected()){
		    if(seg.toString().equalsIgnoreCase(searchText)){
			//System.err.println("found:"+searchText+" at Pos:"+caretPos+"replPoss:"+replacePossible);
			lastFoundPos=caretPos;
			return caretPos;
		    }
		}
		else{
		    if(seg.toString().equals(searchText)){
			//System.err.println("found:"+searchText+" at Pos:"+caretPos+"replPoss:"+replacePossible);
			lastFoundPos=caretPos;
			//System.err.println("3 "+(System.currentTimeMillis()-milstart));
			return caretPos;
		    }
		}
		caretPos++;
	    }
	}
	catch (BadLocationException e){
	    panel.getToolkit().beep(); 
	    e.printStackTrace();
	}
	return -1; //falls nix gefunden;
    }

    /***PanelContent*****************************/
    public boolean panelClosing() { return true; }
    public void panelClosed() {
	doc = null; caret = null;
	interfacePanel.getButtonManager().uninstallButton(closeButton);
	interfacePanel = null;
	textcomp.requestFocus();
    }
    public Component getComponent() { return panel; }
}
