package com.make.swing.text.view;

import java.awt.Component;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.Graphics;
import java.awt.Point;
import javax.swing.SizeRequirements;
import javax.swing.text.BadLocationException;
import com.make.swing.text.Element;
import com.make.swing.text.DocumentEvent;
import com.make.swing.text.ContainerChange;

public class BoxView extends CompositeView {
    /***properties***********************************/
    protected int allocX, allocY;
    protected int axis;
    protected int width, height;
    protected boolean xValid = false, yValid = false;
    protected SizeRequirements xRequest, yRequest;
    protected boolean xAllocValid = false, yAllocValid = false;
    protected int[] xOffsets, xSpans, yOffsets, ySpans;
    protected boolean minorCursorWrap = true, virtualLast = false;
    /***Constructor*************************************/
    public BoxView(Element elem, int _axis) {
	super(elem);
	axis = _axis;
	xOffsets = new int[0];
	xSpans = new int[0];
	yOffsets = new int[0];
	ySpans = new int[0];
    }
    /***CompositeView*******************************/
    protected void loadChildren(ViewFactory f) {
	super.loadChildren(f);
    }
    /***Layout**********************************************/
    public int getAxis() {
	return axis;
    }
    public void setAxis(int axis) {
	this.axis = axis;
	preferenceChanged(null, true, true);
    }
    public void layoutChanged(int axis) {
	if (axis==X_AXIS)
	    xAllocValid = false;
	else
	    yAllocValid = false;
    }
    public void replace(int offset, int length, View[] elems) {
	super.replace(offset, length, elems);
	// invalidate cache 
	int nInserted = elems.length;
	xOffsets = updateLayoutArray(xOffsets, offset, nInserted);
	xSpans = updateLayoutArray(xSpans, offset, nInserted);
	xValid = false;
	xAllocValid = false;
	yOffsets = updateLayoutArray(xOffsets, offset, nInserted);
	ySpans = updateLayoutArray(xSpans, offset, nInserted);
	yValid = false;
	yAllocValid = false;
    }
    int[] updateLayoutArray(int[] oldArray, int offset, int nInserted) {
	int n = getViewCount();
	int[] newArray = new int[n];
	System.arraycopy(oldArray, 0, newArray, 0, offset);
	System.arraycopy(oldArray, offset, 
			 newArray, offset + nInserted, n - nInserted - offset);
	return newArray;
    }
    /***View*****************************/
    public void preferenceChanged(View child, boolean width, boolean height) {
	if (width) {
	    xValid = false;
	    xAllocValid = false;
	}
	if (height) {
	    yValid = false;
	    yAllocValid = false;
	}
	super.preferenceChanged(child, width, height);
    }
    public int getResizeWeight(int axis) {
	checkRequests();
        switch (axis) {
        case X_AXIS:
	    if ((xRequest.preferred != xRequest.minimum) &&
		(xRequest.preferred != xRequest.maximum))
		return 1;
	    return 0;
        case Y_AXIS:
	    if ((yRequest.preferred != yRequest.minimum) &&
		(yRequest.preferred != yRequest.maximum))
		return 1;
            return 0;
        default:
            throw new IllegalArgumentException("Invalid axis: " + axis);
        }
    }
    public void setSize(int w, int h) {
	if (w!=width)
	    xAllocValid = false;
	if (h!=height)
	    yAllocValid = false;
	if (!xAllocValid || !yAllocValid) {
	    width = w; height = h;
	    layout(width - getLeftInset() - getRightInset(), 
		   height - getTopInset() - getBottomInset());
	}
    }
    public void paint(Graphics g, Shape allocation) {
	// alloc
	Rectangle alloc = allocation.getBounds();
	allocX = alloc.x; allocY = alloc.y;
	setSize(alloc.width, alloc.height);
	// children
	final int n = getViewCount();
	final int x = alloc.x + getLeftInset();
	final int y = alloc.y + getTopInset();
	Rectangle clip = g.getClipBounds();
	for (int i=0; i<n; i++) {
	    Rectangle r = (Rectangle)getChildAllocation(i, alloc);
	    if (r.intersects(clip)) {
		View child = getView(i);
		child.paint(g, r);
	    }
	}
    }
    public Shape getChildAllocation(int index, Shape a) {
	if (a==null) return null;
	Shape ca = super.getChildAllocation(index, a);
	if (ca!=null && !isAllocationValid()) {
	    Rectangle r = (ca instanceof Rectangle) ? 
		(Rectangle) ca : ca.getBounds();
	    if (r.width==0 && r.height==0)
		return null;
	}
	return ca;
    }
    public Shape modelToView(int pos, Shape a) throws BadLocationException {
	if (!isAllocationValid()) {
	    Rectangle alloc = a.getBounds();
	    setSize(alloc.width, alloc.height);
	}
	return super.modelToView(pos, a);
    }
    public int viewToModel(float x, float y, Shape a) {
	if (!isAllocationValid()) {
	    Rectangle alloc = a.getBounds();
	    setSize(alloc.width, alloc.height);
	}
	return super.viewToModel(x, y, a);
    }
    public float getAlignment(int axis) {
	checkRequests();
	switch (axis) {
	case X_AXIS:
	    return xRequest.alignment;
	case Y_AXIS:
	    return yRequest.alignment;
	default:
	    throw new IllegalArgumentException("Invalid axis: " + axis);
	}
    }
    public float getPreferredSpan(int axis) {
	checkRequests();
	switch (axis) {
	case X_AXIS:
	    return ((float)xRequest.preferred) + getLeftInset() + getRightInset();
	case Y_AXIS:
	    return ((float)yRequest.preferred) + getTopInset() + getBottomInset();
	default:
	    throw new IllegalArgumentException("Invalid axis: " + axis);
	}
    }
    public float getMinimumSpan(int axis) {
	checkRequests();
	switch (axis) {
	case X_AXIS:
	    return ((float)xRequest.minimum) + getLeftInset() + getRightInset();
	case Y_AXIS:
	    return ((float)yRequest.minimum) + getTopInset() + getBottomInset();
	default:
	    throw new IllegalArgumentException("Invalid axis: " + axis);
	}
    }
    public float getMaximumSpan(int axis) {
	checkRequests();
	switch (axis) {
	case X_AXIS:
	    return ((float)xRequest.maximum) + getLeftInset() + getRightInset();
	case Y_AXIS:
	    return ((float)yRequest.maximum) + getTopInset() + getBottomInset();
	default:
	    throw new IllegalArgumentException("Invalid axis: " + axis);
	}
    }
    /***internal****************************/
    protected boolean isAllocationValid() {
	return (xAllocValid && yAllocValid);
    }
    /***CompositeView***************************/
    protected boolean isBefore(int x, int y, Rectangle innerAlloc) {
	return axis==X_AXIS
	    ?x<innerAlloc.x
	    :y<innerAlloc.y;
    }
    protected boolean isAfter(int x, int y, Rectangle innerAlloc) {
	return axis==X_AXIS
	    ?x>(innerAlloc.width+innerAlloc.x)
	    :y>(innerAlloc.height+innerAlloc.y);
    }
    protected View getViewAtPoint(int x, int y, Rectangle alloc) {
	final int n = getViewCount();
	if (n==0) return null;
	if (axis==X_AXIS) {
	    if (x<(alloc.x+xOffsets[0])) {
		childAllocation(0, alloc);
		return getView(0);
	    }
	    for (int i=0; i<n; i++)
		if (x<(alloc.x+xOffsets[i])) {
		    childAllocation(i-1, alloc);
		    return getView(i-1);
		}
	    childAllocation(n-1, alloc);
	    return getView(n-1);
	} else {
	    if (y<(alloc.y+yOffsets[0])) {
		childAllocation(0, alloc);
		return getView(0);
	    }
	    for (int i=0; i<n; i++)
		if (y<(alloc.y+yOffsets[i])) {
		    childAllocation(i-1, alloc);
		    return getView(i-1);
		}
	    childAllocation(n-1, alloc);
	    return getView(n-1);
	}
    }
    protected void childAllocation(int index, Rectangle alloc) {
	alloc.x += xOffsets[index];
	alloc.y += yOffsets[index];
	alloc.width = xSpans[index];
	alloc.height = ySpans[index];
    }
    /***Layout*********************************************/
    protected void layout(int width, int height) {
	checkRequests();
	if (axis == X_AXIS) {
	    if (!xAllocValid)
		layoutMajorAxis(width, X_AXIS, xOffsets, xSpans);
	    if (!yAllocValid)
		layoutMinorAxis(height, Y_AXIS, yOffsets, ySpans);
	} else {
	    if (!xAllocValid)
		layoutMinorAxis(width, X_AXIS, xOffsets, xSpans);
	    if (!yAllocValid)
		layoutMajorAxis(height, Y_AXIS, yOffsets, ySpans);
	}
	xAllocValid = true;
	yAllocValid = true;
	final int n=getViewCount();
	for (int i=0; i<n; i++)
	    getView(i).setSize(xSpans[i], ySpans[i]);
    }
    protected void layoutMajorAxis(int targetSpan, int axis, int[] offsets, int[] spans) {
	SizeRequirements.calculateTiledPositions(targetSpan, null, getChildRequests(axis),
						 offsets, spans);
    }
    protected void layoutMinorAxis(int targetSpan, int axis, int[] offsets, int[] spans) {
	int n = getViewCount();
	for (int i = 0; i < n; i++) {
	    View v = getView(i);
	    final int min = (int) v.getMinimumSpan(axis);
	    final int max = (int) v.getMaximumSpan(axis);
	    if (max<targetSpan) {
		float align = v.getAlignment(axis);
		offsets[i] = (int) ((targetSpan - max) * align);
		spans[i] = max;
	    } else {
		offsets[i] = 0;
		spans[i] = Math.max(min, targetSpan);
	    }
	}
    }
    /***Requirements*****************************************/
    protected SizeRequirements calculateMajorAxisRequirements(int axis, SizeRequirements r) {
	float min = 0;
	float pref = 0;
	float max = 0;
	final int n = getViewCount();
	if (n==0)
	    max = Integer.MAX_VALUE;
	else
	    for (int i=0; i<n; i++) {
		View v = getView(i);
		min += v.getMinimumSpan(axis);
		pref += v.getPreferredSpan(axis);
		max += v.getMaximumSpan(axis);
	    }
	if (r==null)
	    r = new SizeRequirements();
	r.alignment = 0.5f;
	r.minimum = (int)min;
	r.preferred = (int)pref;
	r.maximum = (int)max;
	return r;
    }
    protected SizeRequirements calculateMinorAxisRequirements(int axis, SizeRequirements r) {
	int min = 0;
	int pref = 0;
	int max = 0;
	final int count = getViewCount();
	if (count==0)
	    max = Integer.MAX_VALUE;
	else
	    for (int i=0; i<count; i++) {
		View v = getView(i);
		min = Math.max((int)v.getMinimumSpan(axis), min);
		pref = Math.max((int)v.getPreferredSpan(axis), pref);
		max = Math.max((int)v.getMaximumSpan(axis), max);
	    }
	if (r==null) {
	    r = new SizeRequirements();
	    r.alignment = 0.5f;
	}
	r.preferred = pref;
	r.minimum = min;
	r.maximum = max;
	return r;
    }
    protected SizeRequirements[] getChildRequests(int axis) {
	int n = getViewCount();
	SizeRequirements[] reqs = new SizeRequirements[n];
	for (int i = 0; i < n; i++) {
	    View v = getView(i);
	    int min = (int) v.getMinimumSpan(axis);
	    int pref = (int) v.getPreferredSpan(axis);
	    int max = (int) v.getMaximumSpan(axis);
	    float a = v.getAlignment(axis);
	    reqs[i] = new SizeRequirements(min, pref, max, a);
	}
	return reqs;
    }
    protected void checkRequests() {
	if (axis==X_AXIS) {
	    if (!xValid)
		xRequest = calculateMajorAxisRequirements(X_AXIS, xRequest);
	    if (!yValid)
		yRequest = calculateMinorAxisRequirements(Y_AXIS, yRequest);
	} else {
	    if (!xValid)
		xRequest = calculateMinorAxisRequirements(X_AXIS, xRequest);
	    if (!yValid)
		yRequest = calculateMajorAxisRequirements(Y_AXIS, yRequest);
	}
	yValid = true;
	xValid = true;
    }
    /***Magic********************************************/
    protected int getViewIndexAtPixel(int x) {
	final int count = getViewCount();
	if (count<2) return 0;
	final int[] offsets = getAxis()==X_AXIS ? xOffsets : yOffsets;
	if (x<offsets[1]) return 0;
	for (int z=1; z<count; z++)
	    if (x<offsets[z])
		return z-1;
	return count-1;
    }
    protected int getViewIndexAtMagic() throws BadLocationException {
	return getViewIndexAtPixel(getAxis()==X_AXIS?getMagicX()-allocX:getMagicY()-allocY);
    }
    /***FindNext*******************************************/
    protected int findBackwardView(int vIndex, int pos, int direction) 
	throws BadLocationException {
	for (; vIndex>=0; vIndex--) {
	    View v = getView(vIndex);
	    pos = v.getNextVisualPositionFrom(pos, direction);
	    if (pos!=-1) return pos;
	}
	return -1;
    }
    protected int findForwardView(int vIndex, int pos, int direction) 
	throws BadLocationException {
	final int count = getViewCount();
	for (; vIndex<count; vIndex++) {
	    View v = getView(vIndex);
	    pos = v.getNextVisualPositionFrom(pos, direction);
	    if (pos!=-1) return pos;
	}
	return -1;
    }
    protected int findNextView(int vIndex, int pos,
			       boolean forward, int direction) 
	throws BadLocationException {
	return forward 
	    ?findForwardView(vIndex, pos, direction)
	    :findBackwardView(vIndex, pos, direction);
    }
    /***BoxView***********************************************/
    protected int getNextMajorAxisPosition(int pos, boolean forward, int direction) 
	throws BadLocationException {
	if (DUMP_NEXTVISUAL)
	    System.err.println(this+": Entering nextMajor pos="+pos+" fwd="+forward);
	if (virtualLast) {
	    if (!forward && pos==-1)
		return getEndOffset()-1;
	    if (forward && pos>=(getEndOffset()-1))
		return -1;
	}
	final int count = getViewCount();
	int vIndex;
	if (count==0)
	    return pos==-1 ? getStartOffset() : -1;
	else if (pos==-1)
	    vIndex = forward ? 0 : count-1;
	else
	    vIndex = getViewIndexAtPosition(pos);
	int res = findNextView(vIndex, pos, forward, direction);
	if (virtualLast && res==-1 && forward)
	    res = getEndOffset()-1;
	if (DUMP_NEXTVISUAL)
	    System.err.println(this+": nextMajor pos="+pos+" idx="+vIndex+" res="+res);
	return res;
    }
    protected int getNextMinorAxisPosition(int pos, boolean forward, int direction)
	throws BadLocationException {
	if (DUMP_NEXTVISUAL)
	    System.err.println(this+": Entering nextMinor pos="+pos+" fwd="+forward);
	final int count = getViewCount();
	int np;
	if (count==0)
	    return pos==-1 ? getStartOffset() : -1;
	else if (minorCursorWrap && pos==-1 && count>=2) {
	    np = -1;
	    int idx = forward ? 0 : count-1;
	    View v = getView(idx);
	    np = v.getNextVisualPositionFrom(-1, direction);
	} else {
	    int vIndex = pos==-1
		?getViewIndexAtMagic()
		:getViewIndexAtPosition(pos);
	    View v = getView(vIndex);
	    np = v.getNextVisualPositionFrom(pos, direction);
	}
	if (minorCursorWrap && count>=2 && np==-1) {
	    int idx = pos==-1
		? (forward ? 0 : count-1)
		: getViewIndexAtPosition(pos);
	    int res = forward
		?findForwardView(idx+1, np, direction)
		:findBackwardView(idx-1, np, direction);
	    if (DUMP_NEXTVISUAL) {
		System.err.println(this+": nextMinor pos="+pos+" fwd="+forward+" res="+res);
	    }
	    return res;
	}
	if (DUMP_NEXTVISUAL)
	    System.err.println(this+": nextMinor pos="+pos+" fwd="+forward+" res="+np);
	return np;
    }
    protected int getNextNorthSouthVisualPositionFrom(int pos, int direction)
	throws BadLocationException {
	return getAxis()==Y_AXIS
	    ?getNextMajorAxisPosition(pos, direction==SOUTH, direction)
	    :getNextMinorAxisPosition(pos, direction==SOUTH, direction);
    }
    protected int getNextEastWestVisualPositionFrom(int pos, int direction)
	throws BadLocationException {
	return getAxis()==X_AXIS
	    ?getNextMajorAxisPosition(pos, direction==EAST, direction)
	    :getNextMinorAxisPosition(pos, direction==EAST, direction);
    }
}
