package com.make.swing.text.view;

import java.util.Vector;
import com.make.util.Debug;

public abstract class BreakUtil {
    public static class BreakInfo {
	public View view;
	public int weight;
	public int index;
	public float span;
    }
    public static BreakInfo getBreakWeight(BreakableView bv, int axis, int position, 
					   float x, float len) {
	BreakInfo bi = new BreakInfo();
	bi.weight = View.BadBreakWeight;
	bi.index = -1;
	bi.span = 0;
	bi.view = null;
	float span = 0;
	final int first = position==-1?0:bv.getPoolViewIndexAt(position);
	if (Debug.DEBUG && first<0)
	    System.err.println("BreakUtil::getBreakWeight bv="+bv+" pos="+position);
	final int last = bv.getPoolViewCount()-1;
        for (int i=first; i<=last && span<len; i++) {
            View v = bv.getPoolView(i);
	    if (position>v.getStartOffset()) {
		v = (View)v.createFragment(position, v.getEndOffset());
		v.setParent(bv.getPoolParent());
	    }
	    int w = v.getBreakWeight(axis, position, x+span, len-span);
	    if (w>=bi.weight) {
		bi.view = v;
		bi.weight = w;
		bi.index = i;
		bi.span = span;
		if (w>=View.ForcedBreakWeight)
		    return bi;
	    }
            span += v.getPreferredSpan(axis);
	    position = v.getEndOffset();
        }
	return bi;
    }
    public static View createFragment(BreakableView bv, int p0, int p1) {
	if (p0<=bv.getPoolParent().getStartOffset() && p1>=bv.getPoolParent().getEndOffset())
	    return bv.getPoolParent();
	int pos = p0;
	Vector views = new Vector();
	final int first = bv.getPoolViewIndexAt(p0);
	final int last = bv.getPoolViewCount()-1;
	for (int idx=first; pos<p1 && idx<=last; idx++) {
	    View v = bv.getPoolView(idx);
	    final int end = Math.min(p1, v.getEndOffset()); 
	    v = (View)bv.getPoolView(idx).createFragment(pos, end);
	    views.addElement(v);
	    pos = end;
	}
	return bv.createView(views);
    }
    public static View breakView(BreakableView bv, int axis, int offset, float x, float len) {
	if (bv.getPoolViewCount()==0)
	    return bv.getPoolParent();
	BreakInfo bi = BreakUtil.getBreakWeight(bv, axis, offset, x, len);
	if (bi.view==null) return bv.getPoolParent();
	Vector views = new Vector();
	final int first = bv.getPoolViewIndexAt(offset);
	if (bi.index==first)
	    views.addElement(bi.view.breakView(axis, offset, x, len));
	else {
	    View v = bv.getPoolView(first);
	    float span;
	    // frag first
	    if (offset>v.getStartOffset()) {
		v = (View)v.createFragment(offset, v.getEndOffset());
		v.setParent(bv.getPoolParent());
	    }
	    if (v.getEndOffset()>offset) {
		span = v.getPreferredSpan(axis);
		x += span; len -= span; offset = v.getEndOffset();
		views.addElement(v); 
	    }
	    // middle
	    for (int z=first+1; z<bi.index; z++) {
		v = bv.getPoolView(z);
		if (v.getEndOffset()>offset) {
		    views.addElement(bv.getPoolView(z));
		    span = v.getPreferredSpan(axis);
		    x += span; len -= span;
		    offset = v.getEndOffset();
		}
	    }
	    // last
	    v = bv.getPoolView(bi.index);
	    if (!View.isVertical(v)) {
		span = v.getPreferredSpan(axis);
		if (span>len) {
		    v = v.breakView(axis, offset, x, len);
		    v.setParent(bv.getPoolParent());
		}
		if (v.getEndOffset()>offset)
		    views.addElement(v);
	    }
	}
	if (views.isEmpty())
	    return new EmptyView(bv.getPoolParent().getElement(), offset);
	else
	    return bv.createView(views);
    }
    public static View fillView(BreakableView bv, int axis, int position, 
				float x, float spanLeft) {
	final float preX = x;
	final float availableSpan = spanLeft;
	boolean forcedBreak = false;
	Vector row = new Vector();
	int idx = bv.getPoolViewIndexAt(position);
	final int last = bv.getPoolViewCount()-1;
	for (; idx<=last && spanLeft>0; idx++) {
	    // get view
	    View v = bv.getPoolView(idx);
	    if (v instanceof GenericBreakView) {
		//System.err.println("found GenericBreakView!");
		row.addElement(v); return bv.createView(row);
	    }
	    if (View.isVertical(v)) {
		if (row.size()==0) {
		    row.addElement(v); return bv.createView(row);
		} else
		    break;
	    }
	    if (position>v.getStartOffset()) {
		v = v.createFragment(position, v.getEndOffset());
		v.setParent(bv.getPoolParent());
	    }
            float chunkSpan = v.getPreferredSpan(axis);
	    // forced?
	    int w = v.getBreakWeight(axis, position, x, spanLeft);
	    if (w>=View.ForcedBreakWeight) {
		v = v.breakView(axis, position, x, spanLeft);
		v.setParent(bv.getPoolParent());
		if (View.isVertical(v)) {
		    if (row.size()==0)
			row.addElement(v);
		    return bv.createView(row);
		}
		chunkSpan = v==null ? 0 : v.getPreferredSpan(axis);
		forcedBreak = true;
	    }
	    // add to row
	    if (v!=null && v.getEndOffset()>position) {
		row.addElement(v);
		position = v.getEndOffset();
		// calc
		spanLeft -= chunkSpan;
		x += chunkSpan;
	    }
	    if (forcedBreak)
		break;
        }
        if (spanLeft<0)
            adjustView(bv, row, axis, preX, availableSpan);
	if (row.size()==0 && idx<=last) {
            View v = bv.getPoolView(bv.getPoolViewIndexAt(position));
	    row.addElement(v.createFragment(position, v.getEndOffset()));
	}
	return bv.createView(row);
    }
    protected static void adjustView(BreakableView bv, Vector row, int axis, 
				     float x, float desiredSpan) {
        final int n = row.size();
        float span = 0;
        int bestWeight = View.BadBreakWeight;
        float bestSpan = 0;
        int bestIndex = -1;
        View v;
        for (int i=0; i<n && span<desiredSpan; i++) {
            v = (View)row.elementAt(i);
            float spanLeft = desiredSpan - span;
            int w = v.getBreakWeight(axis, v.getStartOffset(), x+span, spanLeft);
            if (w >= bestWeight) {
                bestWeight = w;
                bestIndex = i;
                bestSpan = span;
                if (w>=View.ForcedBreakWeight)
                    break;
            }
            span += v.getPreferredSpan(axis);
        }
        if (bestIndex<0)
            return;
        float spanLeft = desiredSpan - bestSpan;
	View ev = (View)row.elementAt(bestIndex);
        v = ev.breakView(axis, ev.getStartOffset(), x+bestSpan, spanLeft);
	while (row.size()>bestIndex)
	    row.removeElementAt(row.size()-1);
	row.addElement(v);
    }
}
