package com.make.swing.text.view;

import java.awt.Shape;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Graphics;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Hashtable;
import javax.swing.text.Utilities;
import javax.swing.SwingConstants;
import javax.swing.text.BadLocationException;
import com.make.util.Debug;
import com.make.util.Strings;
import com.make.swing.text.Element;
import com.make.swing.text.Document;
import com.make.swing.text.TextComponent;
import com.make.swing.text.TextUI;
import com.make.swing.text.Caret;
import com.make.swing.text.DocumentEvent;

public abstract class View implements SwingConstants {
    /***CONST*****************************/
    public static final boolean DUMP_NEXTVISUAL = false;
    /***static*******************************/
    // break
    public static final int BadBreakWeight = 0;
    public static final int GoodBreakWeight = 1000;
    public static final int ExcellentBreakWeight = 2000;
    public static final int ForcedBreakWeight = 3000;
    // axis
    public static final int X_AXIS = HORIZONTAL;
    public static final int Y_AXIS = VERTICAL;
    /***Static*****************************/
    public static boolean isVertical(View v) {
	return v.isVertical();
    }
    /***properties*************************/
    private View parent;
    private Element elem;
    /***Constructor*************************/
    public View(Element _e) {
	elem = _e;
    }
    public String toString() {
	return "["+super.toString()+": start="+getStartOffset()+" end="+getEndOffset()+" el="+getElement()+"]";
    }
    /***Tree*********************************/
    public View getParent() { return parent; }
    public void setParent(View parent) {
	this.parent = parent;
    }
    public Document getDocument() { return (Document)elem.getDocument(); }
    public int getStartOffset() { return elem.getStartOffset(); }
    public int getEndOffset() { return elem.getEndOffset(); }
    public Element getElement() { return elem; }
    public TextComponent getContainer() {
	View v = getParent();
	return v!=null ? v.getContainer() : null;
    }
    public ViewFactory getViewFactory() {
	View v = getParent();
	return v!=null ? v.getViewFactory() : null;
    }
    /***View*******************************/
    public boolean isVertical() {
	return false; //TODO
	//return getViewCount()==1 && isVertical(getView(0));
    }
    protected final int getMagicX(int pos) 
	throws BadLocationException {
	TextComponent text = getContainer();
	Caret c = text.getCaret();
	Point magicPoint = c==null?null:c.getMagicCaretPosition();
	if (magicPoint==null) {
	    Shape posBounds = text.getUI().modelToView(text, pos);
	    return posBounds==null ? 0 : posBounds.getBounds().x;
	} else
	    return magicPoint.x;
    }
    protected final int getMagicY(int pos) 
	throws BadLocationException {
	TextComponent text = getContainer();
	Caret c = text.getCaret();
	Point magicPoint = c==null?null:c.getMagicCaretPosition();
	if (magicPoint==null) {
	    Shape posBounds = text.getUI().modelToView(text, pos);
	    return posBounds==null ? 0 : posBounds.getBounds().y;
	} else
	    return magicPoint.y;
    }
    protected final int getMagicX() throws BadLocationException {
	TextComponent text = getContainer();
	Caret c = text.getCaret();
	Point magicPoint = (c!=null) ? c.getMagicCaretPosition() : null;
	return magicPoint==null ? 0 : magicPoint.x;
    }
    protected final int getMagicY() throws BadLocationException {
	TextComponent text = getContainer();
	Caret c = text.getCaret();
	Point magicPoint = (c!=null) ? c.getMagicCaretPosition() : null;
	return magicPoint==null ? 0 : magicPoint.y;
    }
    /***Overwrite******************************/
    public int getBreakWeight(int axis, int offset, float x, float len) {
	return len>getPreferredSpan(axis)
	    ?GoodBreakWeight:BadBreakWeight;
    }
    public View breakView(int axis, int offset, float x, float len) {
	return this;
    }
    public View createFragment(int p0, int p1) {
	return this;
    }
    /***Abstract*******************************/
    public abstract void documentChanged(DocumentEvent evt, Shape a, ViewFactory vf);
    /***Span**********************************/
    public abstract float getPreferredSpan(int axis);
    public float getMinimumSpan(int axis) {
	return getResizeWeight(axis)==0?getPreferredSpan(axis):0;
    }
    public float getMaximumSpan(int axis) {
	return getResizeWeight(axis)==0?getPreferredSpan(axis):Integer.MAX_VALUE;
    }
    public float getAlignment(int axis) { return 0.5f; }
    /***Update*************************************/
    public void preferenceChanged(View child, boolean width, boolean height) {
	View parent = getParent();
	if (parent!=null)
	    parent.preferenceChanged(this, width, height);
    }
    /***Paint***************************************/
    public abstract void paint(Graphics g, Shape allocation);
    /***Util***************************************/
    public final View getRootView() {
	return parent==null
	    ?this
	    :parent.getRootView();
    }
    public final int getPositionAbove(int offs, int x) 
	throws BadLocationException {
	TextComponent target = getContainer();
	TextUI ui = target.getUI();
	int lastOffs = getRootView().getRowStart(offs)-1;
	int bestSpan = Short.MAX_VALUE;
	int y = 0;
	Rectangle r = null;
	if (lastOffs >= 0) {
	    r = ui.modelToView(target, lastOffs);
	    y = r.y;
	}
	while (r!=null && y==r.y) {
	    int span = Math.abs(r.x-x);
	    if (span<bestSpan) {
		offs = lastOffs;
		bestSpan = span;
	    }
	    lastOffs -= 1;
	    r = lastOffs>=0
		?ui.modelToView(target, lastOffs)
		:null;
	}
	return offs;
    }
    public final int getPositionBelow(int offs, int x) 
	throws BadLocationException {
	TextComponent target = getContainer();
	TextUI ui = target.getUI();
	int lastOffs = getRootView().getRowEnd(offs)+1;
	int bestSpan = Short.MAX_VALUE;
	int n = target.getDocument().getRootElement().getLength();
	int y = 0;
	Rectangle r = null;
	if (lastOffs <= n) {
	    r = ui.modelToView(target, lastOffs);
	    y = r.y;
	}
	while ((r != null) && (y == r.y)) {
	    int span = Math.abs(x - r.x);
	    if (span < bestSpan) {
		offs = lastOffs;
		bestSpan = span;
	    }
	    lastOffs += 1;
	    r = lastOffs<=n
		?ui.modelToView(target, lastOffs)
		:null;
	}
	return offs;
    }
    /***Control**************************************/
    public int getRowStart(int pos) { return -1; }
    public int getRowEnd(int pos) { return -1; }
    public int getNextVisualPositionFrom(int pos, int direction)
	throws BadLocationException {
	if (Debug.DEBUG)
	    System.err.println("WARNING: in View::getNextVisualPositionFrom this="+this);
	switch (direction) {
	case NORTH: {
	    TextComponent target = getContainer();
	    TextUI ui = target.getUI();
	    Rectangle r = ui.modelToView(target, pos);
	    pos = getPositionAbove(pos, r.x);
	}
	break;
	case SOUTH: {
	    TextComponent target = getContainer();
	    TextUI ui = target.getUI();
	    Rectangle r = ui.modelToView(target, pos);
	    pos = getPositionBelow(pos, r.x);
	}
	break;
	case WEST:
	    pos = pos==-1
		?Math.max(0, getEndOffset()-1)
		:Math.max(0, pos-1);
	    break;
	case EAST:
	    pos = pos==-1
		?getStartOffset()
		:Math.min(pos+1, getDocument().getRootElement().getLength());
	    break;
	default:
	    throw new IllegalArgumentException("Bad direction: " + direction);
	}
	return pos;
    }
    public abstract Shape modelToView(int pos, Shape a)
	throws BadLocationException;
    public Shape modelToView(int p0, int p1, Shape a) 
	throws BadLocationException {
	Shape s0 = modelToView(p0, a);
	Shape s1;
	if (p1==getEndOffset()) {
	    try {
		s1 = modelToView(p1, a);
	    } catch (BadLocationException ble) {
		s1 = null;
	    }
	    if (s1 == null) {
		Rectangle alloc = (a instanceof Rectangle)
		    ?(Rectangle)a:a.getBounds();
		s1 = new Rectangle(alloc.x+alloc.width-1, alloc.y,
				   1, alloc.height);
	    }
	} else
	    s1 = modelToView(p1, a);
	Rectangle r0 = s0.getBounds();
	Rectangle r1 = (s1 instanceof Rectangle)
	    ?(Rectangle)s1:s1.getBounds();
	if (r0.y!=r1.y) {
	    Rectangle alloc = (a instanceof Rectangle) ? (Rectangle)a :
		              a.getBounds();
	    r0.x = alloc.x;
	    r0.width = alloc.width;
	}
	r0.add(r1);
	return r0;
    }
    public abstract int viewToModel(float x, float y, Shape a);
    public int getResizeWeight(int axis) { return 0; }
    public void setSize(int width, int height) {}
}
