package com.make.util;

import java.io.IOException;
import java.io.EOFException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.OutputStreamWriter;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.Writer;
import java.net.URL;
import java.net.HttpURLConnection;
import java.util.Hashtable;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

public abstract class Streams {
    private static final byte[] buffer = new byte[8192];
    private static boolean bufferInUse = false;
    /***Static******************************/
    public static void copy(InputStream is, OutputStream os, int maxLen)
	throws IOException {
	int ch;
	while (maxLen-->0 && (ch=is.read())!=-1)
	    os.write(ch);
    }
    public static void copy(InputStream is, OutputStream os)
	throws IOException {
	if (bufferInUse) {
	    int ch;
	    while ((ch=is.read())!=-1)
		os.write(ch);
	} else 
	    try {
		bufferInUse = true;
		synchronized(buffer) {
		    for (;;) {
			int len = is.read(buffer);
			if (len==-1) break;
			os.write(buffer, 0, len);
		    }
		}
	    } finally {
		bufferInUse = false;
	    }
    }
    public static void copy(InputStream is, OutputStream os1, OutputStream os2)
	throws IOException {
	int ch;
	while ((ch=is.read())!=-1) {
	    os1.write(ch);
	    os2.write(ch);
	}
    }
    public static void copy(Reader is, Writer os)
	throws IOException {
	int ch;
	while ((ch=is.read())!=-1)
	    os.write(ch);
    }
    public static byte[] charArrayToByteArray(char[] src, String encoding) 
	throws IOException {
	ByteArrayOutputStream baos = new ByteArrayOutputStream(src.length*2);
	Writer wr = new OutputStreamWriter(baos, encoding);
	for (int z=0; z<src.length; z++)
	    wr.write((int)src[z]);
	wr.close();
	return baos.toByteArray();
    }
    public static byte[] inputStreamToByteArray(InputStream is, int maxLen) 
	throws IOException {
	ByteArrayOutputStream baos = new ByteArrayOutputStream();
	copy(is, baos, maxLen);
	return baos.toByteArray();
    }
    public static void append(StringBuffer sb, Reader rd) 
	throws IOException {
	int ch;
	while ((ch=rd.read())!=-1)
	    sb.append((char)ch);
    }
    public static void append(StringBuffer sb, InputStream is, String enc) 
	throws IOException {
	append(sb, new InputStreamReader(is, enc));
    }
    public static String readWord(Reader rd) throws IOException {
	StringBuffer sb = new StringBuffer();
	int ch = rd.read();
	while (ch!=-1 && Character.isWhitespace((char)ch))
	    ch = rd.read();
	if (ch==-1)
	    return null;
	if (ch=='\'' || ch=='"') {
	    int last = ch;
	    while ((ch=rd.read())!=-1 && ch!=last)
		sb.append((char)ch);
	    if (ch==-1)
		throw new EOFException();
	} else {
	    sb.append((char)ch);
	    while ((ch=rd.read())!=-1 && !Character.isWhitespace((char)ch))
		sb.append((char)ch);
	}
	return sb.toString();
    }
    public static String readResult(InputStream is) throws IOException {
	StringBuffer sb = new StringBuffer();
	append(sb, is, "ISO-8859-1");
	return Strings.trim(sb.toString());
    }
    public static String readResult(Reader rd) throws IOException {
	StringBuffer sb = new StringBuffer();
	append(sb, rd);
	return Strings.trim(sb.toString());
    }
    public static String getReasonMessage(HttpURLConnection c, IOException ex,
					  ResourceBundle msg) {
	String reason = c.getHeaderField("Reason");
	if (reason==null)
	    try {
		reason = c.getResponseMessage();
		if (reason!=null)
		    reason = Strings.trim(Strings.translate(reason.toUpperCase(), ' ', '_'));
	    } catch (IOException e) {
	    }
	if (reason==null) 
	    try {
		reason = ""+c.getResponseCode()+" "+c.getResponseMessage();
	    } catch (IOException e) {
	    }
	if (reason==null) 
	    return ex==null ? null : ex.getLocalizedMessage();
	try {
	    return msg.getString(reason+".message");
	} catch (MissingResourceException e) {
	    return reason;
	}
    }
    public static String call(URL url, ResourceBundle msg) throws IOException {
	if ("http".equals(url.getProtocol())) {
	    HttpURLConnection c = (HttpURLConnection)url.openConnection();
	    try {
		return readResult(c.getInputStream());
	    } catch (IOException e) {
		throw new IOException(getReasonMessage(c, e, msg));
	    }
	} else
	    return readResult(url.openStream());
    }
    public static Hashtable decodeAssignments(Reader rd, Hashtable ht) 
	throws IOException {
	if (ht==null) ht = new Hashtable();
	int ch;
	String name = null;
	StringBuffer sb = new StringBuffer();
	while ((ch=rd.read())!=-1) {
	    switch (ch) {
	    case '\r': break;
	    case '\n':
		if (name==null)
		    ht.put(sb.toString(), "");
		else {
		    ht.put(name, sb.toString());
		    name = null;
		}
		sb.setLength(0);
		break;
	    case '=':
		if (name==null) {
		    name = sb.toString();
		    sb.setLength(0);
		    break;
		}
	    default:
		sb.append((char)ch);
	    }
	}
	if (sb.length()>0 || name!=null) {
	    if (name==null)
		ht.put(sb.toString(), "");
	    else
		ht.put(name, sb.toString());
	}
	return ht;
    }
}
