package com.make.util;

import java.util.StringTokenizer;
import java.util.Vector;
import java.util.Date;
import java.util.Locale;
import java.text.ParseException;
import java.text.DateFormat;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;

public abstract class Strings {
    /***CONST********************************/
    public static final int DATETIME = 0;
    public static final int DATE = 1;
    public static final int TIME = 2;
    /***static********************************/
    public static final DateFormat[] storeDateFormats = {
	new SimpleDateFormat("yyyy/MM/dd HH:mm"),
	new SimpleDateFormat("yyyy/MM/dd"),
	new SimpleDateFormat("HH:mm"),
    };
    public static DateFormat[] displayDateFormats = {
	DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.SHORT),
	DateFormat.getDateInstance(DateFormat.SHORT),
	DateFormat.getTimeInstance(DateFormat.SHORT),
    };
    public static NumberFormat storeIntegerFormat
	= NumberFormat.getNumberInstance(Locale.US);
    public static NumberFormat displayIntegerFormat
	= NumberFormat.getNumberInstance();
    static {
	storeIntegerFormat.setParseIntegerOnly(true);
	storeIntegerFormat.setGroupingUsed(false);
	displayIntegerFormat.setParseIntegerOnly(true);
    }
    /***Static*******************************/
    public static String trim(String s) {
	if (s==null) return null;
	s = s.trim();
	return s.length()==0?null:s;
    }
    public static String normalize(String s, boolean preserveLeft, 
				   boolean preserveRight) {
	if (s==null || "".equals(s)) return "";
	StringBuffer sb = new StringBuffer();
	char[] data = s.toCharArray();
	int z = 0;
	if (!preserveLeft)
	    while (z<data.length && Character.isWhitespace(data[z])) z++;
	boolean wasSpace = false;
	for (; z<data.length; z++) {
	    if (Character.isWhitespace(data[z]))
		wasSpace = true;
	    else {
		if (wasSpace) {
		    sb.append(' ');
		    wasSpace = false;
		}
		sb.append(data[z]);
	    }
	}
	if (wasSpace && preserveRight)
	    sb.append(' ');
	return sb.toString();
    }
    public static String normalize(String s) {
	return normalize(s, false, false);
    }
    public static boolean equals(String a, String b) {
	return a==null
	    ?b==null
	    :a.equals(b);
    }
    public static String[] tokenize(String input) {
	Vector v = new Vector();
	StringTokenizer t = new StringTokenizer(input);
	while (t.hasMoreTokens())
	    v.addElement(t.nextToken());
	String[] cmd = new String[v.size()];
	for (int i = 0; i < cmd.length; i++)
	    cmd[i] = (String) v.elementAt(i);
	return cmd;
    }
    public static int lookup(Object[] list, Object s) {
	for (int z=0; z<list.length; z++)
	    if (s.equals(list[z]))
		return z;
	return -1;
    }
    public static int lookup(String[] list, String s) {
	s = trim(s);
	if (s==null) return -1;
	for (int z=0; z<list.length; z++)
	    if (s.equals(list[z]))
		return z;
	return -1;
    }
    public static int lookup(byte[] list, byte s) {
	for (int z=0; z<list.length; z++)
	    if (s==list[z])
		return z;
	return -1;
    }
    public static Object translate(Object[] list1, Object[] list2, Object o) {
	if (Debug.DEBUG && list1.length!=list2.length)
	    throw new IllegalArgumentException();
	final int idx = lookup(list1, o);
	return idx==-1 ? null : list2[idx];
    }
    public static String translate(String[] list1, String[] list2, String s) {
	if (Debug.DEBUG && list1.length!=list2.length)
	    throw new IllegalArgumentException();
	final int idx = lookup(list1, s);
	return idx==-1 ? null : list2[idx];
    }
    public static byte translate(String[] list1, byte[] list2, String s, byte def) {
	final int idx = lookup(list1, s);
	return idx==-1 ? def : list2[idx];
    }
    public static String translate(byte[] list1, String[] list2, byte s, String def) {
	final int idx = lookup(list1, s);
	return idx==-1 ? def : list2[idx];
    }
    public static String translate(String s, char a, char b) {
	StringBuffer sb = new StringBuffer(s.length());
	for (int z=0; z<s.length(); z++) {
	    char ch = s.charAt(z);
	    if (ch==a)
		ch = b;
	    sb.append(ch);
	}
	return sb.toString();
    }
    public static String escapeURLParameter(String s) {
	final int len = s.length();
	StringBuffer sb = new StringBuffer(len+8);
	for (int z=0; z<len; z++) {
	    final char ch = s.charAt(z);
	    if ((ch>='a' && ch<='z') ||
		(ch>='A' && ch<='Z') ||
		(ch>='0' && ch<='9'))
		sb.append(ch);
	    else if (ch==' ')
		sb.append('+');
	    else {
		sb.append('%');
		int b;
		try {
		    b = new String(""+ch).getBytes("ISO-8859-1")[0];
		} catch (Exception e) {
		    b = (byte)(((int)ch)%256);
		}
		if (b<0) b += 256;
		int digit;
		digit = (b/16)%16;
		if (digit>=10)
		    sb.append((char)('A'+digit-10));
		else
		    sb.append((char)('0'+digit));
		digit = b%16;
		if (digit>=10)
		    sb.append((char)('A'+digit-10));
		else
		    sb.append((char)('0'+digit));
	    }
	}
	return sb.toString();
    }
    public static String escapeXMLText(String s) {
	if (s==null) return "";
	final int len = s.length();
	StringBuffer str = new StringBuffer((len*5)/4);
	for (int i=0; i<len; i++) {
	    final char ch = s.charAt(i);
	    if (Character.isWhitespace(ch))
		str.append(' ');
	    else switch ( ch ) {
            case '<':
		str.append("&lt;");
		break;
            case '>':
		str.append("&gt;");
		break;
            case '&':
		str.append("&amp;");
		break;
            case '"':
		str.append("&quot;");
		break;
            default:
		str.append(ch);
	    }
	}
	return str.toString();
    }
    public static String escapeWDDXText(String s) {
	if (s==null) return "";
	final int len = s.length();
	StringBuffer str = new StringBuffer((len*3)/2);
	for (int i=0; i<len; i++) {
	    final char ch = s.charAt(i);
	    if (Character.isWhitespace(ch))
		str.append(' ');
	    else if (Character.isDigit(ch) || (ch>='a' && ch<='z') ||
		     (ch>='A' && ch<='Z') || ch=='/' || ch==':' || ch=='-'
		     || ch=='.' || ch==',' || ch=='_' || ch==';' || ch=='+'
		     || ch=='=' || ch=='-' || ch=='\\' || ch=='(' || ch==')')
		str.append(ch);
	    else {
		str.append("&amp;#");
		str.append((int)ch);
		str.append(';');
	    }
	}
	return str.toString();
    }
    public static final long MILLI = 1;
    public static final long SECOND = MILLI*1000;
    public static final long MINUTE = SECOND*60;
    public static final long HOUR = MINUTE*60;
    public static final long DAY = HOUR*24;
    public static final long WEEK = DAY*7;
    public static final long YEAR = (DAY*36525)/100;
    public static final long MONTH = YEAR/12;
    /*public static String formatTimeSpan(long ts) {
	StringBuffer sb = new StringBuffer();
	if (ts>=YEAR) {
	    long u = ts/YEAR;
	    span.append(u); span.append('y');
	    ts -= u*YEAR;
	}
	if (ts>=MONTH) {
	}
	}*/
    public static long parseTimeSpan(String ar) 
	throws ParseException {
	char[] arc = ar.toCharArray();
	int pos = 0;
	long span = 0;
	long factor = 1;
	while (pos<arc.length) {
	    if (arc[pos]=='+') {
		factor = 1; pos++;
	    } else if (arc[pos]=='-') {
		factor = -1; pos++;
	    } else {
		int letter = pos;
		while (letter<arc.length)
		    if (Character.isDigit(arc[letter]))
			letter++;
		    else if (Character.isLetter(arc[letter])) {
			if (letter==pos)
			    throw new ParseException("", letter);
			int nr;
			try {
			    nr = Integer.parseInt(ar.substring(pos, letter));
			} catch (NumberFormatException e) {
			    throw new ParseException(e.getLocalizedMessage(), pos);
			}
			switch (arc[letter]) {
			case 'y': span += factor*nr*YEAR; break;
			case 'M': span += factor*nr*MONTH; break;
			case 'd': span += factor*nr*DAY; break;
			case 'h': span += factor*nr*HOUR; break;
			case 'm': span += factor*nr*MINUTE; break;
			case 's': span += factor*nr*SECOND; break;
			default: throw new ParseException("Unknown unit: "+arc[letter], letter);
			}
			break;
		    } else
			throw new ParseException("", letter);
		if (letter>=arc.length)
		    throw new ParseException("", letter);
		pos = letter+1;
	    }
	}
	return span;
    }
    public static Date parseDateInput(String d) throws ParseException {
	d = trim(d);
	if (d==null)
	    return null;
	ParseException exception = null;
	for (int z=0; z<displayDateFormats.length; z++)
	    try {
		return displayDateFormats[z].parse(d);
	    } catch (ParseException e) {
		exception = e;
	    }
	throw exception;
    }
}

