package com.make.xml;

import java.util.Vector;
import java.util.Enumeration;
import java.util.Random;
import org.w3c.dom.Attr;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import com.make.util.Strings;

public class AttributeType {
    /***CONST******************************/
    public final static int T_TEXT=1;
    public final static int T_ID=2;
    public final static int T_IDREF=3;
    public final static int T_URL=4;
    /*public final static int T_INDEX=6;
      public final static int T_INDEXREF=7;
      public final static int T_INDEXNAME=8;*/
    public final static int T_EMBEDDED=9;
    public final static int T_TIMESTAMP=9;
    /***properties*****************************/
    protected static Random random = new Random();
    protected String name, def;
    protected boolean required;
    protected int type;
    /***Constructor******************************/
    public AttributeType(Element attr) {
	name = attr.getAttribute("name");
	required = "yes".equals(attr.getAttribute("required"));
	type = parseType(attr.getAttribute("type"));
	def = attr.getAttribute("default");
	if ("".equals(def)) def=null;
    }
    /***Info*********************************/
    public String getName() { return name; }
    public int getType() { return type; }
    public String createDefaultValue() {
	if (def!=null) return def;
	switch (type) {
	case T_ID: return ""+Math.abs(random.nextInt()); //TODO: long?
	default: return "";
	}
    }
    public boolean isReference() {
	return type==T_IDREF || type==T_URL;
    }
    public boolean isRequired() { return required; }
    public Attr createDefault(Document doc) {
	if (required || def!=null) {
	    Attr a = doc.createAttribute(name);
	    a.setValue(createDefaultValue());
	    return a;
	} else
	    return null;
    }
    /***internal********************************/
    protected int parseType(String type) {
	type = Strings.trim(type);
	if (type==null)
	    return T_TEXT;
	if (type.equals("text"))
	    return T_TEXT;
	else if (type.equals("id"))
	    return T_ID;
	else if (type.equals("idref"))
	    return T_IDREF;
	else if (type.equals("url"))
	    return T_URL;
	else if (type.equals("embedded"))
	    return T_EMBEDDED;
	else if (type.equals("timestamp"))
	    return T_TIMESTAMP;
	else
	    throw new IllegalArgumentException("invalid attribute type: "+type);
    }
}
