package com.make.xml;

import java.io.InputStream;
import java.io.Reader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.util.Hashtable;
import java.net.URL;
import java.net.MalformedURLException;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;
import org.xml.sax.EntityResolver;

public class ConfigPath {
    /***properties**********************/
    private URL path;
    private EntityResolver entityResolver;
    private Hashtable cache = new Hashtable();
    /***Constructor************************/
    public ConfigPath(URL _p, EntityResolver _er) {
	path = _p; entityResolver = _er;
    }
    /***Info*****************************/
    public URL get(String p) {
	try {
	    return path==null
		?null
		:new URL(path, p);
	} catch (MalformedURLException e) {
	    // if this is a path which doesn't exist, 
	    // a MalformedURLException is thrown - 
	    // in this case, we ignore the exception and return null
	    /*if (!p.endsWith("/")) {
	      System.err.println("while getting config: "+p);
	      e.printStackTrace();
	      }*/
	    return null;
	} catch (Exception e) {
	    System.err.println("while getting config: "+p);
	    e.printStackTrace();
	    return null;
	}
    }
    public InputStream open(String p) {
	try {
	    URL url = get(p);
	    return url==null
		?null
		:url.openStream();
	} catch (Exception e) {
	    System.err.println("while opening config: "+p);
	    e.printStackTrace();
	    return null;
	}
    }
    public Reader read(String p) {
	try {
	    InputStream is = open(p);
	    return is==null
		?null
		:new InputStreamReader(is);
	} catch (Exception e) {
	    System.err.println("while reading config: "+p);
	    e.printStackTrace();
	    return null;
	}
    }
    public Element parse(String p) {
	try {
	    InputStream is = open(p);
	    if (is==null)
		return null;
	    Document doc = DOMUtil.parse(is, entityResolver);
	    if (doc==null)
		return null;
	    return doc.getDocumentElement();
	} catch (Exception e) {
	    System.err.println("while parsing config: "+p);
	    e.printStackTrace();
	    return null;
	}
    }
    public Object getDescription(String p) {
	Object n = cache.get(p);
	if (n==null) {
	    n = parse(p+".xml");
	    if (n!=null && p.indexOf('?')==-1)
		putDescription(p, n);
	}
	return n;
    }
    public void putDescription(String p, Object o) {
	cache.put(p, o);
    }
}
