package com.make.xml;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Locale;
import java.io.IOException;
import java.io.InputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;
import java.io.Reader;
import java.net.URL;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Text;
import org.w3c.dom.Attr;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.DocumentHandler;
import org.xml.sax.Parser;
import org.xml.sax.EntityResolver;
import org.apache.xerces.parsers.DOMParser;
import org.apache.xerces.parsers.SAXParser;
import org.apache.xerces.dom.DocumentImpl;
import org.apache.xerces.dom.NodeImpl;
import com.make.util.Debug;
import com.make.util.Strings;

public abstract class DOMUtil {
    /***CONST********************************/
    public static final int DATETIME = Strings.DATETIME;
    public static final int DATE = Strings.DATE;
    public static final int TIME = Strings.TIME;
    public static final Locale locale = Locale.getDefault();
    public static final String language = locale.getLanguage();
    /***static********************************/
    public static final DateFormat[] dateFormat = Strings.storeDateFormats;
    public static final EntityResolver emptyEntityResolver = new EntityResolver() {
	    private byte[] emptyArray = {};
	    public InputSource resolveEntity(String publicId,
					     String systemId) {
		return new InputSource(new ByteArrayInputStream(emptyArray));
	    }
	};
    /***Static********************************/
    public static Element[] getChildElements(Element el) {
	if (el==null) return null;
	NodeList nl = el.getChildNodes();
	final int c = nl.getLength();
	Vector vc = new Vector(c);
	for (int z=0; z<c; z++) {
	    Node n = nl.item(z);
	    if (n instanceof Element)
		vc.addElement(n);
	}
	Element[] ret = new Element[vc.size()];
	for (int z=0; z<ret.length; z++)
	    ret[z] = (Element)vc.elementAt(z);
	return ret;
    }
    public static Element[] getChildElements(Element el, String tag) {
	if (el==null) return null;
	NodeList nl = el.getChildNodes();
	final int c = nl.getLength();
	Vector vc = new Vector(c);
	for (int z=0; z<c; z++) {
	    Node n = nl.item(z);
	    if (n instanceof Element && n.getNodeName().equals(tag))
		vc.addElement(n);
	}
	Element[] ret = new Element[vc.size()];
	for (int z=0; z<ret.length; z++)
	    ret[z] = (Element)vc.elementAt(z);
	return ret;
    } 
    public static Element getChildElementByAttribute(Element el, String name, String attribute, String compareTo){
	Element[] childs = DOMUtil.getChildElements(el,name);
	for(int i=0;i<childs.length;i++){
	    if (childs[i].getAttribute(attribute).equals(compareTo))
		return childs[i];
	}
	return null;
    }
    public static boolean childTagExists(Element el, String tag) {
	return el!=null && getChildElements(el, tag).length>0;
    }
    public static Element getUniqueTag(Element el, String tag) {
	if (el==null) return null;
	NodeList nl = el.getChildNodes();
	final int c = nl.getLength();
	for (int z=0; z<c; z++) {
	    Node n = nl.item(z);
	    if (n instanceof Element && n.getNodeName().equals(tag))
		return (Element)n;
	}
	return null;
    }
    public static Element getLocalizedUniqueTag(Element el, String tag) {
	Element[] ch = getChildElements(el, tag);
	if (ch==null || ch.length==0)
	    return null;
	if (language!=null)
	    for (int z=0; z<ch.length; z++)
		if (language.equals(ch[z].getAttribute("xml:lang")))
		    return ch[z];
	return ch[0];
    }
    public static Element createUniqueTag(Element el, String tag) {
	if (el==null) return null;
	Element ut = getUniqueTag(el, tag);
	if (ut!=null)
	    return ut;
	ut = el.getOwnerDocument().createElement(tag);
	el.appendChild(ut);
	return ut;
    }
    public static Element createDeepUniqueTag(Element el, String tag) {
	if (el==null) return null;
	if (tag==null) return el;
	StringTokenizer st = new StringTokenizer(tag, "/");
	while (el!=null && st.hasMoreTokens()) {
	    String t = Strings.trim(st.nextToken());
	    if (t!=null && !t.equals("."))
		el = createUniqueTag(el, t);
	}
	return el;
    }
    /*public static String getDeepNode(Node n, String path) {
      if (n==null || path==null) return null;
      StringTokenizer st = new StringTokenizer(tag, "/");
      while (el!=null && st.hasMoreTokens()) {
      String t = Strings.trim(st.nextToken());
      if (t==null || t.equals(".")) continue;
      if (t.startsWith("@")) {
      String attrName = t.substring(1);
      return Strings.trim(el.getAttribute(attrName));
      } else
      el = getUniqueTag(el, t);
      }
      return getElementString(el);
      }*/
    public static String getElementString(Element el) {
	if (el==null)
	    return null;
	Node child = el.getFirstChild();
	if (child==null)
	    return null;
	else if (child instanceof Text) {
	    String data = ((Text)child).getData();
	    for (child=child.getNextSibling(); child instanceof Text; 
		 child=child.getNextSibling())
		data += ((Text)child).getData();
	    return Strings.normalize(data, false, false);
	} else {
	    System.err.println("WARNING: text expected, found "+child.getNodeName());
	    return null;
	}
    }
    public static String getElementString(Element el, String tag) {
	return getElementString(getUniqueTag(el, tag));
    }
    public static String getLocalizedElementString(Element el, String tag) {
	return getElementString(getLocalizedUniqueTag(el, tag));
    }
    public static String getNodeText(Node n) {
	if (n==null)
	    return " ";
	if (n instanceof Text)
	    return n.getNodeValue();
	if (n.hasChildNodes()) {
	    StringBuffer sb = new StringBuffer();
	    NodeList nl = n.getChildNodes();
	    for (int z=0; z<nl.getLength(); z++)
		sb.append(getNodeText(nl.item(z)));
	    return sb.toString();
	}
	return " ";
    }
    public static String getDeepString(Element el, String tag) {
	if (el==null || tag==null) return null;
	StringTokenizer st = new StringTokenizer(tag, "/");
	while (el!=null && st.hasMoreTokens()) {
	    String t = Strings.trim(st.nextToken());
	    if (t==null || t.equals(".")) continue;
	    if (t.startsWith("@")) {
		String attrName = t.substring(1);
		return Strings.trim(el.getAttribute(attrName));
	    } else
		el = getUniqueTag(el, t);
	}
	return getElementString(el);
    }
    public static void setDeepString(Node n, String path, String value) {
	value = Strings.trim(value);
	StringTokenizer st = new StringTokenizer(path, "/");
	while (st.hasMoreTokens()) {
	    String t = Strings.trim(st.nextToken());
	    if (t==null || t.equals(".")) continue;
	    if (t.startsWith("@")) {
		String attrName = t.substring(1);
		if (value==null)
		    ((Element)n).removeAttribute(attrName);
		else
		    ((Element)n).setAttribute(attrName, value);
		return;
	    } else {
		Element nn = getUniqueTag((Element)n, t);
		if (nn==null) {
		    nn = n.getOwnerDocument().createElement(t);
		    n.appendChild(nn);
		}
		n = nn;
	    }
	}
	setElementString((Element)n, value);
    }
    public static void setElementString(Element el, String value) {
	while (el.hasChildNodes())
	    el.removeChild(el.getLastChild());
	if (value!=null)
	    el.appendChild(el.getOwnerDocument().createTextNode(value));
    }
    public static void removeElements(Element p, String tag) {
	Element[] ch = getChildElements(p, tag);
	for (int z=0; z<ch.length; z++)
	    p.removeChild(ch[z]);
    }
    public static void setElementString(Element el, String tag, String value) {
	if (value==null)
	    removeElements(el, tag);
	else
	    setElementString(createUniqueTag(el, tag), value);
    }
    public static Date getElementDate(Element el, int which)
	throws ParseException {
	String s = getElementString(el);
	return s==null?null:dateFormat[which].parse(s);
    }
    public static Date getElementDate(Element el, String tag, int which)
	throws ParseException {
	String s = getElementString(el, tag);
	return s==null?null:dateFormat[which].parse(s);
    }
    public static void setElementDate(Element el, Date date, int which) {
	setElementString(el, dateFormat[which].format(date));
    }
    public static void setElementDate(Element el, String tag, Date date, int which) {
	if (date==null)
	    removeElements(el, tag);
	else
	    setElementDate(createUniqueTag(el, tag), date, which);
    }
    /***Factory***************************************************/
    public static Document createDocument() {
	return new DocumentImpl();
    }
    public static Element createDocument(String tag) {
	Document doc = createDocument();
	Element root = doc.createElement(tag);
	doc.appendChild(root);
	return root;
    }
    public static Node cloneNode(Document doc, Node n, boolean recursive) {
	Node result;
	switch (n.getNodeType()) {
	case n.ATTRIBUTE_NODE:
	    result = doc.createAttribute(n.getNodeName());
	    break;
	case n.COMMENT_NODE:
	    result = doc.createComment(n.getNodeValue());
	    break;
	case n.DOCUMENT_NODE:
	    if (Debug.DEBUG && doc!=null)
		throw new InternalError("doc must be null for Document");
	    doc = createDocument();
	    result = doc;
	    break;
	case n.ELEMENT_NODE: {
	    Element el = (Element)n;
	    result = doc.createElement(el.getTagName());
	    NamedNodeMap as = el.getAttributes();
	    final int ac = as.getLength();
	    for (int z=0; z<ac; z++)
		((Element)result).setAttributeNode
		    ((Attr)cloneNode(doc, as.item(z), recursive));
	    break;
	}
	case n.ENTITY_REFERENCE_NODE:
	    result = doc.createEntityReference(n.getNodeName());
	    break;
	case n.TEXT_NODE:
	    result = doc.createTextNode(n.getNodeValue());
	    break;
	default:
	    throw new IllegalArgumentException("unknown node: "+n);
	}
	if (recursive) {
	    NodeList ch = n.getChildNodes();
	    final int cc = ch.getLength();
	    for (int z=0; z<cc; z++)
		result.appendChild(cloneNode(doc, ch.item(z), true));
	}
	return result;
    }
    public static Element createStringElement(Document doc, String tag, String text) {
	Element el = doc.createElement(tag);
	if (text!=null)
	    el.appendChild(doc.createTextNode(text));
	return el;
    }
    public static Element createDateElement(Document doc, String tag, Date v, int which) {
	return createStringElement(doc, tag, dateFormat[which].format(v));
    }
    /***UserData************************************/
    public static void setUserData(Node n, Object o) {
	((NodeImpl)n).setUserData(o);
    }
    public static void setUserData(Node n, Object key, Object o) {
	Hashtable ht = (Hashtable)getUserData(n);
	if (ht==null) {
	    ht = new Hashtable();
	    setUserData(n, ht);
	}
	ht.put(key, o);
    }
    public static Object getUserData(Node n) {
	return ((NodeImpl)n).getUserData();
    }
    public static Object getUserData(Node n, Object key) {
	Hashtable ht = (Hashtable)getUserData(n);
	return ht==null
	    ?null
	    :ht.get(key);
    }
    /***DocType**************************************/
    public static Document getOwnerDocument(Node n) {
	return n instanceof Document
	    ?(Document)n
	    :n.getOwnerDocument();
    }
    public static String getTypeName(Node n) {
	return getOwnerDocument(n).getDocumentElement().getTagName();
    }
    /***Parse****************************************/
    // simple parse
    public static Document parse(InputSource is) 
	throws IOException, SAXException {
	DOMParser parser = new DOMParser();
	parser.parse(is);
	Document doc = parser.getDocument();
	return doc;
    }
    public static Document parse(Reader is) 
	throws IOException, SAXException {
	return parse(new InputSource(is));
    }
    public static Document parse(InputStream is) 
	throws IOException, SAXException {
	return parse(new InputStreamReader(is, "ISO-8859-1"));
    }
    public static Document parse(byte[] file) 
	throws IOException, SAXException {
	return parse(new ByteArrayInputStream(file));
    }
    public static Document parse(File file) 
	throws IOException, SAXException {
	return parse(new FileInputStream(file));
    }
    public static Document parse(URL url) 
	throws IOException, SAXException {
	return parse(url.openStream());
    }
    // parse with dochandler
    public static void parse(InputSource is, DocumentHandler dh) 
	throws IOException, SAXException {
	Parser parser = new SAXParser();
	parser.setDocumentHandler(dh);
	parser.parse(is);
    }
    public static void parse(Reader is, DocumentHandler dh) 
	throws IOException, SAXException {
	parse(new InputSource(is), dh);
    }
    public static void parse(InputStream is, DocumentHandler dh) 
	throws IOException, SAXException {
	parse(new InputStreamReader(is, "ISO-8859-1"), dh);
    }
    // parse with entity resolver
    public static Document parse(InputSource is, EntityResolver er) 
	throws IOException, SAXException {
	DOMParser parser = new DOMParser();
	if (er!=null)
	    try {
		parser.setEntityResolver(er);
	    } catch (Throwable e) {
	    }
	parser.parse(is);
	Document doc = parser.getDocument();
	return doc;
    }
    public static Document parse(Reader is, EntityResolver er) 
	throws IOException, SAXException {
	return parse(new InputSource(is), er);
    }
    public static Document parse(InputStream is, EntityResolver er) 
	throws IOException, SAXException {
	return parse(new InputStreamReader(is, "ISO-8859-1"), er);
    }
    public static Document parse(byte[] file, EntityResolver er) 
	throws IOException, SAXException {
	return parse(new ByteArrayInputStream(file), er);
    }
    /***Serialize****************************************/
    public static byte[] toByteArray(Document doc) {
	try {
	    ByteArrayOutputStream baos = new ByteArrayOutputStream();
	    new DOMWriter(baos).print(doc);
	    return baos.toByteArray();
	} catch (IOException e) {
	    e.printStackTrace(); return null;
	}
    }
    public static byte[] toByteArray(Document doc, boolean cfuml) {
	try {
	    ByteArrayOutputStream baos = new ByteArrayOutputStream();
	    new DOMWriter(baos, cfuml).print(doc);
	    return baos.toByteArray();
	} catch (IOException e) {
	    e.printStackTrace(); return null;
	}
    }
    public static byte[] toByteArray(Document doc, DocumentType dt) {
	try {
	    ByteArrayOutputStream baos = new ByteArrayOutputStream();
	    DOMWriter dw = new DOMWriter(baos);
	    dw.setDocumentType(dt);
	    dw.print(doc);
	    return baos.toByteArray();
	} catch (IOException e) {
	    e.printStackTrace(); return null;
	}
    }
}
