package com.make.xml;

import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.net.URL;
import java.net.MalformedURLException;
import java.io.FileNotFoundException;
import java.io.InputStream;
import org.w3c.dom.Node;
import org.w3c.dom.Attr;
import org.w3c.dom.NodeList;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;
import org.xml.sax.EntityResolver;
import org.apache.xalan.xpath.XPath;
import org.apache.xalan.xpath.XObject;
import org.apache.xalan.xpath.XNull;
import com.make.util.Strings;
import com.make.xml.DOMUtil;

public class DocumentType extends ConfigPath {
    /***static*********************************/
    private static Hashtable pathCache = new Hashtable();
    /***Static********************************/
    public static XPath parseXPath(String path) throws SAXException {
	XPath pt = (XPath)pathCache.get(path);
	if (pt==null) {
	    pt = XPathUtil.parse(path);
	    pathCache.put(path, pt);
	}
	return pt;
    }
    /***propertes**********************************/
    protected String name, publicId, systemId;
    protected Hashtable elements = new Hashtable();
    protected Hashtable lists = new Hashtable();
    protected XPath idPath;
    private Element root;
    private URL path;
    /***Constructor*******************************/
    public DocumentType(URL _path, EntityResolver _er) 
	throws FileNotFoundException {
	super(_path, _er);
	root = parse("format.xml");
	if (root==null)
	    throw new FileNotFoundException("format.xml");
	Element head = DOMUtil.getUniqueTag(root, "head");
	name = DOMUtil.getElementString(head, "name");
	publicId = DOMUtil.getElementString(head, "public");
	systemId = DOMUtil.getElementString(head, "system");
	loadStructure(DOMUtil.getUniqueTag(root, "structure"));
	// idPath
	String ip = DOMUtil.getElementString(root, "id-path");
	if (ip!=null)
	    try {
		idPath = parseXPath(ip);
	    } catch (Exception e) {
		e.printStackTrace();
	    }
    }
    protected ElementType loadElementType(Element el) {
	return new ElementType(this, el);
    }
    protected void loadStructure(Element struc) {
	Element[] els = DOMUtil.getChildElements(struc, "element");
	for (int z=0; z<els.length; z++) {
	    ElementType et = loadElementType(els[z]);
	    elements.put(els[z].getAttribute("tag"), et);
	}
    }
    protected void loadLists(Element ts) {
	Element[] tsa = DOMUtil.getChildElements(ts);
	for (int z=0; z<tsa.length; z++) {
	    String name = tsa[z].getAttribute("name");
	    String tag = tsa[z].getTagName();
	    if (tag.equals("lookup-table"))
		lists.put(name, new LookupTable(tsa[z]));
	    else if (tag.equals("list")) {
		Element[] items = DOMUtil.getChildElements(tsa[z], "item");
		String[] list = new String[items.length];
		for (int a=0; a<items.length; a++)
		    list[a] = DOMUtil.getElementString(items[a]);
		lists.put(name, list);
	    } else
		System.err.println("WARNING: could not load list "+tag+" named "+name);
	}
    }
    /***Embedded********************************/
    protected void getEmbeddedNames(Vector vc, Node n) {
	if (n instanceof Element) {
	    Element el = (Element)n;
	    NamedNodeMap nnm = el.getAttributes();
	    final int ac = nnm.getLength();
	    for (int z=0; z<ac; z++) {
		Attr a = (Attr)nnm.item(z);
		ElementType et = getElementType(el.getTagName());
		if (et==null) continue;
		AttributeType at = et.getAttribute(a.getName());
		if (at==null) continue;
		if (at.getType()==at.T_EMBEDDED) {
		    String name = Strings.trim(a.getValue());
		    if (name!=null && !vc.contains(name))
			vc.addElement(name);
		}
	    }
	}
	NodeList nl = n.getChildNodes();
	final int cc = nl.getLength();
	for (int z=0; z<cc; z++)
	    getEmbeddedNames(vc, nl.item(z));
    }
    public String[] getEmbeddedNames(Node el) {
	Vector vc = new Vector();
	getEmbeddedNames(vc, el);
	String[] n = new String[vc.size()];
	for (int z=0; z<n.length; z++)
	    n[z] = (String)vc.elementAt(z);
	return n;
    }
    /***Info************************************/
    public String getName() { return name; }
    public String getPublicId() { return publicId; }
    public String getSystemId() { return systemId; }
    public Enumeration getElementTags() { return elements.keys(); }
    public ElementType getElementType(String tag) {
	return (ElementType)elements.get(tag);
    }
    public Element getMisc(String tag) {
	return DOMUtil.getUniqueTag(root, tag);
    }
    public Object getList(String name) {
	return lists.get(name);
    }
    public boolean equals(Object o) {
	if (o instanceof DocumentType)
	    return o==this ||
		((DocumentType)o).getName().equals(getName());
	else
	    return super.equals(o);
    }
    public Object extractId(Node el) {
	try {
	    XObject result = XPathUtil.execute(idPath, el);
	    return result instanceof XNull
		?null
		:Strings.trim(result.str());
	} catch (SAXException e) {
	    e.printStackTrace();
	    return null;
	}
    }
}
