package com.make.xml;

import java.util.Vector;
import java.util.Enumeration;
import java.util.Random;
import org.w3c.dom.Attr;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;
import org.w3c.dom.Document;
import com.make.util.Strings;

public class ElementType {
    static class Child {
	/***properties*************************/
	String tag;
	int policy;
	/***Constructor************************/
	public Child(Element el) {
	    tag = el.getAttribute("element");
	    policy = parsePolicy(el.getAttribute("policy"));
	}
	protected static int parsePolicy(String policy) {
	    policy = Strings.trim(policy);
	    if (policy==null)
		return P_ANY;
	    if (policy.equals("any"))
		return P_ANY;
	    if (policy.equals("more"))
		return P_MORE;
	    else if (policy.equals("optional"))
		return P_OPTIONAL;
	    else if (policy.equals("once"))
		return P_ONCE;
	    else {
		System.err.println("WARNING: unknown policy type: "+policy);
		return P_UNKNOWN;
	    }
	}
    }
    /***CONST********************/
    public final static int P_UNKNOWN=0;
    public final static int P_ANY=1;
    public final static int P_MORE=2;
    public final static int P_OPTIONAL=3;
    public final static int P_ONCE=4;
    public final static int O_GIVEN=1;
    public final static int O_ANY=2;
    public final static int O_FIXED=3;
    /***properties********************/
    protected DocumentType docType;
    protected String tag;
    protected int order;
    protected boolean mayContainText = false;
    protected boolean flowing = false;
    protected Child[] children;
    protected AttributeType[] attributes;
    /***Constructor********************/
    public ElementType(DocumentType _dt, Element et) {
	docType = _dt; tag = et.getAttribute("tag");
	order = parseOrder(et.getAttribute("order"));
	flowing = "yes".equals(et.getAttribute("flowing"));
	Element text = DOMUtil.getUniqueTag(et, "text");
	mayContainText = text!=null;
	Element[] cs = DOMUtil.getChildElements(et, "child");
	if (cs.length>0) {
	    children = new Child[cs.length];
	    for (int z=0; z<children.length; z++)
		children[z] = new Child(cs[z]);
	}
	Element[] as = DOMUtil.getChildElements(et, "attribute");
	if (as.length>0) {
	    attributes = new AttributeType[as.length];
	    for (int z=0; z<attributes.length; z++)
		attributes[z] = new AttributeType(as[z]);
	}
    }
    protected static int parseOrder(String order) {
	order = Strings.trim(order);
	if (order==null)
	    return O_GIVEN;
	if (order.equals("given"))
	    return O_GIVEN;
	else if (order.equals("any"))
	    return O_ANY;
	else if (order.equals("fixed"))
	    return O_FIXED;
	else
	    throw new IllegalArgumentException("unknown order type: "+order);
    }
    /***Info***************************************/
    public String getTag() { return tag; }
    public DocumentType getDocumentType() { return docType; }
    public boolean mayContainText() { return mayContainText; }
    public boolean isFlowing() { return flowing; }
    public String getDefaultChildTag() { 
	return children==null?null:children[0].tag;
    }
    public AttributeType[] getAttributes() { return attributes; }
    public AttributeType getAttribute(String name) {
	if (attributes!=null)
	    for (int z=0; z<attributes.length; z++)
		if (attributes[z].getName().equals(name))
		    return attributes[z];
	return null;
    }
    public int getOrder() { return order; }
    public boolean tagAllowed(String tag) {
	if (children==null) return false;
	for (int z=0; z<children.length; z++)
	    if (tag.equals(children[z].tag))
		return true;
	return false;
    }
    public int getPolicy(String tag) {
	if (children==null) return P_ANY;
	for (int z=0; z<children.length; z++)
	    if (tag.equals(children[z].tag))
		return children[z].policy;
	return P_UNKNOWN;
    }
    public Enumeration getAllowedTags() {
	return new Enumeration() {
	    protected int index = 0;
	    public boolean hasMoreElements() {
		return index<children.length;
	    }
	    public Object nextElement() {
		return children[index++].tag;
	    }
	};
    }
    public boolean isEmpty() {
	return !mayContainText && (children==null || children.length==0);
    }
    public boolean isTextOnly() {
	return mayContainText && (children==null || children.length==0);
    }
    public boolean attributesAllowed() {
	return attributes!=null && attributes.length>0;
    }
    /***Document*******************************/
    public Node createDefaultChild(Document doc) {
	if (mayContainText())
	    return doc.createTextNode("");
	else {
	    String tag = getDefaultChildTag();
	    if (tag==null)
		return null;
	    else
		return doc.createElement(tag);
	}
    }
    public void appendElement(Element parent, Element child) {
	if (!parent.getTagName().equals(tag))
	    throw new InternalError("parent.tag="+parent.getTagName()+" should be "+tag);
	if (order==O_FIXED) {
	    String childTag = child.getTagName();
	    int z;
	    for (z=0; z<children.length; z++)
		if (children[z].tag.equals(childTag)) {
		    for (;;) {
			z++;
			if (z==children.length)
			    break;
			Element[] tgs = DOMUtil.getChildElements(parent, children[z].tag);
			if (tgs.length>0) {
			    parent.insertBefore(child, tgs[0]);
			    return;
			}
		    }
		}
	}
	parent.appendChild(child);
    }
}
